<?php
	error_reporting(E_ALL ^ E_NOTICE);

	if($_SERVER['HTTP_USER_AGENT']) {
	    // Inside a Browser, Error Out
	    print "<html><head><title>CCS Setup Script</title></head><body>You must run this script from the command line.</body></html>";
	    exit(0);
	}

	fwrite(STDOUT,"Welcome to the CCS technical setup script.\n");

	// Check PHP Version
	if(substr(PHP_VERSION,0,1) < 5) { // Version 4 or less
	    print "You have PHP version ".PHP_VERSION.". You must have version 5 or later to run CCS/PIWI.";
	    exit(0);
	}

	// Function for collecting Options
	function getOption($caption,$filter = NULL,$default = NULL) {
		print "\n".str_repeat('*',60)."\n\n";
		$answer = '';
		fwrite(STDOUT,$caption);
		$answer = trim(fgets(STDIN));
		while(($default == NULL) && (strlen($answer) < 1)) {
			fwrite(STDOUT,"You must answer the question.\n");
			fwrite(STDOUT,$caption);
			$answer = trim(fgets(STDIN));
		}

		if(strlen($answer) < 1) // Can't happen unless there's a default
			$answer = $default;

		if(is_array($filter)) {
			while(!in_array($answer,$filter)) {
				fwrite(STDOUT,"You have entered an invalid answer.\n");
				fwrite(STDOUT,$caption);
				$answer = trim(fgets(STDIN));
			}
		}
		return $answer;
	}

	// Load CCS Options
	if (file_exists('./ccs_options.xml'))
	    $ccsOptions = simplexml_load_file('./ccs_options.xml');

	if($ccsOptions) {
	    $cwd = getcwd();
	    chdir('../projects/ccs');
	    // Load TCA Options
	    if($ccsOptions && $ccsOptions->{'tca-options'}->{'tca-etc'} && file_exists((string)$ccsOptions->{'tca-options'}->{'tca-etc'}))
		$tcaOptions = simplexml_load_file((string)$ccsOptions->{'tca-options'}->{'tca-etc'}.'/tucows-client.xml');

	    // Load Plat Options
	    if($ccsOptions && $ccsOptions->{'driver-options'}->{'billing-module'}->{'billing-options'}->{'wow-options-file'} && file_exists((string)$ccsOptions->{'driver-options'}->{'billing-module'}->{'billing-options'}->{'wow-options-file'}))
		$wowOptions = simplexml_load_file((string)$ccsOptions->{'driver-options'}->{'billing-module'}->{'billing-options'}->{'wow-options-file'});

	    chdir($cwd);
	}

	function loadOption($optionsFile,$path) {
	    if(!$optionsFile)
		return false;

	    $xmlPath = ''; // Create clear Path

	    $tags = explode("->",$path); // Split Path tags

	    // Build Path for eval
	    foreach($tags as $tag) {
		$xmlPath .= "->{'".$tag."'}";
	    }

	    return eval('return (string)$optionsFile'.$xmlPath.';');
	}

	function checkForService($optionsFile,$serviceName) {
	    $retval = false;
	    if($optionsFile->{'driver-options'}->{'service-modules'}->{'service-module'}) {
		foreach($optionsFile->{'driver-options'}->{'service-modules'}->{'service-module'} as $module) {
		    if(($module->{'name'} == $serviceName) && (strcasecmp((string)$module->{'active'},'true') == 0)) {
			$retval = true;
			break;
		    }
		}
	    }
	    return $retval;
	}

	function loadProvider($optionsFile,$providerName,$keyName) {
	    $retval = false;
	    if($optionsFile) {
			foreach($optionsFile->{'communication'}->{'providers'}->{'provider'} as $provider) {
				if(strcasecmp((string)$provider->{'name'},$providerName) == 0) {
					$ep = NULL;
					// find service endpoint
					foreach($optionsFile->{'communication'}->{'service-endpoint-list'}->{'service-endpoint'} as $endpoint) {
						if((string)$endpoint->{'name'} == (string)$provider->{'reseller-settings'}->{'service-endpoint-ref'}) {
						$ep = $endpoint;
						break;
						}
					}

					// construct retval array
					$retval = array('username'=>(string)$provider->{'reseller-settings'}->{'reseller-auth-info'}->{'username'},
							'password'=>(string)$provider->{'reseller-settings'}->{'reseller-auth-info'}->$keyName,
							'host' => (string)$ep->{'hostname'},
							'port' => (string)$ep->{'port'});
					if($ep->{'uri'})
						$retval['uri'] = (string)$ep->{'uri'};
				}
			}
	    }
	    return $retval;
	}

	$options = array(); // Collection of options
	$option = array();

	// Verbose Option (Hidden)
	$oldval = loadOption($ccsOptions,'driver-options->verbose');
	$option['question'] = "";
	$option['filter'] = array('Y','y','N','n');
	$option['default'] = 'no';
	$option['value'] = 'no';
	$option['hide'] = true;
	$options['verbose'] = $option;


	// Log Path
	$oldval = loadOption($ccsOptions,'driver-options->log-path');
	$option['question'] = "Where would you like your CCS log files created?\nThese files contain private information that should not be accessible through the web such as usernames and passwords.\nPlease ensure that the user running this script and the USER RUNNING YOUR WEB SERVER have write permission to the directory entered.".($oldval?"(".$oldval.")":"")."\n";
	$option['filter'] = NULL;
	$option['default'] = $oldval?$oldval:'../../etc';
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['log-path'] = $option;

	// TCA Path
	$oldval = loadOption($ccsOptions,'tca-options->tca-etc');
	$option['question'] = "Where would you like your CCS configuration files created?\nThese files contain private information that should not be accessible through the web such as usernames and keys.\nPlease ensure that the user running this script has write permission to the directory entered.".($oldval?"(".$oldval.")":"")."\n";
	$option['filter'] = NULL;
	$option['default'] = $oldval?$oldval:'../../etc';
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['tca-path'] = $option;

	/*
	// Plat Switch
	$oldval = loadOption($ccsOptions,'driver-options->billing-module->project-receiver');
	$option['question'] = "Are you using Platypus with this installation? ".(stristr($oldval,'plat')?"(Y/n)":"(N/y)")."\n";
	$option['filter'] = array('Y','y','N','n');
	$option['default'] = (stristr($oldval,'plat')?'Y':'N');
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = true;
	$options['plat-enabled'] = $option;
	*/

	//if ($option['value'] == 'Y' || $option['value'] == 'y') { // Plat Options

	// Plat Path
	$oldval = loadOption($ccsOptions,'driver-options->billing-module->billing-options->wow-options-file');
	if($oldval) // Strip filename
	$oldval = substr($oldval,0,strpos($oldval,"wow_options.xml")-1);
	$option['question'] = "Where would you like your wow_options.xml configuration file created?\nThis file contains a platypus staff username and password and should not be accessible through the web.\nPlease ensure that the user running this script has permission to the directory entered.".($oldval?" (".$oldval.")":"")."\n";
	$option['filter'] = NULL;
	$option['default'] = $oldval?$oldval:'../../etc';
	$option['value'] = $options['tca-path']['value'];
	$option['hide'] = true;
	$options['plat-path'] = $option;

	// Plat Username
	$oldval = loadOption($wowOptions,'wow-username');
	$option['question'] = "What is the username for the Platypus staff member used to gather rate information?".($oldval?" (".$oldval.")":"")."\n";
	$option['filter'] = NULL;
	$option['default'] = $oldval?$oldval:NULL;
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['plat-user'] = $option;

	// Plat Password
	$oldval = loadOption($wowOptions,'wow-password');
	$option['question'] = "What is the password for the Platypus staff member used to gather rate information?".($oldval?" (".$oldval.")":"")."\n";
	$option['filter'] = NULL;
	$option['default'] = $oldval?$oldval:NULL;
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['plat-pass'] = $option;

	// Plat IP
	$oldval = loadOption($wowOptions,'wow-address');
	$option['question'] = "What is the IP address of your WOW server? ".($oldval?"(".$oldval.")":"(127.0.0.1)")."\nNote: This is the windows server running the platypus WOW server.\n";
	$option['filter'] = NULL;
	$option['default'] = $oldval?$oldval:'127.0.0.1';
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['plat-ip'] = $option;

	// Plat Secure
	$oldval = loadOption($wowOptions,'wow-secure');
	$option['question'] = "Does your WOW server use an SSL connection? ".(strcasecmp($oldval,'true')?"(N/y)":"(Y/n)")."\nNote: This requires a public and private key pair to be installed on your WOW server in the System32 directory.\nThe files must be named: \nwowcert.pem (public key)\nwowprivkey.pem (private key).\n";
	$option['filter'] = array('Y','y','N','n');
	$option['default'] = strcasecmp($oldval,'true')?'N':'Y';
	$option['value'] = 'Y';//getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['plat-secure'] = $option;

	// Plat Port
	$oldval = loadOption($wowOptions,'wow-port');
	$option['question'] = "What is the server port of your WOW server? ".($oldval?"(".$oldval.")":"(5565/5566)")."\n";
	$option['filter'] = NULL;
	$option['default'] = $oldval?$oldval:5565;
	$option['value'] = ((strtoupper($options['plat-secure']['value']) == 'Y')?'5566':'5565'); // This option is no longer prompted for
	$option['hide'] = false;
	$option['hide'] = true;
	$options['plat-port'] = $option;
	//}

	// CCS URL prefix
	$oldval = loadOption($ccsOptions,'driver-options->url-prefix');
	$option['question'] = "What is the URL prefix for your pages ".(strcasecmp($oldval,'https')?"(http/https)":"(https/http)")."\nThis option is used to generate module URLs.\n";
	$option['filter'] = array('http','https');
	$option['default'] = strcasecmp($oldval,'https')?'http':'https';
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['ccs-url-prefix'] = $option;

	// Offering Domain?
	$oldval = checkForService($ccsOptions,'domain');
	$option['question'] = "Are you offering OpenSRS domains? ".($oldval?"(Y/n)":"(N/y)")."\n";
	$option['filter'] = array('Y','y','N','n');
	$option['default'] = $oldval?'Y':'N';
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['domain-enabled'] = $option;

	if ($option['value'] == 'Y' || $option['value'] == 'y') { // Domain Options

	    $oldpro = loadProvider($tcaOptions,'tucows-services','private-key');

	    // Domain Username
	    $oldval = $oldpro['username'];
	    $option['question'] = "What is your OpenSRS reseller username?".($oldval?" (".$oldval.")":"")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:NULL;
	    $option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    $option['hide'] = false;
	    $options['domain-username'] = $option;

	    // Domain Private Key
	    $oldval = $oldpro['password'];
	    $option['question'] = "What is your OpenSRS reseller private key?".($oldval?" (".$oldval.")":"")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:NULL;
	    $option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    $option['hide'] = false;
	    $options['domain-key'] = $option;

	    // Advanced Domain Options?
	    $option['question'] = "Would you like to change the default OpenSRS server settings?\nBy default, this installation will connect to the OpenSRS test system (horizon.opensrs.net). (N/y)\n";
	    $option['filter'] = array('Y','y','N','n');
	    $option['default'] = 'N';
	    $option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    $option['hide'] = false;
	    $option['hide'] = true;
	    $options['domain-advanced'] = $option;

	    // Domain Server Address
	    $oldval = $oldpro['host'];
	    $option['question'] = "What is the address to your OpenSRS host? ".($oldval?"(".$oldval.")":"(horizon.opensrs.net)")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:'horizon.opensrs.net';
	    if($options['domain-advanced']['value'] == 'Y' || $options['domain-advanced']['value'] == 'y')
		$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    else
		$option['value'] = $option['default'];
	    $option['hide'] = false;
	    $options['domain-hostname'] = $option;

	    // Domain Server Port
	    $oldval = $oldpro['port'];
	    $option['question'] = "What is the port number used on your OpenSRS host? ".($oldval?"(".$oldval.")":"(55443)")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:55443;
	    if($options['domain-advanced']['value'] == 'Y' || $options['domain-advanced']['value'] == 'y')
		$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    else
		$option['value'] = $option['default'];
	    $option['hide'] = false;
	    $options['domain-port'] = $option;
	} // Domain Options

	/*
	// Offering Blogware?
	$oldval = checkForService($ccsOptions,'blogware');
	$option['question'] = "Are you offering Tucows Blogware? ".($oldval?"(Y/n)":"(N/y)")."\n";
	$option['filter'] = array('Y','y','N','n');
	$option['default'] = $oldval?'Y':'N';
	$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	$option['hide'] = false;
	$options['blogware-enabled'] = $option;

	if ($option['value'] == 'Y' || $option['value'] == 'y') { // Blogware Options

	    $oldpro = loadProvider($tcaOptions,'tucows-blogware','password');

	    // Blogware Username
	    $oldval = $oldpro['username'];
	    $option['question'] = "What is your blogware reseller username?".($oldval?" (".$oldval.")":"")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:NULL;
	    $option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    $option['hide'] = false;
	    $options['blogware-username'] = $option;

	    // Blogware Password
	    $oldval = $oldpro['password'];
	    $option['question'] = "What is your blogware reseller password?".($oldval?" (".$oldval.")":"")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:NULL;
	    $option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    $option['hide'] = false;
	    $options['blogware-password'] = $option;

	    // Advanced Blogware Options?
	    $option['question'] = "Would you like to change the default blogware server settings? (N/y)\n";
	    $option['filter'] = array('Y','y','N','n');
	    $option['default'] = 'N';
	    $option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    $option['hide'] = false;
	    $options['blogware-advanced'] = $option;

	    // Blogware Server Address
	    $oldval = $oldpro['host'];
	    $option['question'] = "What is the address to your blogware host? ".($oldval?"(".$oldval.")":"(blogware.com)")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:'blogware.com';
	    if($options['blogware-advanced']['value'] == 'Y' || $options['blogware-advanced']['value'] == 'y')
		$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    else
		$option['value'] = $option['default'];
	    $option['hide'] = false;
	    $options['blogware-hostname'] = $option;

	    // Blogware Server Port
	    $oldval = $oldpro['post'];
	    $option['question'] = "What is the port number used on your blogware host? ".($oldval?"(".$oldval.")":"(443)")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:443;
	    if($options['blogware-advanced']['value'] == 'Y' || $options['blogware-advanced']['value'] == 'y')
		$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    else
		$option['value'] = $option['default'];
	    $option['hide'] = false;
	    $options['blogware-port'] = $option;

	    // Blogware Server URI
	    $oldval = $oldpro['uri'];
	    $option['question'] = "What is the URI on your blogware host? ".($oldval?"(".$oldval.")":"(/resellers/xmlrpc.cgi)")."\n";
	    $option['filter'] = NULL;
	    $option['default'] = $oldval?$oldval:'/resellers/xmlrpc.cgi';
	    if($options['blogware-advanced']['value'] == 'Y' || $options['blogware-advanced']['value'] == 'y')
		$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    else
		$option['value'] = $option['default'];
	    $option['hide'] = false;
	    $options['blogware-uri'] = $option;


	} // Blogware Options
	*/

	function checkForPlat($options) {
	    /*
		if(in_array($options['plat-enabled']['value'],array('Y','y')))
		    return true;
	    else
		    return false;
		*/
		return true;
	}

	if(checkForPlat($options)) {
	    // Helpdesk Options
	    $oldval = loadOption($ccsOptions,'helpdesk-options->staff-enabled');
	    $option['question'] = "Would you like to enable the Wombat helpdesk pages? ".($oldval?"(Y/n)":"(N/y)")."\n";
	    $option['filter'] = array('Y','y','N','n');
	    $option['default'] = $oldval?'Y':'N';
	    $option['value'] = getOption($option['question'],$option['filter'],$option['default']);
	    $option['hide'] = false;
	    $options['helpdesk-enabled'] = $option;

	    if ($option['value'] == 'Y' || $option['value'] == 'y') { // Help Desk Enabled
		// Staff Enabled
		$oldval = loadOption($ccsOptions,'helpdesk-options->staff-enabled');
		$option['question'] = "Grant staff access to the Wombat helpdesk pages?".($oldval?"(Y/n)":"(N/y)")."\n";
		$option['filter'] = array('Y','y','N','n');
		$option['default'] = $oldval?'Y':'N';
		$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
		$option['hide'] = false;
		$options['helpdesk-staff'] = $option;

		// Customer Enabled
		$oldval = loadOption($ccsOptions,'helpdesk-options->customer-enabled');
		$option['question'] = "Grant customers access to the Wombat helpdesk pages (customer access rights apply)?".($oldval?"(Y/n)":"(N/y)")."\n";
		$option['filter'] = array('Y','y','N','n');
		$option['default'] = $oldval?'Y':'N';
		$option['value'] = getOption($option['question'],$option['filter'],$option['default']);
		$option['hide'] = false;
		$options['helpdesk-customer'] = $option;

	    }
	}

	function writeTCAFile($options) {
	    // Check for existing file
	    $oDomOldTCA = NULL;
	    $opt = $options['tca-path'];
	    if($opt['value'] == '../../etc') // relative location to receiver
		    $filePath = './tucows-client.xml';
	    else
		    $filePath =$opt['value'].'/tucows-client.xml';

	    if (file_exists($filePath)) {
		$oDomOldTCA = new domDocument();
		$oDomOldTCA->load($filePath);
	    }

	    if(!$oDomOldTCA) { // New File
		$oDomTCA = new domDocument();
		$root = $oDomTCA->createElement('tucows-client-config');
		$oDomTCA->appendChild($root);

		// Much of this file is preset and does not use many of the user defined options.
		// For this reason, the function is poorly commented

		// Add communications
		$communication = $root->appendChild($oDomTCA->createElement('communication'));
		    $connectionTypeList = $communication->appendChild($oDomTCA->createElement('connection-type-list'));

			$connectionType = $connectionTypeList->appendChild($oDomTCA->createElement('connection-type'));
			$name = $connectionType->appendChild($oDomTCA->createElement('name'));
			$name->appendChild($oDomTCA->createTextNode('cbc'));
			$class= $connectionType->appendChild($oDomTCA->createElement('class'));
			$class->appendChild($oDomTCA->createTextNode('TCA_CbcConnector'));
			$payloadClass = $connectionType->appendChild($oDomTCA->createElement('payload-class'));
			$payloadClass->appendChild($oDomTCA->createTextNode('TCA_OPS'));

			$connectionType = $connectionTypeList->appendChild($oDomTCA->createElement('connection-type'));
			$name = $connectionType->appendChild($oDomTCA->createElement('name'));
			$name->appendChild($oDomTCA->createTextNode('https'));
			$class= $connectionType->appendChild($oDomTCA->createElement('class'));
			$class->appendChild($oDomTCA->createTextNode('TCA_HttpsConnector'));
			$payloadClass = $connectionType->appendChild($oDomTCA->createElement('payload-class'));
			$payloadClass->appendChild($oDomTCA->createTextNode('TCA_OPS'));

			$connectionType = $connectionTypeList->appendChild($oDomTCA->createElement('connection-type'));
			$name = $connectionType->appendChild($oDomTCA->createElement('name'));
			$name->appendChild($oDomTCA->createTextNode('xmlrpc'));
			$class= $connectionType->appendChild($oDomTCA->createElement('class'));
			$class->appendChild($oDomTCA->createTextNode('TCA_XMLRPCConnector'));
			$payloadClass = $connectionType->appendChild($oDomTCA->createElement('payload-class'));
			$payloadClass->appendChild($oDomTCA->createTextNode('TCA_XMLRPCBlogware'));

		    // Add Encryption
		    $encryptionTypeList = $communication->appendChild($oDomTCA->createElement('encryption-type-list'));

			$encryptionType = $encryptionTypeList->appendChild($oDomTCA->createElement('encryption-type'));
			$name = $encryptionType->appendChild($oDomTCA->createElement('name'));
			$name->appendChild($oDomTCA->createTextNode('nothing'));
			$class= $encryptionType->appendChild($oDomTCA->createElement('class'));
			$class->appendChild($oDomTCA->createTextNode('TCA_NullCipher'));

			$encryptionType = $encryptionTypeList->appendChild($oDomTCA->createElement('encryption-type'));
			$name = $encryptionType->appendChild($oDomTCA->createElement('name'));
			$name->appendChild($oDomTCA->createTextNode('blowfish'));
			$class= $encryptionType->appendChild($oDomTCA->createElement('class'));
			$class->appendChild($oDomTCA->createTextNode('TCA_BlowfishCipher'));

			$encryptionType = $encryptionTypeList->appendChild($oDomTCA->createElement('encryption-type'));
			$name = $encryptionType->appendChild($oDomTCA->createElement('name'));
			$name->appendChild($oDomTCA->createTextNode('des'));
			$class= $encryptionType->appendChild($oDomTCA->createElement('class'));
			$class->appendChild($oDomTCA->createTextNode('TCA_DesCipher'));

		    // idn Converter
		    $converter = $communication->appendChild($oDomTCA->createElement('idn-converter'));
		    $class= $converter->appendChild($oDomTCA->createElement('class'));
		    $class->appendChild($oDomTCA->createTextNode('IdnConverter'));
		    $host = $converter->appendChild($oDomTCA->createElement('hostname'));
		    $host->appendChild($oDomTCA->createTextNode('horizon.opensrs.net'));
		    $port= $converter->appendChild($oDomTCA->createElement('port'));
		    $port->appendChild($oDomTCA->createTextNode('55000'));

		    // Add Service Endpoints
		    $serviceEndpointList = $communication->appendChild($oDomTCA->createElement('service-endpoint-list'));

			$serviceEndpoint = $serviceEndpointList->appendChild($oDomTCA->createElement('service-endpoint'));

			    $name = $serviceEndpoint->appendChild($oDomTCA->createElement('name'));
			    $name->appendChild($oDomTCA->createTextNode('horizon-https'));
			    $type = $serviceEndpoint->appendChild($oDomTCA->createElement('type'));
			    $type->appendChild($oDomTCA->createTextNode('https'));

			    // Set by user
			    $opt = $options['domain-hostname'];
			    $hostname = $serviceEndpoint->appendChild($oDomTCA->createElement('hostname'));
			    $hostname->appendChild($oDomTCA->createTextNode($opt['value']));
			    $opt = $options['domain-port'];
			    $port = $serviceEndpoint->appendChild($oDomTCA->createElement('port'));
			    $port->appendChild($oDomTCA->createTextNode($opt['value']));

			    $connectionTypeRef = $serviceEndpoint->appendChild($oDomTCA->createElement('connection-type-ref'));
			    $connectionTypeRef->appendChild($oDomTCA->createTextNode('https'));
			    $encryptionTypeRef= $serviceEndpoint->appendChild($oDomTCA->createElement('encryption-type-ref'));
			    $encryptionTypeRef->appendChild($oDomTCA->createTextNode('nothing'));

			$serviceEndpoint = $serviceEndpointList->appendChild($oDomTCA->createElement('service-endpoint'));

			    $name = $serviceEndpoint->appendChild($oDomTCA->createElement('name'));
			    $name->appendChild($oDomTCA->createTextNode('blogware-xmlrpc'));
			    $type = $serviceEndpoint->appendChild($oDomTCA->createElement('type'));
			    $type->appendChild($oDomTCA->createTextNode('xmlrpc'));

			    // Set by user
			    $hostname = $serviceEndpoint->appendChild($oDomTCA->createElement('hostname'));
			    $hostname->appendChild($oDomTCA->createTextNode('blogware.com'));
			    $port = $serviceEndpoint->appendChild($oDomTCA->createElement('port'));
			    $port->appendChild($oDomTCA->createTextNode('443'));
			    $uri = $serviceEndpoint->appendChild($oDomTCA->createElement('uri'));
			    $uri->appendChild($oDomTCA->createTextNode('/resellers/xmlrpc.cgi'));

			    $connectionTypeRef = $serviceEndpoint->appendChild($oDomTCA->createElement('connection-type-ref'));
			    $connectionTypeRef->appendChild($oDomTCA->createTextNode('xmlrpc'));
			    $encryptionTypeRef= $serviceEndpoint->appendChild($oDomTCA->createElement('encryption-type-ref'));
			    $encryptionTypeRef->appendChild($oDomTCA->createTextNode('nothing'));

		    // Providers
		    $providers = $communication->appendChild($oDomTCA->createElement('providers'));

			$provider = $providers->appendChild($oDomTCA->createElement('provider'));

			    $name = $provider->appendChild($oDomTCA->createElement('name'));
			    $name->appendChild($oDomTCA->createTextNode('tucows-services'));

			    $reseller = $provider->appendChild($oDomTCA->createElement('reseller-settings'));
			    $resellerAuth = $reseller->appendChild($oDomTCA->createElement('reseller-auth-info'));
				$type = $resellerAuth->appendChild($oDomTCA->createElement('type'));
				$type->appendChild($oDomTCA->createTextNode('tucows-services'));
				$class = $resellerAuth->appendChild($oDomTCA->createElement('class'));
				$class->appendChild($oDomTCA->createTextNode('TCA_ResellerAuthInfoTucows'));

				// Set By User
				$opt = $options['domain-username'];
				$user = $resellerAuth->appendChild($oDomTCA->createElement('username'));
				$user->appendChild($oDomTCA->createTextNode($opt['value']));
				$opt = $options['domain-key'];
				$privateKey = $resellerAuth->appendChild($oDomTCA->createElement('private-key'));
				$privateKey->appendChild($oDomTCA->createTextNode($opt['value']));

			    $serviceEndpointRef = $reseller->appendChild($oDomTCA->createElement('service-endpoint-ref'));
			    $serviceEndpointRef->appendChild($oDomTCA->createTextNode('horizon-https'));

			$provider = $providers->appendChild($oDomTCA->createElement('provider'));

			    $name = $provider->appendChild($oDomTCA->createElement('name'));
			    $name->appendChild($oDomTCA->createTextNode('tucows-blogware'));

			    $reseller = $provider->appendChild($oDomTCA->createElement('reseller-settings'));
			    $resellerAuth = $reseller->appendChild($oDomTCA->createElement('reseller-auth-info'));
				$type = $resellerAuth->appendChild($oDomTCA->createElement('type'));
				$type->appendChild($oDomTCA->createTextNode('blogware'));
				$class = $resellerAuth->appendChild($oDomTCA->createElement('class'));
				$class->appendChild($oDomTCA->createTextNode('TCA_ResellerAuthInfoBlogware'));

				// Set By User
				$opt = $options['blogware-username'];
				$user = $resellerAuth->appendChild($oDomTCA->createElement('username'));
				$user->appendChild($oDomTCA->createTextNode($opt['value']));
				$opt = $options['blogware-password'];
				$privateKey = $resellerAuth->appendChild($oDomTCA->createElement('password'));
				$privateKey->appendChild($oDomTCA->createTextNode($opt['value']));

			    $serviceEndpointRef = $reseller->appendChild($oDomTCA->createElement('service-endpoint-ref'));
			    $serviceEndpointRef->appendChild($oDomTCA->createTextNode('blogware-xmlrpc'));

		    // Service Managers
		    $serviceManagers = $root->appendChild($oDomTCA->createElement('service-managers'));
			$serviceManager = $serviceManagers->appendChild($oDomTCA->createElement('service-manager'));
			    $name = $serviceManager->appendChild($oDomTCA->createElement('name'));
			    $name->appendChild($oDomTCA->createTextNode('TUCOWS_domains'));
			    $provider = $serviceManager->appendChild($oDomTCA->createElement('provider'));
			    $provider->appendChild($oDomTCA->createTextNode('tucows-services'));
			$serviceManager = $serviceManagers->appendChild($oDomTCA->createElement('service-manager'));
			    $name = $serviceManager->appendChild($oDomTCA->createElement('name'));
			    $name->appendChild($oDomTCA->createTextNode('TUCOWS_blogware'));
			    $provider = $serviceManager->appendChild($oDomTCA->createElement('provider'));
			    $provider->appendChild($oDomTCA->createTextNode('tucows-blogware'));

		// Write File
		$oDomTCA->formatOutput = true;
		$opt = $options['tca-path'];
		if($opt['value'] == '../../etc') // relative location to receiver
			$oDomTCA->save('./tucows-client.xml');
		else
			$oDomTCA->save($opt['value'].'/tucows-client.xml');
	    } else { // File already there
		$node = $oDomOldTCA->getElementsByTagName('service-endpoint');
		for($i = 0; $node->item($i); $i++) {
		    if($node->item($i)->getElementsByTagName('name')->item(0)->nodeValue == 'horizon-https') {
			// set Domain Connection
			$opt = $options['domain-hostname'];
			$node->item($i)->getElementsByTagName('hostname')->item(0)->nodeValue = $opt['value'];
			$opt = $options['domain-port'];
			$node->item($i)->getElementsByTagName('port')->item(0)->nodeValue = $opt['value'];
		    } /* elseif($node->item($i)->getElementsByTagName('name')->item(0)->nodeValue == 'blogware-xmlrpc') {
			// set Blogware connection
			$opt = $options['blogware-hostname'];
			$node->item($i)->getElementsByTagName('hostname')->item(0)->nodeValue = $opt['value'];
			$opt = $options['blogware-port'];
			$node->item($i)->getElementsByTagName('port')->item(0)->nodeValue = $opt['value'];
			$opt = $options['blogware-uri'];
			$node->item($i)->getElementsByTagName('uri')->item(0)->nodeValue = $opt['value'];
		    } */
		}

		$node = $oDomOldTCA->getElementsByTagName('provider');
		for($i = 0; $node->item($i); $i++) {
		    if($node->item($i)->getElementsByTagName('name')->item(0)->nodeValue == 'tucows-services') {
			// set Domain login info
			$authInfo = $node->item($i)->getElementsByTagName('reseller-auth-info');
			$opt = $options['domain-username'];
			$authInfo->item(0)->getElementsByTagName('username')->item(0)->nodeValue = $opt['value'];
			$opt = $options['domain-key'];
			$authInfo->item(0)->getElementsByTagName('private-key')->item(0)->nodeValue = $opt['value'];
		    } elseif($node->item($i)->getElementsByTagName('name')->item(0)->nodeValue == 'tucows-blogware') {
			// set Blogware login info
			$authInfo = $node->item($i)->getElementsByTagName('reseller-auth-info');
			$opt = $options['blogware-username'];
			$authInfo->item(0)->getElementsByTagName('username')->item(0)->nodeValue = $opt['value'];
			$opt = $options['blogware-password'];
			$authInfo->item(0)->getElementsByTagName('password')->item(0)->nodeValue = $opt['value'];
		    }
		}
		$oDomOldTCA->formatOutput = true;
		$oDomOldTCA->save($filePath);
	    } // File Exists?
	}

	function writeWOWFile($options) {
	    // Check for existing file
	    $oDomOldWOW = NULL;

	    $opt = $options['plat-path'];
	    if($opt['value'] == '../../etc') // relative location to receiver
		    $filePath = './wow_options.xml';
	    else
		    $filePath =$opt['value'].'/wow_options.xml';

	    if (file_exists($filePath)) {
		$oDomOldWOW = new domDocument();
		$oDomOldWOW->load($filePath);
	    }

	    if(!$oDomOldWOW) { // New File
		$oDomWOW = new domDocument();
		$root = $oDomWOW->createElement('wow-options');
		$oDomWOW->appendChild($root);

		// Add Address
		$address = $root->appendChild($oDomWOW->createElement('wow-address'));
		$opt = $options['plat-ip'];
		$address->appendChild($oDomWOW->createTextNode($opt['value']));

		// Add port
		$port= $root->appendChild($oDomWOW->createElement('wow-port'));
		$opt = $options['plat-port'];
		$port->appendChild($oDomWOW->createTextNode($opt['value']));

		// Add Secure
		$secure = $root->appendChild($oDomWOW->createElement('wow-secure'));
		$opt = $options['plat-secure'];
		$secure->appendChild($oDomWOW->createTextNode(in_array($opt['value'],array('Y','y'))?'true':'false'));

		// Add Username
		$user = $root->appendChild($oDomWOW->createElement('wow-username'));
		$opt = $options['plat-user'];
		$user->appendChild($oDomWOW->createTextNode($opt['value']));

		// Add Password
		$pass = $root->appendChild($oDomWOW->createElement('wow-password'));
		$opt = $options['plat-pass'];
		$pass->appendChild($oDomWOW->createTextNode($opt['value']));

		// Write File
		$oDomWOW->formatOutput = true;
		$opt = $options['plat-path'];
		if($opt['value'] == '../../etc') // relative location to receiver
			$oDomWOW->save('./wow_options.xml');
		else
			$oDomWOW->save($opt['value'].'/wow_options.xml');
	    } else { // File already there
		$oDomOldWOW->getElementsByTagName('wow-address')->item(0)->nodeValue = $options['plat-ip']['value'];
		$oDomOldWOW->getElementsByTagName('wow-port')->item(0)->nodeValue = $options['plat-port']['value'];
		$oDomOldWOW->getElementsByTagName('wow-secure')->item(0)->nodeValue = in_array($options['plat-secure']['value'],array('Y','y'))?'true':'false';
		$oDomOldWOW->getElementsByTagName('wow-username')->item(0)->nodeValue = $options['plat-user']['value'];
		$oDomOldWOW->getElementsByTagName('wow-password')->item(0)->nodeValue = $options['plat-pass']['value'];
		$oDomOldWOW->formatOutput = true;
		$oDomOldWOW->save($filePath);
	    } // File Exist?
	}

	function writeCCSFile($options) {
	    // Check for existing file
	    $oDomOldCCS = NULL;
	    $filePath = './ccs_options.xml';

	    if (file_exists($filePath)) {
		$oDomOldCCS = new domDocument();
		$oDomOldCCS->load($filePath);
	    }

	    if(!$oDomOldCCS) { // New File
		$oDomCCS = new domDocument();
		$root = $oDomCCS->createElement('ccs-options');
		$oDomCCS->appendChild($root);

		// Add Driver Options
		$driver_options = $root->appendChild($oDomCCS->createElement('driver-options'));

			// Verbose
			$url_prefix = $driver_options->appendChild($oDomCCS->createElement('verbose'));
			$url_prefix->appendChild($oDomCCS->createTextNode($options['verbose']['value']));

			// Log Path
			$url_prefix = $driver_options->appendChild($oDomCCS->createElement('log-path'));
			$url_prefix->appendChild($oDomCCS->createTextNode($options['log-path']['value']));

			// Add url prefix
			$url_prefix = $driver_options->appendChild($oDomCCS->createElement('url-prefix'));
			$opt = $options['ccs-url-prefix'];
			$url_prefix->appendChild($oDomCCS->createTextNode($opt['value']));

			// Add project receiver
			$project_receiver = $driver_options->appendChild($oDomCCS->createElement('project-receiver'));
			$project_receiver->appendChild($oDomCCS->createTextNode('/ccs/index.php'));

			// Add Log XML
			$log = $driver_options->appendChild($oDomCCS->createElement('capture-xml-communication-log'));
			$log->appendChild($oDomCCS->createTextNode('no'));

			// Add XML log file
			$log = $driver_options->appendChild($oDomCCS->createElement('xml-communication-log-file'));
			$log->appendChild($oDomCCS->createTextNode($options['log-path']['value'].'/xml_communication.log'));

			// Add Billing Module
			$billing_module = $driver_options->appendChild($oDomCCS->createElement('billing-module'));

				if(checkForPlat($options)) {
				    // Add project receiver
				    $project_receiver = $billing_module->appendChild($oDomCCS->createElement('project-receiver'));
				    $project_receiver->appendChild($oDomCCS->createTextNode('/modules/plat_billing.php'));

				    // Add Billing Options
				    $billing_options = $billing_module->appendChild($oDomCCS->createElement('billing-options'));
					$plat_path= $billing_options->appendChild($oDomCCS->createElement('wow-options-file'));
					$opt = $options['plat-path'];
					$plat_path->appendChild($oDomCCS->createTextNode($opt['value'].'/wow_options.xml'));

					$newnode = $billing_options->appendChild($oDomCCS->createElement('rate-auth'));
					$newnode->appendChild($oDomCCS->createTextNode('yes'));

					$newnode = $billing_options->appendChild($oDomCCS->createElement('service-preauth'));
					$newnode->appendChild($oDomCCS->createTextNode('yes'));
				} else {
				    // Add project receiver
				    $project_receiver = $billing_module->appendChild($oDomCCS->createElement('project-receiver'));
				    $project_receiver->appendChild($oDomCCS->createTextNode('/modules/base_billing.php'));

				    // Add Billing Options
				    $billing_options = $billing_module->appendChild($oDomCCS->createElement('billing-options'));
				}

			// Add Service Modules
			$service_modules = $driver_options->appendChild($oDomCCS->createElement('service-modules'));

				// Add a new module (domains)
				$service_module_domain = $service_modules->appendChild($oDomCCS->createElement('service-module'));

					// Add Name
					$name = $service_module_domain->appendChild($oDomCCS->createElement('name'));
					$name->appendChild($oDomCCS->createTextNode('domain'));

					// Add active flag
					$active = $service_module_domain->appendChild($oDomCCS->createElement('active'));
					$opt = $options['domain-enabled'];
					$active->appendChild($oDomCCS->createTextNode(in_array($opt['value'],array('Y','y'))?'true':'false'));

					// Add receiver
					$project_receiver = $service_module_domain->appendChild($oDomCCS->createElement('project-receiver'));
					$project_receiver->appendChild($oDomCCS->createTextNode('/modules/domain.php'));

					// Add desc
					$description = $service_module_domain->appendChild($oDomCCS->createElement('description'));
					$description->appendChild($oDomCCS->createTextNode('OpenSRS domain offerings'));

					// Add Manager Flag
					$manage = $service_module_domain->appendChild($oDomCCS->createElement('manage'));
					$manage->appendChild($oDomCCS->createTextNode('yes'));

					// Add service options
					$service_options = $service_module_domain->appendChild($oDomCCS->createElement('service-options'));

						// Add allow PW recovery
						$recovery = $service_options->appendChild($oDomCCS->createElement('allow-password-recovery'));
						$recovery->appendChild($oDomCCS->createTextNode('yes'));

						// Add allow PW updates
						$pwupdates = $service_options->appendChild($oDomCCS->createElement('allow-password-updates'));
						$pwupdates->appendChild($oDomCCS->createTextNode('yes'));

						// Add default status locked
						$allow_lock = $service_options->appendChild($oDomCCS->createElement('default-status-locked'));
						$allow_lock->appendChild($oDomCCS->createTextNode('yes'));

						// Add allow domain lock
						$allow_lock = $service_options->appendChild($oDomCCS->createElement('allow-domain-lock-change'));
						$allow_lock->appendChild($oDomCCS->createTextNode('yes'));

						// Add allow change Ownership
						$changeOwnership = $service_options->appendChild($oDomCCS->createElement('allow-change-ownership'));
						if(checkForPlat($options))
						    $changeOwnership->appendChild($oDomCCS->createTextNode('no'));
						else
						    $changeOwnership->appendChild($oDomCCS->createTextNode('yes'));

						// Add Custom Tech Info
						$cust_tech = $service_options->appendChild($oDomCCS->createElement('allow-custom-tech-info'));
						$cust_tech->appendChild($oDomCCS->createTextNode('yes'));

						// Add Custon NS
						$cust_ns = $service_options->appendChild($oDomCCS->createElement('allow-custom-nameservers'));
						$cust_ns->appendChild($oDomCCS->createTextNode('yes'));

						// Add Manage NS
						$manage_ns = $service_options->appendChild($oDomCCS->createElement('allow-manage-nameservers'));
						$manage_ns->appendChild($oDomCCS->createTextNode('yes'));

						// Add Register Ns
						$register_ns = $service_options->appendChild($oDomCCS->createElement('allow-register-nameservers'));
						$register_ns->appendChild($oDomCCS->createTextNode('yes'));

						// Add Renew Change
						$register_ns = $service_options->appendChild($oDomCCS->createElement('allow-renew-change'));
						$register_ns->appendChild($oDomCCS->createTextNode('no'));

						// Add Renew Default
						$register_ns = $service_options->appendChild($oDomCCS->createElement('allow-renew-default'));
						$register_ns->appendChild($oDomCCS->createTextNode('no'));

						// Add Handle Action
						$handle_action = $service_options->appendChild($oDomCCS->createElement('handle-action'));
						$handle_action->appendChild($oDomCCS->createTextNode('process'));

						// Add Whois Server
						$whois_server = $service_options->appendChild($oDomCCS->createElement('whois-server'));
						$whois_server->appendChild($oDomCCS->createTextNode('whois.opensrs.net'));

						// Add Related TLD Groups
						$relatedTLDs = $service_options->appendChild($oDomCCS->createElement('related-tld-groups'));

						// Add Groups
						$group = $relatedTLDs->appendChild($oDomCCS->createElement('group'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('default'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.com'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.net'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.org'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.info'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.biz'));

						$group = $relatedTLDs->appendChild($oDomCCS->createElement('group'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.eu'));

						$group = $relatedTLDs->appendChild($oDomCCS->createElement('group'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.ca'));

						$group = $relatedTLDs->appendChild($oDomCCS->createElement('group'));
						$tld = $group->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.mobi'));

						// Service Offerings
						$service_offering_list = $service_options->appendChild($oDomCCS->createElement('service-offering-list'));

						// Service Offering (.com)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.com'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('yes'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));

						// Service Offering (.net)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.net'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('yes'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));

						// Service Offering (.org)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.org'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('yes'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));

						// Service Offering (.ca)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.ca'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('yes'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('19.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('19.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('19.99'));

						// Service Offering (.biz)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.biz'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('yes'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));

						// Service Offering (.info)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.info'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('yes'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));

						// Service Offering (.eu)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.eu'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('no'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('13.00'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('13.00'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('13.00'));

						// Service Offering (.mobi)
						$service_offering = $service_offering_list->appendChild($oDomCCS->createElement('service-offering'));
						$tld = $service_offering->appendChild($oDomCCS->createElement('tld'));
						$tld->appendChild($oDomCCS->createTextNode('.mobi'));
						$allow_locking = $service_offering->appendChild($oDomCCS->createElement('allow-locking'));
						$allow_locking->appendChild($oDomCCS->createTextNode('yes'));
						$active = $service_offering->appendChild($oDomCCS->createElement('active'));
						$active->appendChild($oDomCCS->createTextNode('yes'));
						$price = $service_offering->appendChild($oDomCCS->createElement('price'));
						$attribute = $price->appendChild($oDomCCS->createElement('new'));
						$attribute->appendChild($oDomCCS->createTextNode('15.00'));
						$attribute = $price->appendChild($oDomCCS->createElement('renew'));
						$attribute->appendChild($oDomCCS->createTextNode('15.00'));
						$attribute = $price->appendChild($oDomCCS->createElement('transfer'));
						$attribute->appendChild($oDomCCS->createTextNode('15.00'));

				// Add a new module (blogware)
				$service_module_blogware = $service_modules->appendChild($oDomCCS->createElement('service-module'));

					// Add Charge Mode
					$name = $service_module_blogware->appendChild($oDomCCS->createElement('name'));
					$name->appendChild($oDomCCS->createTextNode('blogware'));

					// Add active flag
					$active = $service_module_blogware->appendChild($oDomCCS->createElement('active'));
					$opt = $options['blogware-enabled'];
					$active->appendChild($oDomCCS->createTextNode(in_array($opt['value'],array('Y','y'))?'true':'false'));

					// Add Caption
					$hostname = $service_module_blogware->appendChild($oDomCCS->createElement('caption'));
					$hostname->appendChild($oDomCCS->createTextNode('Blogware'));

					// Add receiver
					$project_receiver = $service_module_blogware->appendChild($oDomCCS->createElement('project-receiver'));
					$project_receiver->appendChild($oDomCCS->createTextNode('/modules/blogware.php'));

					// Add desc
					$description = $service_module_blogware->appendChild($oDomCCS->createElement('description'));
					$description->appendChild($oDomCCS->createTextNode('Tucows blogware offerings'));

					// Add Manager Flag
					$manage = $service_module_blogware->appendChild($oDomCCS->createElement('manage'));
					$manage->appendChild($oDomCCS->createTextNode('yes'));

					// Add service options
					$service_options = $service_module_blogware->appendChild($oDomCCS->createElement('service-options'));

						// Add allow PW recovery
						$recovery = $service_options->appendChild($oDomCCS->createElement('allow-password-recovery'));
						$recovery->appendChild($oDomCCS->createTextNode('yes'));

						// Add hostname
						$hostname = $service_options->appendChild($oDomCCS->createElement('hostname'));
						$hostname->appendChild($oDomCCS->createTextNode('blogware.com'));

						// Add Introductory Package
						$package = $service_options->appendChild($oDomCCS->createElement('package'));
						$attribute = $package->appendChild($oDomCCS->createElement('name'));
						$attribute->appendChild($oDomCCS->createTextNode('Miniblog'));
						$attribute = $package->appendChild($oDomCCS->createElement('active'));
						$attribute->appendChild($oDomCCS->createTextNode('true'));
						$attribute = $package->appendChild($oDomCCS->createElement('caption'));
						$attribute->appendChild($oDomCCS->createTextNode('Introductory Package'));
						$attribute = $package->appendChild($oDomCCS->createElement('trial'));
						$attribute->appendChild($oDomCCS->createTextNode('yes'));
						$attribute = $package->appendChild($oDomCCS->createElement('price'));
						$attribute->appendChild($oDomCCS->createTextNode('7.99'));
						$attribute = $package->appendChild($oDomCCS->createElement('includes'));
						$attribute->appendChild($oDomCCS->createTextNode("100MB Diskspace/1GB Bandwidth"));
						$attribute = $package->appendChild($oDomCCS->createElement('desc'));
						$attribute->appendChild($oDomCCS->createTextNode("The first of three packages , this blog comes with 100MB of diskspace and 1GB of bandwidth per month. Try us out for one month for free! It.s a perfect way to start blogging."));
						$upgrades = $package->appendChild($oDomCCS->createElement('upgrades'));
						$attribute = $upgrades->appendChild($oDomCCS->createElement('package'));
						$attribute->appendChild($oDomCCS->createTextNode('basic'));
						$attribute = $upgrades->appendChild($oDomCCS->createElement('package'));
						$attribute->appendChild($oDomCCS->createTextNode('Ultra'));

						// Add Basic Package
						$package = $service_options->appendChild($oDomCCS->createElement('package'));
						$attribute = $package->appendChild($oDomCCS->createElement('name'));
						$attribute->appendChild($oDomCCS->createTextNode('basic'));
						$attribute = $package->appendChild($oDomCCS->createElement('active'));
						$attribute->appendChild($oDomCCS->createTextNode('true'));
						$attribute = $package->appendChild($oDomCCS->createElement('caption'));
						$attribute->appendChild($oDomCCS->createTextNode('Basic Package'));
						$attribute = $package->appendChild($oDomCCS->createElement('trial'));
						$attribute->appendChild($oDomCCS->createTextNode('no'));
						$attribute = $package->appendChild($oDomCCS->createElement('price'));
						$attribute->appendChild($oDomCCS->createTextNode('9.99'));
						$attribute = $package->appendChild($oDomCCS->createElement('includes'));
						$attribute->appendChild($oDomCCS->createTextNode("1GB Diskspace/1GB Bandwidth"));
						$attribute = $package->appendChild($oDomCCS->createElement('desc'));
						$attribute->appendChild($oDomCCS->createTextNode("Our second package includes 1GB of diskspace and 1GB of bandwidth per month. This blog is ideal for the beginner blogger."));
						$upgrades = $package->appendChild($oDomCCS->createElement('upgrades'));
						$attribute = $upgrades->appendChild($oDomCCS->createElement('package'));
						$attribute->appendChild($oDomCCS->createTextNode('Ultra'));

						// Add Ultra Package
						$package = $service_options->appendChild($oDomCCS->createElement('package'));
						$attribute = $package->appendChild($oDomCCS->createElement('name'));
						$attribute->appendChild($oDomCCS->createTextNode('Ultra'));
						$attribute = $package->appendChild($oDomCCS->createElement('active'));
						$attribute->appendChild($oDomCCS->createTextNode('true'));
						$attribute = $package->appendChild($oDomCCS->createElement('caption'));
						$attribute->appendChild($oDomCCS->createTextNode('Ultra Package'));
						$attribute = $package->appendChild($oDomCCS->createElement('trial'));
						$attribute->appendChild($oDomCCS->createTextNode('no'));
						$attribute = $package->appendChild($oDomCCS->createElement('price'));
						$attribute->appendChild($oDomCCS->createTextNode('14.99'));
						$attribute = $package->appendChild($oDomCCS->createElement('includes'));
						$attribute->appendChild($oDomCCS->createTextNode("2GB Diskspace/2GB Bandwidth"));
						$attribute = $package->appendChild($oDomCCS->createElement('desc'));
						$attribute->appendChild($oDomCCS->createTextNode("Our final package . this is for the serious blogger. Loaded with all the same great features, it comes with 2GB of diskspace and 2GB of bandwidth!"));

		    // Add Email Options
		    $email_options = $driver_options->appendChild($oDomCCS->createElement('email-options'));

			// Add Send Confirmation Switch
			$emailoption = $email_options->appendChild($oDomCCS->createElement('send-confirmation'));
			$emailoption->appendChild($oDomCCS->createTextNode('no'));

			// Add Send Reseller Confirmation Switch
			$emailoption = $email_options->appendChild($oDomCCS->createElement('send-reseller-confirmation'));
			$emailoption->appendChild($oDomCCS->createTextNode('no'));

		    // Add TCA Options
		    $tca_options = $root->appendChild($oDomCCS->createElement('tca-options'));

			// Add TCA Home
			$tca_home = $tca_options->appendChild($oDomCCS->createElement('tca-home'));
			$tca_home->appendChild($oDomCCS->createTextNode('../../libs/tucows/api'));

			// Add TCA Cartridges
			$tca_cartridges = $tca_options->appendChild($oDomCCS->createElement('tca-cartridges'));
			$tca_cartridges->appendChild($oDomCCS->createTextNode('../../libs/tucows/api/cartridges'));

			// Add TCA Etc
			$tca_etc = $tca_options->appendChild($oDomCCS->createElement('tca-etc'));
			$opt = $options['tca-path'];
			$tca_etc->appendChild($oDomCCS->createTextNode($opt['value']));

		    // Add Helpdesk Options
		    if ($options['helpdesk-enabled']['value'] == 'Y' || $options['helpdesk-enabled']['value'] == 'y') { // Help Desk Enabled
			$helpdesk_options = $root->appendChild($oDomCCS->createElement('helpdesk-options'));

			// Add Staff Access
			$new_node = $helpdesk_options->appendChild($oDomCCS->createElement('staff-enabled'));
			$new_node->appendChild($oDomCCS->createTextNode(in_array($options['helpdesk-staff']['value'],array('Y','y'))?'true':'false'));

			// Add Customer Access
			$new_node = $helpdesk_options->appendChild($oDomCCS->createElement('customer-enabled'));
			$new_node->appendChild($oDomCCS->createTextNode(in_array($options['helpdesk-customer']['value'],array('Y','y'))?'true':'false'));
		    }
		    $oDomCCS->formatOutput = true;
		    $oDomCCS->save('./ccs_options.xml');

		} else { // File already there
		    $domainChangeOwnership = true;

		    $node = $oDomOldCCS->getElementsByTagName('billing-module')->item(0)->getElementsByTagName('project-receiver')->item(0);
		    if(checkForPlat($options)) {
			$node->nodeValue = '/modules/plat_billing.php';
			$node = $oDomOldCCS->getElementsByTagName('wow-options-file')->item(0);
			if($node) {
			    $node->nodeValue = $options['plat-path']['value'];
			} else {
			    $node = $oDomOldCCS->getElementsByTagName('billing-options')->item(0);
			    $plat_path= $node->appendChild($oDomOldCCS->createElement('wow-options-file'));
			    $plat_path->appendChild($oDomOldCCS->createTextNode($options['plat-path']['value'].'/wow_options.xml'));
			}

			// Fix Domain Change Ownership
			$domainChangeOwnership = false;
		    } else
			$node->nodeValue = '/modules/base_billing.php';

		    // Add Log Path if not there
		    $node = $oDomOldCCS->getElementsByTagName('log-path')->item(0);
		    if($node) { // Node is there
			$node->nodeValue = $options['log-path']['value'];
		    } else { // New Node
			// Add Log Path
			$driver_options = $oDomOldCCS->getElementsByTagName('driver-options')->item(0);
			$newnode = $driver_options->appendChild($oDomOldCCS->createElement('log-path'));
			$newnode->appendChild($oDomOldCCS->createTextNode($options['log-path']['value']));
		    }

		    //---- Add new TLDs if not there
		    //---- Get in the correct service module
		    $node = $oDomOldCCS->getElementsByTagName('service-module');
		    for ($i=0; $node->item($i); $i++){
			if ($node->item($i)->getElementsByTagName("name")->item(0)->nodeValue == "domain"){
			    $node = $node->item($i);
			    break;
			}
		    }
		    //---- get a reference to the groups
		    $nodeGroups = $node->getElementsByTagName("group");
		    //---- add .eu
		    $groupFound = false;
		    for ($i=0; $nodeGroups->item($i); $i++){
			if ($nodeGroups->item($i)->getElementsByTagName("tld")->item(0)->nodeValue == '.eu'){
			    $groupFound = true;
			    break;
			}
		    }
		    if (!$groupFound){
			$relatedTLDs = $node->getElementsByTagName('related-tld-groups')->item(0);
			$newnode     = $relatedTLDs->appendChild($oDomOldCCS->createElement('group'));
			$newnode->appendChild($oDomOldCCS->createElement('tld', '.eu'));
		    }
		    //---- get a reference to the services
		    $nodeServices = $node->getElementsByTagName("service-offering");
		    $serviceFound = false;
		    for ($i=0; $nodeServices->item($i); $i++){
			if ($nodeServices->item($i)->getElementsByTagName("tld")->item(0)->nodeValue == '.eu'){
			    $groupFound = true;
			    break;
			}
		    }
		    if (!$groupFound){
			$serviceOfferings  = $node->getElementsByTagName('service-offering-list')->item(0);
			$newnode           = $serviceOfferings->appendChild($oDomOldCCS->createElement('service-offering'));
			$newnode->appendChild($oDomOldCCS->createElement('tld', '.eu'));
			$newnode->appendChild($oDomOldCCS->createElement('allow-locking', 'no'));
			$newnode->appendChild($oDomOldCCS->createElement('active', 'yes'));
			$priceNode = $newnode->appendChild($oDomOldCCS->createElement('price'));
			$priceNode->appendChild($oDomOldCCS->createElement('new', '13.00'));
			$priceNode->appendChild($oDomOldCCS->createElement('renew', '13.00'));
			$priceNode->appendChild($oDomOldCCS->createElement('transfer', '13.00'));
		    }

		    //---- add .mobi
		    $groupFound = false;
		    for ($i=0; $nodeGroups->item($i); $i++){
			if ($nodeGroups->item($i)->getElementsByTagName("tld")->item(0)->nodeValue == '.mobi'){
			    $groupFound = true;
			    break;
			}
		    }
		    if (!$groupFound){
			$relatedTLDs = $node->getElementsByTagName('related-tld-groups')->item(0);
			$newnode     = $relatedTLDs->appendChild($oDomOldCCS->createElement('group'));
			$newnode->appendChild($oDomOldCCS->createElement('tld', '.mobi'));
		    }
		    //---- get a reference to the services
		    $nodeServices = $node->getElementsByTagName("service-offering");
		    $serviceFound = false;
		    for ($i=0; $nodeServices->item($i); $i++){
			if ($nodeServices->item($i)->getElementsByTagName("tld")->item(0)->nodeValue == '.mobi'){
			    $groupFound = true;
			    break;
			}
		    }
		    if (!$groupFound){
			$serviceOfferings  = $node->getElementsByTagName('service-offering-list')->item(0);
			$newnode           = $serviceOfferings->appendChild($oDomOldCCS->createElement('service-offering'));
			$newnode->appendChild($oDomOldCCS->createElement('tld', '.mobi'));
			$newnode->appendChild($oDomOldCCS->createElement('allow-locking', 'yes'));
			$newnode->appendChild($oDomOldCCS->createElement('active', 'yes'));
			$priceNode = $newnode->appendChild($oDomOldCCS->createElement('price'));
			$priceNode->appendChild($oDomOldCCS->createElement('new', '15.00'));
			$priceNode->appendChild($oDomOldCCS->createElement('renew', '15.00'));
			$priceNode->appendChild($oDomOldCCS->createElement('transfer', '15.00'));
		    }

		    // Add Verbose if not there
		    $node = $oDomOldCCS->getElementsByTagName('verbose')->item(0);
		    if($node) { // Node is there
			$node->nodeValue = $options['verbose']['value'];
		    } else { // New Node
			// Add Verbose
			$driver_options = $oDomOldCCS->getElementsByTagName('driver-options')->item(0);
			$newnode = $driver_options->appendChild($oDomOldCCS->createElement('verbose'));
			$newnode->appendChild($oDomOldCCS->createTextNode($options['verbose']['value']));
		    }

		    $oDomOldCCS->getElementsByTagName('url-prefix')->item(0)->nodeValue = $options['ccs-url-prefix']['value'];
		    $oDomOldCCS->getElementsByTagName('wow-options-file')->item(0)->nodeValue = $options['plat-path']['value'].'/wow_options.xml';

		    $mods = $oDomOldCCS->getElementsByTagName('service-module');

		    for($i = 0; $mods->item($i); $i++) {
			if($mods->item($i)->getElementsByTagName('name')->item(0)->nodeValue == 'domain') {
			    $mods->item($i)->getElementsByTagName('active')->item(0)->nodeValue = in_array($options['domain-enabled']['value'],array('Y','y'))?'true':'false';
			    // Fix Domain Change Ownership
			    $ownership = $mods->item($i)->getElementsByTagName('service-options')->item(0)->getElementsByTagName('allow-change-ownership')->item(0);
			    if(!$ownership) {// Create it
				$parentNode = $mods->item($i)->getElementsByTagName('service-options')->item(0);
				$ownership = $parentNode->appendChild($oDomOldCCS->createElement('allow-change-ownership'));
			    }

			    $ownership->nodeValue = ($domainChangeOwnership?'yes':'no');

			} elseif($mods->item($i)->getElementsByTagName('name')->item(0)->nodeValue == 'blogware') {
			    $mods->item($i)->getElementsByTagName('active')->item(0)->nodeValue = in_array($options['blogware-enabled']['value'],array('Y','y'))?'true':'false';
			    //$mods->item($i)->getElementsByTagName('hostname')->item(0)->nodeValue = $options['blogware-hostname']['value'];
			}
		    }

		    $oDomOldCCS->getElementsByTagName('tca-etc')->item(0)->nodeValue = $options['tca-path']['value'];

		    if ($options['helpdesk-enabled']['value'] == 'Y' || $options['helpdesk-enabled']['value'] == 'y') { // Help Desk Enabled
			// Add helpdesk if not there
			$node = $oDomOldCCS->getElementsByTagName('helpdesk-options')->item(0);
			if($node) { // Node is there
			    $node->getElementsByTagName("staff-enabled")->item(0)->nodeValue = in_array($options['helpdesk-staff']['value'],array('Y','y'))?'true':'false';
			    $node->getElementsByTagName("customer-enabled")->item(0)->nodeValue = in_array($options['helpdesk-customer']['value'],array('Y','y'))?'true':'false';
			} else { // New Node
			    $ccs_options = $oDomOldCCS->getElementsByTagName('ccs-options')->item(0);
			    // Add Helpdesk Options
			    $helpdesk_options = $ccs_options->appendChild($oDomOldCCS->createElement('helpdesk-options'));

			    // Add Staff Access
			    $new_node = $helpdesk_options->appendChild($oDomOldCCS->createElement('staff-enabled'));
			    $new_node->appendChild($oDomOldCCS->createTextNode(in_array($options['helpdesk-staff']['value'],array('Y','y'))?'true':'false'));

			    // Add Customer Access
			    $new_node = $helpdesk_options->appendChild($oDomOldCCS->createElement('customer-enabled'));
			    $new_node->appendChild($oDomOldCCS->createTextNode(in_array($options['helpdesk-customer']['value'],array('Y','y'))?'true':'false'));
			}
		    }

		    $oDomOldCCS->formatOutput = true;
		    $oDomOldCCS->save('./ccs_options.xml');

		} // File Exists?
	}

	$happy = false;
	$outstr = '';
	$answer = '';

	while(!$happy) {
	    fwrite(STDOUT,"\n\nPlease verify this information before proceeding.\nIf something is incorrect, enter the number of that option to correct it.  If all is correct, hit enter.\n");
	    $optionIndex = array();
	    $i = 0;
	    $key = $option = NULL;
	    foreach($options as $key=>$option) {
		if(!$option['hide']) {
		    $outstr = '('.$i.') '.$key.' => '.$option['value'];
		    if($i % 2 == 1) { // Left Side
			$outstr = str_pad($outstr, 50);
		    } else
			$outstr .= "\n"; //Add LF

		    fwrite(STDOUT,$outstr);
		    $optionIndex[$i] = $key;
		    $i++;
		}
	    }

	    fwrite(STDOUT,"\n:");

	    $answer = trim(fgets(STDIN));

	    if(strlen($answer) == 0)
		    $happy = true;
	    else {
		    if($optionIndex[$answer]) { // Don't need to check dependency b/c it wouldn't be displayed
			$options[$optionIndex[$answer]]['value'] = getOption($options[$optionIndex[$answer]]['question'],$options[$optionIndex[$answer]]['filter'],$options[$optionIndex[$answer]]['default']);
		    } else
			fwrite(STDOUT,"That was an invalid number.\n");
	    }
	}

	// Write Options
	writeCCSFile($options);

	if(checkForPlat($options))
		writeWOWFile($options);

	writeTCAFile($options);
?>
