<?php
require_once('acf/acf.inc.php');			// Includes all base classes

class ACF_ControllerXML extends ACF_Controller {
	private $projectpath;

	function __construct($initManagers, $layers)
	{
		parent::__construct(false, $layers);
		
		if($initManagers) {
			$this->processManager = new ACF_ProcessManagerXML($layers);
			//$this->activityManager = new ACF_ActivityManagerXML($layers);
			$this->executableWrapperManager = new ACF_ExecutableWrapperManagerXML($layers);
			$this->viewDefinitionManager = new ACF_ViewDefinitionManagerXML($layers);
		}
		return;
	} 

	function SetProjectPath($path) {
	    $this->projectpath = $path;
	    $this->executableWrapperManager->projectIdentifier = $this->projectpath; // Added for executable Caching (System wide)
	}

	function SaveFramework() {
		$indent = "    ";

		foreach($this->layers as $layer) {
			$xmlstring = "<?xml version=\"1.0\"?>\n<layer name=\"".$layer."\">\n";

			$xmlstring .= $indent."<event-responses>\n";
			$xmlstring .= $this->processManager->SaveResponsesToXML($layer);
			$xmlstring .= $indent."</event-responses>\n";

			$xmlstring .= $indent."<executables>\n";
			$xmlstring .= $this->executableWrapperManager->SaveExecutableWrappersToXML($layer);
			$xmlstring .= $indent."</executables>\n";

			$xmlstring .= $indent."<views>\n";
			$xmlstring .= $this->viewDefinitionManager->SaveViewDefinitionsToXML($layer);
			$xmlstring .= $indent."</views>\n";

			$xmlstring .= "</layer>\n";

			$filename = "../".$this->projectpath."/".$layer.".xml";
			$handle = fopen($filename, 'w');
			fwrite($handle, $xmlstring);
			fclose($handle);
		}
	}

	function LoadFramework() {
		foreach($this->layers as $layer) {
			$filename = "../".$this->projectpath."/".$layer.".xml";
			if (file_exists($filename)) {
				$xml = simplexml_load_file($filename);
				$this->processManager->LoadAllResponsesFromXML($xml,$layer);
				$this->executableWrapperManager->LoadAllExecutableWrappersFromXML($xml,$layer);
				$this->viewDefinitionManager->LoadAllViewDefinitionsFromXML($xml,$layer);
			}
		}
	}

	function CreateParameterList() {
		return new ACF_ParameterListXML();
	}
	
	function CreateInvocationList() {
		return new ACF_InvocationListXML();
	}
}

class ACF_ProcessManagerXML extends ACF_ProcessManager  {
	
	function __construct($layers)
	{
		parent::__construct($layers);
	}
	
	// Return one or more EventResponses used to modify the invocation stack
	// in response to this event
	
	function ResponseTo(ACF_EventSource $eventSource, ACF_Event $event)
	{
		$eventResponse = parent::ResponseTo($eventSource, $event);
		return $eventResponse;
	}

	function LoadAllResponsesFromXML($xml,$layer) {
		$mainindexname = "event-responses";
		$responses = $xml->$mainindexname;
   
		if($responses) { 
			if($responses->response) {
				foreach($responses->response as $response) {
					$indexname = "event-name";
					$eventname = (string) $response->$indexname;
					$event = new ACF_Event($eventname);

					$indexname = "source-realm";
					$sourcerealm = (string) $response->$indexname;
					$indexname = "source-name";
					$sourcename = (string) $response->$indexname;
					$eventSource = new ACF_EventSource($sourcerealm,$sourcename);

					$stackAdvice = new ACF_InvocationStackAdvice();
					$indexname = "popstack";
					$popstack = (string) $response->$indexname;
					$stackAdvice->SetPopStack($popstack == 'true');

					$indexname = "clearstack";
					$clearstack = (string) $response->$indexname;
					$stackAdvice->SetClearStack($clearstack == 'true');
				    
					// Load invocations, if there are any
					$invocationList = new ACF_InvocationListXML();
					$indexname = "invocations";
					$invocations = $response->$indexname;
					if($invocations)
						$invocationList->LoadSetFromXML($invocations);
								
					$newresponse = new ACF_EventResponse($stackAdvice, $invocationList);
					$this->AddResponse($eventSource, $event, $newresponse, $layer);
				}
			}
		}
	}

	function SaveResponsesToXML($currentlayer) {
		$xmlstring = "";

		// For Reading
		$baseindent = "        ";
		$indent = "    ";

		$responses = $this->Responses();
		foreach($responses as $layer => $realms) { // Loop through each response
			if($layer == $currentlayer) {
				foreach($realms as $sourcerealm => $sources) {
					foreach($sources as $sourcename	=> $events) {
						foreach($events as $eventname => $response) {

							$xmlstring .= $baseindent."<response>\n";
							$xmlstring .= $baseindent.$indent."<event-name>".$eventname."</event-name>\n";
							$xmlstring .= $baseindent.$indent."<source-realm>".$sourcerealm."</source-realm>\n";
							$xmlstring .= $baseindent.$indent."<source-name>".$sourcename."</source-name>\n";
							
							$StackAdvice = $response->InvocationStackAdvice();

							if($StackAdvice->ShouldPopStack())
								$xmlstring .= $baseindent.$indent."<popstack>true</popstack>\n";
							else
								$xmlstring .= $baseindent.$indent."<popstack>false</popstack>\n";

							if($StackAdvice->ShouldClearStack())
								$xmlstring .= $baseindent.$indent."<clearstack>true</clearstack>\n";
							else
								$xmlstring .= $baseindent.$indent."<clearstack>false</clearstack>\n";

							$xmlstring .= $baseindent.$indent."<invocations>\n";

							$invocationList = $response->InvocationList();
							$xmlstring .= $invocationList->SaveSetToXML(); // Save invocations;

							$xmlstring .= $baseindent.$indent."</invocations>\n";
							$xmlstring .= $baseindent."</response>\n";
						}
					}
				}
			}
		}
		return $xmlstring;
	}
}

class ACF_ExecutableWrapperManagerXML extends ACF_ExecutableWrapperManager  {
	public $projectIdentifier; // Project ID for Filename (Added by JCW 02/07/05)
	
	function __construct($layers)
	{
		parent::__construct($layers);
	}

	function LoadAllExecutableWrappersFromXML($xml,$layer) {
		$executables = $xml->executables;
		
		if($executables) { 
		    if($executables->executable) { 
			foreach($executables->executable as $executable) {
			    $name = (string)  $executable->name;
			    $indexname = "call-type";    
			    $callType = (string)  $executable->$indexname;
			    $indexname = "return-type";    
			    $returnType =  (string)  $executable->$indexname;

			    // Caching (Added JCW 02/07/05)
			    $indexname = "cache-timeout";    
			    $cacheTimeout =  (int)  $executable->$indexname;
			    $indexname = "cache-global";    
			    $cacheGlobal = ((string) $executable->$indexname == 'true')?true:false;

			    if(empty($returnType))
				    throw new ACF_Exception("Executable '$name' has no return type in database");
			    
			    switch($callType) {
			    case 'localphp' :
				    $indexname = "resource-name";
				    $resourcename = (string) $executable->$indexname;
				    $indexname = "resource-location";
				    $resourcelocation = (string) $executable->$indexname;
				    $wrapper = new ACF_LocalPHPWrapper($name, $returnType, $resourcelocation, $resourcename);
				    break;
			    case 'xmlrpc' :
				    $indexname = "server";
				    $server = (string) $executable->$indexname;
				    $indexname = "port";
				    $port= (string) $executable->$indexname;
				    $indexname = "uri";
				    $uri= (string) $executable->$indexname;
				    $indexname = "function-name";
				    $functionname = (string) $executable->$indexname;
				    $wrapper = new ACF_XMLRPCWrapper($name, $returnType, $server, $port, $uri, $functionname);
				    break;	    
			    case 'soap' :
				    $indexname = "serverpath";
				    $serverpath = (string) $executable->$indexname;
				    $indexname = "uri";
				    $uri= (string) $executable->$indexname;
				    $indexname = "function-name";
				    $functionname = (string) $executable->$indexname;
				    $wrapper = new ACF_SoapWrapper($name, $returnType, $serverpath, $uri, $functionname);
				    break;	    
			    default:
				throw new ACF_Exception("Unrecognized call type for executable '$name' in database: '$callType'");
			    }

			    $req_inparams = array();
			    $indexname = "req-inparams";
			    $subname = "req-inparam";
			    if($executable->$indexname->$subname) {
				    foreach($executable->$indexname->$subname as $param) {
					    array_push($req_inparams,(string) $param);
				    }
			    }

			    $opt_inparams = array();
			    $indexname = "opt-inparams";
			    $subname = "opt-inparam";
			    if($executable->$indexname->$subname) {
				    foreach($executable->$indexname->$subname as $param) {
					    array_push($opt_inparams,(string) $param);
				    }
			    }

			    $req_outparams = array();
			    $indexname = "req-outparams";
			    $subname = "req-outparam";
			    if($executable->$indexname->$subname) {
				    foreach($executable->$indexname->$subname as $param) {
					    array_push($req_outparams,(string) $param);
				    }
			    }

			    $opt_outparams = array();
			    $indexname = "opt-outparams";
			    $subname = "opt-outparam";
			    if($executable->$indexname->$subname) {
				    foreach($executable->$indexname->$subname as $param) {
					    array_push($opt_outparams,(string) $param);
				    }
			    }

			    $output_events = array();
			    $indexname = "output-events";
			    $subname = "output-event";
			    if($executable->$indexname->$subname) {
				    foreach($executable->$indexname->$subname as $event) {
					    array_push($output_events,(string) $event);
				    }
			    }

			    $wrapper->SetRequiredInputParameters($req_inparams);
			    $wrapper->SetOptionalInputParameters($opt_inparams);
			    $wrapper->SetRequiredOutputParameters($req_outparams);
			    $wrapper->SetOptionalOutputParameters($opt_outparams);

			    $wrapper->SetOutputEvents($output_events, $layer);

			    $wrapper->SetCachingOptions($cacheTimeout,$cacheGlobal);

			    $wrapper->projectIdentifier = $this->projectIdentifier;			
			    
			    $this->AddExecutableWrapper($wrapper,$layer); // Added Layer (JCW 8/10/04)
		    }
		}
	    }
	}
	
	function SaveExecutableWrappersToXML($currentlayer) {
		$xmlstring = "";

		// For Reading
		$baseindent = "        ";
		$indent = "    ";

		if($this->wrappers) {
			foreach($this->wrappers as $layer => $wraps) { // Loop through each wrapper
				if($layer == $currentlayer) {
					foreach($wraps as $wrapper) {
						$xmlstring .= $baseindent."<executable>\n";
						$xmlstring .= $baseindent.$indent."<name>".$wrapper->Name()."</name>\n";
						$xmlstring .= $baseindent.$indent."<call-type>".$wrapper->callType()."</call-type>\n";

						// Caching (Added JCW 02/07/05)
						$xmlstring .= $baseindent.$indent."<cache-timeout>".$wrapper->CacheTimeout()."</cache-timeout>\n";
						$xmlstring .= $baseindent.$indent."<cache-global>";

						if($wrapper->CacheGlobal())
							$xmlstring .= "true";
						else
							$xmlstring .= "false";
						$xmlstring .= "</cache-global>\n";

						switch($wrapper->callType()) {
						case 'localphp' :
							$xmlstring .= $baseindent.$indent."<resource-location>".$wrapper->filename()."</resource-location>\n";
							$xmlstring .= $baseindent.$indent."<resource-name>".$wrapper->FunctionName()."</resource-name>\n";
							break;
						case 'xmlrpc' :
							$xmlstring .= $baseindent.$indent."<server>".$wrapper->Server()."</server>\n";
							$xmlstring .= $baseindent.$indent."<port>".$wrapper->Port()."</port>\n";
							$xmlstring .= $baseindent.$indent."<uri>".$wrapper->URI()."</uri>\n";
							$xmlstring .= $baseindent.$indent."<function-name>".$wrapper->MethodName()."</function-name>\n";
							break;
						case 'soap' :
							$xmlstring .= $baseindent.$indent."<serverpath>".$wrapper->ServerPath()."</serverpath>\n";
							$xmlstring .= $baseindent.$indent."<uri>".$wrapper->URI()."</uri>\n";
							$xmlstring .= $baseindent.$indent."<function-name>".$wrapper->MethodName()."</function-name>\n";
							break;
						default:
							throw new ACF_Exception("Unrecognized call type for executable '$wrapper->Name()' in database: '$wrapper->callType()'");
						}

						// Required Input Parameters
						$xmlstring .= $baseindent.$indent."<req-inparams>\n";
						$params = $wrapper->Names('required_input');
						foreach($params as $p) {
							$xmlstring .= $baseindent.$indent.$indent."<req-inparam>".$p."</req-inparam>\n";
						}
						$xmlstring .= $baseindent.$indent."</req-inparams>\n";

						// Optional Input Parameters
						$xmlstring .= $baseindent.$indent."<opt-inparams>\n";
						$params = $wrapper->Names('optional_input');
						foreach($params as $p) {
							$xmlstring .= $baseindent.$indent.$indent."<opt-inparam>".$p."</opt-inparam>\n";
						}
						$xmlstring .= $baseindent.$indent."</opt-inparams>\n";

						// Required Output Parameters
						$xmlstring .= $baseindent.$indent."<req-outparams>\n";
						$params = $wrapper->Names('required_output');
						foreach($params as $p) {
							$xmlstring .= $baseindent.$indent.$indent."<req-outparam>".$p."</req-outparam>\n";
						}
						$xmlstring .= $baseindent.$indent."</req-outparams>\n";

						// Optional Output Parameters
						$xmlstring .= $baseindent.$indent."<opt-outparams>\n";
						$params = $wrapper->Names('optional_output');
						foreach($params as $p) {
							$xmlstring .= $baseindent.$indent.$indent."<opt-outparam>".$p."</opt-outparam>\n";
						}
						$xmlstring .= $baseindent.$indent."</opt-outparams>\n";

						$xmlstring .= $baseindent.$indent."<return-type>".$wrapper->ReturnType()."</return-type>\n";
						
						if($wrapper->ReturnType() == "event") {
							$xmlstring .= $baseindent.$indent."<output-events>\n";
							$outevents = $wrapper->Names('output_event');
							foreach($outevents as $oe) {
								$xmlstring .= $baseindent.$indent.$indent."<output-event>".$oe."</output-event>\n";
							}
							$xmlstring .= $baseindent.$indent."</output-events>\n";
						}

						$xmlstring .= $baseindent."</executable>\n";
					}
				}
			}
		}
		return $xmlstring;
	}
}

class ACF_ViewDefinitionManagerXML extends ACF_ViewDefinitionManager  {
	
	function __construct($layers=array('defaultlayer'))
	{
		parent::__construct($layers);
	}
	
	function LoadAllViewDefinitionsFromXML($xml,$layer)
	{
		$views = $xml->views;
    
		if($views) {
		    if($views->view) {
			foreach($views->view as $view) {
			    $name = (string) $view->name;
			    $definition = new ACF_ViewDefinition($name);

			    $req_inparams = array();
			    $indexname = "req-inparams";
			    $subname = "req-inparam";
			    foreach($view->$indexname->$subname as $param) {
				    array_push($req_inparams,(string) $param);
			    }
			    $definition->SetRequiredInputParameters($req_inparams);

			    $opt_inparams = array();
			    $indexname = "opt-inparams";
			    $subname = "opt-inparam";
			    foreach($view->$indexname->$subname as $param) {
				    array_push($opt_inparams,(string) $param);
			    }
			    $definition->SetOptionalInputParameters($opt_inparams);

			    $dataCollectorList = new ACF_InvocationListXML($this->db);

			    $datacollectors = $view->datacollectors;
			    if($datacollectors)
				    $dataCollectorList->LoadSetFromXML($datacollectors);

			    $definition->SetDataCollectorList($dataCollectorList);
			    $this->AddViewDefinition($definition,$layer);
			}
		    }
		}
	}
	
	function SaveViewDefinitionsToXML($currentlayer) {
		$xmlstring = "";

		// For Reading
		$baseindent = "        ";
		$indent = "    ";

		$definitions = $this->Definitions();
		foreach($definitions as $layer => $defs) { // Loop through each difinition
			if($layer == $currentlayer) {
				foreach($defs as $definition) {
					$xmlstring .= $baseindent."<view>\n";
					$xmlstring .= $baseindent.$indent."<name>".$definition->Name()."</name>\n";

					// Required Input Parameters
					$xmlstring .= $baseindent.$indent."<req-inparams>\n";
					$params = $definition->Names('required_input');
					foreach($params as $p) {
						$xmlstring .= $baseindent.$indent.$indent."<req-inparam>".$p."</req-inparam>\n";
					}
					$xmlstring .= $baseindent.$indent."</req-inparams>\n";

					// Optional Input Parameters
					$xmlstring .= $baseindent.$indent."<opt-inparams>\n";
					$params = $definition->Names('optional_input');
					foreach($params as $p) {
						$xmlstring .= $baseindent.$indent.$indent."<opt-inparam>".$p."</opt-inparam>\n";
					}
					$xmlstring .= $baseindent.$indent."</opt-inparams>\n";

					$xmlstring .= $baseindent.$indent."<datacollectors>\n";
					$Datacollectors = $definition->DataCollectors();
					$xmlstring .= $Datacollectors->SaveSetToXML(); // Save Datacollectors
					$xmlstring .= $baseindent.$indent."</datacollectors>\n";

					$xmlstring .= $baseindent."</view>\n";
				}
			}
		}
		return $xmlstring;
	}
}

// Responsible for parameter persistency

class ACF_ParameterListXML extends ACF_ParameterList
{
	function __construct ()
	{
		parent::__construct();
	}
	
	function LoadSetFromXML($parameters)
	{
		foreach($parameters->parameter as $parameter) {
			$name = (string) $parameter->name;
			$parameterType = (string) $parameter->type;
			
			switch($parameterType) {
			case 'direct' :
				$value = (string) $parameter->value;
				$parameter = new ACF_DirectParameter($name, $value);
				break;
				
			case 'indirect' :
				$reference = (string) $parameter->reference;
				if((string) $parameter->optional == 'true')
					$optional = true;
				else
					$optional = false;
				$parameter = new ACF_IndirectParameter($name, $reference, $optional);
				break;
				
			default:
				throw new ACF_Exception("Unrecognized parameter type for parameter '$name' in database: '$parameterType'");
			}
			$this->AddParameter($parameter);
		}
	}

	function SaveSetToXML() {
		$xmlstring = "";

		// For Reading
		$baseindent = "                        ";
		$indent = "    ";

		$parameters = $this->ParametersArray();
		if($parameters) {
			foreach($parameters as $param) { // Loop for each parameter in list
				$xmlstring .= $baseindent."<parameter>\n";
				$xmlstring .= $baseindent.$indent."<name>".$param->Name()."</name>\n";
				
				if($param->IsDirect()) { // Direct, insert value
					$xmlstring .= $baseindent.$indent."<type>direct</type>\n";
					$xmlstring .= $baseindent.$indent."<value>".$param->Value()."</value>\n";
				} else {
					$xmlstring .= $baseindent.$indent."<type>indirect</type>\n";
					$xmlstring .= $baseindent.$indent."<reference>".$param->Reference()."</reference>\n";
					if($param->IsOptional())
					    $xmlstring .= $baseindent.$indent."<optional>true</optional>\n";
					else
					    $xmlstring .= $baseindent.$indent."<optional>false</optional>\n";
				}

				$xmlstring .= $baseindent."</parameter>\n";
			}
		}

		return $xmlstring;
	}
}

// Responsible for data collector set persistency
 
class ACF_InvocationListXML extends ACF_InvocationList {
	
	function __construct ()
	{
		parent::__construct();
	}
	
	function LoadSetFromXML($invocations)
	{
		foreach($invocations->invocation as $invocation) {
			
			$indexname = "task-name";
			$taskName = (string) $invocation->$indexname;
			$indexname = "task-type";
			$taskType =  (string) $invocation->$indexname;
			
			$parameterList = new ACF_ParameterListXML();
			$parameters = $invocation->parameters;
			$parameterList->LoadSetFromXML($parameters);

			switch($taskType) {
			case 'view' :
				$invocation = new ACF_ViewInvocation($taskName, $parameterList);
				break;
			case 'activity' :
				$invocation = new ACF_ActivityInvocation($taskName, $parameterList);
				break;
			case 'datacollector' :
				$invocation = new ACF_DataCollectorInvocation($taskName, $parameterList);
				break;
			default:
				throw new ACF_Exception("Unrecognized type for invocation $taskName' in database: '$taskType'");
			}
			
			$this->AddInvocation($invocation);
		}
	}


	function SaveSetToXML() {
		$xmlstring = "";

		// For Reading
		$baseindent = "                ";
		$indent = "    ";

		$invocations = $this->InvocationsArray();

		if($invocations) {
			foreach($invocations as $invocation) { // Loop for each invocation in list (already ordered by sequence)
				$xmlstring .= $baseindent."<invocation>\n";
				$xmlstring .= $baseindent.$indent."<task-type>".$invocation->TaskType()."</task-type>\n";
				$xmlstring .= $baseindent.$indent."<task-name>".$invocation->TaskName()."</task-name>\n";

				$xmlstring .= $baseindent.$indent."<parameters>\n";
				$parameterList = $invocation->ParameterList();
				$xmlstring .= $parameterList->SaveSetToXML();	// Save Parameters
				$xmlstring .= $baseindent.$indent."</parameters>\n";
				$xmlstring .= $baseindent."</invocation>\n";
			}
		} // Need to report error otherwise

		return $xmlstring;
	}
}
?>
