<?php

// This class represents the source of an event

class ACF_EventSource {
	private $realm;	// internal or external
	private $name;
	
	function __construct($realm, $name)
	{
	 	$this->SetRealm($realm);
	 	$this->SetName($name);
	}
	
	function SetRealm($realm)
	{
		if($realm!='internal' && $realm!='external')
			throw new ACF_Exception('Realm name must be "internal" or "external"');
		$this->realm = $realm;
	}
	
	function SetName($name)
	{
		if(empty($name))
			throw new ACF_Exception('ACF_EventSource name may not be empty');
		$this->name = $name;
	}
	
	function Realm() {
		return $this->realm;
	}
	
	function Name() {
		return $this->name;
	}
}

// This class holds an event for the controller to respond to
class ACF_Event {
	private $name;
	private $parameterList;
	
	function __construct($name, $parameterList=NULL)
	{
		$this->SetName($name);
		if($parameterList)
			$this->SetParameterList($parameterList);
		else
			$this->SetParameterList(new ACF_ParameterList());
	}
	
	function SetName($name) {
		if(empty($name))
			throw new ACF_Exception('Event name may not be null');
		return $this->name = $name;
	}
	
	function SetParameterList(ACF_ParameterList $parameterList) {
		if(!$parameterList)
			throw new ACF_Exception('parameterList may be empty but may not be null');
		return $this->parameterList = $parameterList;
	}
	
	function ParameterList() {
		return $this->parameterList;
	}
	 	    	
	function Name() {
		return $this->name;
	}
}

// This class is returned by the decoder and passed to the controller for processing

class ACF_Request {
	private $event;
	private $eventSource;
	
	function __construct(ACF_EventSource $eventSource, ACF_Event $event)
	{
		$this->SetEvent($event);
		$this->SetEventSource($eventSource);
	}
	
	function SetEvent(ACF_Event $event) 	{
		$this->event = $event;
	}
	
	function SetEventSource(ACF_EventSource $eventSource) 	{
		$this->eventSource = $eventSource;
	}
		
	function Event() 	{
		return $this->event;
	}
	
	function EventSource() 	{
		return $this->eventSource;
	}
} 
?>
