<?php

class ACF_LocalPHPWrapper extends ACF_ExecutableWrapper {
    private $fileName;
    private $functionName;
    
    function __construct($name, $returnType, $fileName, $functionName)
    {
        parent::__construct($name, $returnType);
        $this->SetFunction($functionName);
        $this->SetFileName($fileName);
        $this->callType = 'localphp';
    }
    
    function SetFunction($functionName) {   // Call this for sure
        $this->functionName = trim($functionName);
    }
    
    function SetFileName($fileName)         // Call this if source file may not have been included
    {
        $this->fileName = trim($fileName);
    }
    
    function _Execute(ACF_ParameterList $parameterList)
    {
        if($this->fileName)
            require_once($this->fileName);
        $functionName = $this->functionName;

        return $functionName($parameterList);
    }
    
    function Validate()
    {
        if(empty($this->functionName))
            throw new ACF_Exception('NULL function name in '.__CLASS__);

        if(empty($this->fileName)) {        // Okay if the function has been included for us
            if(!function_exists($this->functionName))
                throw new ACF_Exception('No include file specified and function does not exist in '.__CLASS__);
        }
    }

    function FunctionName() {
        return $this->functionName;
    }

    function FileName() {
        return $this->fileName;
    }

}
?>
