<?php
class ACF_SoapWrapper extends ACF_ExecutableWrapper {
    private $serverpath;
    private $uri;
    private $methodName;

    function __construct($name, $returnType, $serverPath, $uri, $methodname)
    {
        parent::__construct($name, $returnType);
        $this->SetServerPath($serverPath);
        $this->SetURI($uri);
        $this->SetMethodName($methodname);
        $this->callType = 'soap';
    }
    
    function SetServerPath($serverPath) {
        $this->serverpath = trim($serverPath);
    }
    
    function SetURI($uri) {
        $this->uri = trim($uri);
    }
    
    function SetMethodName($method) {
        $this->methodName = trim($method);
    }
    
    function ServerPath() {
        return $this->serverpath;
    }
    
    function URI() {
        return $this->uri;
    }

    function MethodName() {
        return $this->methodName;
    }

    function _Execute(ACF_ParameterList $parameterList) {
        $client = new SoapClient(NULL,array(
            "location" => $this->serverpath,
            "uri"      => $this->uri,
            "style"    => SOAP_RPC,
            "use"      => SOAP_ENCODED));

        // Build Parameters
        $inputParameters = array();
        $params = $parameterList->ParametersArray();
        foreach($params as $param) {
            array_push($inputParameters,new SoapParam($param->Value(),$param->Name()));
        }

        try {
            $parameters = $client->__call($this->methodName,$inputParameters);
        } catch(Exception $e) {
            throw new ACF_Exception('Error Executing Soap Call: '.$this->methodName.' Error: '.$e->faultstring);
        }

        if($parameters) {
            foreach($parameters as $name => $value) {
                if($name == 'ACF_Return_Event')
                    $retEvent = new ACF_Event($value);
                else {
                    if(!$retparameterList)
                        $retparameterList = new ACF_ParameterList();
                    $retparameterList->AddParameter(new ACF_DirectParameter($name, $value));
                }
            }
        }

        if($retEvent) { 
            $retEvent->SetParameterList($retparameterList);
            return $retEvent;
        }

        return $retparameterList;
    }
    
    function Validate()
    {
        if(empty($this->serverpath))
            throw new ACF_Exception('NULL ServerPath in '.__CLASS__);

        if(empty($this->uri))
            throw new ACF_Exception('NULL URI in '.__CLASS__);

        if(empty($this->methodName))
            throw new ACF_Exception('NULL Method Name in '.__CLASS__);
    }
}
?>
