<?php

function xmlrpc_execute($host, $uri, $port, $request) {
    $fp = fsockopen($host, $port, $errno, $errstr);

    $query = "POST ".$uri." HTTP/1.0\r\n";
    $query .= "User_Agent: ACF XMLRPC Client 1.0\r\n";
    $query .= "Host: ".$host."\r\n";
    $query .= "Content-Type: text/xml\r\n";
    $query .= "Content-Length: ".strlen($request)."\r\n\r\n";
    $query .= $request."\r\n";

    if (!fputs($fp, $query, strlen($query))) {
        $errstr = "Write error";
        return 0;
    }

    $getxml = false;
    $xml = '';
    while (!feof($fp)) {
        $content =  fgets($fp);
        if($getxml)
            $xml .= $content;
        if(!$getxml && stristr($content,"content-length"))
            $getxml = true;
    }

    fclose($fp);
    return $xml;
}

function xmlrpc_buildrequest($methodName,$parameters) {
    $params = $parameters->ParametersArray();
    $request = "<?xml version=\"1.0\"?>\r\n<methodCall>\r\n<methodName>".$methodName."</methodName>";
    $request .= "\r\n\t<params>\r\n\t\t<param>\r\n\t\t\t<value>\r\n\t\t\t\t<struct>";
    foreach($params as $param) {
        $request .= "\r\n\t\t\t\t\t<member>";
        $request .= "\r\n\t\t\t\t\t\t<name>".$param->Name()."</name>";
        $request .= "\r\n\t\t\t\t\t\t<value><string>".$param->Value()."</string></value>";
        $request .= "\r\n\t\t\t\t\t</member>";
    }
    $request .= "\r\n\t\t\t\t</struct>\r\n\t\t\t</value>\r\n\t\t</param>\r\n\t</params>\r\n</methodCall>";
    return $request;
}

function xmlrpc_parseresponse($response) {
    $xr = substr($response,strpos($response,"\r\n\r\n") + 4);
    $xml = simplexml_load_string($xr);
    $parameters = array();
    if(isset($xml->params->param->value->struct->member)) {
        foreach($xml->params->param->value->struct->member as $member) {
            $name = (string) $member->name;
            foreach($member->value as $type) {
                foreach($type as $val) {
                    $value = (string) $val;
                }
            }
            $parameters[$name] = $value;
        }
    }
    return $parameters;
}

function xmlrpc_checkforerror($response) {
    $xr = substr($response,strpos($response,"\r\n\r\n") + 4);
    $xml = simplexml_load_string($xr);

    if(isset($xml->fault)) {
        $parameters = array();
        foreach($xml->fault->value->struct->member as $member) {
            $name = (string) $member->name;
            foreach($member->value as $type) {
                foreach($type as $val) {
                    $value = (string) $val;
                }
            }
            $parameters[$name] = $value;
        }
        return $parameters;
    }
    return null;
}

class ACF_XMLRPCWrapper extends ACF_ExecutableWrapper {
    private $server;
    private $port;
    private $uri;
    private $methodName;
    
    function __construct($name, $returnType, $server, $port, $uri, $methodname) {
        parent::__construct($name, $returnType);
        $this->SetServer($server);
        $this->SetPort($port);
        $this->SetURI($uri);
        $this->SetMethodName($methodname);
        $this->callType = 'xmlrpc';
    }
    
    function SetServer($server) {
        $this->server = trim($server);
    }
    
    function SetPort($port) {
        $this->port = trim($port);
    }
    
    function SetURI($uri) {
        $this->uri = trim($uri);
    }
    
    function SetMethodName($method) {
        $this->methodName = trim($method);
    }
    
    function Server() {
        return $this->server;
    }
    
    function Port() {
        return $this->port;
    }
    
    function URI() {
        return $this->uri;
    }

    function MethodName() {
        return $this->methodName;
    }

    function _Execute(ACF_ParameterList $parameterList) {
        $request = xmlrpc_buildrequest($this->methodName,$parameterList);

        $response = xmlrpc_execute($this->server,$this->uri,$this->port,$request);

        $parameters = xmlrpc_checkforerror($response);
        if($parameters)
            throw new ACF_Exception('Error Executing XMLRPC Call: '.$this->methodName.' Error: '.$parameters['faultString']);
        else
            $parameters = xmlrpc_parseresponse($response);

        if($parameters) {
            foreach($parameters as $name => $value) {
                if($name == 'ACF_Return_Event')
                    $retEvent = new ACF_Event($value);
                else {
                    if(!$retparameterList)
                        $retparameterList = new ACF_ParameterList();
                    $retparameterList->AddParameter(new ACF_DirectParameter($name, $value));
                }
            }
        }
       
        if($retEvent) { 
            $retEvent->SetParameterList($retparameterList);
            return $retEvent;
        }

        return $retparameterList;
    }
    
    function Validate() {
        if(empty($this->server))
            throw new ACF_Exception('NULL Server in '.__CLASS__);
            
        if(empty($this->port))
            throw new ACF_Exception('NULL Port in '.__CLASS__);
            
        if(empty($this->uri))
            throw new ACF_Exception('NULL URI in '.__CLASS__);

        if(empty($this->methodName))
            throw new ACF_Exception('NULL Method Name in '.__CLASS__);
    }
}
?>
