<?php

abstract class ACF_ProcessManager {
	private $responses;
	private $layers;
	
	function __construct($layers= array('defaultlayer'))
	{
		$this->SetLayers($layers);
		$this->ClearCache();
	}
	
	function ClearCache()
	{
		$this->responses = array();
	}

	function ImportManager($ProcessManager,$ParentController) {
		$responses = $ProcessManager->Responses();
		if($responses) {
			foreach($responses as $layer => $realms) {
				foreach($realms as $sourcerealm => $sources) {
					foreach($sources as $sourcename => $events) {
						foreach($events as $eventname => $response) {
							$stackadvice = $response->InvocationStackAdvice();
							$newinvocationList = $ParentController->CreateInvocationList();
							$newinvocationList->ImportInvocationList($response->InvocationList(),$ParentController);
							$newresponse = new ACF_EventResponse($stackadvice,$newinvocationList);
							$this->responses[$layer][$sourcerealm][$sourcename][$eventname] = $newresponse;
						}
					}
				}
			}
		}
	}
	
	function SetLayers($layers)
	{
		$this->layers = array();
		foreach($layers as $layer)
			$this->layers[] = strtolower($layer);
	}
	
	function AddResponse(ACF_EventSource $source, ACF_Event $event, ACF_EventResponse $response, $layer = 'defaultlayer')
	{
		$sourceName = strtolower($source->Name());
		$sourceRealm = strtolower($source->Realm());
		$eventName = strtolower($event->Name());
		$layer = strtolower($layer);
		
		if(!$layer || !in_array($layer, $this->layers))
			throw new ACF_Exception(__METHOD__." passed invalid layer name '$layer'");
		
		$this->responses[$layer][$sourceRealm][$sourceName][$eventName] = $response;
	}

	
	// Return one or more EventResponses used to modify the invocation stack
	// in response to this event
	
	function ResponseTo(ACF_EventSource $source, ACF_Event $event)
	{
		$sourceName = strtolower($source->Name());
		$sourceRealm = strtolower($source->Realm());
		$eventName = strtolower($event->Name());

		foreach($this->layers as $layer) {
			if(isset($this->responses[$layer][$sourceRealm][$sourceName][$eventName]))
				return clone $this->responses[$layer][$sourceRealm][$sourceName][$eventName];
		}
		return NULL;
	}

	function Responses() { // Added by JCW 8/16/04
		return $this->responses;
	}
	
	function FindResponse($layer,$sourcerealm,$sourcename,$eventname) { // Added by JCW 8/19/04
		$llayer = strtolower($layer);
		$lsourcerealm = strtolower($sourcerealm);
		$lsourcename = strtolower($sourcename);
		$leventname = strtolower($eventname);
		return $this->responses[$layer][$sourcerealm][$sourcename][$eventname];
	}
	
	function RemoveResponse($layer,$sourcerealm,$sourcename,$eventname) {
		$llayer = strtolower($layer);
		$lsourcerealm = strtolower($sourcerealm);
		$lsourcename = strtolower($sourcename);
		$leventname = strtolower($eventname);
		if(isset($this->responses[$layer][$sourcerealm][$sourcename][$eventname])) {
			unset($this->responses[$layer][$sourcerealm][$sourcename][$eventname]);
			
			// Clean up array (floating indexes)
			if(count($this->responses[$layer][$sourcerealm][$sourcename]) == 0)
			    unset($this->responses[$layer][$sourcerealm][$sourcename]);
			if(count($this->responses[$layer][$sourcerealm]) == 0)
			    unset($this->responses[$layer][$sourcerealm]);
			if(count($this->responses[$layer]) == 0)
			    unset($this->responses[$layer]);
		}
    
	}
	
	function MoveResponse($layer,$OldEventSource,$OldEvent,$NewEventSource,$NewEvent) { // Add by JCW 8/24/04
		$llayer = strtolower($layer);
		$losourceName = strtolower($OldEventSource->Name());
		$losourceRealm = strtolower($OldEventSource->Realm());
		$loeventName = strtolower($OldEvent->Name());

		$lnsourceName = strtolower($NewEventSource->Name());
		$lnsourceRealm = strtolower($NewEventSource->Realm());
		$lneventName = strtolower($NewEvent->Name());

		if(isset($this->responses[$llayer][$losourceRealm][$losourceName][$loeventName])) {
			$this->responses[$llayer][$lnsourceRealm][$lnsourceName][$lneventName] = $this->responses[$llayer][$losourceRealm][$losourceName][$loeventName];
			
			// Remove Old Value
			$this->RemoveResponse($llayer,$losourceRealm,$losourceName,$loeventName);
			//unset($this->responses[$llayer][$losourceRealm][$losourceName][$loeventName]);
		}
	}
}


/*
 * Something we do in response to an event
 *
 */
 
class ACF_EventResponse {
	private $invocationStackAdvice;
	private $invocationList;
	
	function __construct(ACF_InvocationStackAdvice $advice, ACF_InvocationList $invocationList)
	{
		$this->SetInvocationStackAdvice($advice);
		$this->SetInvocationList($invocationList);		
	}
	
	function __clone()
	{
		$this->invocationList = clone $this->invocationList;
		$this->invocationStackAdvice = clone $this->invocationStackAdvice;
	}
	
	function SetInvocationStackAdvice(ACF_InvocationStackAdvice $advice)
	{
		$this->invocationStackAdvice = $advice;
	}
	
	function SetInvocationList(ACF_InvocationList $invocationList)
	{
		$this->invocationList = $invocationList;
	}
	
	function InvocationList() {
		return $this->invocationList;
	}
	
	function InvocationsArray()
	{
		return $this->invocationList->InvocationsArray();
	}
	
	function InvocationStackAdvice() {
		return $this->invocationStackAdvice;
	}
}


/*
 * holds a list of responses to an event
 *
 */
 
class ACF_EventResponseList {
	private $responses;
	
	function __construct()
	{
		$this->responses = array();
	}
	
	function __clone()
	{
		$this->responses = CloneObjectArray($this->responses);
	}
	
	function AddResponse(ACF_EventResponse $response)
	{
		$this->responses[] = $response;
	}
	
	function Responses()
	{
		return $this->responses;
	}
}

?>
