<?php 

function encoder(ACF_ViewInvocation $viewInvocation, $layers = array('defaultlayer'))
{
    global $ACF_TemplateDir;
    global $ACF_TemplateExt;

    $langs = array();

    /* http://www.thefutureoftheweb.com/blog/use-accept-language-header */
    if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
        // break up string into pieces (languages and q factors)
        preg_match_all('/([a-z]{1,8}(-[a-z]{1,8})?)\s*(;\s*q\s*=\s*(1|0\.[0-9]+))?/i', $_SERVER['HTTP_ACCEPT_LANGUAGE'], $lang_parse);

        if (count($lang_parse[1])) {
            // create a list like "en" => 0.8
            $langs = array_combine($lang_parse[1], $lang_parse[4]);
            // set default to 1 for any without q factor
            foreach ($langs as $lang => $val) {
                if ($val === '') $langs[$lang] = 1;
            }

            // sort list based on value
            arsort($langs, SORT_NUMERIC);
        }
    }

    $viewName = $viewInvocation->TaskName();
    $parameters = $viewInvocation->ParameterList()->ParametersArray();
    $te = new TemplateEngine();

    if(!is_array($parameters))
        throw new exception('encoder(): $parameters is a '.gettype($parameters).' instead of an array of name/value pairs.');
    foreach($parameters as $parameter) {
        $te->assign($parameter->Name(), $parameter->Value());
        //echo "<p>".$parameter->Name()."=". $parameter->Value()."</p>";
    }
    $te->assign('_viewName', $viewName);
    
    $searchedlayers = '';
    while(!empty($layers)) {    // Iterate through prioritized layers
        // look through sorted language list and use first one that matches our languages
        foreach ($langs as $lang => $val) {
            // If the next language is English, abort language specific checks and use default filename
            if (strpos($lang, 'en') === 0) {
                break;
            } 
            $filename = $viewName.'.'.$lang.$ACF_TemplateExt;
            $tname = $layers[0].'/'.$filename;
            if(file_exists($ACF_TemplateDir.'/'.$tname)) {
                $te->assign('_layerName', $layers[0]);
                $te->display($tname);
                return;
            }
        }
        $filename = $viewName.$ACF_TemplateExt;
        $tname = $layers[0].'/'.$filename;
        if(file_exists($ACF_TemplateDir.'/'.$tname)) {
            $te->assign('_layerName', $layers[0]);
            $te->display($tname);
            return;
        }
        if($searchedlayers)
            $searchedlayers .= ', ';
        $searchedlayers .= array_shift($layers);    // Record for exception
    }
    throw new Exception('Could not find template "'.$filename."\" in any of the specified layers ($searchedlayers).");
}

function SaveExceptionAndExit(ACF_Exception $e)
{
    global $ACF_LastErrorTemplate;
    
    $te = new TemplateEngine();
    $te->register_outputfilter('SaveErrorAndExit');
    
    $te->assign('sourcelocation', $e->GetSourceLocation());
    $te->assign('message', $e->GetMessage());
    $te->assign('trace', explode("\n",$e->GetTraceAsString()));
    
    $te->display($ACF_LastErrorTemplate);    // Will not return because of output filter that exits.
}

function SaveErrorAndExit($output, &$smarty)
{
    global $ACF_LastErrorFile;
    
    $fp = fopen($ACF_LastErrorFile, 'w');
    if(!$fp)
        return;
    fwrite($fp, $output);
    fclose($fp);
    exit();
}
?>