<?php
class WOW_Request {
    //--- WOW XML Options
    private $wowobject;         //--- Required
    private $method;            //--- Required
    private $properties;        //--- Optional
    private $parameters;        //--- Optional
    private $attributes;        //--- Optional
    private $username;          //--- Optional
    private $password;          //--- Optional
    private $xml;               //--- Generated
    private $debugpath;         //--- Optional
    private $apiversion;        //--- Read From Response

    //--- Communication Options
    private $serverPort;        //--- Required
    private $serverAddress;     //--- Required
    private $serverSecure;      //--- Optional
    private $serverFingerprint; //--- Optional
    private $serverContext;

    //--- Global Properties
    public $lastError;

    function __construct($username='',$password='',$object='addusr') {            //--- Constructor
        $this->method = '';
        $this->wowobject = $object;
        $this->lastError = "";

        //--- Setup Object
        $this->flush();
        $this->loadOptions();
    }

    function setMethod($name) { //--- Set WOW Method Name
        $this->method = $name;

        //--- if method name is changed, flush object
        $this->flush();
    }

    private function convertSimpleXMLToArray($xmlobj) {
        $knownArrays = array(); // Array of known keys
        foreach($xmlobj as $key=>$value) {
            if(!$retval) //--- Must be done here, would check class type, but even end nodes are SimpleXMLElements
                $retval = array();

            if($key == "data_block" || $key == "data_block_item")
                array_push($retval,$this->convertSimpleXMLToArray($value));
            else if($retval[$key]) { // double keys, build array
                if(!$knownArrays[$key]) { // not known, learn it
                    $retval[$key] = array($retval[$key]); // shift array
                    $knownArrays[$key] = true; // set flag
                }
                array_push($retval[$key], $this->convertSimpleXMLToArray($value)); // stack onto array (we must have a return value, this is recursive.
            } else
                $retval[$key] = $this->convertSimpleXMLToArray($value); // main recursive call
        }
        if(!$retval)
            $retval = (string) $xmlobj; // no kids, get value
        return $retval;
    }

    private function loadOptions() {
        $this->serverAddress = $_SESSION['wow']['address'];
        $this->serverPort = $_SESSION['wow']['port'];
        $this->serverSecure = $_SESSION['wow']['secure'];
        $this->serverFingerprint = $_SESSION['wow']['fingerprint'];
        if($_SESSION['ccs']['options']['driver-options']['capture-xml-communication-log'] === "yes")
            $this->debugpath = $_SESSION['ccs']['options']['driver-options']['xml-communication-log-file'];
        if (strlen($this->serverFingerprint) == 0 || strtoupper($this->serverFingerprint) == "DISABLE") {
            $this->context=stream_context_create(array(
                "ssl"=>array(
                    "verify_peer"=>false,
                    "verify_peer_name"=>false,
                    "crypto_method"=>STREAM_CRYPTO_METHOD_TLS_CLIENT,
                ),
            ));
        } else {
            $this->context=stream_context_create(array(
                "ssl"=>array(
                    "verify_peer"=>false,
                    "verify_peer_name"=>false,
                    "crypto_method"=>STREAM_CRYPTO_METHOD_TLS_CLIENT,
                    "peer_fingerprint"=>$this->serverFingerprint,
                ),
            ));
        }
    }

    function setUsername($name) { //--- Set WOW Username
        $this->username = $name;
    }

    function setPassword($pass) { //--- Set WOW Password
        $this->password = $pass;
    }

    function addParameter($name,$value,$type = 'string') { //--- Add Parameter to parameters
        $this->parameters[$name] = array();
        $this->parameters[$name]['value'] = $value;
        $this->parameters[$name]['type'] = $type;
    }

    function addProperty($name,$value,$type = 'string') { //--- Add Property to properties
        $this->properties[$name] = array();
        $this->properties[$name]['value'] = $value;
        $this->properties[$name]['type'] = $type;
    }

    function addAttribute($name,$value,$type = 'string') { //--- Add Attribute to attributes
        $this->attributes[$name] = array();
        $this->attributes[$name]['value'] = $value;
        $this->attributes[$name]['type'] = $type;
    }

        /*
    function getAttribute($index, $key = false) { //--- Get Attribute from attributes
        if($key)
            return $this->attributes[$key];
        else
            return $this->attributes;
    }
        */

    function getAttribute($index, $key = false) { //--- Get Attribute from attributes
        if($key)
            return $this->attributes[$index][$key];
        else
            return $this->attributes[$index];
    }

    function getAttributesArray() { //--- Get Attributes Array
        return $this->attributes;
    }

    function getAPIVersion() { //--- Get the version returned by the Platypus API Call
        return $this->apiversion;
    }

    function closeConnection($socket) {
        fwrite($socket,"QUIT\r\n",strlen("QUIT\r\n"));
        fclose($socket);
    }

    function execute() { //------ Query WOW
        // Clear last Error (Grant's Idea) 01/05/2006
        //$this->lastError = '';

        if(!$this->serverAddress)
            $this->loadOptions();

        if($this->debugpath)
            $debugfile = fopen($this->debugpath,"a");

        if($this->method) {

            //------ Make XML Package
            $this->GenerateXML();
            //------ Make HTML Header
            $package = "content-length:".strlen($this->xml)."\r\n\r\n".$this->xml;
//print "OUT:".$this->xml."<br>";
            if($debugfile)
                fwrite($debugfile,"*****".date("r")."*****\nOUT:\n".$this->xml."\n");
            //--- Process Request
            if($this->serverSecure) {
                $socket = stream_socket_client ("ssl://".$this->serverAddress.":".$this->serverPort, $errorNumber, $errorString, 15, STREAM_CLIENT_CONNECT, $this->context);
            } else {
                $socket = stream_socket_client ("tcp//".$this->serverAddress.":".$this->serverPort, $errorNumber, $errorString, 15);
            }
            if($socket) {
                //--- Write to socket
                if(fwrite($socket,$package,strlen($package)) === false) {
                    $this->lastError = "Cannot Write to Server.";
                    if($debugfile)
                        fclose($debugfile);
                    return false;
                }

                //--- Set Socket Timeout (Some calls can take some time)
                stream_set_timeout($socket,240); //--- Times out in 4 min.

                //--- Read From Socket
                $contentHeader = fgets($socket);  //--- Gets one line
                if($contentHeader === FALSE) {
                    $this->lastError = "Failed to read from socket.";
                    $this->closeConnection($socket);
                    if($debugfile) {
                        fwrite($debugfile,$this->lastError."\n");
                        fwrite($debugfile,$package."\n*****".date("r")."*****\n");
                        fclose($debugfile);
                    }
                    return false;
                }

                fgets($socket);  //--- Gets one line (Clear Blank Line)
                                 //--- We can assume that there are no other html headers b/c this is the wow
                                 //--- And we wrote the response.

                //--- Check Packet and Find Size
                if(stristr($contentHeader,"content-length:")) {
                    //--- Get Package Size
                    $parseString = strstr($contentHeader,":");
                    $size = intval(substr($parseString,1));
                    //--- Get the Package
                    $buffer = fread($socket,$size);  //--- 2K Reads

                    if($buffer) {
                        $readSize = strlen($buffer);
                        $package = $buffer;
                        while($readSize < $size) {
                            $buffer = fread($socket,($size - $readSize));  //--- 2K Reads
                            if($buffer) {
                                $readSize += strlen($buffer);
                                $package .= $buffer;
                            } else {
                                $this->lastError = "Cannot Complete Package Read From Server.";
                                $this->closeConnection($socket);
                                if($debugfile) {
                                    fwrite($debugfile,$this->lastError."\n*****".date("r")."*****\n");
                                    fclose($debugfile);
                                }
                                return false;
                            }
                        }
                    } else {
                        $this->lastError = "Cannot Read Package From Server.";
                        $this->closeConnection($socket);
                        if($debugfile) {
                            fwrite($debugfile,$this->lastError."\n*****".date("r")."*****\n");
                            fclose($debugfile);
                        }
                        return false;
                    }
                } else {
                    $this->lastError = "Server Sent Bad Package:".$contentHeader;
                    $this->closeConnection($socket);
                    if($debugfile) {
                        fwrite($debugfile,$this->lastError."\n");
                        fwrite($debugfile,$package."\n*****".date("r")."*****\n");
                        fclose($debugfile);
                    }
                    return false;
                }

                $this->closeConnection($socket);
            } else {
                $this->lastError = "Cannot Open Socket to Server.";
                if($debugfile) {
                    fwrite($debugfile,$this->lastError."\n*****".date("r")."*****\n");
                    fclose($debugfile);
                }
                return false;
            }
//print "IN:".$package."<br>";
            if($debugfile)
                fwrite($debugfile,"IN:\n".$package."\n*****".date("r")."*****\n");
            //--- Parse Response

            $packageClean = utf8_encode($package);

            $xml = @simplexml_load_string($packageClean);

            //--- Check For XML return
            if(!$xml) {
                $this->lastError = "Error Parsing Package: ".$package;
                if($debugfile)
                    fclose($debugfile);
                return false;
            }

            //--- Check For Plat Error
            if((int) $xml->body->data_block->is_success != 1) {
                if(isset($xml->body->data_block->response_text->PRE))
                    $this->lastError = (string) $xml->body->data_block->response_text->PRE;
                else if(isset($xml->body->data_block->response_text->pre))
                    $this->lastError = (string) $xml->body->data_block->response_text->pre;
                else if(isset($xml->body->data_block->response_text))
                    $this->lastError = (string) $xml->body->data_block->response_text;
                else
                    $this->lastError = "Platypus Error in Package: ".$package;

                if(!$this->lastError)
                    $this->lastError = "Error Message Not Available.";

                if($debugfile)
                    fclose($debugfile);
                return false;
            }

            if(isset($xml->body->data_block->version))
                $this->apiversion = (string) $xml->body->data_block->version;
            else
                $this->apiversion = "unknown";

            if($xml->body->data_block->attributes[0])
                $this->attributes = $this->convertSimpleXMLToArray($xml->body->data_block->attributes[0]);
            elseif($xml->body->data_block->data_block[0]) //Added for backward-compatibility with Plat web object versions prior to 6.1.69
                $this->attributes = $this->convertSimpleXMLToArray($xml->body->data_block->data_block[0]); //that contained a bug in the Chk_Multi_Sec method - faldridge, 10-21-08
            else
                $this->attributes = array();

            if($this->attributes['data_block'])
                $this->attributes = $this->attributes['data_block'];

            if($debugfile)
                fclose($debugfile);

            return true;
        } else
            $this->lastError = "Cannot execute without method name.";
    
        if($debugfile)
            fclose($debugfile);
        return false;
    }

    private function addToXML($name,$value,$parentNode,$domDoc) { //--- Recursive Function For Constructing XML
        if(is_int($name)) { //--- insert tag name as row if index is a number
            $changed = true;                 //--- used below to determine whether the tagname for this node was changed from an array index to 'item'
            $id      = $name;                //--- used below to create array item's id attribute
            $group   = $parentNode->tagName; //--- used below to create array item's group attribute
            $name    = "item";               //--- use the tagname 'item' to designate an array of children within a node to conform to the Web Object API
        }

        // Create Node with attribute if present
        $newNode = $parentNode->appendChild($domDoc->createElement($name));

        if($value['value'] || is_bool($value['value']) || $value['value'] === 0 || $value['value'] === '0') { //--- check to see if there is a value or is this an empty tag

            if($changed && !is_bool($value['value'])) {
                $idAttr = $newNode->appendChild($domDoc->createAttribute('id'));
                $idAttr->appendChild($domDoc->createTextNode($id));
                $groupAttr = $newNode->appendChild($domDoc->createAttribute('group'));
                $groupAttr->appendChild($domDoc->createTextNode($group));
            } else if (is_bool($value['value'])) {
                $attr = $newNode->appendChild($domDoc->createAttribute('type'));
                $attr->appendChild($domDoc->createTextNode('boolean'));
            } else {
                $attr = $newNode->appendChild($domDoc->createAttribute('type'));
                $attr->appendChild($domDoc->createTextNode($value['type']));
            }

            if(is_array($value['value'])) {
                foreach($value['value'] as $key => $val) {
                    $newvalue = array();
                    $newvalue['value'] = $val;
                    $newvalue['type'] = 'string';
                    $this->addToXML($key,$newvalue,$newNode,$domDoc); //--- Recursive Call
                }
            } else {
                if(is_bool($value['value'])) {
                    if($value['value'])
                        $newNode->appendChild($domDoc->createTextNode('true'));
                    else
                        $newNode->appendChild($domDoc->createTextNode('false'));
                } else
                    $newNode->appendChild($domDoc->createTextNode($value['value']));
            }
        }
    }

    private function GenerateXML() { //--- Function For Constructing XML
        $domDoc = new domDocument();

        // Root Node
        $root = $domDoc->createElement('PLATXML');
        $domDoc->appendChild($root);

        // Header
        $header = $root->appendChild($domDoc->createElement('header'));

        // Body
        $body = $root->appendChild($domDoc->createElement('body'));

        // Data Block
        $dataBlock = $body->appendChild($domDoc->createElement('data_block'));

        // Protocol
        $newNode = $dataBlock->appendChild($domDoc->createElement('protocol'));
        $newNode->appendChild($domDoc->createTextNode('Plat'));

        // Object
        $newNode = $dataBlock->appendChild($domDoc->createElement('object'));
        $newNode->appendChild($domDoc->createTextNode($this->wowobject));

        // Action
        $newNode = $dataBlock->appendChild($domDoc->createElement('action'));
        $newNode->appendChild($domDoc->createTextNode($this->method));

        // Username
        $newNode = $dataBlock->appendChild($domDoc->createElement('username'));
        $newNode->appendChild($domDoc->createTextNode($this->username));

        // Password
        $newNode = $dataBlock->appendChild($domDoc->createElement('password'));
        $newNode->appendChild($domDoc->createTextNode($this->password));

        // Add Attributes
        if($this->attributes) {
            foreach($this->attributes as $name => $value)
                    $this->addToXML($name,$value,$dataBlock,$domDoc);
        }

        // Add Parameters
        if($this->parameters) {
            $newNode = $dataBlock->appendChild($domDoc->createElement('parameters'));
            foreach($this->parameters as $name => $value)
                $this->addToXML($name,$value,$newNode,$domDoc);
        }

        // Add Properties
        if($this->properties) {
            $newNode = $dataBlock->appendChild($domDoc->createElement('properties'));
            foreach($this->properties as $name => $value) {
                $this->addToXML($name,$value,$newNode,$domDoc);
            }
        }


        $domDoc->formatOutput = true;
        $this->xml = $domDoc->saveXML();
    }

    private function flush() {
        $this->properties = array();
        $this->parameters = array();
        $this->attributes = array();
    }
}
?>
