<?php

require_once($_ENV['TCA_HOME']."/cartridges/action_parameters.inc.php");

interface TCA_ServiceOffering {
    function getName();
    
    function getActions();
    
    function getActionByName($name);
}

class TCA_SimpleServiceOffering implements TCA_ServiceOffering {
    private $name;
    private $actions = array();

    public function __construct($name) {
    	$this->name = $name;
    }
    
    public function getName() {
    	return $this->name;
    }
    
    public function getActions() {
    	return $this->actions;
    }
    
    public function getActionByName($name) {
    	return $this->actions[$name];
    }
    
    public function addAction(TCA_ServiceOfferingAction $action) {
    	$this->actions[$action->getName()] = $action;
    }
}

interface TCA_ServiceOfferingAction {
    public function getName();

    public function getNumberOfSteps();
    
    // returns TCA_ActionParameterContainerI
    public function getParametersContainer($step = 1);

    // returns TCA_ActionResultI
    public function perform($step = 1);
}

abstract class TCA_AbstractServiceOfferingAction {
    private $name;
    private $steps;
    protected $paramsContainerAr = array();
    protected $serviceManager;

    // $steps must be greater than 0
    public function __construct($name, $steps, TCA_AbstractServiceManager $serviceManager) {
    	$this->name = $name;    	
    	$this->steps = $steps;    	
    	$this->serviceManager = $serviceManager;
    	for ($i = 0; $i < $steps; $i++) {
    		$this->paramsContainerAr[$i] = new TCA_ActionParameterContainer("main");
    	}
    }
    
    public function getName() {
    	return $this->name;
    }

    public function getNumberOfSteps() {
    	return $this->steps;
    }
    
    public function getParametersContainer($step = 1) {
    	return $this->paramsContainerAr[$step - 1];
    }
    
    public function getServiceManager() {
    	return $this->serviceManager;
    }
    
    // "deep" converter from TCA_ServiceResponse to TCA_ActionResultI
    public static function convertToActionResult(TCA_ServiceResponse $serviceResponse) {
    	$actionResult = new TCA_ActionResult($serviceResponse->isSuccess(),
    		$serviceResponse->getResponseCode(), $serviceResponse->getResponseText());
    	
    	$serviceResponseData = $serviceResponse->getResponseData();    	
    	self::populateParameterContainerFromParameterSet($actionResult->getResultData(), $serviceResponseData);    	
    	
    	return $actionResult;    	
    }
    
    private static function populateParameterContainerFromParameterSet(TCA_ActionParameterContainerI $cont,
    		TCA_ParameterSetI $pset) {
    	foreach ($pset->keys() as $key) {
    		$element = $pset->getElement($key);
    		if ($element instanceof TCA_ParameterI) {
    			$param = new TCA_ActionValueParameter($key, $key . " parameter", "", false,
    				new TCA_GenericValidator(TCA_DummyValidationRule::getInstance()));
    				
    			$param->setValue($element->getValue());
    			$cont->addParameter($param);
    		} elseif ($element instanceof TCA_ParameterSetI) {
    			$childCont = new TCA_ActionParameterContainer($key);
    			self::populateParameterContainerFromParameterSet($childCont, $element);
    			$cont->addChildContainer($childCont);
    		}
    	}
    }    
}

?>
