<?php

class TCA_CartridgeManager {
    const CLASS_PREFIX = "TCA_";
    const CLASS_SUFIX = "_cartridge";
    const CARTRIDGE_CLASS_FILE = "cartridge.inc.php";

    // TCA_CartridgeManager is a singleton
    private static $cartridgeManager = null;

    private $cartridges = array();

    private function __construct() {
    	if (!isset($_ENV['TCA_CARTRIDGES']))
    		throw new Exception("TCA_CARTRIDGES is not set");
    	if (!is_dir($_ENV['TCA_CARTRIDGES']))
    		throw new Exception("TCA_CARTRIDGES is not a dir");

    	$this->loadCartridges($_ENV['TCA_CARTRIDGES']);
    }

    private function loadCartridges($path) {
    	$dh = opendir($path);
    	while (($file = readdir($dh)) !== false) {
    		$matches = array();
    		if (preg_match("/^crt_(.*)/", $file, $matches)) {
    			require_once($file . "/" . self::CARTRIDGE_CLASS_FILE);
    			
    			$name = self::CLASS_PREFIX . $matches[1] . self::CLASS_SUFIX;
    			$cartridge = call_user_func(array($name, 'getCartridge'));
    			$cartridge->check(); // will throw an exception on failure

    			$this->cartridges[$name] = $cartridge;
    		}
    	}
    }

    public static function getCartridgeManager() {
    	if (self::$cartridgeManager === null) {
    		self::$cartridgeManager = new TCA_CartridgeManager();
    	}
    	return self::$cartridgeManager;
    }

    public function getCartridges() {
    	return $this->cartridges;
    }

    public function getCartridgeByName($name) {
    	return $this->cartridges[$name];
    }
}

?>
