<?php

require_once($_ENV['TCA_HOME']."/servicemanager.inc.php");
require_once($_ENV['TCA_HOME']."/util/parameter.inc.php");
require_once($_ENV['TCA_HOME']."/util/validator.inc.php");

// This class contains rules for parameter validation. This info could be
// loaded from an external file in the future (XML or something). Or it 
// could be fetched from the server side. Or the implementation could 
// solely rely on the server side validation as seems to be the direction
// for Mercury.
class TCA_BlogwareServiceValidator extends TCA_ServiceValidator {
    
    function __construct() {
	$this->validator = new TCA_Validator();	
	$this->initGenericValidationRuleList();
    }

    function initGenericValidationRuleList() {
	$this->genericRuleList = new TCA_ValidationRuleList();
	//$rule1 = new TCA_MinMaxValidationRule();
	//$this->genericRuleList->addRule($rule1);
    }
    
    function validateServiceRequest(TCA_ValidationRuleList $ruleList, TCA_ServiceRequest $request) {
	$validationResult = $this->validator->validate($ruleList, $request->toArray()); 
	// XXX for now throw exception
	if ( !$validationResult->isValid() )
	    throw new TCA_InvalidServiceRequestException($validationResult->__toString());
    }    
}

// This class contains rules for different API calls (2nd level of checking)
class TCA_BlogwareActionValidationMapper extends TCA_ValidationMapper {
    
    function __construct() {
	$this->initValidationMap();
    }
     
    function initValidationMap() {
	$this->map = array(
	    'check_user' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_username" => TCA_RegexValidationRule::createRule(1, "/^([a-z]||[A-Z]||[0-9])+$/"),
		)),
	    )),
	    'get_csstemplates' => TCA_ValidationRuleList::createRuleList(array(
	    )),
	    'get_componentprofilelist' => TCA_ValidationRuleList::createRuleList(array(
	    )),
	    'update_blogpackage' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_id" => TCA_NullValidationRule::createRule(1),
		    "package" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'update_bloghostnames' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_id" => TCA_NullValidationRule::createRule(1),
		    "primary_hostname" => TCA_NullValidationRule::createRule(1),
		    "secondary_hostnames" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'update_blogstatus' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_id" => TCA_NullValidationRule::createRule(1),
		    "status" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'delete_blog' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_id" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'activate_trialblog' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_id" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'add_blog' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "struct" => TCA_BlogStructValidationRule::createRule(1),
		    "create_user" => TCA_EnumValidationRule::createRule(0, array(0,1))
		)),
	    )),
	    'get_packagelist' => TCA_ValidationRuleList::createRuleList(array(
	    )),
	    'get_statuslist' => TCA_ValidationRuleList::createRuleList(array(
	    )),
	    'get_blogs' => TCA_ValidationRuleList::createRuleList(array(
	    )),
	    'get_timezones' => TCA_ValidationRuleList::createRuleList(array(
	    )),
	    'send_userpasswordbyhostname' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "hostname" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'get_bloginfo' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_id" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'get_usersblogs' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_username" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'add_bandwidth' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_id" => TCA_NullValidationRule::createRule(1),
		    "quantity" => TCA_EnumValidationRule::createRule(0, range(1,10))
		)),
	    )),
	    'get_bloginfobyhostname' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "hostname" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'check_hostname' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "hostname" => TCA_RegexValidationRule::createRule(1, "/^(?:[a-z0-9](?:[-a-z0-9]*[a-z0-9])*\.)+[a-z]{2,}/"),
		)),
	    )),
	    'send_userpassword' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_username" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'auth_user' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "blog_username" => TCA_NullValidationRule::createRule(1),
		    "blog_password" => TCA_NullValidationRule::createRule(1)
		)),
	    ))
	);	
    }
} 

class TCA_BlogwareActionMapper extends TCA_ActionMapper {

    function __construct() {
	$this->actionList = new TCA_ApiActionList();
	$this->initActions();	
    }

    function initActions() {
    	$registrant_ip = $_ENV['REMOTE_ADDR'];
    	
	// add api definitions here
	$this->actionList->addApiAction('check_user', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'checkUser', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_csstemplates', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getCSSTemplates', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_componentprofilelist', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getComponentProfileList', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('update_blogpackage', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'updateBlogPackage', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('update_bloghostnames', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'updateBlogHostnames', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('update_blogstatus', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'updateBlogStatus', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('delete_blog', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'deleteBlog', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('activate_trialblog', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'activateTrialBlog', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('add_blog', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'addBlog', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_packagelist', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getPackageList', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_statuslist', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getStatusList', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_blogs', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getBlogs', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_timezones', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getTimezones', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('send_userpasswordbyhostname', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'sendUserPasswordByHostname', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_bloginfo', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getBlogInfo', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_usersblogs', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getUsersBlogs', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('add_bandwidth', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'addBandwidth', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_bloginfobyhostname', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'getBlogInfoByHostname', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('check_hostname', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'checkHostname', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('send_userpassword', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'sendUserPassword', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('auth_user', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XMLRPC', action => 'authUser', object => 'reseller', registrant_ip => $registrant_ip),
	)));
	// end api definitions
    }

}

//*****************************************************************************************
class TCA_BlogwareServiceManager extends TCA_AbstractServiceManager {
    private $requestValidator;
    private $actionPrefix = 'TUCOWS_blogware';
    private $actions = array();
    private $blogwareDispatcher;

    function __construct() {
	parent::__construct();
	$this->requestValidator = new TCA_BlogwareServiceValidator();
	$this->actionMapper = new TCA_BlogwareActionMapper();
	$this->actionValidationMapper = new TCA_BlogwareActionValidationMapper();
	

	// list all actions here (or we could use introspection to build this)
	$this->actions = array(
	   'check_user' => 1,
	   'get_csstemplates' => 1,
	   'get_componentprofilelist' => 1,
	   'update_blogpackage' => 1,
	   'update_bloghostnames' => 1,
	   'update_blogstatus' => 1,
	   'delete_blog' => 1,
	   'activate_trialblog' => 1,
	   'add_blog' => 1,
	   'get_packagelist' => 1,
	   'get_statuslist' => 1,
	   'get_blogs' => 1,
	   'get_timezones' => 1,
	   'send_userpasswordbyhostname' => 1,
	   'get_bloginfo' => 1,
	   'get_usersblogs' => 1,
	   'add_bandwidth' => 1,
	   'get_bloginfobyhostname' => 1,
	   'check_hostname' => 1,
	   'send_userpassword' => 1,
	   'auth_user' => 1
	);
	
	$this->blogwareDispatcher = new TCA_BlogwareDispatcher();		
    }

    function getActionPrefix() {
	return $this->actionPrefix;
    }

    function getActions() {
	return $this->actions;
    }

    function getRequestValidator() {
	return $this->requestValidator;
    }
    
    function init() {}
    
    protected function check_user(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('check_user', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_csstemplates(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_csstemplates', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_componentprofilelist(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_componentprofilelist', $request);	
	return $serviceResponseArr[0];
    }

    protected function update_blogpackage(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('update_blogpackage', $request);	
	return $serviceResponseArr[0];
    }

    protected function update_bloghostnames(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('update_bloghostnames', $request);	
	return $serviceResponseArr[0];
    }

    protected function update_blogstatus(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('update_blogstatus', $request);	
	return $serviceResponseArr[0];
    }

    protected function delete_blog(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('delete_blog', $request);	
	return $serviceResponseArr[0];
    }

    protected function activate_trialblog(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('activate_trialblog', $request);	
	return $serviceResponseArr[0];
    }

    protected function add_blog(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('add_blog', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_packagelist(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_packagelist', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_statuslist(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_statuslist', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_blogs(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_blogs', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_timezones(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_timezones', $request);	
	return $serviceResponseArr[0];
    }

    protected function send_userpasswordbyhostname(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('send_userpasswordbyhostname', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_bloginfo(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_bloginfo', $request);	
	return $serviceResponseArr[0];
    }
    
    protected function get_usersblogs(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_usersblogs', $request);	
	return $serviceResponseArr[0];
    }

    protected function add_bandwidth(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('add_bandwidth', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_bloginfobyhostname(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('get_bloginfobyhostname', $request);	
	return $serviceResponseArr[0];
    }

    protected function check_hostname(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('check_hostname', $request);	
	return $serviceResponseArr[0];
    }

    protected function send_userpassword(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('send_userpassword', $request);	
	return $serviceResponseArr[0];
    }

    protected function auth_user(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('auth_user', $request);	
	return $serviceResponseArr[0];
    }
}

?>
