<?php

require_once($_ENV['TCA_HOME']."/cartridges/abstract_cartridge.inc.php");
require_once($_ENV['TCA_HOME']."/cartridges/abstract_serviceoffering.inc.php");

// include the actions file
require_once("checkavail.inc.php");
require_once("query.inc.php");
require_once("refund.inc.php");
require_once("renew.inc.php");
require_once("transfer.inc.php");
require_once("provision.inc.php");

require_once("lib/domain.inc.php");
require_once("lib/domainmanager.inc.php");

class TCA_TUCOWS_domains_cartridge extends TCA_AbstractCartridge implements TCA_Cartridge {
    const PROVIDER = "TUCOWS";
    const SERVICE = "domains";
    const VERSION = "1.0";
    
    private static $domainsCartridge = null;
    
    private function __construct() {
    	$this->serviceManager = new TCA_DomainServiceManager();

    	// create actions
    	$checkAvailableAction = new TCA_CheckAvailableDomainServiceAction($this->serviceManager);
    	$queryAction = new TCA_QueryDomainServiceAction($this->serviceManager);
    	$refundAction = new TCA_RefundDomainServiceAction($this->serviceManager);
    	$provisionAction = new TCA_ProvisionDomainServiceAction($this->serviceManager);    	
    	$transferAction = new TCA_TransferDomainServiceAction($this->serviceManager);
    	$renewAction = new TCA_RenewDomainServiceAction($this->serviceManager);
    	
    	// .com service offering
    	$comServiceOffering = new TCA_SimpleServiceOffering(".com");
    	$comServiceOffering->addAction($checkAvailableAction);
    	$comServiceOffering->addAction($queryAction);
    	$comServiceOffering->addAction($refundAction);
    	$comServiceOffering->addAction($provisionAction);
    	$comServiceOffering->addAction($transferAction);
    	$comServiceOffering->addAction($renewAction);
    	$this->addServiceOffering($comServiceOffering);

    	// .net service offering
    	$netServiceOffering = new TCA_SimpleServiceOffering(".net");
    	$netServiceOffering->addAction($checkAvailableAction);
    	$netServiceOffering->addAction($queryAction);
    	$netServiceOffering->addAction($refundAction);
    	$netServiceOffering->addAction($provisionAction);
    	$netServiceOffering->addAction($transferAction);
    	$netServiceOffering->addAction($renewAction);
    	$this->addServiceOffering($netServiceOffering);

    	// .org service offering
    	$orgServiceOffering = new TCA_SimpleServiceOffering(".org");
    	$orgServiceOffering->addAction($checkAvailableAction);
    	$orgServiceOffering->addAction($queryAction);
    	$orgServiceOffering->addAction($refundAction);
    	$orgServiceOffering->addAction($provisionAction);
    	$orgServiceOffering->addAction($transferAction);
    	$orgServiceOffering->addAction($renewAction);
    	$this->addServiceOffering($orgServiceOffering);	
    	
    	// .ca service offering
    	$caServiceOffering = new TCA_SimpleServiceOffering(".ca");
    	$caServiceOffering->addAction($checkAvailableAction);
    	$caServiceOffering->addAction($queryAction);
    	$caServiceOffering->addAction($refundAction);
    	$caServiceOffering->addAction(new TCA_CaPrivisionDomainServiceAction($this->serviceManager));
    	$caServiceOffering->addAction($transferAction);
    	$caServiceOffering->addAction($renewAction);
    	$this->addServiceOffering($caServiceOffering);
    	
    	// .info service offering
    	$infoServiceOffering = new TCA_SimpleServiceOffering(".info");
    	$infoServiceOffering->addAction($checkAvailableAction);
    	$infoServiceOffering->addAction($queryAction);
    	$infoServiceOffering->addAction($refundAction);
    	$infoServiceOffering->addAction($provisionAction);
    	$infoServiceOffering->addAction($transferAction);
    	$infoServiceOffering->addAction($renewAction);
    	$this->addServiceOffering($infoServiceOffering);
    	
    	// .biz service offering
    	$bizServiceOffering = new TCA_SimpleServiceOffering(".biz");
    	$bizServiceOffering->addAction($checkAvailableAction);
    	$bizServiceOffering->addAction($queryAction);
    	$bizServiceOffering->addAction($refundAction);
    	$bizServiceOffering->addAction($provisionAction);
    	$bizServiceOffering->addAction($transferAction);
    	$bizServiceOffering->addAction($renewAction);
    	$this->addServiceOffering($bizServiceOffering);
    }

    public static function getCartridge() {
    	if (self::$domainsCartridge === null) {
    		self::$domainsCartridge = new TCA_TUCOWS_domains_cartridge();
    	}
    	return self::$domainsCartridge;
    }

    public function getVersion() {
    	return self::VERSION;
    }

    public function getProvider() {
    	return self::PROVIDER;
    }

    function getService() {
    	return self::SERVICE;
    }
}

?>
