<?PHP
require_once($_ENV['TCA_HOME']."/cartridges/abstract_serviceoffering.inc.php");
require_once("lib/domainutils.inc.php");

class TCA_CheckAvailableDomainServiceAction extends TCA_AbstractServiceOfferingAction
    	implements TCA_ServiceOfferingAction {
    		
    const DOMAIN = 'domain';
    const ATTRIBUTES = 'attributes';
    const ACTION = 'lookup_domain';
        
    private $statusParam;
    		
    public function __construct(TCA_DomainServiceManager $serviceManager) {
    	parent::__construct("check_available", 1, $serviceManager);
    	
    	$domainParam = TCA_ActionParameterFactory::newStringParameter(self::DOMAIN, "Domain Name",
    		"", true, "/^.+$/s", "Valid domain name");
    	
    	$attrContainer = new TCA_ActionParameterContainer(self::ATTRIBUTES);
    	$attrContainer->addParameter($domainParam);
    		
    	$this->getParametersContainer()->addChildContainer($attrContainer);
    	
    	$this->statusParam =
    		new TCA_ActionBooleanParameter("is_available", "Whether the domain is available", "", true);
    }
        
    public function perform($step = 1) {    	
    	$attrContainer = $this->getParametersContainer()->getChildContainerByName(self::ATTRIBUTES);
    	$serviceRequestParams =
    		TCA_ServiceRequestParameterSet::createServiceRequestParameterSet($attrContainer->toParameterSet());
    	$serviceRequest = $this->serviceManager->createServiceRequest(self::ACTION);
    	$serviceRequest->addParameterSet(self::ATTRIBUTES, $serviceRequestParams);
    	
    	$serviceResponse = $this->serviceManager->processServiceRequest($serviceRequest);
    	
    	$actionResult = new TCA_ActionResult($serviceResponse->isSuccess(), $serviceResponse->getResponseCode(),
    		$serviceResponse->getResponseText());

    	if ($actionResult->isSuccess()) {
    		$responseData = $serviceResponse->getResponseData();
    		$status = $responseData->getElement(self::ATTRIBUTES)->getElement('status')->getValue();
    		$this->statusParam->setValue($status == "available" ? 1 : 0);
    		$actionResult->getResultData()->addParameter($this->statusParam);
    	}
    	
    	return $actionResult;
    }
}

?>