<?php

// marker interfaces
require_once($_ENV['TCA_CARTRIDGES'].'/interface.inc.php');

class TCA_DomainDispatcher {
    
    private $tld_map = array(
	"generic"	=> "TCA_GenericDomain",
	"com"		=> "TCA_ComDomain",
	"net"		=> "TCA_NetDomain",
	"ca"		=> "TCA_CaDomain",
	"org"		=> "TCA_OrgDomain",
	"biz"		=> "TCA_BizDomain",
	"info"		=> "TCA_InfoDomain",
	"eu"		=> "TCA_EuDomain",
	"mobi"		=> "TCA_MobiDomain"
    );
    
    function getDomain($domainName) {
    	
    	$domainName = strtolower($domainName);
    	preg_match("/[^\.\/]+$/", $domainName, $matches);
    	$tld = $matches[0];
    	if (key_exists($tld, $this->tld_map) )
	    return new $this->tld_map{$tld}($domainName);
    	else
	    throw new TCA_IllegalArgumentsException("Invalid tld: $tld.");
    }
    
    function getTldCatalogue() {    	
    	return array_keys($this->tld_map);
    }
    
}

abstract class TCA_Domain {
    
    protected $tld;
    protected $maxRegistrationPeriod = 10;
    protected $minRegistrationPeriod = 1;
    protected $transferPeriod = 1;
    protected $minNameLength = 1;
    protected $maxNameLength;
    protected $domainName;
    protected $ruleList;
    protected $contactDispatcher;
    
    function __construct($domainName) {
	$this->domainName = $domainName;
	$this->contactDispatcher = new TCA_DomainContactDispatcher();
    }
    
    function getContact($type) {
	return $this->contactDispatcher->getContact($this->tld, $type);
    }

    function getContacts() {
    	$types = array('owner', 'admin', 'tech', 'billing');
    	$contacts = array();
    	
    	foreach ( $types as $type ) {
	    array_push($contacts, $this->getContact($type));
    	}
	return $contacts;    	
    }
    
    function getMaxRegistrationPeriod() {
	return $this->maxRegistrationPeriod;
    }
    
    function getMinRegistrationPeriod() {
    	return $this->minRegistrationPeriod;
    }
    
    function getTransferPeriod() {
    	return $this->transferPeriod;
    }
    
    function getMinNameLength() {
	return $this->minNameLength;
    }

    function getMaxNameLength() {
	return $this->maxNameLength;
    }
    
    function initValidationRuleList() {
	// commmon traits shared by all tlds
	// add more specifics in each tld's class
	$gRuleList = TCA_ValidationRuleList::createRuleList(array(
	    "attributes" => TCA_ValidationRuleList::createRuleList(array(
		"period" => TCA_ValidationRuleList::createRuleList(array(
		    TCA_MinValidationRule::createRule(0, $this->getMinRegistrationPeriod()),
		    TCA_MaxValidationRule::createRule(0, $this->getMaxRegistrationPeriod())
		)),
		"domain" => TCA_ValidationRuleList::createRuleList(array(
		    TCA_MinLengthValidationRule::createRule(0, $this->getMinNameLength()),
		    TCA_MaxLengthValidationRule::createRule(0, $this->getMaxNameLength()),
		    TCA_DomainSyntaxValidationRule::createRule(0),
		)),
	    )),
	));
	$attrRuleList = $gRuleList->getElement('attributes');
	$attrRuleList->addRuleList("domain_name", $attrRuleList->getElement("domain"));
	$this->ruleList['generic'] = $gRuleList;
    }

    final function getRuleList($action = 'generic') {
    	if ( key_exists($action, $this->ruleList) ) {
	    return $this->ruleList[$action];
    	} else {
    	    return null;
    	}
    }
        
}

class TCA_GenericDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable {
    private static $domainRuleList = null;
    	
    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'generic';
    	$this->maxNameLength = 67;

	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	    	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
    }        
}

class TCA_ComDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable {
    private static $domainRuleList = null;
    	
    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'com';
    	$this->maxNameLength = 67;

	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	    	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
        
}

class TCA_NetDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable  {
    private static $domainRuleList = null;

    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'net';
    	$this->maxNameLength = 67;

	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	

    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
    	
}

class TCA_OrgDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable  {
    private static $domainRuleList = null;

    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'org';
    	$this->maxNameLength = 67;

	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	    	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
	
	// add more rules here as needed
    }

}

class TCA_CaDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable  {
    private static $domainRuleList = null;
	
    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'ca';
    	$this->maxNameLength = 67;

	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
	$this->ruleList['sw_register_domain'] = TCA_ValidationRuleList::createRuleList(array(
	    "attributes" => TCA_ValidationRuleList::createRuleList(array(
		"legal_type" => TCA_NullValidationRule::createRule(1)
	    )),
	));
    }

    function getContacts() {
    	$types = array('owner', 'admin', 'tech');
    	$contacts = array();
    	
    	foreach ( $types as $type ) {
	    array_push($contacts, $this->getContact($type));
    	}
	return $contacts;    	
    }

}

class TCA_BizDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable  {
    private static $domainRuleList = null;
	
    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'biz';
    	$this->maxNameLength = 67;
    	$this->minNameLength = 7;

	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();	
    }
}

class TCA_InfoDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable  {
    private static $domainRuleList = null;
	
    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'info';
    	$this->maxNameLength = 67;
    	$this->minNameLength = 8;
    	
	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
}

class TCA_EuDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable  {
    private static $domainRuleList = null;
	
    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'eu';
    	$this->maxNameLength = 67;
    	
	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
    }

    function getContacts() {
    	$types = array('tech', 'owner');
    	$contacts = array();
    	
    	foreach ( $types as $type ) {
	    array_push($contacts, $this->getContact($type));
    	}
	return $contacts;    	
    }
}

class TCA_MobiDomain extends TCA_Domain 
    implements TCA_Renewable , TCA_Manageable , TCA_Lockable , TCA_Transferable  {
    private static $domainRuleList = null;
	
    function __construct($domainName) {
	parent::__construct($domainName);
    	$this->tld = 'mobi';
    	$this->maxNameLength = 68;
    	$this->minNameLength = 8;
    	
	if (self::$domainRuleList == null) {
	    $this->initValidationRuleList();
	    self::$domainRuleList = $this->ruleList;
	} else {
	    $this->ruleList = self::$domainRuleList;
	}	
    }
    
    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
}

class TCA_DomainContactDispatcher {
    
    private $tld_map = array(
	"generic"	=> "TCA_GenericContact",
	"com"		=> "TCA_ComContact",
	"net"		=> "TCA_NetContact",
	"org"		=> "TCA_OrgContact",
	"biz"		=> "TCA_BizContact",
	"info"		=> "TCA_InfoContact",
	"ca"		=> "TCA_CaContact",
	"eu"		=> "TCA_EuContact",
	"mobi"		=> "TCA_MobiContact"
    );
    
    function getContact($tld, $type) {
    	
    	if ( key_exists($tld, $this->tld_map) )
	    return new $this->tld_map{$tld}($type);
    	else
	    throw new TCA_IllegalArgumentsException("Invalid tld: $tld.");
    }
    
    function getTldCatalogue() {    	
    	return array_keys($this->tld_map);
    }
        
}

abstract class TCA_DomainContact {
    protected $type;
    protected $ruleList;
    
    function __construct($type) {
    	$this->type = $type;
    	$this->initValidationRuleList();
    }
    
    final function getRuleList() {
	return $this->ruleList;
    }

    function getType() {
	return $this->type;
    }    
    
    function initValidationRuleList() {
	// commmon traits shared by all tlds
	// add more specifics in each tld's class
	$cList = TCA_ValidationRuleList::createRuleList(array(
	    "contact_set" => TCA_ValidationRuleList::createRuleList(array(
		$this->type => TCA_ValidationRuleList::createRuleList(array(
		    "first_name" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
			TCA_MaxLengthValidationRule::createRule(1, 64),
		    )),
		    "last_name" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
			TCA_MaxLengthValidationRule::createRule(1, 64),
		    )),
		    "address1" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
			TCA_MaxLengthValidationRule::createRule(1, 100),
		    )),
		    "address2" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(0),
			TCA_MaxLengthValidationRule::createRule(0, 100),
		    )),
		    "address3" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(0),
			TCA_MaxLengthValidationRule::createRule(0, 100),
		    )),
		    "org_name" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
		    )),
		    "city" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
			TCA_MaxLengthValidationRule::createRule(1, 64),
		    )),
		    "state" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(0),
			TCA_MaxLengthValidationRule::createRule(0, 32),
		    )),
		    "country" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
			TCA_MaxLengthValidationRule::createRule(1, 2),
		    )),
		    "phone" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
			TCA_MaxLengthValidationRule::createRule(1, 20),
			TCA_PhoneValidationRule::createRule(1),
		    )),
		    "fax" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(0),
			TCA_MaxLengthValidationRule::createRule(0, 20),
		    )),
		    "email" => TCA_ValidationRuleList::createRuleList(array(
			TCA_NullValidationRule::createRule(1),
			TCA_MaxLengthValidationRule::createRule(1, 255),
			TCA_EmailAddressValidationRule::createRule(1),
		    )),
		)),
	    )),
	));
	$this->ruleList = TCA_ValidationRuleList::createRuleList(array( attributes => $cList ));

    }

    function getContactSetRuleList() {
    	$rList = $this->ruleList->getElement('attributes');
	$rList = $rList->getElement('contact_set');
	return $rList->getElement($this->type);
    }   
}

class TCA_GenericContact extends TCA_DomainContact {
	
    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
    
}

class TCA_EPPDomainContact extends TCA_DomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
	
	$rList = $this->getContactSetRuleList();
	$rList->setRuleList("phone", TCA_ValidationRuleList::createRuleList(array(
	    TCA_NullValidationRule::createRule(1),
	    TCA_MaxLengthValidationRule::createRule(1, 20),
	    TCA_PhoneValidationRuleEPP::createRule(1),
	)));
    }
}

class TCA_ComContact extends TCA_DomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
}

class TCA_NetContact extends TCA_DomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
}

class TCA_OrgContact extends TCA_EPPDomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
    }
}

class TCA_BizContact extends TCA_EPPDomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();	
    }
}

class TCA_InfoContact extends TCA_EPPDomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
    }    
}

class TCA_CaContact extends TCA_DomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
	
	$rList = $this->getContactSetRuleList();
	$rList->addRuleList("legal_type", TCA_ValidationRuleList::createRuleList(array(
	    TCA_NullValidationRule::createRule(0),
	)));
	if ( $this->type == 'admin' || $this->type == 'tech' ) {
	    $rList->addRuleList("lang", TCA_ValidationRuleList::createRuleList(array(
		TCA_NullValidationRule::createRule(0),
	    )));
	}	
    }
 
}

class TCA_EuContact extends TCA_EPPDomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
	
	$rList = $this->getContactSetRuleList();
	if ($this->type == 'tech' || $this->type == 'owner'){
	    $rList->addRuleList("lang", TCA_ValidationRuleList::createRuleList(array(
		TCA_NullValidationRule::createRule(0),
	    )));
	    $rList->addRuleList("vat", TCA_ValidationRuleList::createRuleList(array(
		TCA_NullValidationRule::createRule(0),
	    )));
	}
    }
 
}

class TCA_MobiContact extends TCA_EPPDomainContact {

    function initValidationRuleList() {
	parent::initValidationRuleList();
    }    
}
?>
