<?php

require_once($_ENV['TCA_HOME']."/servicemanager.inc.php");
require_once($_ENV['TCA_HOME']."/util/parameter.inc.php");
require_once($_ENV['TCA_HOME']."/util/validator.inc.php");

// This class contains rules for parameter validation. This info could be
// loaded from an external file in the future (XML or something). Or it 
// could be fetched from the server side. Or the implementation could 
// solely rely on the server side validation as seems to be the direction
// for Mercury.
class TCA_DomainServiceValidator extends TCA_ServiceValidator {
    
    function __construct() {
	$this->validator = new TCA_Validator();	
	$this->initGenericValidationRuleList();
    }

    function initGenericValidationRuleList() {
	$this->genericRuleList = new TCA_ValidationRuleList();
	//$rule1 = new TCA_MinMaxValidationRule();
	//$this->genericRuleList->addRule($rule1);
    }
    
    function validateServiceRequest(TCA_ValidationRuleList $ruleList, TCA_ServiceRequest $request) {
	$validationResult = $this->validator->validate($ruleList, $request->toArray()); 
	// XXX for now throw exception
	if ( !$validationResult->isValid() )
	    throw new TCA_InvalidServiceRequestException($validationResult->__toString());
    }    
}

// This class contains rules for different API calls (2nd level of checking)
class TCA_DomainActionValidationMapper extends TCA_ValidationMapper {
    
    function __construct() {
	$this->initValidationMap();
    }
     
    function initValidationMap() {
	$this->map = array(
	    'set_cookie' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		    "reg_username" => TCA_NullValidationRule::createRule(1),
		    "reg_password" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'update_cookie' => TCA_ValidationRuleList::createRuleList(array(
		"cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		    "domain_new" => TCA_NullValidationRule::createRule(1),
		    "reg_username" => TCA_NullValidationRule::createRule(1),
		    "reg_password" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'delete_cookie' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "cookie" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'lookup_domain' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'ca_blocker_list_domain' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'query_registrant_ca' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "rant_no" => TCA_NullValidationRule::createRule(1),
		    "domain" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'bulk_transfer_domain' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain_list" => TCA_TransferListValidationRule::createRule(1),
		    "reg_domain" => TCA_NullValidationRule::createRule(0),
		    "reg_username" => TCA_NullValidationRule::createRule(1),
		    "reg_password" => TCA_DomainPasswordValidationRule::createRule(1),
		    "affiliate_id" => TCA_MaxValidationRule::createRule(0, 100),
		    "handle" => TCA_EnumValidationRule::createRule(0, array('save','process')),
		    "custom_tech_contact" => TCA_MaxValidationRule::createRule(0, 1),
		    "contact_set" => TCA_NullValidationRule::createRule(1)
		)),
	    )),
	    'sw_register_domain' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		    "reg_username" => TCA_NullValidationRule::createRule(1),
		    "reg_password" => TCA_DomainPasswordValidationRule::createRule(1),
		    "reg_type" => TCA_EnumValidationRule::createRule(0, array('new','transfer')),
		    "period" => TCA_EnumValidationRule::createRule(0, range(1,10)),
		    "custom_nameservers" => TCA_EnumValidationRule::createRule(1, array(0,1)),
		    "custom_tech_contact" => TCA_EnumValidationRule::createRule(1, array(0,1)),
		    "contact_set" => TCA_NullValidationRule::createRule(1),
		    "affiliate_id" => TCA_MaxValidationRule::createRule(0, 100),
		    "auto_renew" => TCA_EnumValidationRule::createRule(0, array(0,1)),
		    "handle" => TCA_EnumValidationRule::createRule(0, array('save','process')),
		    "link_domains" => TCA_EnumValidationRule::createRule(0, array(0,1)),
		    "rant_no" => TCA_NullValidationRule::createRule(0),
		    "change_contact" => TCA_EnumValidationRule::createRule(0, array(0,1))
		)),
	    )),
	    'get_domain' => TCA_ValidationRuleList::createRuleList(array(
	    	"cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "type" => TCA_EnumValidationRule::createRule(1, array(
			'waiting_history', 'all_info', 'nameservers', 'owner',
			'admin', 'billing', 'tech', 'list', 'auto_renew_flag',
			'forwarding_email', 'domain_auth_info', 'rsp_whois_info',
			'nexus_info', 'status', 'trademark', 'expire_action',
		    )),
		    "page" => TCA_RegexValidationRule::createRule(0, "/^\d+$/"),
		)),
	    )),
	    'renew_domain' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		    "currentexpirationyear" => TCA_RegexValidationRule::createRule(1, "/^\d{4}$/"),
		    "period" => TCA_EnumValidationRule::createRule(0, range(1,10)),
		    "auto_renew" => TCA_EnumValidationRule::createRule(1, array(0,1)),
		    "handle" => TCA_EnumValidationRule::createRule(1, array('save','process')),
		)),
	    )),
	    'modify_domain' => TCA_ValidationRuleList::createRuleList(array(
	    	"cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "data" => TCA_NullValidationRule::createRule(1),
		    "affect_domains" => TCA_EnumValidationRule::createRule(1, array(0,1)), 
		    "flag" => TCA_EnumValidationRule::createRule(0, array('Y','N')), 
		    "trademark" => TCA_EnumValidationRule::createRule(0, array('Y','N')), 
		    "all" => TCA_EnumValidationRule::createRule(0, array(0,1)), 
		    "let_expire" => TCA_EnumValidationRule::createRule(0, array(0,1)), 
		    "report_email" => TCA_EmailAddressValidationRule::createRule(0), 
		    "nameserver_list" => TCA_ValidationRuleList::createRuleList(array(
			"sortorder" => TCA_EnumValidationRule::createRule(0, range(1,1000)),
			"name" => TCA_HostnameValidationRule::createRule(0),
			"action" => TCA_EnumValidationRule::createRule(0, array('add','update','remove'))
		    )),
		    "contact_set" => TCA_ValidationRuleList::createRuleList(array(
			"also_apply_to" => TCA_SubsetValidationRule::createRule(0, array('owner','admin','billing','tech')),
		    )),
		)),
	    )),
	    'modify_ca_owner' => TCA_ValidationRuleList::createRuleList(array(
	    	"cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "data" => TCA_NullValidationRule::createRule(1),
		    "affect_domains" => TCA_EnumValidationRule::createRule(1, array(0,1)), 
		    "contact_set" => TCA_ValidationRuleList::createRuleList(array(
			"also_apply_to" => TCA_SubsetValidationRule::createRule(0, array('owner','admin','billing','tech')),
		    )),
		)),
	    )),
	    'refund_domain' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		    "reseller" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'get_domain_price' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		    "reg_type" => TCA_EnumValidationRule::createRule(0, array('new','transfer','renewal')),
		)),
	    )),
	    'send_authcode' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain_name" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'send_password_domain' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain_name" => TCA_NullValidationRule::createRule(1),
		    "send_to" => TCA_EnumValidationRule::createRule(1,array('owner','admin')),
		    "sub_user" => TCA_EnumValidationRule::createRule(1,array('0','1')),
		)),
	    )),
	    'belongs_to_rsp' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'cira_email_password' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		)),
	    )),
	    'add_nameserver' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "all" => TCA_EnumValidationRule::createRule(1, array(0,1)),
		    "fqdn" => TCA_HostnameValidationRule::createRule(1),
		    "tld" => TCA_RegexValidationRule::createRule(1, "/^\./"),
		)),
	    )),
	    'create_nameserver' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "name" => TCA_HostnameValidationRule::createRule(1),
		    "ipaddress" => TCA_IPAddressValidationRule::createRule(1),
		)),
	    )),
	    'check_nameserver' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "tld" => TCA_RegexValidationRule::createRule(1, "/^\./"),
		    "fqdn" => TCA_HostnameValidationRule::createRule(1),
		)),
	    )),
	    'get_nameserver' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "name" => TCA_EnumValidationRule::createRule(1, array('all')),
		)),
	    )),
	    'modify_nameserver' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "name" => TCA_HostnameValidationRule::createRule(1),
		    "new_name" => TCA_HostnameValidationRule::createRule(0),
		    "ipaddress" => TCA_IPAddressValidationRule::createRule(1),
		)),
	    )),
	    'delete_nameserver' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "name" => TCA_HostnameValidationRule::createRule(1),
		    "ipaddress" => TCA_IPAddressValidationRule::createRule(1),
		)),
	    )),	    	    
	    'check_transfer' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain" => TCA_NullValidationRule::createRule(1),
		    "get_request_address" => TCA_EnumValidationRule::createRule(0,array(0,1)),
		)),
	    )),	    	    
	    'process_transfer' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "order_id" => TCA_NullValidationRule::createRule(1),
		    "reseller" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'send_password_transfer' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "domain_name" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'cancel_transfer' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "order_id" => TCA_NullValidationRule::createRule(1),
		    "reseller" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'get_order_info' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "order_id" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'check_status_of_queued_req' => TCA_ValidationRuleList::createRuleList(array(
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "request_id" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'process_pending_order' => TCA_ValidationRuleList::createRuleList(array(		
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "order_id" => TCA_NullValidationRule::createRule(1),
		    "command" => TCA_EnumValidationRule::createRule(0, array('cancel')),
		    "fax_received" => TCA_EnumValidationRule::createRule(0, array(0,1)),
		    "owner_address" => TCA_EmailAddressValidationRule::createRule(0),
		)),
	    )),	    	    
	    'add_subuser' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
	        "username" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "sub_username" => TCA_NullValidationRule::createRule(1),
		    "sub_permission" => TCA_NullValidationRule::createRule(1),
		    "sub_password" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'get_subuser' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
	        "username" => TCA_NullValidationRule::createRule(1),
	    )),	    	    
	    'modify_subuser' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
	        "username" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "sub_username" => TCA_NullValidationRule::createRule(1),
		    "sub_permission" => TCA_NullValidationRule::createRule(1),
		    "sub_password" => TCA_NullValidationRule::createRule(1),
		    "sub_id" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'delete_subuser' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
	        "username" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "sub_id" => TCA_NullValidationRule::createRule(1),
		)),
	    )),	    	    
	    'get_user_info' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
	    )),	    	    
	    'change_ownership' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "username" => TCA_RegexValidationRule::createRule(1,"/^[a-z0-9]+$/"),
		    "password" => TCA_DomainPasswordValidationRule::createRule(1),
		    "move_all" => TCA_NullValidationRule::createRule(0),
		    "reg_domain" => TCA_NullValidationRule::createRule(0),
		)),
	    )),	    	    
	    'change_password' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "reg_password" => TCA_DomainPasswordValidationRule::createRule(1),
		)),
	    )),	    	    
	    'advanced_update_nameservers' => TCA_ValidationRuleList::createRuleList(array(
	        "cookie" => TCA_NullValidationRule::createRule(1),
		"attributes" => TCA_ValidationRuleList::createRuleList(array(
		    "op_type" => TCA_EnumValidationRule::createRule(0, array('add_remove','assign')),
		    "remove_ns" => TCA_NullValidationRule::createRule(0),
		    "add_ns" => TCA_NullValidationRule::createRule(0),
		    "assign_ns" => TCA_NullValidationRule::createRule(0),
		    "add_ns_to_foreign_registries" => TCA_EnumValidationRule::createRule(0, array(0,1)),
		)),
	    )),
	);	
    }
} 

class TCA_DomainActionMapper extends TCA_ActionMapper {

    function __construct() {
	$this->actionList = new TCA_ApiActionList();
	$this->initActions();	
    }

    function initActions() {
    	$registrant_ip = $_ENV['REMOTE_ADDR'];
    	
	// add api definitions here
	$this->actionList->addApiAction('set_cookie', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'set', object => 'cookie', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('update_cookie', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'update', object => 'cookie', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('delete_cookie', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'delete', object => 'cookie', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('lookup_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'lookup', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('ca_blocker_list_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'get_ca_blocker_list', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('query_registrant_ca', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'query_registrant', object => 'ca', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('bulk_transfer_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'bulk_transfer', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('sw_register_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'sw_register', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'get', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('renew_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'renew', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('modify_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'modify', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('modify_ca_owner', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'modify', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('refund_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'revoke', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_domain_price', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'get_price', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('send_authcode', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'send_authcode', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('send_password_domain', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'send_password', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('belongs_to_rsp', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'belongs_to_rsp', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('cira_email_password', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'cira_email_pwd', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('add_nameserver', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'registry_add_ns', object => 'nameserver', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('create_nameserver', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'create', object => 'nameserver', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('check_nameserver', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'registry_check_nameserver', object => 'nameserver', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_nameserver', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'get', object => 'nameserver', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('modify_nameserver', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'modify', object => 'nameserver', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('delete_nameserver', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'delete', object => 'nameserver', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('check_transfer', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'check_transfer', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('process_transfer', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'process_transfer', object => 'transfer', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('send_password_transfer', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'send_password', object => 'transfer', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('cancel_transfer', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'cancel_transfer', object => 'transfer', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_order_info', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'get_order_info', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('check_status_of_queued_req', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'query_queued_request', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('process_pending_order', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'process_pending', object => 'domain', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('add_subuser', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'add', object => 'subuser', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_subuser', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'get', object => 'subuser', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('modify_subuser', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'modify', object => 'subuser', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('delete_subuser', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'delete', object => 'subuser', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('get_user_info', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'get', object => 'userinfo', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('change_ownership', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'change', object => 'ownership', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('change_password', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'change', object => 'password', registrant_ip => $registrant_ip),
	)));
	$this->actionList->addApiAction('advanced_update_nameservers', new TCA_ApiAction(array( 
	    simple => array(protocol => 'XCP', action => 'advanced_update_nameservers', object => 'domain', registrant_ip => $registrant_ip),
	)));
	// end api definitions
    }

}

//*****************************************************************************************
class TCA_DomainServiceManager extends TCA_AbstractServiceManager {
    private $requestValidator;
    private $actionPrefix = 'TUCOWS_domains';
    private $actions = array();
    private $domainDispatcher;

    function __construct() {
	parent::__construct();
	$this->requestValidator = new TCA_DomainServiceValidator();
	$this->actionMapper = new TCA_DomainActionMapper();
	$this->actionValidationMapper = new TCA_DomainActionValidationMapper();
	

	// list all actions here (or we could use introspection to build this)
	$this->actions = array(
	   'set_cookie' => 1,
	   'update_cookie' => 1,
	   'delete_cookie' => 1,
	   'lookup_domain' => 1,
	   'ca_blocker_list_domain' => 1,
	   'query_registrant_ca' => 1,
	   'bulk_transfer_domain' => 1,
	   'sw_register_domain' => 1,
	   'get_domain' => 1,
	   'renew_domain' => 1,
	   'modify_domain' => 1,
	   'modify_ca_owner' => 1,
	   'refund_domain' => 1,
	   'get_domain_price' => 1,
	   'send_authcode' => 1,
	   'send_password_domain' => 1,
	   'belongs_to_rsp' => 1,
	   'cira_email_password' => 1,
	   'add_nameserver' => 1,
	   'create_nameserver' => 1,
	   'check_nameserver' => 1,
	   'get_nameserver' => 1,
	   'modify_nameserver' => 1,
	   'delete_nameserver' => 1,
	   'check_transfer' => 1,
	   'process_transfer' => 1,
	   'send_password_transfer' => 1,
	   'cancel_transfer' => 1,
	   'get_order_info' => 1,
	   'check_status_of_queued_req' => 1,
	   'process_pending_order' => 1,
	   'add_subuser' => 1,
	   'get_subuser' => 1,
	   'modify_subuser' => 1,
	   'delete_subuser' => 1,
	   'get_user_info' => 1,
	   'change_ownership' => 1,
	   'change_password' => 1,
	   'advanced_update_nameservers' => 1,
	);
	
	$this->domainDispatcher = new TCA_DomainDispatcher();		
    }

    function getActionPrefix() {
	return $this->actionPrefix;
    }

    function getActions() {
	return $this->actions;
    }

    function getRequestValidator() {
	return $this->requestValidator;
    }
    
    function init() {}
    
    private function domainValidate(TCA_Domain $dom, TCA_ServiceRequest $request) {
    	$this->requestValidator->validateServiceRequest($dom->getRuleList(), $request);
    	$funcRuleList = $dom->getRuleList($request->getActionPostfix());
    	if ($funcRuleList != null) {
    	    $this->requestValidator->validateServiceRequest($funcRuleList, $request);
    	}
    }
    
    protected function set_cookie(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('set_cookie', $request);	
	return $serviceResponseArr[0];
    }
    
    protected function update_cookie(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('update_cookie', $request);	
	return $serviceResponseArr[0];
    }

    protected function delete_cookie(TCA_ServiceRequest $request) {	
	$serviceResponseArr = $this->processOrder('delete_cookie', $request);	
	return $serviceResponseArr[0];
    }
            
    protected function sw_register_domain(TCA_ServiceRequest $request) {
	$reqArr = $request->toArray();
    	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());
	$this->domainValidate($domain, $request);
	
	//contact validation
	$contacts = $domain->getContacts();
	foreach ( $contacts as $contact ) {
	    if ( key_exists($contact->getType(), $reqArr['attributes']['contact_set']) ) {
		$this->requestValidator->validateServiceRequest($contact->getRuleList(), $request);
	    }
	}
	//nameserver list validation
	$nsRuleList = TCA_ValidationRuleList::createRuleList(array(
	    "sortorder" => TCA_RegexValidationRule::createRule(1, "/^\d+$/"),
	    "name" => TCA_HostnameValidationRule::createRule(1),
	));
	$nsArrRuleList = new TCA_ValidationRuleList();
	$ruleList = TCA_ValidationRuleList::createRuleList(array(
	    "attributes" => TCA_ValidationRuleList::createRuleList(array(
		"nameserver_list" => $nsArrRuleList,
	    )),
	));
	foreach ($reqArr['attributes']['nameserver_list'] as $key => $value) {
	    $nsArrRuleList->addRuleList($key, $nsRuleList);
	}
	$this->requestValidator->validateServiceRequest($ruleList, $request);
	
	$serviceResponseArr = $this->processOrder('sw_register_domain', $request);	
	return $serviceResponseArr[0];
    }    

    protected function lookup_domain(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('lookup_domain', $request);
	return $serviceResponseArr[0];
    }
    
    protected function ca_blocker_list_domain(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('ca_blocker_list_domain', $request);
	return $serviceResponseArr[0];
    }
    
	protected function query_registrant_ca(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('query_registrant_ca', $request);
	return $serviceResponseArr[0];
    }
    
	protected function bulk_transfer_domain(TCA_ServiceRequest $request) {
		$serviceResponseArr = $this->processOrder('bulk_transfer_domain', $request);
		return $serviceResponseArr[0];
    }

    protected function get_domain(TCA_ServiceRequest $request) {

    	$serviceResponseArr = $this->processOrder('get_domain', $request);	
	return $serviceResponseArr[0];
    }    

    protected function renew_domain(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('renew_domain', $request);
	return $serviceResponseArr[0];
    }    
    
    protected function modify_domain(TCA_ServiceRequest $request) {
	$reqArr = $request->toArray();
    	$domain = $this->domainDispatcher->getDomain('dummy.generic');	
	
    	$contacts = $domain->getContacts();	
	foreach ( $contacts as $contact ) {
	    if ( $reqArr['attributes']['contact_set'] != null &&
		key_exists($contact->getType(), $reqArr['attributes']['contact_set']) ) {
		$this->requestValidator->validateServiceRequest($contact->getRuleList(), $request);
	    }
	}

	$serviceResponseArr = $this->processOrder('modify_domain', $request);	
	return $serviceResponseArr[0];
    }    

    protected function modify_ca_owner(TCA_ServiceRequest $request) {
/*
	$reqArr = $request->toArray();
    	$domain = $this->domainDispatcher->getDomain('dummy.generic');	
	
    	$contacts = $domain->getContacts();	
	foreach ( $contacts as $contact ) {
	    if ( $reqArr['attributes']['contact_set'] != null &&
		key_exists($contact->getType(), $reqArr['attributes']['contact_set']) ) {
		$this->requestValidator->validateServiceRequest($contact->getRuleList(), $request);
	    }
	}
*/

	$serviceResponseArr = $this->processOrder('modify_domain', $request);	
	return $serviceResponseArr[0];
    }    

    protected function refund_domain(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('refund_domain', $request);
	return $serviceResponseArr[0];
    }        

    protected function get_domain_price(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('get_domain_price', $request);
	return $serviceResponseArr[0];
    }        

    protected function send_authcode(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain_name")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('send_authcode', $request);
	return $serviceResponseArr[0];
    }  
    
    protected function send_password_domain(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain_name")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('send_password_domain', $request);
	return $serviceResponseArr[0];
    }  

    protected function belongs_to_rsp(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('belongs_to_rsp', $request);
	return $serviceResponseArr[0];
    }  
    
    protected function cira_email_password(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('cira_email_password', $request);
	return $serviceResponseArr[0];
    }  
    
    protected function add_nameserver(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('add_nameserver', $request);
	return $serviceResponseArr[0];
    }

    protected function create_nameserver(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('create_nameserver', $request);	
	return $serviceResponseArr[0];
    }
    
    protected function check_nameserver(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('check_nameserver', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_nameserver(TCA_ServiceRequest $request) {    
	$serviceResponseArr = $this->processOrder('get_nameserver', $request);	
	return $serviceResponseArr[0];
    }

    protected function modify_nameserver(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('modify_nameserver', $request);	
	return $serviceResponseArr[0];
    }

    protected function delete_nameserver(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('delete_nameserver', $request);	
	return $serviceResponseArr[0];
    }

    protected function check_transfer(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain")->getValue());	
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('check_transfer', $request);
	return $serviceResponseArr[0];
    }  
    
    protected function process_transfer(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('process_transfer', $request);	
	return $serviceResponseArr[0];
    }

    protected function send_password_transfer(TCA_ServiceRequest $request) {
	$domain = $this->domainDispatcher->getDomain(
	    $request->getElement("attributes")->getElement("domain_name")->getValue());	
	
	// domain specific validation
	$this->domainValidate($domain, $request);
	
	$serviceResponseArr = $this->processOrder('send_password_transfer', $request);
	return $serviceResponseArr[0];
    }  

    protected function cancel_transfer(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('cancel_transfer', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_order_info(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('get_order_info', $request);	
	return $serviceResponseArr[0];
    }

    protected function check_status_of_queued_req(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('check_status_of_queued_req', $request);	
	return $serviceResponseArr[0];
    }

    protected function process_pending_order(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('process_pending_order', $request);	
	return $serviceResponseArr[0];
    }

    protected function add_subuser(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('add_subuser', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_subuser(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('get_subuser', $request);	
	return $serviceResponseArr[0];
    }

    protected function modify_subuser(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('modify_subuser', $request);	
	return $serviceResponseArr[0];
    }

    protected function delete_subuser(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('delete_subuser', $request);	
	return $serviceResponseArr[0];
    }

    protected function get_user_info(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('get_user_info', $request);	
	return $serviceResponseArr[0];
    }

    protected function change_ownership(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('change_ownership', $request);	
	return $serviceResponseArr[0];
    }

    protected function change_password(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('change_password', $request);	
	return $serviceResponseArr[0];
    }

    protected function advanced_update_nameservers(TCA_ServiceRequest $request) {
	$serviceResponseArr = $this->processOrder('advanced_update_nameservers', $request);
	return $serviceResponseArr[0];
    }    
}

?>
