<?php

require_once($_ENV['TCA_HOME']."/cartridges/action_parameters.inc.php");

class TCA_DomainNameserverContainer extends TCA_ActionParameterContainer {
    const NAME = "name";
    const SORT_ORDER = "sortorder";

    function __construct($name) {
    	parent::__construct($name);
    	
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::NAME,
    		"The fully qualified domain name of the nameserver",
    		"", false, "/^.+$/", "Valid domain name"));
    		
    	$this->addParameter(TCA_ActionParameterFactory::newIntegerParameter(self::SORT_ORDER,
    		"The sort order of the nameserver",
    		"Must start at 1 and should not contain any gaps in sequence", false, 1, 1000));
    }

    public function getNameParam() {
	return $this->getParameterByName(self::NAME);
    }
        
    public function getSortorderParam() {
    	return $this->getParameterByName(self::SORT_ORDER);
    }
}

class TCA_DomainNameserverListContainer extends TCA_ActionParameterContainer {
    private $sortorder = 1;
    
    public function addNameserver(TCA_DomainNameserverContainer $nameserver) {
    	$nameserver->getSortorderParam()->setValue($this->sortorder++);
    	$this->addChildContainer($nameserver);
    }

    // overrides super's method
    public function toParameterSet() {
    	$parameterSet = new TCA_ParameterSet();
    	foreach (array_values($this->getChildContainers()) as $i => $nameserverContainer) {
    		$parameterSet->addParameterSet($i, $nameserverContainer->toParameterSet());
    	}
    	return $parameterSet;
    }
}

class TCA_DomainContactSetContainer extends TCA_ActionParameterContainer {    
    public function addContact(TCA_ContactContainer $contact) {
    	$this->addChildContainer($contact);
    }
    
    public function getAdminContact() {
    	return $this->getChildContainerByName(TCA_ContactContainer::ADMIN);
    }
    
    public function getBillingContact() {
    	return $this->getChildContainerByName(TCA_ContactContainer::BILLING);
    }
    
    public function getOwnerContact() {
    	return $this->getChildContainerByName(TCA_ContactContainer::OWNER);
    }
    
    public function getTechContact() {
    	return $this->getChildContainerByName(TCA_ContactContainer::TECH);
    }
}

class TCA_ContactContainer extends TCA_ActionParameterContainer {	
    const ADMIN = "admin";
    const BILLING = "billing";
    const OWNER = "owner";
    const TECH = "tech";
	
    const FIRST_NAME = "first_name";
    const LAST_NAME = "last_name";
    const ORG_NAME = "org_name";
    const ADDRESS1 = "address1";
    const ADDRESS2 = "address2";
    const ADDRESS3 = "address3";
    const CITY = "city";
    const STATE = "state";
    const POSTAL_CODE = "postal_code";
    const COUNTRY = "country";
    const PHONE = "phone";
    const FAX = "fax";
    const EMAIL = "email";
    const URL = "url";    
    
    public function __construct($name) {
    	parent::__construct($name);
    	
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::FIRST_NAME, "First Name",
    		"", true, "/^.{0,64}$/", "Max 64 chars"));
    		
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::LAST_NAME, "Last Name",
    		"", true, "/^.{0,64}$/", "Max 64 chars"));
    	
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::ORG_NAME, "Organization",
    		"", true, "/^.{0,64}$/", "Max 64 chars"));
    		
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::ADDRESS1, "Address 1",
    		"", true, "/^.{0,100}$/", "Max 100 chars"));

    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::ADDRESS2, "Address 2",
    		"", false, "/^.{0,100}$/", "Max 100 chars"));

    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::ADDRESS3, "Address 3",
    		"", false, "/^.{0,100}$/", "Max 100 chars"));

    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::CITY, "City",
    		"", true, "/^.{0,64}$/", "Max 64 chars"));
    		
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::STATE, "State",
    		"", true, "/^.{0,32}$/", "Max 32 chars"));
    		
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::POSTAL_CODE, "Postal Code",
    		"", true, "/^.{0,32}$/", "Max 32 chars"));
    		
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::PHONE, "Phone",
    		"", true, "/^.{0,20}$/", "Max 20 chars"));
    	
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::FAX, "Fax",
    		"", false, "/^.{0,20}$/", "Max 20 chars"));
    	
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::EMAIL, "Email",
    		"", true, "/^.{0,100}$/", "Max 100 chars"));
    	
    	$this->addParameter(TCA_ActionParameterFactory::newStringParameter(self::URL, "URL",
    		"", false, "/^.{0,255}$/", "Max 255 chars"));

    	// country param
    	require("countries.inc.php");
    	$countryElements = array();
    	preg_match_all("/([A-Z]+) (.+)/m", $countriesStr, $matches);
    	foreach ($matches[1] as $i => $code) {
    		$countryElements[] = new TCA_EnumerationElement($code, $matches[2][$i]);
    	}
    	    	
    	$countryParam = TCA_ActionParameterFactory::newEnumeratedParameter(self::COUNTRY, "Country", "",
    		true, new TCA_Enumeration("countries", $countryElements));
    	
	$this->addParameter($countryParam);
    }

    public static function newAdminContact() {
    	return new TCA_ContactContainer(self::ADMIN);
    }

    public static function newBillingContact() {
    	return new TCA_ContactContainer(self::BILLING);
    }

    public static function newOwnerContact() {
    	return new TCA_ContactContainer(self::OWNER);
    }

    public static function newTechContact() {
    	return new TCA_ContactContainer(self::TECH);
    }
        
    public function getFirstNameParam() {
    	return $this->getParameterByName(self::FIRST_NAME);
    }

    public function getLastNameParam() {
    	return $this->getParameterByName(self::LAST_NAME);
    }

    public function getOrgNameParam() {
    	return $this->getParameterByName(self::ORG_NAME);
    }
    
    public function getAddress1Param() {
    	return $this->getParameterByName(self::ADDRESS1);
    }
    
    public function getAddress2Param() {
    	return $this->getParameterByName(self::ADDRESS2);
    }
    
    public function getAddress3Param() {
    	return $this->getParameterByName(self::ADDRESS3);
    }
    
    public function getCityParam() {
    	return $this->getParameterByName(self::CITY);
    }
    
    public function getStateParam() {
    	return $this->getParameterByName(self::STATE);
    }

    public function getCountryParam() {
    	return $this->getParameterByName(self::COUNTRY);
    }
        
    public function getPostalCodeParam() {
    	return $this->getParameterByName(self::POSTAL_CODE);
    }
    
    public function getPhoneParam() {
    	return $this->getParameterByName(self::PHONE);
    }
    
    public function getFaxParam() {
    	return $this->getParameterByName(self::FAX);
    }
    
    public function getEmailParam() {
    	return $this->getParameterByName(self::EMAIL);
    }

    public function getUrlParam() {
    	return $this->getParameterByName(self::URL);
    }
}

?>