<?PHP

require_once($_ENV['TCA_HOME']."/cartridges/abstract_serviceoffering.inc.php");
require_once("lib/domainutils.inc.php");

abstract class TCA_AbstractSwRegisterDomainAction extends TCA_AbstractServiceOfferingAction {
    // constants used in request
    const ACTION = 'sw_register_domain';
    const ATTRIBUTES = 'attributes';
    const REG_TYPE = 'reg_type';
    const CONTACT_SET = 'contact_set';
    const NAMESERVER_LIST = 'nameserver_list';
    const AFFILIATE_ID = 'affiliate_id';
    const DOMAIN = 'domain';    
    const ENCODING_TYPE = 'encoding_type';
    const REG_DOMAIN = 'reg_domain';
    const REG_USERNAME = 'reg_username';
    const REG_PASSWORD = 'reg_password';
    const HANDLE = 'handle';
    const CUSTOM_TECH_CONTACT = 'custom_tech_contact';
    const CUSTOM_NAMESERVERS = 'custom_nameservers';
    const TLD_DATA = 'tld_data';
    const AUTO_RENEW = 'auto_renew';
    const LINK_DOMAINS = 'link_domains';
    const MASTER_ORDER_ID = 'master_order_id';
    
    // constants used in response
    const ADMIN_EMAIL = 'admin_email';
    const ERROR = 'error';
    const ID = 'id';
    const REGISTRATION_CODE = 'registration_code';
    const REGISTRATION_TEXT = 'registration_text';
    const QUEUE_REQUEST_ID = 'queue_request_id';
    const FORCED_PENDING = 'forced_pending';

    // action result data params
    protected $adminEmailParam;
    protected $errorParam;
    protected $idParam;
    protected $registrationCodeParam;
    protected $registrationTextParam;
    protected $queueRequestIdParam;
    protected $forcedPendingParam;
    
    public function __construct($name, TCA_DomainServiceManager $serviceManager) {
    	parent::__construct($name, 1, $serviceManager);
    	
    	$attrContainer = new TCA_ActionParameterContainer(self::ATTRIBUTES);

    	$affiliateIdParam = TCA_ActionParameterFactory::newStringParameter(self::AFFILIATE_ID,
    		"Unique identifier of an RSP s affiliate", "", false, "/^.{0,100}$/", "Maximum 100 char");    	
    	
    	$domainParam = TCA_ActionParameterFactory::newStringParameter(self::DOMAIN,
    		"The domain name", "", true, "/^.+$/", "Valid domain name");

    	require("lib/languages.inc.php");
    	$encodingElements = array();
    	preg_match_all("/([a-z]+) (.+)/m", $encodings, $matches);
    	foreach ($matches[1] as $i => $code) {
    		$encodingElements[] = new TCA_EnumerationElement($code, $matches[2][$i]);
    	}    	    	
    	$encodingTypeParam = TCA_ActionParameterFactory::newEnumeratedParameter(self::ENCODING_TYPE,
    		"The encoding type for the domain", "", true, new TCA_Enumeration("encodings", $encodingElements));
	// set default encoding
    	$encodingTypeParam->setValue('ascii');

    	$regDomainParam = TCA_ActionParameterFactory::newStringParameter(self::REG_DOMAIN,
    		"An existing domain name owned by the registrant",
    		"It will cause the newly registered/transferred domain to be linked with the profile of the already existing domain",
    		true, "/^.+$/", "Valid domain name");
    		
    	$regUserNameParam = TCA_ActionParameterFactory::newStringParameter(self::REG_USERNAME,
    		"The username of the registrant", "", true, "/^.+$/s", "Valid username");
    	
    	$regPasswordParam = TCA_ActionParameterFactory::newStringParameter(self::REG_PASSWORD,
    		"The registrant's password", "", true, "/^.+$/s", "Valid password");

    	$handleElements = array();
    	$handleElements[] = new TCA_EnumerationElement("save", "Save order");
    	$handleElements[] = new TCA_EnumerationElement("process", "Process order");
    	$handleParam = TCA_ActionParameterFactory::newEnumeratedParameter(self::HANDLE,
    		"How the order is to be processed", "", true, new TCA_Enumeration("handle", $handleElements));
    		
    	$customTechContactParam = TCA_ActionParameterFactory::newBooleanParameter(self::CUSTOM_TECH_CONTACT,
    		"Whether to use the RSP's tech contact info, or the tech contact info provided in the request",    		
    		"", true);

    	$customNameServersParam = TCA_ActionParameterFactory::newBooleanParameter(self::CUSTOM_NAMESERVERS, 
    		"Whether to use the RSP's default nameservers, or those provided in the request",
    		 "", true);

    	$autoRenewParam = TCA_ActionParameterFactory::newBooleanParameter(self::AUTO_RENEW,
    		"Whether the domain should be set to auto-renew", "", false);

    	$attrContainer->addParameter($domainParam);
    	$attrContainer->addParameter($regUserNameParam);
    	$attrContainer->addParameter($regPasswordParam);
    	$attrContainer->addParameter($handleParam);
    	$attrContainer->addParameter($customTechContactParam);
    	$attrContainer->addParameter($customNameServersParam);
    	$attrContainer->addParameter($affiliateIdParam);
    	$attrContainer->addParameter($autoRenewParam);
    	$attrContainer->addParameter($regDomainParam);
    	$attrContainer->addParameter($encodingTypeParam);
    	
    	// although tld_data is required parameter according to the API spec, no one except for .us and
    	// .name (tlds that are not supported by current client) seem to really use it
    	$tldDataContainer = new TCA_ActionParameterContainer(self::TLD_DATA);
    	
    	$nameserverList = new TCA_DomainNameserverListContainer(self::NAMESERVER_LIST);
    	$nameserverList->addNameserver(new TCA_DomainNameserverContainer("ns1"));
    	$nameserverList->addNameserver(new TCA_DomainNameserverContainer("ns2"));
    	$nameserverList->addNameserver(new TCA_DomainNameserverContainer("ns3"));
    	$nameserverList->addNameserver(new TCA_DomainNameserverContainer("ns4"));
    	
    	$contactSetContainer = new TCA_DomainContactSetContainer("contact_set");
    	$contactSetContainer->addContact(TCA_ContactContainer::newAdminContact());
    	$contactSetContainer->addContact(TCA_ContactContainer::newBillingContact());
    	$contactSetContainer->addContact(TCA_ContactContainer::newOwnerContact());
    	$contactSetContainer->addContact(TCA_ContactContainer::newTechContact());

    	$attrContainer->addChildContainer($tldDataContainer);
    	$attrContainer->addChildContainer($nameserverList);
    	$attrContainer->addChildContainer($contactSetContainer);
    	
    	$this->getParametersContainer()->addChildContainer($attrContainer);
    	
    	// initialize action result data parameters    	
    	$this->adminEmailParam = TCA_ActionParameterFactory::newStringParameter(self::ADMIN_EMAIL,
    		"Returns the admin email contact from the order",
    		"Can be used to send 'thank you' email for .ca based on existing CIRA's registrant profile",
    		false, "/^+.$/", "Valid email address");
    		
    	$this->errorParam = TCA_ActionParameterFactory::newStringParameter(self::ERROR,
    		"Text description of the errors that occurred in a failed transaction",
    		"", false, "/^.+$/s", "Any text");
    	
    	$this->idParam = TCA_ActionParameterFactory::newIntegerParameter(self::ID, "The ID of the order",
    		"Can be used in RWI queries", true, 1, 999999999);
    		
    	$this->registrationCodeParam = TCA_ActionParameterFactory::newIntegerParameter(self::REGISTRATION_CODE,
    		"The registration code returned by the registry", "", true, -999999999, 999999999);
    		
    	$this->registrationTextParam = TCA_ActionParameterFactory::newStringParameter(self::REGISTRATION_TEXT,
    		"The registration text returned by the registry", "", true, "/^.+$/s", "Any text");
    		
    	$this->queueRequestIdParam = TCA_ActionParameterFactory::newIntegerParameter(self::QUEUE_REQUEST_ID,
    		"The ID of the request as it appears in the queue", "", false, 1, 999999999);
    	
    	$this->forcedPendingParam = TCA_ActionParameterFactory::newIntegerParameter(self::FORCED_PENDING,
    		"Orders that cannot be processed will be forced to the pending queue", "", false, 1, 999999999);
    }
    
    protected function convertServiceResponseToActionResult(TCA_ServiceResponse $serviceResponse) {
    	$actionResult = new TCA_ActionResult($serviceResponse->isSuccess(), $serviceResponse->getResponseCode(),
    		$serviceResponse->getResponseText());
    	
    	$attrParamSet = $serviceResponse->getResponseData()->getElement(self::ATTRIBUTES);
    	if (!is_null($attrParamSet)) {
    		$this->adminEmailParam->setValue(
    			self::getElementValue($attrParamSet->getElement(self::ADMIN_EMAIL)));
    		$this->errorParam->setValue(
    			self::getElementValue($attrParamSet->getElement(self::ERROR)));
    		$this->idParam->setValue(
    			self::getElementValue($attrParamSet->getElement(self::ID)));
    		$this->registrationCodeParam->setValue(
    			self::getElementValue($attrParamSet->getElement(self::REGISTRATION_CODE)));
    		$this->registrationTextParam->setValue(
    			self::getElementValue($attrParamSet->getElement(self::REGISTRATION_TEXT)));
    		$this->queueRequestIdParam->setValue(
    			self::getElementValue($attrParamSet->getElement(self::QUEUE_REQUEST_ID)));
    		$this->forcedPendingParam->setValue(
    			self::getElementValue($attrParamSet->getElement(self::FORCED_PENDING)));
    			
    		$actionResultData = $actionResult->getResultData();
    		
    		self::addNotNullParameter($actionResultData, $this->adminEmailParam);
    		self::addNotNullParameter($actionResultData, $this->errorParam);
    		self::addNotNullParameter($actionResultData, $this->idParam);
    		self::addNotNullParameter($actionResultData, $this->registrationCodeParam);
    		self::addNotNullParameter($actionResultData, $this->registrationTextParam);
    		self::addNotNullParameter($actionResultData, $this->queueRequestIdParam);
    		self::addNotNullParameter($actionResultData, $this->forcedPendingParam);
    	}
    	
    	return $actionResult;
    }
    
    private static function getElementValue($element) {
    	return is_null($element) ? null : $element->getValue();
    }
    
    private static function addNotNullParameter(TCA_ActionParameterContainerI $actionResultData,
    						TCA_ActionValueParameterI $actionParam) {
    	if (!is_null($actionParam->getValue())) {
    		$actionResultData->addParameter($actionParam);
    	}
    }
}

class TCA_ProvisionDomainServiceAction extends TCA_AbstractSwRegisterDomainAction implements
    	TCA_ServiceOfferingAction {
    		
    const PERIOD = 'period';
    		
    public function __construct(TCA_DomainServiceManager $serviceManager) {
    	parent::__construct('provision', $serviceManager);

//    	$linkDomainsParam = TCA_ActionParameterFactory::newBooleanParameter(self::LINK_DOMAINS,
//    		"Whether to link this domain with others",
//    		"Only used when ordering multiple domains over multiple calls", false);
//
//    	$masterOrderIdParam =  TCA_ActionParameterFactory::newIntegerParameter(self::MASTER_ORDER_ID,
//    		"The order ID returned by the first call for a group of domains that are to be linked",
//    		"Only required if 'link_domains' is used", false, 1, 3999999999);

    	$periodElements = array();
    	$periodElements[]= new TCA_EnumerationElement("1", "1 Year");
    	$periodElements[]= new TCA_EnumerationElement("2", "2 Years");
    	$periodElements[]= new TCA_EnumerationElement("3", "3 Years");
    	$periodElements[]= new TCA_EnumerationElement("4", "4 Years");
    	$periodElements[]= new TCA_EnumerationElement("5", "5 Years");
    	$periodElements[]= new TCA_EnumerationElement("6", "6 Years");
    	$periodElements[]= new TCA_EnumerationElement("7", "7 Years");
    	$periodElements[]= new TCA_EnumerationElement("8", "8 Years");
    	$periodElements[]= new TCA_EnumerationElement("9", "9 Years");
    	$periodElements[]= new TCA_EnumerationElement("10", "10 Years");	
    	$periodParam = TCA_ActionParameterFactory::newEnumeratedParameter(self::PERIOD,
    		"The length of the registration period", "", true,
    		new TCA_Enumeration("period", $periodElements));
    	
    	$attrContainer = $this->getParametersContainer()->getChildContainerByName(self::ATTRIBUTES);
    	$attrContainer->addParameter($periodParam);
    }
    
    public function perform($step = 1) {
    	$serviceRequest = $this->serviceManager->createServiceRequest(self::ACTION);

    	$attrContainer = $this->getParametersContainer()->getChildContainerByName(self::ATTRIBUTES);
    	
    	if ($attrContainer->getParameterByName(self::CUSTOM_TECH_CONTACT)->getValue() != 1) {
    		$attrContainer->getChildContainerByName('contact_set')->removeChildContainerByName('tech');
    	}
    	
    	// delete those nameservers for which we don't have 'name' entered
    	$nameServerListParam = $attrContainer->getChildContainerByName(self::NAMESERVER_LIST);
    	foreach ($nameServerListParam->getChildContainers() as $nameserverParam) {
    		$nameParam = $nameserverParam->getParameterByName('name');
    		if ($nameParam->isEmpty()) {
    			$nameServerListParam->removeChildContainerByName($nameserverParam->getName());
    		}
    	}
    	    	    	
    	$attrServReqParamSet = TCA_ServiceRequestParameterSet::createServiceRequestParameterSet(
    		$attrContainer->toParameterSet());
    		
    	// if encoding_type is 'ascii' - overwrite it with blank string
    	if ($attrServReqParamSet->getElement(self::ENCODING_TYPE)->getValue() == 'ascii') {
    		$attrServReqParamSet->getElement(self::ENCODING_TYPE)->setValue("");
    	}
    	
    	$attrServReqParamSet->addParameter(new TCA_Parameter(self::REG_TYPE, "new"));
    	
    	$serviceRequest->addParameterSet(self::ATTRIBUTES, $attrServReqParamSet);
    	
    	$serviceResponse = $this->serviceManager->processServiceRequest($serviceRequest);
    	
    	return $this->convertServiceResponseToActionResult($serviceResponse);    	
    }
} 

//***********************************************************************
class TCA_CaPrivisionDomainServiceAction extends TCA_ProvisionDomainServiceAction implements
		TCA_ServiceOfferingAction {

    const LEGAL_TYPE = 'legal_type';

    public function __construct(TCA_DomainServiceManager $serviceManager) {
    	parent::__construct($serviceManager);

    	$legalTypeElements = array();
    	$legalTypeElements[] = new TCA_EnumerationElement("ABO", "Aboriginal");
    	$legalTypeElements[] = new TCA_EnumerationElement("ASS", "Association");
    	$legalTypeElements[] = new TCA_EnumerationElement("CCO", "Canadian Corporation");
    	$legalTypeElements[] = new TCA_EnumerationElement("CCT", "Canadain Citizen");
    	$legalTypeElements[] = new TCA_EnumerationElement("EDU", "Education Institution");
    	$legalTypeElements[] = new TCA_EnumerationElement("GOV", "Government");
    	$legalTypeElements[] = new TCA_EnumerationElement("HOP", "Hospital");
    	$legalTypeElements[] = new TCA_EnumerationElement("INB", "Indian Band");
    	$legalTypeElements[] = new TCA_EnumerationElement("LAM", "Library, Archive or Museum");
    	$legalTypeElements[] = new TCA_EnumerationElement("LGR", "Legal Representative");
    	$legalTypeElements[] = new TCA_EnumerationElement("MAJ", "Her Majestry the Queen");
    	$legalTypeElements[] = new TCA_EnumerationElement("OMK", "Protected by Trademark Acts");
    	$legalTypeElements[] = new TCA_EnumerationElement("PLT", "Political Party");
    	$legalTypeElements[] = new TCA_EnumerationElement("PRT", "Partnership");
    	$legalTypeElements[] = new TCA_EnumerationElement("RES", "Permanent Resident");
    	$legalTypeElements[] = new TCA_EnumerationElement("TDM", "Trademark Owner");
    	$legalTypeElements[] = new TCA_EnumerationElement("TRD", "Trade Union");
    	$legalTypeElements[] = new TCA_EnumerationElement("TRS", "Trust"); 
    	$legalTypeParam = TCA_ActionParameterFactory::newEnumeratedParameter(self::LEGAL_TYPE,
    		"Domain name owner's legal classification", "", true,
    		new TCA_Enumeration(self::LEGAL_TYPE, $legalTypeElements));

    	$attrContainer = $this->getParametersContainer()->getChildContainerByName(self::ATTRIBUTES);
    	$attrContainer->addParameter($legalTypeParam);
    }
}

?>
