<?PHP
require_once($_ENV['TCA_HOME']."/cartridges/abstract_serviceoffering.inc.php");
require_once("lib/domainutils.inc.php");

class TCA_QueryDomainServiceAction extends TCA_AbstractServiceOfferingAction implements
    	TCA_ServiceOfferingAction {
    		
    const ATTRIBUTES = 'attributes';
    const ACTION = 'get_domain';
    const COOKIE = "cookie";
    
    public function __construct(TCA_DomainServiceManager $serviceManager) {
    	parent::__construct("query", 1, $serviceManager);

    	$cookieParam = TCA_ActionParameterFactory::newStringParameter(self::COOKIE, "cookie", "", true,
    		"/^\d+\:\d+$/", "Valid domain cookie");

    	$typeElements = array();
    	$typeElements[] = new TCA_EnumerationElement("all_info", "All information");
    	$typeElements[] = new TCA_EnumerationElement("waiting_history", "Information on asynchronous requests");
    	$typeElements[] = new TCA_EnumerationElement("nameservers", "Nameserver information");
    	$typeElements[] = new TCA_EnumerationElement("owner", "Owner contact information");
    	$typeElements[] = new TCA_EnumerationElement("admin", "Admin contact information");
    	$typeElements[] = new TCA_EnumerationElement("billing", "Billing contact information");
    	$typeElements[] = new TCA_EnumerationElement("tech", "Technical contact information");
    	$typeElements[] = new TCA_EnumerationElement("domain_auth_info", "Domain authorization code");
    	$typeElements[] = new TCA_EnumerationElement("rsp_whois_info", "Name and contact information for RSP");
    	$typeElements[] = new TCA_EnumerationElement("status", "Lock status of the domain");
    	$typeElements[] = new TCA_EnumerationElement("expire_action", "Action to be taken upon domain expiry");
    	    	      
    	$typeParam = TCA_ActionParameterFactory::newEnumeratedParameter("type", "get_domain query type", "", true,
    		new TCA_Enumeration("type", $typeElements));

    	$pageParam = TCA_ActionParameterFactory::newIntegerParameter("page", "Page number to retrieve", "",
    		false, 0, 999999);
    	
    	$minToExpireParam = TCA_ActionParameterFactory::newIntegerParameter("min_to_expire",
    		"Expiration range in days", "", false, -999999, 999999);
    		
    	$maxToExpireParam = TCA_ActionParameterFactory::newIntegerParameter("max_to_expire",
    		"Expiration range in days", "", false, -999999, 999999);
    			
    	$this->getParametersContainer()->addParameter($cookieParam);
        		
    	$attrContainer = new TCA_ActionParameterContainer(self::ATTRIBUTES);
    	$attrContainer->addParameter($typeParam);
    	$attrContainer->addParameter($pageParam);
    	$attrContainer->addParameter($minToExpireParam);
    	$attrContainer->addParameter($maxToExpireParam);
    	
	$this->getParametersContainer()->addChildContainer($attrContainer);
    }
    
    public function perform($step = 1) {
    	$attrContainer = $this->getParametersContainer()->getChildContainerByName(self::ATTRIBUTES);    
    	$attrServiceRequestParams =
    		TCA_ServiceRequestParameterSet::createServiceRequestParameterSet($attrContainer->toParameterSet());
    	    	
    	$serviceRequest = $this->serviceManager->createServiceRequest(self::ACTION);
    	
    	$cookieParam = $this->getParametersContainer()->getParameterByName(self::COOKIE);
    	$cookieServiceRequestParam = new TCA_ServiceRequestParameter($cookieParam->getName(),
    		$cookieParam->getValue());
    		
    	$serviceRequest->addParameter($cookieServiceRequestParam);
    	$serviceRequest->addParameterSet(self::ATTRIBUTES, $attrServiceRequestParams);
    	
	$serviceResponse = $this->serviceManager->processServiceRequest($serviceRequest);
	
	return self::convertToActionResult($serviceResponse);
    }
}

?>