<?PHP
require_once($_ENV['TCA_HOME']."/cartridges/abstract_serviceoffering.inc.php");
require_once("lib/domainutils.inc.php");

class TCA_RefundDomainServiceAction extends TCA_AbstractServiceOfferingAction implements
    TCA_ServiceOfferingAction {
    		
    const ATTRIBUTES = 'attributes';
    const ACTION = 'refund_domain';
    const DOMAIN = "domain";
    const RESELLER = "reseller";
    
    private $chargeParam;
    private $priceParam;
    
    public function __construct(TCA_DomainServiceManager $serviceManager) {
    	parent::__construct("refund", 1, $serviceManager);
    	
    	$domainParam = TCA_ActionParameterFactory::newStringParameter(self::DOMAIN, "Domain to be revoked",
    		"", true, "/^.+$/", "Valid domain name");
        
    	$resellerParam = TCA_ActionParameterFactory::newStringParameter(self::RESELLER, "Reseller Username",
    		"", true, "/^.+$/", "Valid reseller username");

    	$notesParam = TCA_ActionParameterFactory::newStringParameter("notes",
    		"Notes will be saved to domain notes", "", false, "/^.+$/s", "Any text");
    		
    	$attrContainer = new TCA_ActionParameterContainer(self::ATTRIBUTES);
    	$attrContainer->addParameter($domainParam);
    	$attrContainer->addParameter($resellerParam);
    	$attrContainer->addParameter($notesParam);
        	
    	$this->getParametersContainer()->addChildContainer($attrContainer);
    	
    	// init. action result params
    	$this->chargeParam = TCA_ActionParameterFactory::newBooleanParameter("charge",
    		"Whether the reseller has been charged", "", false);
    	$this->priceParam = TCA_ActionParameterFactory::newNumericParameter("price",
    		"The sum of money of that the reseller is charged", "", false, 0, 10000);
    }
    
    public function perform($step = 1) {    	    	
    	$attrContainer = $this->getParametersContainer()->getChildContainerByName(self::ATTRIBUTES);
    	$attrServiceRequestParams =
    		TCA_ServiceRequestParameterSet::createServiceRequestParameterSet($attrContainer->toParameterSet());
    	$serviceRequest = $this->serviceManager->createServiceRequest(self::ACTION);    	    	
    	$serviceRequest->addParameterSet(self::ATTRIBUTES, $attrServiceRequestParams);
    	
    	$serviceResponse = $this->serviceManager->processServiceRequest($serviceRequest);

    	$actionResult = new TCA_ActionResult($serviceResponse->isSuccess(), $serviceResponse->getResponseCode(),
    		$serviceResponse->getResponseText());    	
    	
    	if ($actionResult->isSuccess()) {
    		$responseData = $serviceResponse->getResponseData();
    		
		$charge = $responseData->getElement(self::ATTRIBUTES)->getElement('charge')->getValue();
		$this->chargeParam->setValue($charge);
	
		$price = $responseData->getElement(self::ATTRIBUTES)->getElement('price')->getValue();
		$this->priceParam->setValue($price);
		
		$actionResult->getResultData()->addParameter($this->chargeParam);
		$actionResult->getResultData()->addParameter($this->priceParam);
    	}

    	return $actionResult;
    }
}
?>