<?PHP
require_once($_ENV['TCA_HOME']."/cartridges/abstract_serviceoffering.inc.php");
require_once("lib/domainutils.inc.php");

class TCA_RenewDomainServiceAction extends TCA_AbstractServiceOfferingAction implements
    	TCA_ServiceOfferingAction {
    		
    const ATTRIBUTES = 'attributes';
    const ACTION = 'renew_domain';
    
    const AFFILIATE_ID = "affiliate_id";
    const DOMAIN = "domain";
    const PERIOD = "period";
    const AUTO_RENEW = "auto_renew";
    const HANDLE = "handle";
    const CURRENTEXPIRATIONYEAR = "currentexpirationyear";
    const ORDER_ID = "order_id";
    const ID = "id";
    const QUEUE_REQUEST_ID = "queue_request_id";
    const REGISTRATION_EXPIRATION_DATE = "registration expiration date";
    
    // make "auto renew" parameter to be property of class becacuse it's used in request and response
    private $autoRenewParam;
    private $regExpirationParam;
    private $orderIdParam;
    private $domainIdParam;
    private $queueRequestIdParam;

    public function __construct(TCA_DomainServiceManager $serviceManager) {
    	parent::__construct("renew", 1, $serviceManager);

    	$affiliateIdParam = TCA_ActionParameterFactory::newStringParameter(self::AFFILIATE_ID,
    		"Unique identifier of an RSP s affiliate", "", false, "^.{0,100}$", "Maximum 100 char");
    	
    	$domainParam = TCA_ActionParameterFactory::newStringParameter(self::DOMAIN, "Domain to be renewed",
    		"Domain must be registered, and must exist in both OpenSRS and the appropriate registry",
    		true, "/^.+$/", "Valid domain name");
    	
    	$periodParam = TCA_ActionParameterFactory::newIntegerParameter(self::PERIOD, "The renewal period",
    		"May not exceed 10 years", true, 1, 11);
    		
    	$this->autoRenewParam = TCA_ActionParameterFactory::newBooleanParameter(self::AUTO_RENEW,
    		"Whether the domain should be set to auto-renew", "", true);

	$handleEnumElements = array();
	$handleEnumElements[] = new TCA_EnumerationElement("save", "Pend the order for the RSP's later approval");
	$handleEnumElements[] = new TCA_EnumerationElement("process", "Process the order immediately");
        $handleParam = TCA_ActionParameterFactory::newEnumeratedParameter(self::HANDLE,
        	"Processing instruction", "", true, new TCA_Enumeration("handle", $handleEnumElements));

        $currentExpirationYearParam = TCA_ActionParameterFactory::newIntegerParameter(self::CURRENTEXPIRATIONYEAR,
        	"The domain's current expiration year", "Must match the data in the registry", true, 2000, 2100);
        	
    	$attrContainer = new TCA_ActionParameterContainer(self::ATTRIBUTES);
    	$attrContainer->addParameter($affiliateIdParam);
    	$attrContainer->addParameter($domainParam);
    	$attrContainer->addParameter($periodParam);
    	$attrContainer->addParameter($this->autoRenewParam);
    	$attrContainer->addParameter($handleParam);
    	$attrContainer->addParameter($currentExpirationYearParam);
  
  	$this->getParametersContainer()->addChildContainer($attrContainer);
  	
  	// params used in response
    	$this->regExpirationParam = TCA_ActionParameterFactory::newStringParameter(
    		self::REGISTRATION_EXPIRATION_DATE, "The new expiration date of the domain", "",
    		true, "/^.+$/", "Valid date");
    	$this->orderIdParam = TCA_ActionParameterFactory::newIntegerParameter(self::ORDER_ID,
    		"The order ID of the renewal", "", true, 1, 3999999999);
    	$this->domainIdParam = TCA_ActionParameterFactory::newIntegerParameter(self::ID, "Domain ID",
    		"Returned if order is not queued", false, 1, 3999999999);
    	$this->queueRequestIdParam = TCA_ActionParameterFactory::newIntegerParameter(self::QUEUE_REQUEST_ID,
    		"The ID of the request as it occurs within the queue", "Only returned if order is queued",
    		false, 1, 3999999999);
    }
    
    public function perform($step = 1) {
    	$serviceRequest = $this->serviceManager->createServiceRequest(self::ACTION);
    	$attrContainer = $this->getParametersContainer()->getChildContainerByName(self::ATTRIBUTES);
    	$attrServiceRequestParams = TCA_ServiceRequestParameterSet::createServiceRequestParameterSet(
    		$attrContainer->toParameterSet());
    	$serviceRequest = $this->serviceManager->createServiceRequest(self::ACTION);
    	$serviceRequest->addParameterSet(self::ATTRIBUTES, $attrServiceRequestParams);
    	
    	$serviceResponse = $this->serviceManager->processServiceRequest($serviceRequest);

    	$actionResult = new TCA_ActionResult($serviceResponse->isSuccess(), $serviceResponse->getResponseCode(),
    		$serviceResponse->getResponseText());
    	
    	if ($actionResult->isSuccess()) {    		
    		$responseData = $serviceResponse->getResponseData();
    		$dataAttr = $responseData->getElement(self::ATTRIBUTES);
    		
    		$actionResultData = $actionResult->getResultData();
    		
    		$regExpirDate = $dataAttr->getElement(self::REGISTRATION_EXPIRATION_DATE)->getValue();
    		$this->regExpirationParam->setValue($regExpirDate);
    		$actionResultData->addParameter($this->regExpirationParam);
    		
    		$autoRenew = $dataAttr->getElement(self::AUTO_RENEW)->getValue();
    		$this->autoRenewParam->setValue($autoRenew);
    		$actionResultData->addParameter($this->autoRenewParam);    		
    		
    		$orderId = $dataAttr->getElement(self::ORDER_ID)->getValue();
    		$this->orderIdParam->setValue($orderId);
    		$actionResultData->addParameter($this->orderIdParam);
    		
    		$domainIdElement = $dataAttr->getElement(self::ID);
    		if (!is_null($domainIdElement)) {
    			$this->domainIdParam->setValue($domainIdElement->getValue());
    			$actionResultData->addParameter($this->domainIdParam);
    		}
    		
    		$queueRequestIdElement = $dataAttr->getElement(self::QUEUE_REQUEST_ID);
    		if (!is_null($queueRequestIdElement)) {
    			$this->queueRequestIdParam->setValue($queueRequestIdElement->getValue());
    			$actionResultData->addParameter($this->queueRequestIdParam);
    		}
    	}
    	
    	return $actionResult;
    }   
}

?>
