<?php

require_once("util/parameter.inc.php");
require_once("util/validator.inc.php");
require_once("util/configreader.inc.php");
require_once("util/string.inc.php");
require_once("util/path.inc.php");
require_once("mercury/workflow.inc.php");
require_once("servicemanager.inc.php");
require_once("connector.inc.php");
require_once("cartridges/cartridgemanager.inc.php");
//require_once("cartridges/catalogue.inc.php");

class TCA_Client {
    const BUILD_NR = 1;
    const VERSION = "0.90";
    const REQUIRED_PHP_VERSION = "5.0.0";

    private $config;
    private $serviceProviders;
    
    // managers
    private $workOrderManager;
    private $serviceController;
    //private $servicePackageManager;
    private $cartridgeManager;

    function __construct(TCA_ConfigReader $reader) {
	$this->config = $reader->parseConfig();
	$this->actions = array();     
	
	$this->workOrderManager = new TCA_WorkOrderManager();
		
	$this->init();

        $this->cartridgeManager = TCA_CartridgeManager::getCartridgeManager();
        foreach ($this->cartridgeManager->getCartridges() as $cartridge) {
        	$cartridge->getServiceManager()->setWorkOrderManager($this->getWorkOrderManager());
        }
        
	$this->serviceController =  new TCA_ServiceController();
	$this->serviceController->setWorkOrderManager($this->workOrderManager);
	$this->serviceController->setCartridgeManager($this->cartridgeManager);
	$this->serviceController->init();
	
	//$this->servicePackageManager = $this->createAndConfigureServicePackageManager($this->cartridgeManager);
    }        

    static function checkPHP() {
	if (version_compare(phpversion(),TCA_Client::REQUIRED_PHP_VERSION,"<")){
	    throw new Exception("Need PHP version ".
				 TCA_Client::REQUIRED_PHP_VERSION.
				" or higher. You have: ".phpversion());
	}
    }

    // XXX make it OS-specific
    // I am not moving it to TCA_Path due to bootstrapping issues
    static function getDirSeparator() {
	return ':';
    }

    // set up environment, can be called statically, without client object
    static function initEnv() {
	if ( !isset($_ENV['TCA_HOME'])) $_ENV['TCA_HOME'] = getcwd();
	if ( !isset($_ENV['TCA_CARTRIDGES'])) $_ENV['TCA_CARTRIDGES'] = $_ENV['TCA_HOME']."/cartridges";  
	set_include_path(get_include_path().
			 TCA_Client::getDirSeparator().
			 $_ENV['TCA_HOME']);

	if ( !isset($_ENV['TCA_ETC'])) $_ENV['TCA_ETC'] = $_ENV['TCA_HOME'];
    }

    function setLogFile($filename) {
	$this->workOrderManager->setLogFile($filename);
    }

    function init() {
	TCA_Client::checkPHP();
	TCA_Client::initEnv();

	$connectors = array(); // Array that holds all connectors for Work Orders
	
	$providersArray = $this->config->communication->providers->provider;
	if(!is_array($providersArray)) // If there is only one, it's not an array so make it one
		$providersArray = array($providersArray);
	
	foreach($providersArray as $provider) {
		
		// set the data for the communication layer
		$resSettings = $provider->{"reseller-settings"};
		// In a switch b/c some types may have different auth values (other than username, password)
		switch ($resSettings->{"reseller-auth-info"}->type->value) {
			case "tucows-services":
				$authinfo = new $provider->{"reseller-settings"}->{"reseller-auth-info"}->{"class"}->value (
			    $resSettings->{"reseller-auth-info"}->username->value,
			    $resSettings->{"reseller-auth-info"}->{"private-key"}->value
				);
				break;
			case "blogware":
				$authinfo = new $provider->{"reseller-settings"}->{"reseller-auth-info"}->{"class"}->value (
			    $resSettings->{"reseller-auth-info"}->username->value,
			    $resSettings->{"reseller-auth-info"}->{"password"}->value
			    );
				break;
		}

		$conn = new TCA_Connector(
		  $resSettings->{"service-endpoint"}->{"connection-type"}->class->value,
		  $resSettings->{"service-endpoint"}->{"connection-type"}->{"payload-class"}->value,
		  $resSettings->{"service-endpoint"}->{"encryption-type"}->class->value);
		$conn->setResellerAuthInfo($authinfo);
		
		switch ($resSettings->{"service-endpoint"}->type->value) {
			case "cbc":
				$remoteEnd = new TCA_RemoteEndCBC(
				$resSettings->{"service-endpoint"}->hostname->value,
				$resSettings->{"service-endpoint"}->port->value);
				break;
			case "https":
				$remoteEnd = new TCA_RemoteEndHTTPS(
				$resSettings->{"service-endpoint"}->hostname->value,
				$resSettings->{"service-endpoint"}->port->value);
				break;
			case "xmlrpc":
				$remoteEnd = new TCA_RemoteEndXMLRPC(
				$resSettings->{"service-endpoint"}->hostname->value,
				$resSettings->{"service-endpoint"}->port->value,
				$resSettings->{"service-endpoint"}->uri->value
				);
				break;
		}
				
		$conn->setRemoteEnd($remoteEnd); 
	
		$connectors[$provider->name->value] = $conn; // Insert new connector into list
	}

	$this->workOrderManager->setConnectors($connectors);
	
	// Load Service Providers
	$this->serviceProviders = array();
	
	$providersArray = $this->config->{"service-managers"}->{"service-manager"};
	if(!is_array($providersArray)) // If there is only one, it's not an array so make it one
		$providersArray = array($providersArray);
	
	foreach($providersArray as $provider) {
		$this->serviceProviders[$provider->name->value] = $provider->provider->value;	
	}
    }
/*
    private function createAndConfigureServicePackageManager() {
    	$servicePackageManager = new TCA_ServicePackageManager();
    	
    	// read all the offerings available in cartridges into "name --> offering" map
    	$soMap = array();
    	foreach ($this->cartridgeManager->getCartridges() as $cartridge) {
    		foreach ($cartridge->getServiceOfferings() as $serviceOffering) {
    			if (isset($soMap[$serviceOffering->getName()])) {
    				throw new Exception("Service offering " .
    					$serviceOffering->getName() . " has been already defined");
    			}
    			$soMap[$serviceOffering->getName()] = $serviceOffering;
    		}
    	}

    	// iterate throught service package definitions    	
    	$packageList = $this->config->__get('service-package-list')->__get('service-package');
    	$packages = array();
    	if (is_array($packageList)) {
    		$packages = $packageList;
    	} else {
    		$packages[] = $packageList;
    	}
    	
	foreach ($packages as $package) {
    		$name = $package->__get('name')->value();
    		$description = $package->__get('description')->value();
    		$price = $package->__get('price')->value();
    		$servicePackage = new TCA_ServicePackage($name, $description, $price);
    		
    		$offeringList = $package->__get('service-offering-list')->__get('service-offering');
    		$soNames = array();
		if (is_array($offeringList)) {
			foreach ($offeringList as $offering) {
	    			$soNames[] = $offering->value();
			}
    		} else {
			$soNames[] = $offeringList->value();
    		}
    	
    		foreach ($soNames as $soName) {
    			$so = $soMap[$soName];
    			if (is_null($so)) {
    				throw new Exception("Can't find service offering named: " . $soName);
    			}
    			$servicePackage->addServiceOffering($so);
    		}    		
    		$servicePackageManager->addServicePackage($servicePackage);
    	}
    	
    	return $servicePackageManager;
    }
 */   
    // service manager layer API
    function createServiceRequest($action) {
    $request = new TCA_ServiceRequest($action);
    // Set Provider for Request
    $request->setProvider($this->serviceProviders[$request->getActionPrefix()]);
	return $request;
    }

    function processServiceRequest(TCA_ServiceRequest $req) {
	return $this->serviceController->processServiceRequest($req);
    }

    // Mercury WorkOrder API (called directly)
    function createWorkOrder() {
	return new TCA_WorkOrder();
    }
    
    function processWorkOrder(TCA_WorkOrder $workOrder) {
	$this->workOrderManager->setWorkOrder($workOrder);
	return $this->workOrderManager->process();
    }

    // this is obsolete - remove any usaged of it
    // use getServicePackageManager() instead
    function getCartridgeManager() {
    	return  $this->cartridgeManager;
    }
 /*   
    function getServicePackageManager() {
    	return $this->servicePackageManager;
    }
 */   
    function getWorkOrderManager(){
    	return $this->workOrderManager;
    }
    
}

?>
