<?php

interface TCA_Authenticationable {
    function login();
    function logout();
}

class TCA_Authenticator {
    protected $connector;
    protected $authenticated;
}

class TCA_HttpsAuthenticator extends TCA_Authenticator implements TCA_Authenticationable {
    
    function __construct($connector) {}

    function login() {
     	return true;
    }
    
    function logout() {
	return true;
    }
}

class TCA_CbcAuthenticator extends TCA_Authenticator implements TCA_Authenticationable {
    
    function __construct($connector) {
    	$this->connector = $connector;
    	$this->authenticated = false;
    }

    function __destruct() {
	if ( $this->authenticated ) 
	    $this->logout();
    }
    
    function login() {
    	if ( $this->authenticated ) 
	    return true;

	if ( !$this->connector->getConnectable()->openSocket(
	    $this->connector->getRemoteEnd(),
	    $this->connector->getResellerAuthInfo()) )
	    throw new TCA_ConnectionException('Failed to open socket');
	    
	$response = $this->readData();

	if ( $response['response_code'] == 555 ) {
	    # ip address not accepted
	    throw new TCA_ConnectionException("Authentication failed: ".$response['response_text']);
	} else if ( !preg_match('/OpenSRS\sSERVER/', $response['attributes']['sender']) ||
	    !preg_match('/^(XML|TPP)/', $response['attributes']['version']) )
	{
	    throw new TCA_ConnectionException("Authentication failed: Unrecognized Peer");
	}	
	
	$request = array(
	    action => "check",
	    object => "version",
	    attributes => array(
		sender => "OpenSRS CLIENT",
		version => 'XML:0.1', //conf
		state => "ready"
	    )
	);
	$this->sendData($request);
			
	$request = array(
	    action => "authenticate",
	    object => "user",
	    attributes => array(
		crypt_type => $this->connector->getCipher()->getCryptType(),
		username => $this->connector->getResellerAuthInfo()->getUsername(),
	    )
	);
	$this->sendData($request);
	
	// Encrypt the challenge bytes with our password to generate
	// the session key.
 	// Respond to the challenge with the MD5 checksum of the challenge.
	// Note: no XML encoding
 	$challenge = $this->readData(false, false);
 	
	$request = pack('H*',md5($challenge));
	$this->sendData($request, true, false); // no XML encoding
	
	$response = $this->readData(true);
	
	if ( preg_match('/^2/', $response['response_code']) ) {
	    $this->authenticated = true;
	} else {
	    if ( $response['response_text'] == "No response from server" ) {
		throw new TCA_ConnectionException("Authentication failed: Invalid private key");
	    }
	    throw new TCA_ConnectionException("Authentication failed: " . $response['response_text']);	    
	}	
	
	return $this->authenticated;
    }

    function readData($cipher = false, $xml = true) {

    	$response = $this->connector->getConnectable()->readData();
	if ( $cipher )
	    $response = $this->connector->getCipher()->decrypt($response);
	if ( $xml ) 
	    $response = $this->connector->getPayloadTranslator()->decode($response);

	return $response;
    }
    
    function sendData($in, $cipher = false, $xml = true) {

    	if ( $xml )
    	    $in = $this->connector->getPayloadTranslator()->encode($in);	
    	if ( $cipher )
	    $in = $this->connector->getCipher()->encrypt($in);
	$this->connector->getConnectable()->sendData($in);
    }
    
    function logout() {
	
	$this->sendData( array(
	    action => "quit",
	    object => "session",
	), true);
	$this->authenticated = false;	
    }
}


?>
