<?php

interface TCA_Cipherable {
    function encrypt($msg);
    function decrypt($msg);
    function getCryptType();
    function initialize($key);
}

class TCA_Cipher {
    protected $privateKey;
    protected $cryptType;
    protected $cipher;	
}

// This class doesn't encrypt anything, simply returns the messages
// Provided for when no encryption is required and for completeness
class TCA_NullCipher extends TCA_Cipher implements TCA_Cipherable {
    
    public function __construct() {
	$this->cryptType = 'NullCrypt';
    }

    public function initialize($key) {}
   
    public function getCryptType() {
	return $this->cryptType;
    }
    
    public function encrypt($msg) {
	return $msg;
    }

    public function decrypt($msg) {
	return $msg;
    }
}

class TCA_BlowfishCipher extends TCA_Cipher implements TCA_Cipherable {
        
    public function __construct() {
	$this->cryptType = 'Blowfish';
	unset($this->cipher);
    }

    public function initialize($key) {
	
	//this is here intentionally...do not move up.    	
	require_once 'Crypt/CBC.php';

    	if ( !isset($this->cipher) ) {
	    $this->privateKey = $key;
	    $this->cipher = new Crypt_CBC(pack('H*', $this->privateKey), $this->cryptType );
    
	    if (!$this->cipher instanceof Crypt_CBC)
		throw new TCA_ConnectionException('Failed to instantiate Blowfish cipher');
    	}
    }
    
    public function getCryptType () {
	return $this->cryptType;
    }
        
    public function encrypt($msg) {
	return $this->cipher->encrypt($msg);
    }

    public function decrypt($msg) {
	return $this->cipher->decrypt($msg);
    }
}

class TCA_DesCipher extends TCA_Cipher implements TCA_Cipherable {
    
    public function __construct() {
	$this->cryptType = 'DES';
	unset($this->cipher);
    }

    public function initialize($key) {

	//this is here intentionally...do not move up.    	
	require_once 'Crypt/CBC.php';
    	
    	if ( !isset($this->cipher) ) {
	    $this->privateKey = $key;
	    $this->cipher = new Crypt_CBC(pack('H*', $this->privateKey), $this->cryptType );
    
	    if (!$this->cipher instanceof Crypt_CBC)
		throw new TCA_ConnectionException('Failed to instantiate Des cipher');
    	}
    }
    
    public function getCryptType () {
	return $this->cryptType;
    }
        
    public function encrypt($msg) {
	return $this->cipher->encrypt($msg);
    }

    public function decrypt($msg) {
	return $this->cipher->decrypt($msg);
    }
}
?>
