<?php

class TCA_WorkUnitParameter extends TCA_Parameter {}

class TCA_WorkUnitParameterSet extends TCA_ParameterSet {
	
    function fromArray($ar) {
	$set = new TCA_WorkUnitParameterSet();	
    	foreach ( $ar as $key => $value ) {
 	    if ( is_array($value) ) {
 	    	$set->addParameterSet($key, $this->fromArray($value));
 	    } else {
		$set->addParameter(new TCA_WorkUnitParameter($key, $value));
 	    }
    	}
	return $set;    	
    }
}

abstract class TCA_WorkOrderResponseStatus {
    protected $isSuccess = 0;
    protected $responseCode;
    protected $responseText;

    abstract function isSuccess();
}

class TCA_WorkUnitResponse extends TCA_WorkOrderResponseStatus {
    private $parameterSet;

    function __construct(TCA_ConnectorResponse $res) {
	$response = $res->getResponse();
	$this->isSuccess = $response['is_success'];
	$this->responseCode = $response['response_code'];
	$this->responseText = $response['response_text'];
	$this->parameterSet = new TCA_WorkUnitParameterSet();
	$this->parameterSet = $this->fromConnectorResponse($response, $this->parameterSet);
    }

    function isSuccess() {
	return $this->isSuccess;
    }

    function getResponseCode() {
	return $this->responseCode;
    }
    
    function getResponseText() {
	return $this->responseText;
    }

    function getResponseData() {
	return $this->parameterSet;
    }
    
    private function fromConnectorResponse($response, $paraSet) {
	foreach($response as $key => $value) {

	    if ($value == null || is_string($value) || is_int($value)) {
		$paraSet->addParameter(new TCA_WorkUnitParameter($key, $value));
	    } else if (is_array($value)) {
		$newParaSet = new TCA_WorkUnitParameterSet();
		$subParaSet = $this->fromConnectorResponse($value, $newParaSet);
		$paraSet->addParameterSet($key, $subParaSet);
	    } else if (is_object($value)) {
		$arrval = get_object_vars($value);
		$newParaSet = new TCA_WorkUnitParameterSet();
		$subParaSet = $this->fromConnectorResponse($arrval, $newParaSet);
		$paraSet->addParameterSet($key, $subParaSet);
		}
	}
		
	return $paraSet;
    }

    function __toString() {
	$str = "    WorkUnitResponse:\n";
	$str .= $this->parameterSet->__toString();
	return $str;
    }
}

class TCA_WorkUnitBundleResponse extends TCA_WorkOrderResponseStatus {
    private $workUnitResponseList = array();
    private $isSuccessSet = false;

    function isSuccess() {
	return $this->isSuccess;
    }
    
    function addWorkUnitResponse(TCA_WorkUnitResponse $workUnit) {
	array_push($this->workUnitResponseList, $workUnit);
	if (! $this->isSuccessSet) {
	    $this->isSuccess = $workUnit->isSuccess();
	    $this->isSuccessSet = true;
	} else {
	    $this->isSuccess &= $workUnit->isSuccess();
	}
    }

    function getWorkUnitResponseList() { return $this->workUnitResponseList; }

    function __toString() {
	$str = "    WorkUnitBundleResponse:\n";
	foreach( $this->workUnitResponseList as $unit) {
	    $str .= "    ".$unit->__toString()."\n";
	}

	return $str;
    }

}

class TCA_WorkOrderResponse extends TCA_WorkOrderResponseStatus {
    private $workUnitBundleResponseList = array();
    private $isSuccessSet = false;

    function isSuccess() {
	return $this->isSuccess;
    }
    
    function addWorkUnitBundleResponse( TCA_WorkUnitBundleResponse $bundle) {
	array_push($this->workUnitBundleResponseList, $bundle);
	if (! $this->isSuccessSet) {
	    $this->isSuccess = $bundle->isSuccess();
	    $this->isSuccessSet = true;
	} else {
	    $this->isSuccess &= $bundle->isSuccess();
	}
    }

    function getWorkUnitBundleResponseList() { return $this->workUnitBundleResponseList; }

    function __toString() {
	$str = "WorkOrderResponse:\n";
	foreach( $this->workUnitBundleResponseList as $bundle) {
	    $str .= $bundle->__toString()."\n";
	}

	return $str;
    }
}

class TCA_WorkOrderManager {
    private $workOrder;
    private $connectors;

    function setConnectors($connectors) {
	$this->connectors = $connectors;
    }

    function getConnectors() {
	return $this->connectors;
    }

    function setLogFile($filename) {
	if($this->connectors) {
	    foreach($this->connectors as $connector) {
		$connector->setLogFileName($filename);
	    }
	}
    }

    function setWorkOrder( TCA_WorkOrder $workOrder) {
	$this->workOrder = $workOrder;
    }

    function process($provider) {
	if ( !$this->connectors[$provider]) 
	    throw new TCA_ObjectNotFullyInitializedException();

	// work order may translate to many connector requests:
	// iterate through bundles/units and process one by one
	// them all in the sequence
	$res = new TCA_WorkOrderResponse;
	foreach( $this->workOrder->getWorkUnitBundleList() as $bundle) {
	    $resBundle = new TCA_WorkUnitBundleResponse();
	    foreach( $bundle->getWorkUnitList() as $unit) {
		$req = $unit->toConnectorRequest();
		$connRes = $this->connectors[$provider]->sendCommand($req);
		//$connRes = $this->connectors[$providier]->sendCommand($req);
		$resUnit = new TCA_WorkUnitResponse($connRes);
		$resBundle->addWorkUnitResponse($resUnit);
	    }
	    $res->addWorkUnitBundleResponse($resBundle);
	}
	
	return $res;
    }
}

class TCA_WorkOrder {
    private $workUnitBundleList = array();

    function addWorkUnitBundle( TCA_WorkUnitBundle $bundle) {
	array_push($this->workUnitBundleList, $bundle);
    }

    function getWorkUnitBundleList() { return $this->workUnitBundleList; }

    function __toString() {
	$str = "WorkOrder:\n";
	foreach( $this->workUnitBundleList as $bundle) {
	    $str .= $bundle->__toString()."\n";
	}

	return $str;
    }
}

class TCA_WorkUnitBundle {
    private $workUnitList = array();

    function addWorkUnit( TCA_WorkUnit $workUnit) {
	array_push($this->workUnitList, $workUnit);
    }

    function getWorkUnitList() { return $this->workUnitList; }

    function __toString() {
	$str = "    WorkUnitBundle:\n";
	foreach( $this->workUnitList as $unit) {
	    $str .= "    ".$unit->__toString()."\n";
	}

	return $str;
    }
}

class TCA_WorkUnit {
    private $parameterSet;
    private $notes;

    function __construct() {
	$this->parameterSet = new TCA_WorkUnitParameterSet();
    }

    function addParameter( TCA_WorkUnitParameter $param ) {
	$this->parameterSet->addParameter($param);
    }

    function addParameterSet( $key, TCA_WorkUnitParameterSet $set ) {
	$this->parameterSet->addParameterSet($key, $set);
    }

    function getParameter($key) {
	return $this->parameterSet->getElement($key);
    }
    
    function setNotes($str) {
	$this->notes = $str;
    }

    function getNotes() { return $this->notes; }

    function toConnectorRequest() {
	// XXX send notes to server?
	$req = $this->parameterSet->toArray();
	$connReq = new TCA_ConnectorRequest();
	$connReq->setRequest($req);
	return $connReq;
    }

    function __toString() {
	$str = "    WorkUnit:\n";
	$str .= $this->parameterSet->__toString();
	return $str;
    }
}

?>
