<?php

require_once("apiTest.php");

class domainTest extends apiTest {

    public static $newDomain;
    public $fqdn1;
    public $fqdn2;
    public $fqdn3;
    
    public function setUp() {
	parent::setUp();
	self::$newDomain = "acf-test-".date("Ymd").".com";
	$this->fqdn1 = "ns1.".self::$newDomain;
	$this->fqdn2 = "ns2.".self::$newDomain;
	$this->fqdn3 = "ns3.".self::$newDomain;

	$this->setCookie();
    }

    function testLookupDomain() {
	echo "\n\n---- Test Lookup Domain ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.lookup_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $this->domain));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if ($result->getResponseCode() === "210") return TRUE; else return FALSE;
    }

    function testGetDomain() {
	echo "\n\n---- Test Get Domain ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.get_domain');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('type', 'owner'));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
    }

    private function getNameserverList() {
	$ns1 = new TCA_ServiceRequestParameterSet();
	$ns1->addParameter(new TCA_ServiceRequestParameter("sortorder", 1));
	$ns1->addParameter(new TCA_ServiceRequestParameter("name", "ns1.domaindirect.com"));
	
	$ns2 = new TCA_ServiceRequestParameterSet();
	$ns2->addParameter(new TCA_ServiceRequestParameter("sortorder", 2));
	$ns2->addParameter(new TCA_ServiceRequestParameter("name", "ns2.domaindirect.com"));
	
	$nsSet = new TCA_ServiceRequestParameterSet();
	$nsSet->addParameterSet('0', $ns1);
	$nsSet->addParameterSet('1', $ns2);

	return $nsSet;
    }

    private function getContactSetList() {
	$contact = new TCA_ServiceRequestParameterSet();
	$contact->addParameter(new TCA_ServiceRequestParameter('first_name', 'Jane'));
	$contact->addParameter(new TCA_ServiceRequestParameter('last_name', 'Doe'));
	$contact->addParameter(new TCA_ServiceRequestParameter('org_name', 'ACME administration'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address1', '99 West 1st St.'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address2', ''));
	$contact->addParameter(new TCA_ServiceRequestParameter('city', 'toronto'));
	$contact->addParameter(new TCA_ServiceRequestParameter('state', 'ON'));
	$contact->addParameter(new TCA_ServiceRequestParameter('country', 'CA'));
	$contact->addParameter(new TCA_ServiceRequestParameter('postal_code', 'A!A 9Z9'));
	$contact->addParameter(new TCA_ServiceRequestParameter('phone', '+1.4165350101'));
	$contact->addParameter(new TCA_ServiceRequestParameter('fax', ''));
	$contact->addParameter(new TCA_ServiceRequestParameter('email', 'jane_pheudo@doe.name'));
	
	$contacts = new TCA_ServiceRequestParameterSet();
	$contacts->addParameterSet('admin',$contact);
	$contacts->addParameterSet('tech',$contact);
	$contacts->addParameterSet('billing',$contact);
	$contacts->addParameterSet('owner',$contact);
	
	return $contacts;
    }
    
    function testRegisterDomain() {
	$this->registerDomain();
    }

    function registerDomain($args=array()) {
	echo "\n\n---- Test Register a New Domain ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.sw_register_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', isset($args{'domain'}) ? $args{'domain'} : self::$newDomain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_domain', $this->domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_username', $this->reg_username));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $this->reg_password));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_type', isset($args{'reg_type'}) ? $args{'reg_type'} : 'new'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('handle', isset($args{'handle'}) ? $args{'handle'} : 'process'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('period', '1'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('custom_tech_contact', '0'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('custom_nameservers', '1'));
	$pSet->addParameterSet('contact_set', $this->getContactSetList());
	$pSet->addParameterSet('nameserver_list', $this->getNameserverList());
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);

	return $result;
    }
    
	
    function testRenewDomain() {
	echo "\n\n---- Test Renew Domain ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.renew_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', self::$newDomain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('currentexpirationyear', date("Y")+1));
	$pSet->addParameter(new TCA_ServiceRequestParameter('auto_renew', 0));
	$pSet->addParameter(new TCA_ServiceRequestParameter('handle', 'process'));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
    }
    
    function testModifyDomain() {
	echo "\n\n---- Test Modify Domain ----\n\n";
	$this->updateCookie(self::$newDomain);
	
	$request = self::$client->createServiceRequest('TUCOWS_domains.modify_domain');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('data', "contact_info"));
	$pSet->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
	$contactSet = new TCA_ServiceRequestParameterSet();
	$contactSet->addParameter(new TCA_ServiceRequestParameter('also_apply_to', array("owner", "billing")));
	$contact = new TCA_ServiceRequestParameterSet();
	$contactSet->addParameterSet('admin', $contact);
	$contact->addParameter(new TCA_ServiceRequestParameter('first_name', 'Jane'));
	$contact->addParameter(new TCA_ServiceRequestParameter('last_name', 'Doe'));
	$contact->addParameter(new TCA_ServiceRequestParameter('org_name', 'ACME administration'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address1', '99 West 1st St.'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address2', ''));
	$contact->addParameter(new TCA_ServiceRequestParameter('city', 'toronto'));
	$contact->addParameter(new TCA_ServiceRequestParameter('state', 'ON'));
	$contact->addParameter(new TCA_ServiceRequestParameter('country', 'CA'));
	$contact->addParameter(new TCA_ServiceRequestParameter('postal_code', 'A!A 9Z9'));
	$contact->addParameter(new TCA_ServiceRequestParameter('phone', '+1.4165350101'));
	$contact->addParameter(new TCA_ServiceRequestParameter('fax', '+1.1234567890'));
	$contact->addParameter(new TCA_ServiceRequestParameter('email', 'jane_pheudo@doe.name'));
	$pSet->addParameterSet('contact_set', $contactSet);
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("modify domain failed!");
	}
    }
    
    function testGetDomainPrice() {
	echo "\n\n---- Test Get Domain Price ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.get_domain_price');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', self::$newDomain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('period', 1));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_type', "new"));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("get domain price failed -- ". $result->getResponseText());
	} else {
	    return $result->getResponseData()->getElement("attributes")->getElement('price')->getValue();
	}
    }
    
    function testSendAuthcode() {
	echo "\n\n---- Test Send Authcode ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.send_authcode');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain_name', $this->domain_new));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("send authcode failed!");
	}
    }
    
    function testSendPasswordDomain() {
	echo "\n\n---- Test Send Domain Password ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.send_password_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain_name', self::$newDomain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('send_to', "owner"));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_user', "0"));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("send password - domain failed!");
	}
    }

    function testBelongsToRsp() {
	echo "\n\n---- Test Belongs To RSP ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.belongs_to_rsp');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', self::$newDomain));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("belongs to rsp failed!");
	}
    }

/**
    function testCiraEmailPassword() {
	echo "\n\n---- Test Cira Email Password ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.cira_email_password');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', self::$newDomain));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("cira email password failed!");
	}
    }
*/

    function testNameserver() {
	$this->updateCookie(self::$newDomain);
	
	$this->createNameserver();
	$this->addNameserver();
	$this->checkNameserver();
	$this->modifyNameserver();
	$this->getNameserver();
	$this->deleteNameserver();
    }
    
    function createNameserver() {
	echo "\n\n---- Test Create Nameserver ----\n\n";
	for ($i=1; $i<3; $i++) {
	    $request = self::$client->createServiceRequest('TUCOWS_domains.create_nameserver');
	    $request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	    $pset = new TCA_ServiceRequestParameterSet();
	    $pset->addParameter(new TCA_ServiceRequestParameter('name', $this->{"fqdn$i"}));
	    $pset->addParameter(new TCA_ServiceRequestParameter('ipaddress', "216.40.33.2$i"));
	    $request->addParameterSet('attributes',$pset);
	    $result = $this->process($request);
	    if (!$result->isSuccess()) {
		throw new Exception("create nameserver failed!");
	    }
	}
    }

    function addNameserver() {
	echo "\n\n---- Test Add Nameserver ----\n\n";
	for ($i=1; $i<3; $i++) {
	    $request = self::$client->createServiceRequest('TUCOWS_domains.add_nameserver');
	    $pset = new TCA_ServiceRequestParameterSet();
	    $pset->addParameter(new TCA_ServiceRequestParameter('fqdn', $this->{"fqdn$i"}));
	    $pset->addParameter(new TCA_ServiceRequestParameter('all', '0'));
	    preg_match("/[^\.\/]+$/", self::$newDomain, $matches);
	    $tld = $matches[0];
	    $pset->addParameter(new TCA_ServiceRequestParameter('tld', ".$tld"));
	    $request->addParameterSet('attributes',$pset);
	    $result = $this->process($request);
	    if (!$result->isSuccess()) {
		throw new Exception("add nameserver failed!");
	    }
	}
    }

    function checkNameserver() {
	echo "\n\n---- Test Check Nameserver ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.check_nameserver');
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('fqdn', $this->fqdn1));
	preg_match("/[^\.\/]+$/", self::$newDomain, $matches);
	$tld = $matches[0];
	$pset->addParameter(new TCA_ServiceRequestParameter('tld', ".$tld"));
	$request->addParameterSet('attributes',$pset);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("check nameserver failed!");
	}
    }

    function getNameserver() {
	echo "\n\n---- Test Get Nameserver ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.get_nameserver');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('name', 'all'));
	$request->addParameterSet('attributes',$pset);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("get nameserver failed!");
	}
    }

    function modifyNameserver() {
	echo "\n\n---- Test Modify Nameserver ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.modify_nameserver');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('name', $this->fqdn2));
	$pset->addParameter(new TCA_ServiceRequestParameter('new_name', $this->fqdn3));
	$pset->addParameter(new TCA_ServiceRequestParameter('ipaddress', '216.40.33.23'));
	$request->addParameterSet('attributes',$pset);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("modify nameserver failed!");
	}
    }

    function deleteNameserver() {
	echo "\n\n---- Test Delete Nameserver ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.delete_nameserver');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('name', $this->fqdn1));
	$pset->addParameter(new TCA_ServiceRequestParameter('ipaddress', '216.40.33.21'));
	$request->addParameterSet('attributes',$pset);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("delete nameserver failed!");
	}
    }

    function testRefundDomain() {
	$this->refundDomain();
    }
    
    function refundDomain($domain=null) {
	echo "\n\n---- Test Refund Domain ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.refund_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain==null ? self::$newDomain : $domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('currentexpirationyear', date("Y")+1));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reseller', $this->reseller));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("refund domain failed!");
	}
    }

    function testTransfer() {
	$domain = "acf-test-".date("U").".com";
	$this->registerDomain((array("domain" => $domain)));
	echo "DOMAIN: $domain\n";
	try {
	    $response = $this->registerDomain(array("domain" => $domain, "reg_type" => 'transfer', "handle" => 'save'));
	    $orderId = $response->getResponseData()->getElement('attributes')->getElement('forced_pending')->getValue();
	    $this->checkTransfer($domain);
	    $this->processTransfer($orderId);
	    $this->cancelTransfer($orderId);
	} catch (Exception $e) {
	    $this->refundDomain($domain);
	    throw $e;
	}
    }
 
    function checkTransfer($domain = null) {
	echo "\n\n---- Test Check Transfer ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.check_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain == null ? self::$newDomain : $domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('get_request_address', 1));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("check transfer failed!");
	}
    }   
    
    function processTransfer($orderId) {
	echo "\n\n---- Test Process Transfer ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.process_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('reseller', $this->reseller));
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $orderId));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	$this->assertNotNull($result->getResponseText());
    }
    
/**        
    function SendPasswordTransfer() {
	echo "\n\n---- Test Send Password - Transfer ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.send_password_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain_name', self::$newDomain));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("send password - transfer failed!");
	}
    }
*/

    function cancelTransfer($orderId) {
	echo "\n\n---- Test Cancel Transfer ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.cancel_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('reseller', $this->reseller));
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $orderId));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	$this->assertNotNull($result->getResponseText());
    }
   
    function testOrderInfo() {
	$domain = "acf-test-".date("U").".com";
	$response = $this->registerDomain(array("domain" => $domain, "handle" => 'save'));
	$orderId = $response->getResponseData()->getElement('attributes')->getElement('id')->getValue();
	$response = $this->getOrderInfo($orderId);
	$this->processPendingOrder(array("orderId" => $orderId, "command" => 'cancel'));
    }
    
    function getOrderInfo($orderId) {
	echo "\n\n---- Test Get Order Info ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.get_order_info');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $orderId));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("get order info failed!");
	}
    }   
    
/** 
    function testCheckStatusOfQueuedReq() {
	echo "\n\n---- Test Check Status of Queued Request ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.check_status_of_queued_req');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('request_id', ''));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("check status of queued request failed!");
	}
    }    
*/  

    function processPendingOrder($args=array()) {
	echo "\n\n---- Test Process Pending Order ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.process_pending_order');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $args{'orderId'}));
// optional
	$pSet->addParameter(new TCA_ServiceRequestParameter('command', $args{'command'}));
//	$pSet->addParameter(new TCA_ServiceRequestParameter('fax_received', '1'));
//	$pSet->addParameter(new TCA_ServiceRequestParameter('owner_address', 'email@address.com'));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("process pending order failed!");
	}
    }
    
    function testUser() {
	$this->addSubuser(array(
	    'subUsername' => 'subUser1',
	    'subPermission' => 63,
	    'subPassword' => 'subuser')
	);
	$response = $this->getSubuser();
	$subuserId = $response->getResponseData()->getElement('attributes')->getElement('id')->getValue();
	
	// do not test modifySubsuer for now, as the modified username can not be reused
	// $this->modifySubuser($subuserId);

	$this->deleteSubuser($subuserId);
    }
    
    function addSubuser($args) {
	echo "\n\n---- Test Add Subuser ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.add_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_username', $args{subUsername}));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_permission', $args{subPermission}));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_password', $args{subPassword}));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("add subuser failed!");
	}
    }

    function getSubuser() {
	echo "\n\n---- Test Get Subuser ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.get_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	$result = $this->process($request,true);
	if (!$result->isSuccess()) {
	    throw new Exception("get subuser failed!");
	}
	return $result;
    }
        
    function modifySubuser($subuserId) {
	echo "\n\n---- Test Modify Subuser ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.modify_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_id', $subuserId));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_username', 'changeUserName'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_permission', '1'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_password', 'changePwd'));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("modify subuser failed!");
	}
    }
    
    function deleteSubuser($subuserId) {
	echo "\n\n---- Test Delete Subuser ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.delete_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_id', $subuserId));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("delete subuser failed!");
	}
    }
   
    function testUserInfoPassword() {
	$this->getUserInfo();
	$this->changePassword('new_password');
	$this->changePassword($this->reg_password);
    }
    
    function getUserInfo() {
	echo "\n\n---- Test Get User Info ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.get_user_info');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("get user info failed!");
	}

    }
    
/**        
    function testChangeOwnership() {
	echo "\n\n---- Test Change Ownership ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.change_ownership');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('username', 'qatwo'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('password', 'new_password'));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("change ownership failed!");
	}
    }
*/

    function changePassword($password) {
	echo "\n\n---- Test Change Password ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.change_password');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $password));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("change password failed!");
	}
    }
   
    function testAdvancedUpdateNameservers() {
	$args = array(
	    'op_type' => 'add_remove',
	    'remove_ns' => array(
		'ns5.domaindirect.com',
		'ns6.domaindirect.com',
	    ),
	    'add_ns' => array(
		'ns3.domaindirect.com',
		'ns4.domaindirect.com',
	    ),
	);
	$this->advancedUpdateNameservers($args);
	$args = array(
	    'op_type' => 'assign',
	    'assign_ns' => array(
		'ns1.domaindirect.com',
		'ns2.domaindirect.com',
	    ),
	);
	$this->advancedUpdateNameservers($args);
    }
    
    function advancedUpdateNameservers($args) {
	echo "\n\n---- Test Advanced Update Nameservers (".$args{'op_type'}.") ----\n\n";
	$request = self::$client->createServiceRequest('TUCOWS_domains.advanced_update_nameservers');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', self::$cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('domain', $args{'domain'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('op_type', $args{'op_type'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('remove_ns', $args{'remove_ns'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('add_ns', $args{'add_ns'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('assign_ns', $args{'assign_ns'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('add_ns_to_foreign_registries', 0));
	$request->addParameterSet('attributes',$pset);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("advanced update nameservers failed!");
	}
    }
    
    function tearDown() {
	$this->deleteCookie();
    }
}

$suite = new PHPUnit2_Framework_TestSuite(new ReflectionClass('domainTest'));
PHPUnit2_TextUI_TestRunner::run($suite);


?>
