<?php

define('PHPUnit2_MAIN_METHOD', '');
set_include_path(get_include_path().":..");

require_once "PHPUnit2/Framework/TestSuite.php";
require_once "PHPUnit2/Framework/TestCase.php";
require_once 'PHPUnit2/TextUI/TestRunner.php';

require_once("../util/validator.inc.php");

class validatorTest extends PHPUnit2_Framework_TestCase {
    private $validator;
    private $list;
    private $rule;
    
    public function setUp() {
	$this->validator = new TCA_Validator();
    }

    public function test1() {
	$rule = new TCA_MinValidationRule();
    }

    public function test2() {
	$list = new TCA_ValidationRuleList();
    }

    public function testNullValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_NullValidationRule();
	$rule->setRequired(0);
	$list->addRule("null", $rule);

	$res = $this->validator->validate($list);
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array());
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('null' => ''));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('null' => 3));
	if ( !$res->isValid()) throw new Exception();

	// test static methods
	$list = TCA_ValidationRuleList::createRuleList(array(
	    'null' => TCA_NullValidationRule::createRule(0))
	);
	$res = $this->validator->validate($list);
	if ( !$res->isValid()) throw new Exception();
    }

    public function testNullRequiredValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_NullValidationRule();
	$rule->setRequired(1);
	$list->addRule("null", $rule);

	$res = $this->validator->validate($list, array());
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('null' => ''));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('null' => 3));
	if ( !$res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'null' => TCA_NullValidationRule::createRule(1))
	);
	$res = $this->validator->validate($list, array('null' => ''));
	if ( !$res->isValid()) throw new Exception();
    }

    public function testMinValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_MinValidationRule();
	$rule->setRequired(1);
	$rule->setMinLimit(3);
	$list->addRule("min", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("min" => 2));
	if ( $res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("min" => 3));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("min" => 4));
	if ( !$res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'min' => TCA_MinValidationRule::createRule(1,3))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("min" => 2));
	if ( $res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("min" => 3));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("min" => 4));
	if ( !$res->isValid()) throw new Exception();
    }

    public function testMaxValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_MaxValidationRule();
	$rule->setRequired(1);
	$rule->setMaxLimit(3);
	$list->addRule("max", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("max" => 2));
	if ( !$res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("max" => 3));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("max" => 4));
	if ( $res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'max' => TCA_MaxValidationRule::createRule(1,3))
	);

	$res = $this->validator->validate($list, array("max" => 2));
	if ( !$res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("max" => 3));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("max" => 4));
	if ( $res->isValid()) throw new Exception();
    }

    public function testMinLengthValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_MinLengthValidationRule();
	$rule->setRequired(1);
	$rule->setMinLength(3);
	$list->addRule("minLength", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("minLength" => 'ab'));
	if ( $res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("minLength" => 'abc'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("minLength" => 'abcd'));
	if ( !$res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'minLength' => TCA_MinLengthValidationRule::createRule(1,3))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("minLength" => 'ab'));
	if ( $res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("minLength" => 'abc'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("minLength" => 'abcd'));
	if ( !$res->isValid()) throw new Exception();
    }

    public function testMaxLengthValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_MaxLengthValidationRule();
	$rule->setRequired(1);
	$rule->setMaxLength(3);
	$list->addRule("maxLength", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("maxLength" => 'ab'));
	if ( !$res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("maxLength" => 'abc'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("maxLength" => 'abcd'));
	if ( $res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'maxLength' => TCA_MaxLengthValidationRule::createRule(1,3))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("maxLength" => 'ab'));
	if ( !$res->isValid()) throw new Exception();
	
	$res = $this->validator->validate($list, array("maxLength" => 'abc'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array("maxLength" => 'abcd'));
	if ( $res->isValid()) throw new Exception();
    }

    public function testRegexValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_RegexValidationRule();
	$rule->setRequired(1);
	$rule->setRegex('/^abc/');
	$list->addRule("regex", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('regex' => 'abc'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('regex' => 'xabc'));
	if ( $res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'regex' => TCA_RegexValidationRule::createRule(1, '/^abc/'))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('regex' => 'abc'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('regex' => 'xabc'));
	if ( $res->isValid()) throw new Exception();
    }

    public function testIPAddressValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_IPAddressValidationRule();
	$rule->setRequired(1);
	$list->addRule("ip", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('ip' => '12.12.12.12'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('ip' => 'adfasdf'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('ip' => '12.12.12.333'));
	if ( $res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'ip' => TCA_IPAddressValidationRule::createRule(1))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('ip' => '12.12.12.12'));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('ip' => 'adfasdf'));
	if ( $res->isValid()) throw new Exception();

    }

    public function testHostnameValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_HostnameValidationRule();
	$rule->setRequired(1);
	$list->addRule("host", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => 'host'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => '.com'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => '!!2.com'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => 'host.com'));
	if ( !$res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'host' => TCA_HostnameValidationRule::createRule(1))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => 'host'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => '.com'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => '!!2.com'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('host' => 'host.com'));
	if ( !$res->isValid()) throw new Exception();
    }

    public function testEmailAddressValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_EmailAddressValidationRule();
	$rule->setRequired(1);
	$list->addRule("email", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('email' => 'email'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('email' => '2 3@2.com'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('email' => 'a@eml.com'));
	if ( !$res->isValid()) throw new Exception();

	$list = TCA_ValidationRuleList::createRuleList(array(
	    'email' => TCA_EmailAddressValidationRule::createRule(1))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('email' => 'email'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('email' => '2 3@2.com'));
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('email' => 'a@eml.com'));
	if ( !$res->isValid()) throw new Exception();
    }

    public function testCustomValidationRule() {
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_CustomValidationRule();
	$rule->setRequired(1);
	$rule->setFunction(
		'$a', 
		'if ( $a < 10 ) return 1; else return 0;');
	$list->addRule("custom", $rule);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('custom' => 5));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('custom' => 20));
	if ( $res->isValid()) throw new Exception();

	// test optional data stuff
	$list = new TCA_ValidationRuleList();
	$rule = new TCA_CustomValidationRule();
	$rule->setRequired(1);
	$rule->setFunction(
		'$a', 
		'$country = $optData; 
		 if ( $country == "US" && $a < 10 ) return 1; else return 0;');
	$list->addRule("custom", $rule);

	$res = $this->validator->validate($list, array('custom' => 5), 'US');
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('custom' => 5), 'CA');
	if ( $res->isValid()) throw new Exception();

	// static methods
	$list = TCA_ValidationRuleList::createRuleList(array(
	    'custom' => TCA_CustomValidationRule::createRule(
		1,
		'$a',
		'if ( $a < 10 ) return 1; else return 0;'))
	);

	$res = $this->validator->validate($list);
	if ( $res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('custom' => 5));
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('custom' => 20));
	if ( $res->isValid()) throw new Exception();

	// optional data with static methods
	$list = TCA_ValidationRuleList::createRuleList(array(
	    'custom' => TCA_CustomValidationRule::createRule(
		1,
		'$a',
		'$country = $optData; 
		 if ( $country == "US" && $a < 10 ) return 1; else return 0;'))
	);

	$res = $this->validator->validate($list, array('custom' => 5), 'US');
	if ( !$res->isValid()) throw new Exception();

	$res = $this->validator->validate($list, array('custom' => 5), 'CA');
	if ( $res->isValid()) throw new Exception();
    }
}

$suite = new PHPUnit2_Framework_TestSuite(new ReflectionClass('validatorTest'));
PHPUnit2_TextUI_TestRunner::run($suite);

?>
