<?php

require_once("domainUsage.php");

class domainManagement extends domainUsage {

    public $newDomain;
    public $fqdn1;
    public $fqdn2;
    public $fqdn3;
    
    function __construct() {
	//$this->domain = "";
	//$this->domain_new = "";
	//$this->reg_username = "";
	//$this->reg_password = "";
	$this->newDomain = "acf-test-".date("U").".com";
	$this->fqdn1 = "ns1.".$this->newDomain;
	$this->fqdn2 = "ns2.".$this->newDomain;
	$this->fqdn3 = "ns3.".$this->newDomain;
	
	parent::__construct();
	$this->setCookie();
    }

    function lookupDomain($domain=null) {
	echo "\n\n---- Lookup Domain ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.lookup_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain==null ? $this->domain : $domain));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if ($result->getResponseCode() === "210") return TRUE; else return FALSE;
    }

    function getDomain() {
	echo "\n\n---- Get Domain ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.get_domain');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('type', 'owner'));
#	$pSet->addParameter(new TCA_ServiceRequestParameter('page', '0'));
	$request->addParameterSet('attributes',$pSet);
	var_dump($request->toArray());
	$result = $this->process($request);
    }

    private function getNameserverList() {
	$ns1 = new TCA_ServiceRequestParameterSet();
	$ns1->addParameter(new TCA_ServiceRequestParameter("sortorder", 1));
	$ns1->addParameter(new TCA_ServiceRequestParameter("name", "ns1.domaindirect.com"));
	
	$ns2 = new TCA_ServiceRequestParameterSet();
	$ns2->addParameter(new TCA_ServiceRequestParameter("sortorder", 2));
	$ns2->addParameter(new TCA_ServiceRequestParameter("name", "ns2.domaindirect.com"));
	
	$nsSet = new TCA_ServiceRequestParameterSet();
	$nsSet->addParameterSet('0', $ns1);
	$nsSet->addParameterSet('1', $ns2);

	return $nsSet;
    }

    private function getContactSetList() {
	$contact = new TCA_ServiceRequestParameterSet();
	$contact->addParameter(new TCA_ServiceRequestParameter('first_name', 'Jane'));
	$contact->addParameter(new TCA_ServiceRequestParameter('last_name', 'Doe'));
	$contact->addParameter(new TCA_ServiceRequestParameter('org_name', 'ACME administration'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address1', '99 West 1st St.'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address2', ''));
	$contact->addParameter(new TCA_ServiceRequestParameter('city', 'toronto'));
	$contact->addParameter(new TCA_ServiceRequestParameter('state', 'ON'));
	$contact->addParameter(new TCA_ServiceRequestParameter('country', 'CA'));
	$contact->addParameter(new TCA_ServiceRequestParameter('postal_code', 'A!A 9Z9'));
	$contact->addParameter(new TCA_ServiceRequestParameter('phone', '+1.4165350101'));
	$contact->addParameter(new TCA_ServiceRequestParameter('fax', ''));
	$contact->addParameter(new TCA_ServiceRequestParameter('email', 'jane_pheudo@doe.name'));
	
	$contacts = new TCA_ServiceRequestParameterSet();
	$contacts->addParameterSet('admin',$contact);
	$contacts->addParameterSet('tech',$contact);
	$contacts->addParameterSet('billing',$contact);
	$contacts->addParameterSet('owner',$contact);
	
	return $contacts;
    }
    
    function registerDomain($args) {
	echo "\n\n---- Register a New Domain ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.sw_register_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', isset($args{'domain'}) ? $args{'domain'} : $this->newDomain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_domain', $this->domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_username', $this->reg_username));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $this->reg_password));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_type', isset($args{'reg_type'}) ? $args{'reg_type'} : "new" ));
	$pSet->addParameter(new TCA_ServiceRequestParameter('handle', isset($args{'handle'}) ? $args{'handle'} : "process" ));
	$pSet->addParameter(new TCA_ServiceRequestParameter('period', '1'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('custom_tech_contact', '0'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('custom_nameservers', '1'));
	$pSet->addParameterSet('contact_set', $this->getContactSetList());
	$pSet->addParameterSet('nameserver_list', $this->getNameserverList());
	$request->addParameterSet('attributes',$pSet);
	return $this->process($request);
    }
    
	
    function renewDomain($domain=null) {
	echo "\n\n---- Renew Domain ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.renew_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain==null? $this->newDomain : $domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('currentexpirationyear', date("Y")+1));
	$pSet->addParameter(new TCA_ServiceRequestParameter('auto_renew', 0));
	$pSet->addParameter(new TCA_ServiceRequestParameter('handle', 'process'));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
    }
    
    function modifyDomain() {
	echo "\n\n---- Modify Domain ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.modify_domain');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('data', "contact_info"));
	$pSet->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
	$contactSet = new TCA_ServiceRequestParameterSet();
	$contactSet->addParameter(new TCA_ServiceRequestParameter('also_apply_to', array("owner", "billing")));
	$contact = new TCA_ServiceRequestParameterSet();
	$contactSet->addParameterSet('admin', $contact);
	$contact->addParameter(new TCA_ServiceRequestParameter('first_name', 'Jane'));
	$contact->addParameter(new TCA_ServiceRequestParameter('last_name', 'Doe'));
	$contact->addParameter(new TCA_ServiceRequestParameter('org_name', 'ACME administration'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address1', '99 West 1st St.'));
	$contact->addParameter(new TCA_ServiceRequestParameter('address2', ''));
	$contact->addParameter(new TCA_ServiceRequestParameter('city', 'toronto'));
	$contact->addParameter(new TCA_ServiceRequestParameter('state', 'ON'));
	$contact->addParameter(new TCA_ServiceRequestParameter('country', 'CA'));
	$contact->addParameter(new TCA_ServiceRequestParameter('postal_code', 'A!A 9Z9'));
	$contact->addParameter(new TCA_ServiceRequestParameter('phone', '+1.4165350101'));
	$contact->addParameter(new TCA_ServiceRequestParameter('fax', '+1.1234567890'));
	$contact->addParameter(new TCA_ServiceRequestParameter('email', 'jane_pheudo@doe.name'));
	$pSet->addParameterSet('contact_set', $contactSet);
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
    }
    
    function refundDomain($domain=null) {
	echo "\n\n---- Refund Domain ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.refund_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain==null? $this->newDomain : $domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('currentexpirationyear', date("Y")+1));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reseller', $this->reseller));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
    }
    
    function getDomainPrice($domain=null) {
	echo "\n\n---- Get Domain Price ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.get_domain_price');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain==null? $this->newDomain : $domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('period', 1));
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_type', "new"));
	$request->addParameterSet('attributes',$pSet);
	$result = $this->process($request);
	if (!$result->isSuccess()) {
	    throw new Exception("get domain price failed -- ". $result->getResponseText());
	} else {
	    return $result->getResponseData()->getElement("attributes")->getElement('price')->getValue();
	}
    }
    
    function sendAuthcode($domain=null) {
	echo "\n\n---- Send Authcode ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.send_authcode');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain_name', $domain==null? $this->domain_new : $domain));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }
    
    function sendPasswordDomain($domain=null) {
	echo "\n\n---- Send Domain Password ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.send_password_domain');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain_name', $domain==null? $this->newDomain : $domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('send_to', "owner"));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_user', "0"));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }

    function belongsToRsp($domain=null) {
	echo "\n\n---- Belongs To RSP ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.belongs_to_rsp');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain==null? $this->newDomain : $domain));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }

    function ciraEmailPassword($domain=null) {
	echo "\n\n---- Cira Email Password ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.cira_email_password');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain==null? $this->newDomain : $domain));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }

    function createNameserver() {
	echo "\n\n---- Create Nameserver ----\n\n";
	for ($i=1; $i<3; $i++) {
	    $request = $this->client->createServiceRequest('TUCOWS_domains.create_nameserver');
	    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	    $pset = new TCA_ServiceRequestParameterSet();
	    $pset->addParameter(new TCA_ServiceRequestParameter('name', $this->{"fqdn$i"}));
	    $pset->addParameter(new TCA_ServiceRequestParameter('ipaddress', "216.40.33.2$i"));
	    $request->addParameterSet('attributes',$pset);
	    $this->process($request);
	}
    }

    function addNameserver() {
	echo "\n\n---- Add Nameserver ----\n\n";
	for ($i=1; $i<3; $i++) {
	    $request = $this->client->createServiceRequest('TUCOWS_domains.add_nameserver');
	    $pset = new TCA_ServiceRequestParameterSet();
	    $pset->addParameter(new TCA_ServiceRequestParameter('fqdn', $this->{"fqdn$i"}));
	    $pset->addParameter(new TCA_ServiceRequestParameter('all', '0'));
	    preg_match("/[^\.\/]+$/", $this->newDomain, $matches);
	    $tld = $matches[0];
	    $pset->addParameter(new TCA_ServiceRequestParameter('tld', ".$tld"));
	    $request->addParameterSet('attributes',$pset);
	    $this->process($request);
	}
    }

    function checkNameserver() {
	echo "\n\n---- Check Nameserver ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.check_nameserver');
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('fqdn', $this->fqdn1));
	preg_match("/[^\.\/]+$/", $this->newDomain, $matches);
	$tld = $matches[0];
	$pset->addParameter(new TCA_ServiceRequestParameter('tld', ".$tld"));
	$request->addParameterSet('attributes',$pset);
	$this->process($request);
    }

    function getNameserver() {
	echo "\n\n---- Get Nameserver ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.get_nameserver');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('name', 'all'));
	$request->addParameterSet('attributes',$pset);
	$this->process($request);
    }

    function modifyNameserver() {
	echo "\n\n---- Modify Nameserver ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.modify_nameserver');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('name', $this->fqdn2));
	$pset->addParameter(new TCA_ServiceRequestParameter('new_name', $this->fqdn3));
	$pset->addParameter(new TCA_ServiceRequestParameter('ipaddress', '216.40.33.21'));
	$request->addParameterSet('attributes',$pset);
	$this->process($request);
    }

    function deleteNameserver() {
	echo "\n\n---- Delete Nameserver ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.delete_nameserver');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('name', $this->fqdn3));
	$pset->addParameter(new TCA_ServiceRequestParameter('ipaddress', '216.40.33.21'));
	$request->addParameterSet('attributes',$pset);
	$this->process($request);
    }

    function checkTransfer($domain = null) {
	echo "\n\n---- Check Transfer ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.check_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain == null ? $this->newDomain : $domain));
	$pSet->addParameter(new TCA_ServiceRequestParameter('get_request_address', 1));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }   
    
    function processTransfer($orderId) {
	echo "\n\n---- Process Transfer ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.process_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('reseller', $this->reseller));
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $orderId));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }
        
    function sendPasswordTransfer($domain = null) {
	echo "\n\n---- Send Password - Transfer ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.send_password_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('domain_name', $domain == null ? $this->newDomain : $domain));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }

    function cancelTransfer($orderId) {
	echo "\n\n---- Cancel Transfer ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.cancel_transfer');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('reseller', $this->reseller));
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $orderId));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }
    
    function getOrderInfo($orderId) {
	echo "\n\n---- Get Order Info ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.get_order_info');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $orderId));
	$request->addParameterSet('attributes',$pSet);
	return $this->process($request);
    }   
    
    function checkStatusOfQueuedReq($requestId) {
	echo "\n\n---- Check Status of Queued Request ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.check_status_of_queued_req');
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('request_id', $requestId));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }    
    
    function processPendingOrder($args) {
	echo "\n\n---- Process Pending Order ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.process_pending_order');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('order_id', $args{'orderId'}));
// optional
	$pSet->addParameter(new TCA_ServiceRequestParameter('command', $args{'command'}));
//	$pSet->addParameter(new TCA_ServiceRequestParameter('fax_received', '1'));
//	$pSet->addParameter(new TCA_ServiceRequestParameter('owner_address', 'email@address.com'));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }
        
    function addSubuser($args) {
	echo "\n\n---- Add Subuser ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.add_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_username', $args{subUsername}));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_permission', $args{subPermission}));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_password', $args{subPassword}));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }

    function getSubuser() {
	echo "\n\n---- Get Subuser ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.get_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	return $this->process($request);
    }
        
    function modifySubuser($subuserId) {
	echo "\n\n---- Modify Subuser ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.modify_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_id', $subuserId));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_username', 'changeName'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_permission', '1'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_password', 'changePwd'));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }
    
    function deleteSubuser($subuserId) {
	echo "\n\n---- Delete Subuser ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.delete_subuser');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$request->addParameter(new TCA_ServiceRequestParameter('username', $this->reg_username));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('sub_id', $subuserId));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }
    
    function getUserInfo() {
	echo "\n\n---- Get User Info ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.get_user_info');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$this->process($request);
    }
        
    function changeOwnership() {
	echo "\n\n---- Change Ownership ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.change_ownership');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('username', 'qatwo'));
	$pSet->addParameter(new TCA_ServiceRequestParameter('password', 'new_password'));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }
        
    function changePassword($password) {
	echo "\n\n---- Change Password ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.change_password');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pSet = new TCA_ServiceRequestParameterSet();
	$pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $password));
	$request->addParameterSet('attributes',$pSet);
	$this->process($request);
    }

    function advancedUpdateNameservers($args) {
	echo "\n\n---- Advanced Update Nameservers (".$args{'op_type'}.") ----\n\n";
	$request = $this->client->createServiceRequest('TUCOWS_domains.advanced_update_nameservers');
	$request->addParameter(new TCA_ServiceRequestParameter('cookie', $this->cookie));
	$pset = new TCA_ServiceRequestParameterSet();
	$pset->addParameter(new TCA_ServiceRequestParameter('domain', $args{'domain'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('op_type', $args{'op_type'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('remove_ns', $args{'remove_ns'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('add_ns', $args{'add_ns'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('assign_ns', $args{'assign_ns'}));
	$pset->addParameter(new TCA_ServiceRequestParameter('add_ns_to_foreign_registries', 0));
	$request->addParameterSet('attributes',$pset);
	$this->process($request);
    }
    
    function __destruct() {
	$this->deletecookie();
    }
}

$usage = new domainManagement();
$usage->setShow(false);

$price = $usage->getDomainPrice($usage->newDomain);
print "Domain $usage->newDomain price is $price\n";

$available = $usage->lookupDomain($usage->newDomain);
if ($available === true) {
    echo "Domain ".$usage->newDomain." is available\n";
    $usage->registerDomain(array("domain" => $usage->newDomain));
} else {
    echo "Domain ".$usage->newDomain." is taken\n";
}

$usage->updateCookie($usage->newDomain);
$usage->getDomain($usage->newDomain);
$usage->modifyDomain();
$usage->renewDomain($usage->newDomain);
$usage->sendPasswordDomain($usage->newDomain);
$usage->belongsToRsp($usage->newDomain);

$usage->createNameserver();
$usage->addNameserver();
$usage->checkNameserver();
$usage->modifyNameserver();
$usage->getNameserver();
$usage->deleteNameserver();

$usage->refundDomain($usage->newDomain);

$available = $usage->lookupDomain($usage->newDomain);
if ($available === TRUE) {
    echo "Domain ".$usage->newDomain." is available now\n";
} else {
    echo "Domain ".$usage->newDomain." is taken now\n";
}

$usage->sendAuthcode();

// -----------------------------------------------------------------------------------------
// register domain and try to transfer it.
// check transfer, try to process transfer (should fail), cancel transfer
// for this to work order must be saved - not processed imediately
echo "\nTRANSFER\n";
echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
$usage->setShow(false);
$domain = "acf-test-".date("U").".com";
$response = $usage->registerDomain(array("domain" => $domain));
echo "DOMAIN: $domain\n";
$response = $usage->registerDomain(array("domain" => $domain, "reg_type" => 'transfer', "handle" => 'save'));
$orderId = $response->getResponseData()->getElement('attributes')->getElement('forced_pending')->getValue();
$usage->checkTransfer($domain);
$usage->processTransfer($orderId);
$usage->cancelTransfer($orderId);
// end transfer

// -----------------------------------------------------------------------------------------
// get order info and process pending order
// for this to work order must be saved - not processed imediately
echo "\nORDER\n";
echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
$usage->setShow(false);
$response = $usage->registerDomain(array("domain" => "acf-test-".date("U").".com", "handle" => 'save'));
$orderId = $response->getResponseData()->getElement('attributes')->getElement('id')->getValue();
$response = $usage->getOrderInfo($orderId);
echo "ORDER STATUS BEFORE processPendingOrder: ".
$response->getResponseData()->getElement('attributes')->getElement('field_hash')->getElement('status')->getValue()."\n";
$usage->processPendingOrder(array("orderId" => $orderId, "command" => 'cancel'));
$response = $usage->getOrderInfo($orderId);
echo "ORDER STATUS AFTER processPendingOrder: ".
$response->getResponseData()->getElement('attributes')->getElement('field_hash')->getElement('status')->getValue()."\n";
// end order info

// -----------------------------------------------------------------------------------------
// queued order
// for this to work, disable .info RA (InfoProxy) and enable queueing in RWI
echo "\nQUEUED ORDER\n";
echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
$usage->setShow(false);
$response = $usage->registerDomain( array("domain" => "acf-test-".date("U").".info") );
$requestId = $response->getResponseData()->getElement('attributes')->getElement('queue_request_id')->getValue();
$usage->checkStatusOfQueuedReq($requestId);
//  end queued order

// -----------------------------------------------------------------------------------------
// sub user creation, modification, info, and deletion
echo "\nSUB USER\n";
echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
$usage->setShow(false);
$usage->addSubuser(array( 
    'subUsername' => 'subUser1',
    'subPermission' => 63,
    'subPassword' => 'subuser')
);
$response = $usage->getSubuser();
$subuserId = $response->getResponseData()->getElement('attributes')->getElement('id')->getValue();
$usage->modifySubuser($subuserId);
$response = $usage->getSubuser();
$usage->deleteSubuser($subuserId);
// end sub user

// -----------------------------------------------------------------------------------------
// user info and password change
echo "\nUSER INFO AND PASSWORD\n";
echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
$usage->setShow(false);
$usage->getUserInfo();
$usage->changePassword('new_password');
$fp = fopen("php://stdin","r");
echo "\n\nNow you have to go and check the password in RWI - View Domain ...\n";
echo "Press enter to continue ...\n\n";
fgetc($fp);
$usage->changePassword($usage->_getRegPassword());
echo "\nNow go and check the password again ...\n";
echo "Press enter to continue ...\n\n";
fgetc($fp);
fclose($fp);
// end password change


// -----------------------------------------------------------------------------------------
// advanced update nameservers
echo "\nADVANCED UPDATE NAMESERVERS\n";
echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
$usage->setShow(true);
$args = array(
    'op_type' => 'add_remove',
    'remove_ns' => array(
	'ns5.domaindirect.com',
	'ns6.domaindirect.com',
    ),
    'add_ns' => array(
	'ns3.domaindirect.com',
	'ns4.domaindirect.com',
    ),
);
$usage->advancedUpdateNameservers($args);
$args = array(
    'op_type' => 'assign',
    'assign_ns' => array(
	'ns1.domaindirect.com',
	'ns2.domaindirect.com',
    ),
);
$usage->advancedUpdateNameservers($args);
//  end advanced update nameservers

?>
