<?php

require_once("../client.inc.php");
                                                                                
// instantiate the client object, pass config object
$client = new TCA_Client(new TCA_XMLConfigReader("tucows-client.xml"));

// test few loose rules
$set = new TCA_ValidationRuleList();
$rule = new TCA_MinValidationRule();
$rule->setMinLimit(3);

$rule = new TCA_MaxValidationRule();
$rule->setMaxLimit(8);
echo $rule->validate(8.1)->__toString();

$rule = new TCA_EnumValidationRule();
$rule->setValues(array("one", "two"));
$rule->addValue("five");

#$rule = new TCA_IPAddressValidationRule();
#$rule = new TCA_DomainSyntaxValidationRule();
#$rule = new TCA_HostnameValidationRule();
#$rule = new TCA_EmailAddressValidationRule();
#$rule->setRequired(1);
#$ruleList->addRule("domainName", $rule);

// first compose a test request
//$request = new TCA_ServiceRequest('lookup_domain');
$request = new TCA_ServiceRequest('register_domain');
#$request->addParameter(new TCA_ServiceRequestParameter('domainName', 'example.com'));
#$request->addParameter(new TCA_ServiceRequestParameter('domainName', '34.34.34.34'));
#$request->addParameter(new TCA_ServiceRequestParameter('domainName', 'asda.com'));
$request->addParameter(new TCA_ServiceRequestParameter('regPeriod', 3));
$request->addParameter(new TCA_ServiceRequestParameter('domainName', 'asda.com'));
$set = new TCA_ServiceRequestParameterSet();
$set->addParameter(new TCA_Parameter("lastName", "Doe"));
$set->addParameter(new TCA_Parameter("country", "US"));
$set->addParameter(new TCA_Parameter("zipCode", "11122"));
$set->addParameter(new TCA_Parameter("phone", "+1.123-456-7890 x 345"));
$set->addParameter(new TCA_Parameter("contact_types", array("owner","tech")));
$set->addParameter(new TCA_Parameter("wrong_contact_types", array("owner","tech", "wrong")));

$arrSet1 = new TCA_ServiceRequestParameterSet();
$arrSet1->addParameter(new TCA_ServiceRequestParameter("sortorder", 1));
$arrSet1->addParameter(new TCA_ServiceRequestParameter("name", "ns1.tucows.com"));
$arrSet2 = new TCA_ServiceRequestParameterSet();
$arrSet2->addParameter(new TCA_ServiceRequestParameter("sortorder", 2));
$arrSet2->addParameter(new TCA_ServiceRequestParameter("name", "ns2.tucows.com"));
$nsSet = new TCA_ServiceRequestParameterSet();
$nsSet->addParameterSet('0', $arrSet1);
$nsSet->addParameterSet('1', $arrSet2);
$set->addParameterSet("nameserver_list", $nsSet);
    
$request->addParameterSet("attributes", $set);
//echo "Request: ".$request->__toString()."\n";
//echo "Request (array): "; var_dump($request->toArray());

$validator = new TCA_Validator();

// now compose a set of rules for an API call
$ruleList = new TCA_ValidationRuleList();

// action value
$rule = new TCA_EnumValidationRule();
$rule->setValues(array(2,3,4,5,6,7,8,9,10));
$ruleList->addRule("regPeriod", $rule);

// domainName parameter
$rule = new TCA_DomainSyntaxValidationRule();
$ruleList->addRule("domainName", $rule);

// attributes hash
$list = new TCA_ValidationRuleList();
$rule = new TCA_EnumValidationRule();
$rule->setValues(array("Dre"));
$list->addRule("lastName", $rule);
$ruleList->addRuleList("attributes", $list);

// call validate
$res = $validator->validate($ruleList, $request->toArray());

echo "res: [$res]\n";
echo "res dump: ".$res->__toString()."\n";


// so that was way too verbose, try the same using factory methods
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "regPeriod" => TCA_EnumValidationRule::createRule(1, array(2,3,4,8,9,10)),
    "domainName" => TCA_DomainSyntaxValidationRule::createRule(1),
    "attributes" => TCA_ValidationRuleList::createRuleList(array(
	"lastName" => TCA_EnumValidationRule::createRule(1, array("rre")),
	"country" => TCA_EnumValidationRule::createRule(1, array("CA")),
    )),
));
$res = $validator->validate($ruleList, $request->toArray());
echo "res dump: ".$res->__toString()."\n";

// Try a more complex example, with multiple rules for one field
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "regPeriod" => TCA_ValidationRuleList::createRuleList(array(
	TCA_MinValidationRule::createRule(1, 4),
	TCA_MaxValidationRule::createRule(1, 10),
    )),
));
$res = $validator->validate($ruleList, $request->toArray());
echo "res dump: ".$res->__toString()."\n";

//  try a custom validating class
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "regPeriod" => TCA_CustomValidationRule::createRule(
	1,
	'$regPeriod',
	'if ( $regPeriod < 4 or $regPeriod > 10) return 0; else return 1;')
));
$res = $validator->validate($ruleList, $request->toArray());
echo "res dump: ".$res->__toString()."\n";

//  try a custom validating class on a nested value
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "attributes" => TCA_ValidationRuleList::createRuleList(array(
	"country" => TCA_CustomValidationRule::createRule(
	    1,
	    '$country',
	    'if ( $country != "CA") return 0; else return 1;')
    )),
));
$res = $validator->validate($ruleList, $request->toArray());
echo "res dump: ".$res->__toString()."\n";

# how to pass additional data to the validation rule
echo "now trying custom rule with opt data\n";
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "attributes" => TCA_ValidationRuleList::createRuleList(array(
	"zipCode" => TCA_CustomValidationRule::createRule(
	    1,
	    '$zipCode',
	    '$country = $optData; if ($country == "US" && strlen($zipCode) != 5) return 0; else return 1; ')
    )),
));
$res = $validator->validate($ruleList, $request->toArray(), 'US');
echo "res dump: ".$res->__toString()."\n";

# how to validate if an array value is a subset of another array
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "attributes" => TCA_ValidationRuleList::createRuleList(array(
	"contact_types" => TCA_SubsetValidationRule::createRule(
	            1, array("owner","tech","admin","billing")),
	"wrong_contact_types" => TCA_SubsetValidationRule::createRule(
	            1, array("owner","tech","admin","billing")),
    )),
));
$res = $validator->validate($ruleList, $request->toArray());
echo "res dump: ".$res->__toString()."\n";

# validate a phone number
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "attributes" => TCA_ValidationRuleList::createRuleList(array(
	"phone" => TCA_PhoneValidationRule::createRule(1),
    )),
));
$res = $validator->validate($ruleList, $request->toArray());
echo "res dump: ".$res->__toString()."\n";

# how to validate a real array size
$ruleList = TCA_ValidationRuleList::createRuleList(array(
    "attributes" => TCA_ValidationRuleList::createRuleList(array(
	"nameserver_list" => TCA_MinArrayLengthValidationRule::createRule(1, 3),
    )),
));
$res = $validator->validate($ruleList, $request->toArray());
echo "res dump: ".$res->__toString()."\n";

?>
