<?php 

interface TCA_ConfigReader {

    function parseConfig();
}

class TCA_XMLConfigReader implements TCA_ConfigReader {
    private $_path;
    private $_xml;
    private $_refDefSuffix = "-list";
    private $_refSuffix = "-ref";
    private $_refDefTagName = "name";
    private $_refInternal = "-tag-";

    function __construct($path) {
        if (isset($_ENV['TCA_ETC']) && !TCA_Path::isAbsolutePath($path)) 
            $this->_path = $_ENV['TCA_ETC'].'/'.$path;
        else
            $this->_path = "$path";
        $this->_xml = simplexml_load_file($this->_path);
    }

    function setReferenceDefinitionSuffix($suffix) {
        $this->_refDefSuffix = $suffix;
    }

    function setReferenceSuffix($suffix) {
        $this->_refSuffix = $suffix;
    }

    function parseConfig() {
        return $this->parseXMLConfig($this->_xml, array());
    }

    private function parseXMLConfig($xml, $ref) {
        $config = new TCA_Config();
        $empty = 1;
        foreach ($xml->children() as $key => $value) {
            $empty = 0;
            $child = $this->parseXMLConfig($value, $ref);
            $obj = $child;
            if (is_null($obj)) {
                $tagValue = (string) $value;
                if (preg_match('/^(.+)'.$this->_refSuffix.'$/', $key, $matches)) {
                    $tag = $matches[1];
                    if (!isset($ref["$tag$this->_refInternal$tagValue"])) {
                        throw new TCA_XMLParseException(
                                "Parsing Configuration error:".
                                "Definition of reference $key:$tagValue not found!"
                        );
                    } else {
                        $obj = $ref["$tag$this->_refInternal$tagValue"];
                    }
                } else {
                    $obj = new TCA_Config();
                    $obj->setKey($key);
                    $obj->setValue((string) $value);
                }
            } else {
                $obj->setKey($key);
                if (preg_match('/^(.+)'.$this->_refDefSuffix.'$/', $key, $matches)) {
                    $tag = $matches[1];
                    $refDefs = $obj->$tag;
                    foreach ($refDefs as $def) {
                        $ref["$tag$this->_refInternal".(string)$def->{$this->_refDefTagName}->value] = $def;
                    }
                }
            }
            foreach ($value->attributes() as $aKey => $aValue) {
                $obj->setAttribute($aKey, (string)$aValue);
            }

            $newKey = $obj->getKey();
            if (! $config->$newKey) {
                $config->$newKey = $obj;
            } else {
                $arrConf = is_array($config->$newKey) ? $config->$newKey : array ($config->$newKey);
                array_push($arrConf, $obj);
                $config->$newKey = $arrConf;
            }
        }
        return $empty ? NULL : $config;
    }
}

class TCA_Config {
    private $configs = array();
    private $elementKey;
    private $elementValue;
    private $attrs = array();

    function __construct() {
    }

    function setKey($key) {
        $this->elementKey = $key;
    }

    function getKey() {
        return $this->elementKey;
    }

    function setValue($value) {
        $this->elementValue = $value;
    }

    function value() {
        return $this->elementValue;
    }

    function setAttribute($attrKey, $attrValue) {
        $this->attrs[$attrKey] = $attrValue;
    }

    function attr($attrKey) {
        return $this->attrs[$attrKey];
    }

    function __set($name, $value) {
        $this->configs[$name] = $value;
    }

    function __get($name) {
        if ($name == 'value') {
            return $this->elementValue;
        } else {
            return $this->configs[$name];
        }
    }
}

?>
