<?php

require_once("string.inc.php");

interface TCA_ParameterI {

    function getName();

    function getValue();

    function setValue($value);
}

interface TCA_ParameterSetI {

    function addParameter(TCA_ParameterI $par);

    function addParameterSet($key, TCA_ParameterSetI $set);

    function getElement($key);

    function deleteElement($key);

    function keys();

    function values();

    function toArray();

    function fromArray($ar);
}

// generic parameter type. Inherit from it to get a specific parameter type
class TCA_Parameter implements TCA_ParameterI {
    private $name;
    private $value;

    function __construct($name, $value) {
        $this->name = trim($name);
        if (is_string($value))
            $this->value = trim($value);
        else
            $this->value = $value;
    }

    function getName() {
        return $this->name;
    }

    function getValue() {
        return $this->value;
    }

    function setValue($value) {
        $this->value = $value;
    }

    function __toString() {
        return TCA_String::toString($this);
    }
}

// parameter type holder
class TCA_ParameterSet implements TCA_ParameterSetI {
    private $elements = array();

    // elements of parameter set can be either simple TCA_Parameter
    // or another TCA_ParameterSet
    function addParameter(TCA_ParameterI $par) {
        $this->elements[$par->getName()] = $par;
    }

    function addParameterSet($key, TCA_ParameterSetI $set) {
        $this->elements[$key] = $set;
    }

    function getElement($key) {
        return $this->elements[$key];
    }

    function deleteElement($key) {
        unset($this->elements[$key]);
    }

    function keys() {
        return array_keys($this->elements);
    }

    function values() {
        return array_values($this->elements);
    }

    function toArray() {
        $a = array();
        foreach ($this->elements as $key => $par) {
            if ($par instanceof TCA_ParameterSetI) {
                $a[$key] = $par->toArray();
            } elseif ($par instanceof TCA_ParameterI) {
                $a[$key] = $par->getValue();
            } else {
                throw new Exception("can't go here!!!");
            }
        }
        return $a;
    }

    function fromArray($ar) {
        $set = new TCA_ParameterSet();
        foreach ($ar as $key => $value) {
            if (is_array($value)) {
                $set->addParameterSet($key, $this->fromArray($value));
            } else {
                $set->addParameter(new TCA_Parameter($key, $value));
            }
        }
        return $set;
    }

    function __toString() {
        return TCA_String::toString($this);
    }
}

?>
