<?php

require_once('../../libs/ccs/options.inc.php');

function AH_LoadGlobalOptions(ACF_ParameterList $inParameterList)
{
	// This function parses the options file for global options.
	// Prep Session
    if(!$_SESSION['ccs'])
    	$_SESSION['ccs'] = array();
	$retEvent; // Return Event

	if(loadCCSOptions()) { // Loads options into an array (Located in options.inc.php)
		// Check for services
		if(count($_SESSION['ccs']['options']['driver-options']['service-modules']) < 1) {
			$_SESSION['ccs']['lastSystemError'] = "No services were loaded.";
			return new ACF_Event('loadglobaloptions_failure');
		}
	
		if(!$_SESSION['ccs']['options']['driver-options']['url']) { // url option does not exist.
			$_SESSION['ccs']['lastSystemError'] = "Failed to load global CCS option: URL";
			return new ACF_Event('loadglobaloptions_failure');
		}

		return new ACF_Event('loadglobaloptions_success');
	} else { 
		$_SESSION['ccs']['lastSystemError'] = "Failed to load global CCS options file";
		return new ACF_Event('loadglobaloptions_failure');
	}
}
			
function AH_CheckForBundles(ACF_ParameterList $inParameterList)
{
	if(!$_SESSION['ccs'])
		$_SESSION['ccs'] = array();

	if(!$_SESSION['ccs']['optionsLoaded'])
		loadCCSOptions();

	if($_SESSION['ccs']['bundled'] == true)
		return new ACF_Event('checkforbundles_present');
	else
		return new ACF_Event('checkforbundles_absent');
}

function AH_SelectBundle(ACF_ParameterList $inParameterList)
{
	$_SESSION['ccs']['actionsPreformed']['selectbundle'] = true;
	// This Function Sets up a hop between ACF Projects (To the billing module)

	$retEvent; // Return Event
	$retParams; // Return Parameters
	
	$_SESSION['ccs']['source'] = 'default';

	if($_SESSION['ccs']['options']['driver-options']['billing-module']) {
		$retEvent = new ACF_Event('selectbundle_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/selectbundle'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find billing module URL";
		$retEvent = new ACF_Event('selectbundle_failed');
	}

	return $retEvent;
}

function AH_SelectUser(ACF_ParameterList $inParameterList)
{
	$_SESSION['ccs']['actionsPreformed']['selectuser'] = true;
	// This Function Sets up a hop between ACF Projects (To the billing module)

	$retEvent; // Return Event
	$retParams; // Return Parameters

	if($_SESSION['ccs']['options']['driver-options']['billing-module']) {
		$retEvent = new ACF_Event('selectuser_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/selectuser'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find billing module URL";
		$retEvent = new ACF_Event('selectuser_failed');
	}

	return $retEvent;	
}

function AH_CheckServiceAvailability(ACF_ParameterList $inParameterList) 
{
	if($_SESSION['ccs']['excludeService'])
		return new ACF_Event('checkserviceavailability_exclude');

	$_SESSION['ccs']['actionsPreformed']['checkavailibility'] = true;

	$retEvent; // Return Event
	$retParams; // Return Parameters
	$ServiceType = $_SESSION['modules']['requiredService'];

	if($_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['active'] == 'true') {
		$retEvent = new ACF_Event('checkserviceavailability_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/checkavailability'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find service module URL for ".$ServiceType;
		$retEvent = new ACF_Event('checkserviceavailability_failed');
	}

	return $retEvent;	
}

function AH_GatherServiceInformation(ACF_ParameterList $inParameterList)
{
	if($_SESSION['ccs']['excludeService'])
		return new ACF_Event('gatherserviceinformation_exclude');

	$_SESSION['ccs']['actionsPreformed']['gatherinformation'] = true;
	// This Function Sets up a hop between ACF Projects (To the service module)

	$retEvent; // Return Event
	$retParams; // Return Parameters
	$ServiceType = $_SESSION['modules']['requiredService'];

	if($_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['active'] == 'true') {
		$retEvent = new ACF_Event('gatherserviceinformation_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/gatherinformation'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find service module URL for ".$ServiceType;
		$retEvent = new ACF_Event('gatherserviceinformation_failed');
	}

	return $retEvent;	
}

function AH_ProvisionService(ACF_ParameterList $inParameterList)
{

	if($_SESSION['ccs']['excludeService'])
		return new ACF_Event('provisionservice_exclude');

	$_SESSION['ccs']['actionsPreformed']['provision'] = true;
	// This Function Sets up a hop between ACF Projects (To the service module)
	$retEvent; // Return Event
	$retParams; // Return Parameters
	$ServiceType = $_SESSION['modules']['requiredService'];

	if($_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['active'] == 'true') {
		$retEvent = new ACF_Event('provisionservice_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/provision'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find service module URL for ".$ServiceType;
		$retEvent = new ACF_Event('provisionservice_failed');
	}

	return $retEvent;	
}

function AH_ManageService(ACF_ParameterList $inParameterList)
{
	$_SESSION['ccs']['actionsPreformed']['manage'] = true;
	// This Function Sets up a hop between ACF Projects (To the service module)

	$retEvent; // Return Event
	$retParams; // Return Parameters
	$ServiceType = $_SESSION['modules']['requiredService'];
	
	$type = $inParameterList->ParameterValue('type');
	if($type)
		$ServiceType = $type;

	if($_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['active'] == 'true') {
		$retEvent = new ACF_Event('manageservice_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/manager_manage'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find service module URL for ".$ServiceType;
		$retEvent = new ACF_Event('manageservice_failed');
	}

	return $retEvent;	
}

function AH_ProvisionFailed(ACF_ParameterList $inParameterList)
{
	$source = $inParameterList->ParameterValue('source');

	// This Function Sets up a hop between ACF Projects (To the service module)
	$retEvent; // Return Event
	$retParams; // Return Parameters
	$ServiceType = $_SESSION['modules']['requiredService'];

	$retEvent = new ACF_Event('provisionfailed_return');
	$retParams = new ACF_ParameterList();

	if($source != 'outside')
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/provisionfailed'));
	else	
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/provisionfailed'));

	$retEvent->SetParameterList($retParams);

	return $retEvent;
}

function AH_ReportFailureToService(ACF_ParameterList $inParameterList) {
	$retEvent = new ACF_Event('reportfailuretoservice_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/provisionfailed'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}
	
function AH_Authorize(ACF_ParameterList $inParameterList)
{
	$_SESSION['ccs']['actionsPreformed']['authorize'] = true;
	// This Function Sets up a hop between ACF Projects (To the service module)

	$retEvent; // Return Event
	$retParams; // Return Parameters

	if($_SESSION['ccs']['options']['driver-options']['billing-module']) {
		$retEvent = new ACF_Event('authorize_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/authorize'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find billing module URL";
		$retEvent = new ACF_Event('authorize_failed');
	}

	return $retEvent;	
}

function AH_Commit(ACF_ParameterList $inParameterList)
{
	$_SESSION['ccs']['actionsPreformed']['commit'] = true;
	// This Function Sets up a hop between ACF Projects (To the service module)

	$retEvent; // Return Event
	$retParams; // Return Parameters

	if($_SESSION['ccs']['options']['driver-options']['billing-module']) {
		$retEvent = new ACF_Event('commit_success');
		$retParams = new ACF_ParameterList();
		$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/commit'));
		$retEvent->SetParameterList($retParams);
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Failed to find billing module URL";
		$retEvent = new ACF_Event('commit_failed');
	}

	return $retEvent;	
}

function AH_SetServiceType(ACF_ParameterList $inParameterList)
{
	$_SESSION['ccs']['actionsPreformed']['setservicetype'] = true;
	$ServiceType = $inParameterList->ParameterValue('servicetype');
	$Options = $inParameterList->ParameterValue('options');

	// Optional Passable Args, must be in the options[] array
	if($ServiceType && $Options) {
		foreach($Options as $key=>$opt) {
			$_SESSION['modules'][$ServiceType]['passed'][$key] = $opt;
		}
	}

	if($ServiceType && $_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]) {
		$_SESSION['modules']['requiredService'] = $ServiceType;
		return new ACF_Event('setservicetype_success');
	} else {
		$_SESSION['ccs']['lastSystemError'] = "Service type is invalid.";
		return new ACF_Event('setservicetype_failed');
	}
}

function AH_ProvisionReturn(ACF_ParameterList $inParameterList)
{
	$ServiceType = $_SESSION['modules']['requiredService'];

	if(!$_SESSION['modules'][$ServiceType]['register']['isSuccess'])
		return new ACF_Event('provisionreturn_failed');

	return new ACF_Event('provisionreturn_success');
}

function AH_HopProject(ACF_ParameterList $inParameterList)
{
	// This activity handler redirects a user to the given url (for Project hopping)

	$retEvent; // Return Event
	//$retParams; // Return Parameters

	$url = $inParameterList->ParameterValue('url');
	header("location:".$url); // Send HTML header

	/*
	$retEvent = new ACF_Event('hopproject_success');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('hopurl',$url));
	$retEvent->SetParameterList($retParams);
	//*/
	
	$retEvent = new ACF_Event('hopproject_success');

	return $retEvent;
}

function AH_ParsePassedOptions(ACF_ParameterList $inParameterList) {
	$options = $inParameterList->ParameterValue('options');
	if($options) {
        if(!$_SESSION['ccs']) {
		    $_SESSION['ccs'] = array();
	    }
		$_SESSION['ccs']['options']['passed'] = $options;
    }

	return new ACF_Event('parsepassedoptions_return');
}

function AH_CheckAvailabilityOptions(ACF_ParameterList $inParameterList) {
	$ServiceType = $inParameterList->ParameterValue('servicetype');
	$options = $inParameterList->ParameterValue('options');

	if(!$_SESSION['ccs'])
		$_SESSION['ccs'] = array();

	if(!$_SESSION['ccs']['optionsLoaded'])
		loadCCSOptions();
	
	// Set Service Type
	$_SESSION['modules']['requiredService'] = $ServiceType;

	if($options) {
		foreach($options as $key=>$option) { // Doesn't override options
			$_SESSION['modules'][$ServiceType]['passed'][$key] = $option;
		}
	}

	return new ACF_Event('checkavailabilityoptions_return');
}

function AH_TargetManage(ACF_ParameterList $inParameterList) {
	$ServiceType = $inParameterList->ParameterValue('servicetype');
	$options = $inParameterList->ParameterValue('options');
	$staffid = $inParameterList->ParameterValue('staffid');

	// Set Service Type
	$_SESSION['modules']['requiredService'] = $ServiceType;

	if(!$_SESSION['ccs'])
		$_SESSION['ccs'] = array();

	if(!$_SESSION['ccs']['optionsLoaded'])
		loadCCSOptions();

	if($options) {
		$_SESSION['modules']['billing']['passed'] = array();
		foreach($options as $key=>$option) { // Doesn't override options
			$_SESSION['modules'][$ServiceType]['passed'][$key] = $option;
			$_SESSION['modules']['billing']['passed'][$key] = $option;
		}
	}

	if($staffid) {
		$_SESSION['piwi']['staffid'] = $staffid;
	}

	if(strtolower($ServiceType) == 'domain') { // get domain pricing
		GetDomainPrices($_SESSION['modules']['billing']['passed']['crid']);
	}

	$retEvent = new ACF_Event('targetmanage_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/targetpage'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}

function clearccsinfo() {
	// Flush Tasks & Alternates
	if($_SESSION['modules']) {
		foreach($_SESSION['modules'] as &$module) {
			if(is_array($module)) {
				if($module['task'])
					unset($module['task']);
				if($module['alternate'])
					unset($module['alternate']);
			}
		}
	}

	// Clean State
	if($_SESSION['ccs']['actionsPreformed'])
		unset($_SESSION['ccs']['actionsPreformed']);

	// Clear skipped services
        if($_SESSION['ccs']['excludeService'])
		unset($_SESSION['ccs']['excludeService']);

	// Clear Purchase Step Skips
	if($_SESSION['ccs']['excludeSteps'])
		unset($_SESSION['ccs']['excludeSteps']);

	// Clear Auth Flag
	if($_SESSION['modules']['shared']['authorized'])
		unset($_SESSION['modules']['shared']['authorized']);
}

function AH_CleanSession(ACF_ParameterList $inParameterList) {
	clearccsinfo();

	if($_SESSION['modules']) {
		$status = $_SESSION['modules']['shared']['status'];
		$_SESSION['modules'] = array();
		$_SESSION['modules']['shared']['status'] = $status;
	}

	return new ACF_Event('cleansession_return');
}

function AH_WrapUp(ACF_ParameterList $inParameterList) {
	clearccsinfo();

	// Flush Register Info
	if($_SESSION['modules']) {
		foreach($_SESSION['modules'] as &$module) {
			if(is_array($module) && $module['register']){
				unset($module['register']);
			}
		}
	}

	// Flush Template Status
	if($_SESSION['ccs']['knownSteps'])
		unset($_SESSION['ccs']['knownSteps']);
	if($_SESSION['ccs']['maxStep'])
		unset($_SESSION['ccs']['maxStep']);

	// Flush Billing Info
    if($_SESSION['modules']['billing'])
        unset($_SESSION['modules']['billing']);

	// Flush Passed Options
	if($_SESSION['ccs']['options']['passed'])
		unset($_SESSION['ccs']['options']['passed']);

	return new ACF_Event('wrapup_return');
}

// Item Charge Hops
function AH_ChargeItemAuth(ACF_ParameterList $inParameterList) {
	$retEvent = new ACF_Event('chargeitemauth_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/chargeitem_auth'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}

function AH_ChargeItemCommit(ACF_ParameterList $inParameterList) {
	$retEvent = new ACF_Event('chargeitemcommit_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/chargeitem_commit'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}

function AH_ChargeItemFailed(ACF_ParameterList $inParameterList) {
	$retEvent = new ACF_Event('chargeitemfailed_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['billing-module']['url'].'/chargeitem_failed'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}

function AH_ChargeServiceItemAuth(ACF_ParameterList $inParameterList) {
	$ServiceType = $_SESSION['modules']['requiredService'];
	$retEvent = new ACF_Event('chargeserviceitemauth_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/chargeserviceitem_auth'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}

function AH_ChargeServiceItemCommit(ACF_ParameterList $inParameterList) {
	$ServiceType = $_SESSION['modules']['requiredService'];
	$retEvent = new ACF_Event('chargeserviceitemcommit_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/chargeserviceitem_commit'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}

function AH_ChargeServiceItemFailed(ACF_ParameterList $inParameterList) {
	$ServiceType = $_SESSION['modules']['requiredService'];
	$retEvent = new ACF_Event('chargeserviceitemfailed_return');
	$retParams = new ACF_ParameterList();
	$retParams->AddParameter(new ACF_DirectParameter('url',$_SESSION['ccs']['options']['driver-options']['service-modules'][$ServiceType]['url'].'/chargeserviceitem_failed'));
	$retEvent->SetParameterList($retParams);
	return $retEvent;
}

function AH_SendConfirmationEmail(ACF_ParameterList $inParameterList) {

	if((($_SESSION['ccs']['options']['driver-options']['email-options']['send-confirmation'] == 'yes') && $_SESSION['modules']['shared']['email']) || ($_SESSION['ccs']['options']['driver-options']['email-options']['send-reseller-confirmation'] == 'yes')) {
		// Send Email ****************************************
		require_once('activityhandlers/email.inc.php');
		$obj = new TU_Email;
		$obj->setServer($_SESSION['ccs']['options']['driver-options']['email-options']['server']);
		$obj->setFrom($_SESSION['ccs']['options']['driver-options']['email-options']['sender-address']);
		if($_SESSION['ccs']['options']['driver-options']['email-options']['send-confirmation'] && $_SESSION['modules']['shared']['email'])
			$obj->addTo($_SESSION['modules']['shared']['email']);
		if($_SESSION['ccs']['options']['driver-options']['email-options']['send-reseller-confirmation'])
			$obj->addTo($_SESSION['ccs']['options']['driver-options']['email-options']['reseller-address']);
		$obj->setSubject($_SESSION['ccs']['options']['driver-options']['email-options']['subject']);
		$obj->setBody(file_get_contents($_SESSION['ccs']['options']['driver-options']['email-options']['body-file']));
		$obj->send();
		//if($obj->getlasterror())
	}
	return new ACF_Event('sendconfirmationemail_return');
}

function GetDomainPrices($crid = false) {
	require_once("../../libs/platypus/wow.inc.php");
	require_once('../../libs/ccs/utilities.inc.php');

	$stafflogin = $_SESSION['wow']['staffname'];
	$staffpass  = $_SESSION['wow']['staffpass'];

	$wowlogintype = $_SESSION['wow']['logintype'] ? $_SESSION['wow']['logintype'] : 'staff';

	$wow = new WOW_Request();
	$wow->setUsername($stafflogin);
	$wow->setPassword($staffpass);

	if($crid) {
		$wow->setMethod('SQL');
		$wow->addAttribute('logintype',$wowlogintype);

		$wow->addParameter('query','select rgid from custrate where crid = '.$crid);
		$wow->addParameter('noconn',false,'boolean');
		$wow->addParameter('datatype','XML');

		if($wow->execute()) {
			$rgid = $wow->getAttribute(0,'rgid');
		} else {
			$rgid = false;
		}
	}

	if($rgid) {
		$wow->setMethod('Transaction_GetPrice');
		$wow->addAttribute('logintype',$wowlogintype);

		$wow->addParameter('Service',false,'boolean');
		$wow->addParameter('SubService',false,'boolean');
		$wow->addParameter('Action',false,'boolean');
		$wow->addParameter('Period',false,'boolean');
		$wow->addParameter('RGID',$rgid,'string');
		$wow->addParameter('DataType','XML');

		if($wow->execute()) {
			$allPrices = $wow->getAttributesArray();
		} else {
			$allPrices = false;
		}
	}

	$allowedTLDsByRate = array();
	$tldList = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['related-tld-groups'];

	if(isset($allPrices) && is_array($allPrices)) {
		$priceArray = array();
		foreach($allPrices as $thisPrice) {
			if(!in_array('.'.strtolower($thisPrice['service_subtype']),$allowedTLDsByRate)) {
				array_push($allowedTLDsByRate,'.'.strtolower($thisPrice['service_subtype']));
			}

			switch($thisPrice['transaction_type']) {
				case "Register":
					$type = "new";
					break;
				default:
					$type = strtolower($thisPrice['transaction_type']);
			}

			$type = 'period-pricing-'.$type;
			$tld = ".".strtolower($thisPrice['service_subtype']);
			$year = $thisPrice['period'];

			if(!$priceArray[$tld]) {
				$priceArray[$tld] = array();
			}

			if(!$priceArray[$tld][$type]) {
				$priceArray[$tld][$type] = array();
			}

			$priceArray[$tld][$type][$year-1] = $thisPrice['price'];
		}
	}

	if(isset($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering']) && is_array($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'])) {
		foreach($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'] as &$thisTLD) {
			if(isset($priceArray) && is_array($priceArray)) {
				$thisTLD['price']['period-pricing-new'] = $priceArray[$thisTLD['tld']]['period-pricing-new'];
				$thisTLD['price']['period-pricing-renew'] = $priceArray[$thisTLD['tld']]['period-pricing-renew'];
				$thisTLD['price']['period-pricing-transfer'] = $priceArray[$thisTLD['tld']]['period-pricing-transfer'];
			} else {
				for($thisPeriod = 0;$thisPeriod < 10;$thisPeriod++) {
					$thisTLD['price']['period-pricing-new'][$thisPeriod] = $thisTLD['price']['new'] * ($thisPeriod + 1);
					$thisTLD['price']['period-pricing-renew'][$thisPeriod] = $thisTLD['price']['renew'] * ($thisPeriod + 1);
					$thisTLD['price']['period-pricing-transfer'][-1] = $thisTLD['price']['transfer'];
				}
			}
		}
	}

	if(isset($tldList['group']) && is_array($tldList['group'])) {
		foreach($tldList['group'] as $tldGroupKey=>$tldGroup) {
			if(isset($tldGroup['tld']) && is_array($tldGroup['tld'])) { // more than one TLD in this group
				foreach($tldGroup['tld'] as $thisTLDKey=>$thisTLDName) {
					if(!in_array($thisTLDName,$allowedTLDsByRate) && $thisTLDName != 'default') {
						unset($tldList['group'][$tldGroupKey]['tld'][$thisTLDKey]);
					}
				}
			} else {
				if(!in_array($tldGroup['tld'],$allowedTLDsByRate)) {
					unset($tldList['group'][$tldGroupKey]);
				}
			}
		}
	}

	$_SESSION['ccs']['tldGroups'] = $tldList;
}

?>
