<?php

class TU_Email {
	private $server;	
	private $port;
	private $lastError;
	private $secure;
	private $log;

	// Headers
	private $to_list;
	private $from;
	private $subject;
	private $id;
	private $hostname;

	// Content
	private $body;
	private $_CRLF = "\r\n";

	function __construct($server=null,$port=null) {

		// Set Server and Port (This can be done using set/get functions).
		if($server)
			$this->server = $server;
		if($port)
			$this->port= $port;
		else
			$this->port= 25;

		$this->log = "";
		$this->generateId();
		$this->to_list = array();
	}

	// Property Methods
	function getLastError() { return $this->lastError; }
	function setServer($server) { $this->server = $server; }
	function getServer() { return $this->server; }
	function setBody($body) { $this->body = $body; }
	function getBody() { return $this->body; }
	function setHostname($hostname) { $this->hostname = $hostname; }
	function getHostname() { return $this->hostname; }
	function setId($id) { $this->id = $id; }
	function getId() { return $this->id; }
	function setSubject($subject) { $this->subject = $subject; }
	function getSubject() { return $this->subject; }
	function setFrom($from) { $this->from = $from; }
	function getFrom() { return $this->from; }
	function setPort($port) { $this->port = $port; }
	function getPort() { return $this->port; }
	function addTo($to) { array_push($this->to_list,$to); }
	function getToList() { return $this->to_list; }
	function setSecure($secure) { $this->secure = $secure; }
	function getSecure() { return $this->secure; }
	function getLog() { return $this->log; }

	private function getlocalhostname() {
		$name = getenv('HOSTNAME');
		if(!$name) {
				$ip = getenv('REMOTE_ADDR');
				if (!$ip) 
					$ip = getenv('REMOTE_HOST'); 
				$name = @GetHostByAddr($ip);
		}
		if(!$name)
			$name = 'localhost';
		return $name;
	} 

	private function generateId() {
		if(!$this->hostname) { // No hostname, get one
			$this->hostname = $this->getlocalhostname();
		}

		$prefix = time().'t'.rand().'u';

		$this->id = $prefix.'@'.$this->hostname;
	}
	
	private function buildHeader() {
		$header = 'From: '.$this->from.$this->_CRLF; // Required
		$header .= 'To: '.$this->to.$this->_CRLF; // Required
		$header .= 'Subject: '.$this->subject.$this->_CRLF; // Required
		$header .= 'Date: '.date('r').$this->_CRLF; // print date in rfc822 format (Required)
		$header .= 'Message-ID: '.$this->id.$this->_CRLF; // Required
		$header .= 'MIME-Version: 1.0'.$this->_CRLF; // Optional
		$header .= 'Content-Type: text/plain; charset="iso-8859-1"'.$this->_CRLF; // Optional

		return $header;
	}

	private function checkStatus() {
		$retStatus = true;
		// Check Required Props
		switch (true) {
			case (count($this->to_list) < 1):
				$this->lastError = 'To field not set.';
				$retStatus = false;
				break;
			case (!$this->from):
				$this->lastError = 'From field not set.';
				$retStatus = false;
				break;
			case (!$this->server):
				$this->lastError = 'Server not set.';
				$retStatus = false;
				break;
		}

		return $retStatus;
	}

	private function sendCommand($socket,$command,$successCode = '250') {
		// Com Values
		$response = '';
		$retval = false;

		// for Secure
		if($this->secure)
			$successCode = '+OK';

		// Write To Socket
		$this->log .= $command;
		if(fwrite($socket,$command,strlen($command)) === false) {
			$this->lastError = "Failed to write during command: ".$command;
			return false;
		}
		// Read From Socket
		$response = fgets($socket);  // Gets one line
		$this->log .= $response;
		if(substr($response,0,3) == $successCode) // Success
			$retval = true;
		else
			$this->lastError = 'SMTP error during command: ('.$command.') with error: ('.$response.')';

		return $retval;
	}

	private function checkMTA($socket) {
		// This function checks the response code of the connection to ensure
		// the server is ready to accept messages.

		$retval = false;
		$response = '';
		$response = fgets($socket);  // Gets one line
		if(substr($response,0,3) == '220') // Success
			$retval = true;
		return $retval;
	}

	private function sendBody($socket) {
		// Com Values
		$command = '';
		$response = '';
		$retval = false;

		// Build Message
		$msg = $this->buildHeader().$this->_CRLF.$this->body; // Construct Message

		$command = 'DATA'.$this->_CRLF;
		if(fwrite($socket,$command,strlen($command)) === false) {
			$this->lastError = "Failed to write during command: ".$command;
			return false;
		}
		$response = fgets($socket);  // Gets one line
		if(substr($response,0,3) == '354') {
			if(fwrite($socket,$msg,strlen($msg)) === false) {
				$this->lastError = "Failed to write email body";
				return false;
			}
			$command = $this->_CRLF.'.'.$this->_CRLF;
			if(fwrite($socket,$command,strlen($command)) === false) {
				$this->lastError = "Failed to write email body";
				return false;
			}
			$response = fgets($socket);  // Gets one line
			if(substr($response,0,3) == '250') // Success
				$retval = true;
			else
				$this->lastError = 'SMTP error while sending body with error: ('.$response.')';
		} else
			$this->lastError = 'SMTP error during command: ('.$command.') with error: ('.$response.')';
		return $retval;
	}

	function send() {
		$retval = false;
		// Socket Error Values
	 	$errorNumber = 0;
		$errorString = '';
		$toSuccess = false;

		// Ensure Everything is in order
		if(!$this->checkStatus())
			return false;

		// Open Socket
		if($this->secure)
			$socket = fsockopen('ssl://'.$this->server, $this->port, $errorNumber, $errorString);
		else
			$socket = fsockopen($this->server, $this->port, $errorNumber, $errorString);

		if($socket) {
			// Set Socket Timeout
			stream_set_timeout($socket,120); //--- Times out in 2 min.

			if($this->checkMTA($socket)) {
				// HELO COMMAND
				$command = 'HELO '.$this->hostname.$this->_CRLF;
				if($this->sendCommand($socket,$command)) {
					// MAIL FROM COMMAND
					$command = 'MAIL FROM: '.$this->from.$this->_CRLF;	
					if($this->sendCommand($socket,$command)) {
						// RCPT TO COMMAND
						foreach($this->to_list as $to) {
							$command = 'RCPT TO: '.$to.$this->_CRLF;	
							if($this->sendCommand($socket,$command)) {
								$toSuccess = true;
							} else {
							    $this->lastError = 'Failed to deliver to: '.$to.$this->_CRLF;
							}
						}
						if($toSuccess)		
						    if($this->sendBody($socket))
							$retval = true;
					}
				}
			}
					
			$command = 'QUIT'.$this->_CRLF;	
			$this->sendCommand($socket,$command,'221');
		
			fclose($socket);

		} else {
				$this->lastError = 'Could not connect to server: ('.$errorNumber.') '.$errorString;
		}
		return $retval;
	}
}

?>
