<?php
require_once('acf_refresh.inc.php');

function AH_UpdateLayers(ACF_ParameterList $inParameterList) {
    $receiver = $inParameterList->ParameterValue('receiver');
    $activelayers = $inParameterList->ParameterValue('activelayers');
    $layernames = $inParameterList->ParameterValue('layernames');
    $layerorder = $inParameterList->ParameterValue('layerorder');

    asort($layerorder);
    reset($layerorder);

    $newlayers = array();
    $newbaselayers = array();
    foreach($layerorder as $key=>$value) {
	$newlayer = array();
	if($layernames[$key])
	    $newlayer['name'] = trim($layernames[$key]);
	else
	    $newlayer['name'] = $key;

	array_push($newbaselayers,$newlayer['name']);

	if($activelayers[$key] || ($key == "defaultlayer")) // defaultlayer is always active
	    $newlayer['active'] = "true";
	else
	    $newlayer['active'] = "false";
	
	array_push($newlayers,$newlayer);
    }

    $conn = $_SESSION['currentcontroller'];
    $conn->SetLayers($newbaselayers);
   

    foreach($_SESSION['project']['receivers'] as $key => $recv) {
	if($recv['name'] == $receiver) {
	    $_SESSION['project']['receivers'][$key]['layers'] = $newlayers;
	    break;
	}
    }

    RefreshXML();

    $retEvent = new ACF_Event('UpdateLayer_Success');
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('receiver', $receiver));
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}

function AH_SetLayer(ACF_ParameterList $inParameterList) {
    $layer = $inParameterList->ParameterValue('layer');
    if($layer)
	$_SESSION['layer'] = $layer;

    return new ACF_Event('Events');
}

function RefreshXML() {
    if (file_exists('../../etc/projects.xml'))
	$xml = simplexml_load_file('../../etc/projects.xml');

    $xmlstr = "<?xml version=\"1.0\"?>\n<projects>\n";
    foreach($xml->project as $p) {
	$xmlstr .= "<project>\n";
	$value = (string) $p->name;
	$xmlstr .= "    <name>".$value."</name>\n";
	$value = (string) $p->directory;
	$xmlstr .= "    <directory>".$value."</directory>\n";
	$xmlstr .= "    <receivers>\n";

	if($p->name == $_SESSION['projname']) {
	    foreach($_SESSION['project']['receivers'] as $recv) {
		$xmlstr .= "        <receiver name=\"".$recv['name']."\">\n";
		foreach($recv['layers'] as $layer) {
		    $xmlstr .= "            <layer active=\"".$layer['active']."\">".$layer['name']."</layer>\n";
		}
		$xmlstr .= "        </receiver>\n";
	    }
	} else {
	    foreach($p->receivers->receiver as $recv) {
		$value = (string) $recv['name'];
		$xmlstr .= "        <receiver name=\"".$value."\">\n";
		foreach($recv->layer as $layer) {
		    if((string) $layer['active'] == 'false')
			$value = (string) $layer['active'];
		    else
			$value = 'true';
		    $xmlstr .= "            <layer active=\"".$value."\">";
		    $value = (string) $layer;
		    $xmlstr .= $value."</layer>\n";
		}
		$xmlstr .= "        </receiver>\n";
	    }
	}

	$xmlstr .= "    </receivers>\n";
	$indexname = "persistence-type";
	$value = (string) $p->$indexname;
	$xmlstr .= "    <persistence-type>".$value."</persistence-type>\n";
	if($value == "adodb") {
	    $xmlstr .= "    <persistence-options>\n";
	    $indexname = "persistence-options";
	    $subname = "server-address";
	    $value = (string) $p->$indexname->$subname;
	    $xmlstr .= "        <server-address>".$value."</server-address>\n";
	    $indexname = "persistence-options";
	    $subname = "username";
	    $value = (string) $p->$indexname->$subname;
	    $xmlstr .= "        <username>".$value."</username>\n";
	    $indexname = "persistence-options";
	    $subname = "password";
	    $value = (string) $p->$indexname->$subname;
	    $xmlstr .= "        <password>".$value."</password>\n";
	    $indexname = "persistence-options";
	    $subname = "dbname";
	    $value = (string) $p->$indexname->$subname;
	    $xmlstr .= "        <dbname>".$value."</dbname>\n";
	    $xmlstr .= "    </persistence-options>\n";
	}
	$xmlstr .= "</project>\n";
    }
    $xmlstr .= "</projects>\n";
    $handle = fopen('../../etc/projects.xml', 'w');
    fwrite($handle, $xmlstr);
    fclose($handle); 
}

function AH_RemoveLayer(ACF_ParameterList $inParameterList) {
    $name = $inParameterList->ParameterValue('name');

    if($name != 'defaultlayer') {

	$conn = $_SESSION['currentcontroller'];
	$layerlist = $conn->Layers();

	if(array_search($name,$layerlist) !== false) {

	    $newlayers = array();
	    foreach($_SESSION['project']['receivers'] as &$recv) {
		foreach($recv['layers'] as $key=>$layer) {
		    if($layer['name'] != $name) {
			if(!in_array($layer['name'],$newlayers))
			    array_push($newlayers,$layer['name']);
		    } else if($layer['name'] == $name) {
			// remove layer from receiver
			unset($recv['layers'][$key]);
		    }
		}
	    }
	    $conn->SetLayers($newlayers);

	    RefreshXML();
	}
    }

    return new ACF_Event('removelayer_success');
}

function AH_PutNewLayer(ACF_ParameterList $inParameterList) {
    $name = $inParameterList->ParameterValue('name');

    $conn = $_SESSION['currentcontroller'];
    $layerlist = $conn->Layers();

    if(!array_search($name,$layerlist)) {
	
	// Add Layer to Lists
	array_push($layerlist,$name);
	$conn->SetLayers($layerlist);
	$newfulllayer = array();
	$newfulllayer['name'] = $name;
	$newfulllayer['active'] = "true";

	foreach($_SESSION['project']['receivers'] as &$recv) {
	    array_unshift($recv['layers'],$newfulllayer);
	}

	RefreshXML();	

	$_SESSION['layer'] = $name;
    }

    return new ACF_Event('putnewlayer_success');
}

function AH_LoadProject(ACF_ParameterList $inParameterList) {
    $projectname = $inParameterList->ParameterValue('projname');
    $selectproject = $inParameterList->ParameterValue('selectproject');

    if (file_exists('../../etc/projects.xml'))
       $xml = simplexml_load_file('../../etc/projects.xml');

    $index = 0;
    foreach($xml->project as $project) {
	if($project->name == $projectname) {
	    if($selectproject == "Delete Project") {
		$xmlstr = "<?xml version=\"1.0\"?>\n<projects>\n";
		foreach($xml->project as $p) {
		    if($p->name != $projectname) {
			$xmlstr .= "<project>\n";
			$value = (string) $p->name;
			$xmlstr .= "    <name>".$value."</name>\n";
			$value = (string) $p->directory;
			$xmlstr .= "    <directory>".$value."</directory>\n";
			$xmlstr .= "    <receivers>\n";
			foreach($p->receivers->receiver as $recv) {
			    $value = (string) $recv['name'];
			    $xmlstr .= "        <receiver name=\"".$value."\">\n";
			    foreach($recv->layer as $layer) {
				$value = (string) $layer;
				$xmlstr .= "            <layer active=\"".$layer['active']."\">".$value."</layer>\n";
			    }
			    $xmlstr .= "        </receiver>\n";
			}
			$xmlstr .= "    </receivers>\n";
			$indexname = "persistence-type";
			$value = (string) $p->$indexname;
			$xmlstr .= "    <persistence-type>".$value."</persistence-type>\n";
			if($value == "adodb") {
			    $xmlstr .= "    <persistence-options>\n";
			    $indexname = "persistence-options";
			    $subname = "server-address";
			    $value = (string) $p->$indexname->$subname;
			    $xmlstr .= "        <server-address>".$value."</server-address>\n";
			    $indexname = "persistence-options";
			    $subname = "username";
			    $value = (string) $p->$indexname->$subname;
			    $xmlstr .= "        <username>".$value."</username>\n";
			    $indexname = "persistence-options";
			    $subname = "password";
			    $value = (string) $p->$indexname->$subname;
			    $xmlstr .= "        <password>".$value."</password>\n";
			    $indexname = "persistence-options";
			    $subname = "dbname";
			    $value = (string) $p->$indexname->$subname;
			    $xmlstr .= "        <dbname>".$value."</dbname>\n";
			    $xmlstr .= "    </persistence-options>\n";
			}
			$xmlstr .= "</project>\n";
		    }
		}
		$xmlstr .= "</projects>\n";
	
		$handle = fopen('../../etc/projects.xml', 'w');
		fwrite($handle, $xmlstr);
		fclose($handle); 
		$retEvent = new ACF_Event('defaultevent');
	    }
	    else {
		$_SESSION['projname'] = $projectname;
		$currentproject = array();

		$currentproject['name'] = (string) $project->name;
		$currentproject['dir'] = (string) $project->directory;

		$receivers = array();

		foreach($project->receivers->receiver as $recv) {
		    $newrecv = array();
		    $newrecv['name'] = (string) $recv['name'];
		    $newrecv['layers'] = array(); 
		    foreach($recv->layer as $layer) {
			$newlayer = array();
			$newlayer['name'] = (string) $layer;
			if((string) $layer['active'] == 'false')
			    $newlayer['active'] = (string) $layer['active'];
			else
			    $newlayer['active'] = 'true';
			array_push($newrecv['layers'],$newlayer);
		    }
		    array_push($receivers,$newrecv);
		}
		   
		$indexname = "persistence-type";
		$currentproject['type'] = (string) $project->$indexname;
		$currentproject['receivers'] = $receivers;

		if($currentproject['type'] == 'adodb') {
		    $currentproject['dboptions'] = array();
		    
		    $indexname = "persistence-options";
		    $subname = "server-address";
		    $currentproject['dboptions']['server'] = (string) $project->$indexname->$subname;

		    $subname = "username";
		    $currentproject['dboptions']['username'] = (string) $project->$indexname->$subname;

		    $subname = "password";
		    $currentproject['dboptions']['password'] = (string) $project->$indexname->$subname;

		    $subname = "dbnameresponses[$layer][$sourceRealm][$sourceName][$eventName]";
		    $currentproject['dboptions']['dbname'] = (string) $project->$indexname->$subname;
		}

		$_SESSION['project'] = $currentproject;
		$_SESSION['layer'] = $currentproject['receivers'][0]['layers'][0]['name'];

		$retEvent = new ACF_Event('Events');
		$parameterList = new ACF_ParameterList();
		$parameterList->AddParameter(new ACF_DirectParameter('projname', $_SESSION['projname']));
		$retEvent->SetParameterList($parameterList);
	    }
	    break;
	}
	$index++;
    }

    return $retEvent;
}

function AH_StoreProjectName(ACF_ParameterList $inParameterList) {
    $projectname = $inParameterList->ParameterValue('name');

    if(!$projectname) {
	$retEvent = new ACF_Event('projectwizard_error');
	$parameterList = new ACF_ParameterList();
	$parameterList->AddParameter(new ACF_DirectParameter('message', "Project Name Cannot Be Empty."));
	$retEvent->SetParameterList($parameterList);
	return $retEvent;
    }

    // Check to see if name exists
    if (file_exists('../../etc/projects.xml'))
       $xml = simplexml_load_file('../../etc/projects.xml');
    
    foreach($xml->project as $project) {
	if($project->name == $projectname) {
	    $retEvent = new ACF_Event('projectwizard_error');
	    $parameterList = new ACF_ParameterList();
	    $parameterList->AddParameter(new ACF_DirectParameter('message', "Project Name In Use."));
	    $retEvent->SetParameterList($parameterList);
	    return $retEvent; 
	}
    }

    $_SESSION['newproject_name'] = $projectname;

    return new ACF_Event('StoreProjectName_Success');
}

function AH_StoreProjectDir(ACF_ParameterList $inParameterList) {
    $dirname = $inParameterList->ParameterValue('dirname');

    if(!$dirname)
	return new ACF_Event('projectwizard_error');

    $_SESSION['newproject_dirname'] = $dirname;

    return new ACF_Event('StoreProjectDir_Success');
}

function AH_StoreProjectType(ACF_ParameterList $inParameterList) {
    $type = $inParameterList->ParameterValue('type');

    $_SESSION['newproject_type'] = $type;

    if($type == "xml") {
	SaveNewProject();
	return new ACF_Event('defaultevent');
    }
    else if($type == "adodb")
	return new ACF_Event('StoreProjectType_Success_ADODB');
}

function create_node_from_string($document, $xml_string)
{
    return $document->importNode(dom_import_simplexml(simplexml_load_string($xml_string)),true);
}

function CreateNewProject() {
    exec('mkdir ../'.$_SESSION['newproject_dirname']);
    $indexfile = file_get_contents('./index.php');
    $newindexfile = str_replace('Config Tool',$_SESSION['newproject_name'],$indexfile);
    $handle = fopen('../'.$_SESSION['newproject_dirname'].'/index.php', 'w');
    fwrite($handle, $newindexfile);
    fclose($handle); 
}

function SaveNewProject() {
   
    if (file_exists('../../etc/projects.xml'))
	$xml = simplexml_load_file('../../etc/projects.xml');

    $newproject = "<project>";
    $newproject .= "\n    <name>".$_SESSION['newproject_name']."</name>";
    $newproject .= "\n    <directory>".$_SESSION['newproject_dirname']."</directory>";
    $newproject .= "\n    <receivers>\n        <receiver name=\"index.php\">\n            <layer active=\"true\">defaultlayer</layer>\n        </receiver>\n    </receivers>";
    $newproject .= "\n    <persistence-type>".$_SESSION['newproject_type']."</persistence-type>";
    $newproject .= "\n    <persistence-options>";

    if($_SESSION['newproject_type'] == "adodb") {
	$newproject .= "\n        <server-address>".$_SESSION['newproject_server']."</server-address>";
	$newproject .= "\n        <username>".$_SESSION['newproject_username']."</username>";
	$newproject .= "\n        <password>".$_SESSION['newproject_pass']."</password>";
	$newproject .= "\n        <dbname>".$_SESSION['newproject_dbname']."</dbname>";
    }

    $newproject .= "\n    </persistence-options>";
    $newproject .= "\n</project>\n";

    $dom = dom_import_simplexml($xml);
    $dom->appendChild(create_node_from_string($dom->ownerDocument,$newproject));

    $xmlstr = $xml->asXML();
    $handle = fopen('../../etc/projects.xml', 'w');
    fwrite($handle, $xmlstr);
    fclose($handle);
    
    CreateNewProject();
}

function AH_StoreProjectADODB(ACF_ParameterList $inParameterList) {
    $server = $inParameterList->ParameterValue('server');
    $dbname = $inParameterList->ParameterValue('dbname');
    $username = $inParameterList->ParameterValue('uname');
    $pass = $inParameterList->ParameterValue('pass');

    $_SESSION['newproject_server'] = $server;
    $_SESSION['newproject_dbname'] = $dbname;
    $_SESSION['newproject_username'] = $username;
    $_SESSION['newproject_pass'] = $pass;

    SaveNewProject();

    return new ACF_Event('defaultevent');
}

function AH_RenderLayer(ACF_ParameterList $inParameterList) {
    require_once('./drawproject.php');

    if($_SESSION['layer'])
	$htmltext = drawLayer('../'.$_SESSION['project']['dir'].'/'.$_SESSION['layer'].'.xml');

    $retEvent = new ACF_Event('renderlayer_return');
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('htmltext', $htmltext));
    $retEvent->SetParameterList($parameterList);
    return $retEvent; 
}

function AH_LayerItem(ACF_ParameterList $inParameterList) {
    $type = $inParameterList->ParameterValue('type');
    $targetlayer = $inParameterList->ParameterValue('targetlayer');
    $options = $inParameterList->ParameterValue('options');

    $retevent = new ACF_Event('layeritem_'.$type); // Create Return Event
    $parameterList = new ACF_ParameterList(); // Create Return Parameters

    $Controller = GetCurrentController(true);

    switch ($type) {
    case "event": // Copy Event
	if($targetlayer != $_SESSION['layer']) { // Copy to new Layer
	    $Responses = $Controller->ProcessManager()->Responses();
	    $newEventSource = new ACF_EventSource($options['realm'],$options['source']);
	    $newEvent = new ACF_Event($options['eventname']);
	    $newEventResponse = clone($Responses[$_SESSION['layer']][$options['realm']][$options['source']][$options['eventname']]);
	    $Controller->ProcessManager()->AddResponse($newEventSource,$newEvent,$newEventResponse,$targetlayer);
	    $Controller->SaveFramework();
	}

	// Setup return Event
	$parameterList->AddParameter(new ACF_DirectParameter('eventname', $options['eventname']));
	$parameterList->AddParameter(new ACF_DirectParameter('parentid', 'EVENT_'.$options['realm'].(($options['realm'] == 'external')?'':'/').$options['source']));
	break;
    case "executable": // Copy Executable
	if($targetlayer != $_SESSION['layer']) { // Copy to new Layer
	    $Wrappers = $Controller->ExecutableWrapperManager()->Wrappers();
	    $newWrapper = clone($Wrappers[$_SESSION['layer']][$options['taskname']]); 
	    $Controller->ExecutableWrapperManager()->AddExecutableWrapper($newWrapper,$targetlayer);
	    $Controller->SaveFramework();
	}

	// Setup return Event
	$rev = strrev($options['taskname']);
	$parameterList->AddParameter(new ACF_DirectParameter('executablename', strrev(substr($rev,0,strpos($rev,"/")))));
	$parameterList->AddParameter(new ACF_DirectParameter('parentid', 'EXECUTABLE/'.substr($options['taskname'],0,strlen($options['taskname']) - strpos($rev,"/") - 1)));
	break;
    case "view": // Copy View
	if($targetlayer != $_SESSION['layer']) { // Copy to new Layer
	    $Definitions = $Controller->ViewDefinitionManager()->Definitions();
	    $newDefinition = clone($Definitions[$_SESSION['layer']][$options['parentid'].'/'.$options['viewname']]); 
	    $Controller->ViewDefinitionManager()->AddViewDefinition($newDefinition,$targetlayer);
	    $Controller->SaveFramework();
	}
	
	// Setup return Event
	$parameterList->AddParameter(new ACF_DirectParameter('viewname', $options['viewname']));
	$parameterList->AddParameter(new ACF_DirectParameter('parentid', $options['parentid']));
	break;
    }
    
    $_SESSION['layer'] = $targetlayer;
    $retevent->SetParameterList($parameterList); // Assign return parameters to return event.
    return $retevent;
}

?>
