<?php
require_once('acf_refresh.inc.php');

function AH_UpdateExecutable(ACF_ParameterList $inParameterList) {
    $taskname = $inParameterList->ParameterValue('taskname');
    $name = $inParameterList->ParameterValue('name');
    $calltype = $inParameterList->ParameterValue('calltype');
    $rettype = $inParameterList->ParameterValue('returntype');
    
    $cacheTimeout = $inParameterList->ParameterValue('cache_timeout');
    $cacheGlobal = $inParameterList->ParameterValue('cache_global')?true:false;

    $tmp_req_inparam = $inParameterList->ParameterValue('req_inparam'); 
    $tmp_opt_inparam = $inParameterList->ParameterValue('opt_inparam'); 
    $tmp_req_outparam = $inParameterList->ParameterValue('req_outparam'); 
    $tmp_opt_outparam = $inParameterList->ParameterValue('opt_outparam'); 

    $req_inparam = explode(",",$tmp_req_inparam);
    $opt_inparam = explode(",",$tmp_opt_inparam);
    $req_outparam = explode(",",$tmp_req_outparam);
    $opt_outparam = explode(",",$tmp_opt_outparam);

    if($calltype == 'localphp') {
	    $resourcelocation = $inParameterList->ParameterValue('resourcelocation');
	    $resourcename = $inParameterList->ParameterValue('resourcename');
    } else if($calltype == 'xmlrpc') {
	    $server = $inParameterList->ParameterValue('server');
	    $port = $inParameterList->ParameterValue('port');
	    $uri = $inParameterList->ParameterValue('uri');
	    $functionname = $inParameterList->ParameterValue('functionname');
    } else if($calltype == 'soap') {
	    $serverpath = $inParameterList->ParameterValue('serverpath');
	    $uri = $inParameterList->ParameterValue('uri');
	    $functionname = $inParameterList->ParameterValue('functionname');
    }

    $tmp_outputevents = $inParameterList->ParameterValue('outputevents');
    $outputevents = explode(",",$tmp_outputevents);

    $layer = $_SESSION['layer'];
    $controller = GetCurrentController(true);
    $executableManager = $controller->ExecutableWrapperManager();
    $wrapper = $executableManager->FindWrapper($layer,$taskname);

    $wrapper->SetName($name);
    $wrapper->SetReturnType($rettype);
    $wrapper->SetRequiredInputParameters($req_inparam);
    $wrapper->SetOptionalInputParameters($opt_inparam);
    $wrapper->SetRequiredOutputParameters($req_outparam);
    $wrapper->SetOptionalOutputParameters($opt_outparam);
    
    $wrapper->SetCachingOptions($cacheTimeout,$cacheGlobal);
    
    if($outputevents)
	$wrapper->SetOutputEvents($outputevents);

    if($calltype == 'localphp') {
	$wrapper->SetFileName($resourcelocation);
	$wrapper->SetFunction($resourcename);
    } else if($calltype == 'xmlrpc') {
	$wrapper->SetServer($server);
	$wrapper->SetPort($port);
	$wrapper->SetURI($uri);
	$wrapper->SetMethodName($functionname);
    } else if($calltype == 'soap') {
	$wrapper->SetServerPath($serverpath);
	$wrapper->SetURI($uri);
	$wrapper->SetMethodName($functionname);
    }

    if($taskname != $name) // name change
	$executableManager->MoveWrapper($layer,$taskname,$name);

    $controller->SaveFramework();
    $levels = explode("/",$name);
    $executablename = array_pop($levels);
    $tmpparentid = implode("/",$levels);
    $parentid = "EXECUTABLE/".$tmpparentid;

    $retEvent = new ACF_Event('executable_show');
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('executablename', $executablename));
    $parameterList->AddParameter(new ACF_DirectParameter('parentid', $parentid));
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}

function AH_RemoveExecutable(ACF_ParameterList $inParameterList) {
    $taskname = $inParameterList->ParameterValue('taskname');
    $answer = $inParameterList->ParameterValue('answer');

    if($answer == "No") {
	$levels = explode("/",$taskname);
	$executablename = array_pop($levels);
	$tmpparentid = implode("/",$levels);
	$parentid = "EXECUTABLE/".$tmpparentid;

	$retEvent = new ACF_Event('executable_show');
	$parameterList = new ACF_ParameterList();
	$parameterList->AddParameter(new ACF_DirectParameter('executablename', $executablename));
	$parameterList->AddParameter(new ACF_DirectParameter('parentid', $parentid));
	$retEvent->SetParameterList($parameterList);
	return $retEvent;
    } else {
	$layer = $_SESSION['layer'];
	$controller = GetCurrentController(true);
	$executableManager = $controller->ExecutableWrapperManager();
	$wrapper = $executableManager->FindWrapper($layer,$taskname);
	$executableManager->DeleteWrapper($wrapper,$layer);
	$controller->SaveFramework();
	return new ACF_Event('executables');
    }
}

function AH_WizardStoreName(ACF_ParameterList $inParameterList) {
    $name = $inParameterList->ParameterValue('name');
    $path= $inParameterList->ParameterValue('path');

    if($name)
	$_SESSION['executablewizard_name'] = $path.$name;
    else
	return new ACF_Event('executablewizard_error');

    return new ACF_Event('executablewizard_calltype_show');
}

function AH_WizardStoreCallType(ACF_ParameterList $inParameterList) {
    $calltype = $inParameterList->ParameterValue('calltype');

    if($calltype) {
	$_SESSION['executablewizard_calltype'] = $calltype;
	if($calltype == 'localphp')
	    return new ACF_Event('executablewizard_localphp_show');
	else if($calltype == 'xmlrpc')
	    return new ACF_Event('executablewizard_xmlrpc_success');
	else if($calltype == 'soap')
	    return new ACF_Event('executablewizard_soap_success');
    } else
	return new ACF_Event('executablewizard_error');
}

function AH_WizardStoreLocalPHP(ACF_ParameterList $inParameterList) {
    $resourcelocation = $inParameterList->ParameterValue('resourcelocation');
    $resourcename = $inParameterList->ParameterValue('resourcename');

    if($resourcelocation && $resourcename) {
	$_SESSION['executablewizard_resourcelocation'] = $resourcelocation;
	$_SESSION['executablewizard_resourcename'] = $resourcename;
	return new ACF_Event('executablewizard_parameters_show');
    } else
	return new ACF_Event('executablewizard_error');
}

function AH_WizardStoreXMLRPC(ACF_ParameterList $inParameterList) {
    $server = $inParameterList->ParameterValue('server');
    $port = $inParameterList->ParameterValue('port');
    $uri = $inParameterList->ParameterValue('uri');
    $functionname = $inParameterList->ParameterValue('functionname');

    if($server && $port && $uri && $functionname) {
	$_SESSION['executablewizard_server'] = $server;
	$_SESSION['executablewizard_port'] = $port;
	$_SESSION['executablewizard_uri'] = $uri;
	$_SESSION['executablewizard_functionname'] = $functionname;

	return new ACF_Event('executablewizard_storexmlrpc_success');
    } else
	return new ACF_Event('executablewizard_error');
}

function AH_WizardStoreSoap(ACF_ParameterList $inParameterList) {
    $serverpath = $inParameterList->ParameterValue('serverpath');
    $uri = $inParameterList->ParameterValue('uri');
    $functionname = $inParameterList->ParameterValue('functionname');

    if($serverpath && $uri && $functionname) {
	$_SESSION['executablewizard_serverpath'] = $serverpath;
	$_SESSION['executablewizard_uri'] = $uri;
	$_SESSION['executablewizard_functionname'] = $functionname;

	return new ACF_Event('executablewizard_storesoap_success');
    } else
	return new ACF_Event('executablewizard_error');
}

function AH_WizardStoreParameters(ACF_ParameterList $inParameterList) {
    $req_inparam = $inParameterList->ParameterValue('req_inparam');
    $opt_inparam = $inParameterList->ParameterValue('opt_inparam');
    $req_outparam = $inParameterList->ParameterValue('req_outparam');
    $opt_outparam = $inParameterList->ParameterValue('opt_outparam');

    $_SESSION['executablewizard_req_inparam'] = explode(",",$req_inparam);
    $_SESSION['executablewizard_opt_inparam'] = explode(",",$opt_inparam);
    $_SESSION['executablewizard_req_outparam'] = explode(",",$req_outparam);
    $_SESSION['executablewizard_opt_outparam'] = explode(",",$opt_outparam);
    return new ACF_Event('executablewizard_returntype_show');
}

function AH_WizardStoreReturnType(ACF_ParameterList $inParameterList) {
    $returntype = $inParameterList->ParameterValue('returntype');

    if($returntype == 'parameterlist') {
    
	// Create Wrapper
	if($_SESSION['executablewizard_calltype'] == 'localphp') {
	    $wrapper = new ACF_LocalPHPWrapper($_SESSION['executablewizard_name'], $returntype, $_SESSION['executablewizard_resourcelocation'], $_SESSION['executablewizard_resourcename']);
	}
	if($_SESSION['executablewizard_calltype'] == 'xmlrpc') {
	    $wrapper = new ACF_XMLRPCWrapper($_SESSION['executablewizard_name'], $returntype, $_SESSION['executablewizard_server'], $_SESSION['executablewizard_port'], $_SESSION['executablewizard_uri'], $_SESSION['executablewizard_functionname']);
	}
	if($_SESSION['executablewizard_calltype'] == 'soap') {
	    $wrapper = new ACF_SoapWrapper($_SESSION['executablewizard_name'], $returntype, $_SESSION['executablewizard_serverpath'], $_SESSION['executablewizard_uri'], $_SESSION['executablewizard_functionname']);
	}
	
	$wrapper->SetRequiredInputParameters($_SESSION['executablewizard_req_inparam']);
	$wrapper->SetOptionalInputParameters($_SESSION['executablewizard_opt_inparam']);
	$wrapper->SetRequiredOutputParameters($_SESSION['executablewizard_req_outparam']);
	$wrapper->SetOptionalOutputParameters($_SESSION['executablewizard_opt_outparam']);

	$layer = $_SESSION['layer'];
	$controller = GetCurrentController(true);
	$executableManager = $controller->ExecutableWrapperManager();
	$executableManager->AddExecutableWrapper($wrapper,$layer);

	$controller->SaveFramework();

	return new ACF_Event('executables');
    } else {
	$_SESSION['executablewizard_returntype'] = $returntype;
	return new ACF_Event('executablewizard_outputevents_show');
    }
}

function AH_WizardStoreOutputEvents(ACF_ParameterList $inParameterList) {
    $outputevents = explode(",",$inParameterList->ParameterValue('outputevents'));

    if($outputevents) {
	if($_SESSION['executablewizard_calltype'] == 'localphp') {
	    $wrapper = new ACF_LocalPHPWrapper($_SESSION['executablewizard_name'], $_SESSION['executablewizard_returntype'], $_SESSION['executablewizard_resourcelocation'], $_SESSION['executablewizard_resourcename']);
	}
	if($_SESSION['executablewizard_calltype'] == 'xmlrpc') {
	    $wrapper = new ACF_XMLRPCWrapper($_SESSION['executablewizard_name'], $_SESSION['executablewizard_returntype'], $_SESSION['executablewizard_server'], $_SESSION['executablewizard_port'], $_SESSION['executablewizard_uri'], $_SESSION['executablewizard_functionname']);
	}
	if($_SESSION['executablewizard_calltype'] == 'soap') {
	    $wrapper = new ACF_SoapWrapper($_SESSION['executablewizard_name'], $_SESSION['executablewizard_returntype'], $_SESSION['executablewizard_serverpath'], $_SESSION['executablewizard_uri'], $_SESSION['executablewizard_functionname']);
	}

	$wrapper->SetRequiredInputParameters($_SESSION['executablewizard_req_inparam']);
	$wrapper->SetOptionalInputParameters($_SESSION['executablewizard_opt_inparam']);
	$wrapper->SetRequiredOutputParameters($_SESSION['executablewizard_req_outparam']);
	$wrapper->SetOptionalOutputParameters($_SESSION['executablewizard_opt_outparam']);

	$wrapper->SetOutputEvents($outputevents);

	$layer = $_SESSION['layer'];
	$controller = GetCurrentController(true);
	$executableManager = $controller->ExecutableWrapperManager();
	$executableManager->AddExecutableWrapper($wrapper,$layer);

	$controller->SaveFramework();
	return new ACF_Event('executables');
    } else {
	return new ACF_Event('executablewizard_error');
    }
}
?>
