<?php
require_once('acf_refresh.inc.php');

function AH_MoveUp(ACF_ParameterList $inParameterList) {
    $dataname = $inParameterList->ParameterValue('dataname');
    $datatype = $inParameterList->ParameterValue('datatype');
    $invocname = $inParameterList->ParameterValue('invocname');
    if ($inParameterList->ParameterValue('realm')) {
        $realm = $inParameterList->ParameterValue('realm');
    }
    if ($inParameterList->ParameterValue('source')) {
        $source = $inParameterList->ParameterValue('source');
    }
    $parentid = $inParameterList->ParameterValue('parentid');
    $layer = $_SESSION['layer'];

    $retParameterList = new ACF_ParameterList();
    $retParameterList->AddParameter(new ACF_DirectParameter('parentid', $parentid));

    $controller = GetCurrentController(true);
    switch($datatype) {
        case 'event' :
            $processManager = $controller->ProcessManager();
            $process = $processManager->FindResponse($layer,$realm,$source,$dataname);
            $invocationsList = $process->InvocationList();
            $invocationsList->MoveInvocationOrder($invocname,'up');

            $retEvent = new ACF_Event('moveupEvent_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('eventname', $dataname));

            break;
        case 'view' :
            $viewDefManager = $controller->ViewDefinitionManager();
	    if($parentid)
		$viewDef = $viewDefManager->FindDefinition($layer,$parentid."/".$dataname);
	    else
		$viewDef = $viewDefManager->FindDefinition($layer,$dataname);
            $dataCollectors = $viewDef->DataCollectors();
            $dataCollectors->MoveInvocationOrder($invocname,'up');

            $retEvent = new ACF_Event('moveupView_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('viewname', $dataname));

            break;
        default:
            throw new ACF_Exception("Unrecognized type for datatype ' in database: '$datatype'");
    }

    $controller->SaveFramework();

    $retEvent->SetParameterList($retParameterList);

    return $retEvent;
}

function AH_MoveDown(ACF_ParameterList $inParameterList) {
    $dataname = $inParameterList->ParameterValue('dataname');
    $datatype = $inParameterList->ParameterValue('datatype');
    $invocname = $inParameterList->ParameterValue('invocname');
    if ($inParameterList->ParameterValue('realm')) {
        $realm = $inParameterList->ParameterValue('realm');
    }
    if ($inParameterList->ParameterValue('source')) {
        $source = $inParameterList->ParameterValue('source');
    }
    $parentid = $inParameterList->ParameterValue('parentid');
    $layer = $_SESSION['layer'];

    $retParameterList = new ACF_ParameterList();
    $retParameterList->AddParameter(new ACF_DirectParameter('parentid', $parentid));

    $controller = GetCurrentController(true);
    switch($datatype) {
        case 'event' :
            $processManager = $controller->ProcessManager();
            $process = $processManager->FindResponse($layer,$realm,$source,$dataname);
            $invocationsList = $process->InvocationList();
            $invocationsList->MoveInvocationOrder($invocname,'down');

            $retEvent = new ACF_Event('movedownEvent_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('eventname', $dataname));

            break;
        case 'view' :
            $viewDefManager = $controller->ViewDefinitionManager();
	    if($parentid)
		$viewDef = $viewDefManager->FindDefinition($layer,$parentid."/".$dataname);
	    else
		$viewDef = $viewDefManager->FindDefinition($layer,$dataname);
            $dataCollectors = $viewDef->DataCollectors();
            $dataCollectors->MoveInvocationOrder($invocname,'down');

            $retEvent = new ACF_Event('movedownView_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('viewname', $dataname));

            break;
        default:
            throw new ACF_Exception("Unrecognized type for datatype ' in database: '$datatype'");
    }

    $controller->SaveFramework();

    $retEvent->SetParameterList($retParameterList);

    return $retEvent;
}

function AH_InsertInvocation(ACF_ParameterList $inParameterList) {
    $dataname = $inParameterList->ParameterValue('dataname');
    $datatype = $inParameterList->ParameterValue('datatype');
    $invoctype = $inParameterList->ParameterValue('invoctype');
    $invocname = $inParameterList->ParameterValue('invocname');
    if ($inParameterList->ParameterValue('realm')) {
        $realm = $inParameterList->ParameterValue('realm');
    }
    if ($inParameterList->ParameterValue('source')) {
        $source = $inParameterList->ParameterValue('source');
    }
    $parentid = $inParameterList->ParameterValue('parentid');
    $layer = $_SESSION['layer'];

    $retParameterList = new ACF_ParameterList();
    $retParameterList->AddParameter(new ACF_DirectParameter('parentid', $parentid));

    $controller = GetCurrentController(true);
    $parameterList = $controller->CreateParameterList();

    switch($invoctype) {
        case 'view' :
            $invocation = new ACF_ViewInvocation($invocname, $parameterList);
            break;
        case 'activity' :
            $invocation = new ACF_ActivityInvocation($invocname, $parameterList);
            break;
        case 'datacollector' :
            $invocation = new ACF_DataCollectorInvocation($invocname, $parameterList);
            break;
        default:
            throw new ACF_Exception("Unrecognized type for invocation $invocname' in database: '$invoctype'");
    }

    switch($datatype) {
        case 'event' :
            $processManager = $controller->ProcessManager();
            $process = $processManager->FindResponse($layer,$realm,$source,$dataname);
            $invocationsList = $process->InvocationList();
            $invocationsList->AddInvocation($invocation);
    
            $retEvent = new ACF_Event('insertinvocationEvent_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('eventname', $dataname));

            break;
        case 'view' :
            $viewDefManager = $controller->ViewDefinitionManager();
	    if($parentid)
		$viewDef = $viewDefManager->FindDefinition($layer,$parentid."/".$dataname);
	    else
		$viewDef = $viewDefManager->FindDefinition($layer,$dataname);
            $dataCollectors = $viewDef->DataCollectors();
            $dataCollectors->AddInvocation($invocation);

            $retEvent = new ACF_Event('insertinvocationView_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('viewname', $dataname));

            break;
        default:
            throw new ACF_Exception("Unrecognized type for datatype ' in database: '$datatype'");
    }

    $controller->SaveFramework();

    $retEvent->SetParameterList($retParameterList);

    return $retEvent;
}

function AH_InsertParameter(ACF_ParameterList $inParameterList) {
    $dataname = $inParameterList->ParameterValue('dataname');
    $datatype = $inParameterList->ParameterValue('datatype');
    $invocname = $inParameterList->ParameterValue('invocname');
    if ($inParameterList->ParameterValue('realm')) {
        $realm = $inParameterList->ParameterValue('realm');
    }
    if ($inParameterList->ParameterValue('source')) {
        $source = $inParameterList->ParameterValue('source');
    }
    $parentid = $inParameterList->ParameterValue('parentid');
    $paramtype = $inParameterList->ParameterValue('paramtype');
    $paramname = $inParameterList->ParameterValue('paramname');
    $paramentry = $inParameterList->ParameterValue('paramentry');
    $paramoption = $inParameterList->ParameterValue('paramoption');
    $layer = $_SESSION['layer'];

    $retParameterList = new ACF_ParameterList();
    $retParameterList->AddParameter(new ACF_DirectParameter('parentid', $parentid));

    $controller = GetCurrentController(true);

    switch ($paramtype) {
        case 'direct' :
            $newparam = new ACF_DirectParameter($paramname,$paramentry);
            break;
        case 'indirect' :
            $newparam = new ACF_IndirectParameter($paramname,$paramentry,$paramoption); // last param is optional flag
            break;
        default:
            throw new ACF_Exception("Unrecognized type for parameter $paramname' in database: '$paramtype'");
        }

    switch($datatype) {
        case 'event' :
            $processManager = $controller->ProcessManager();
            $process = $processManager->FindResponse($layer,$realm,$source,$dataname);
            $invocationsList = $process->InvocationList();
            $invocation = $invocationsList->FindInvocation($invocname);
            $parameterList = $invocation->ParameterList();
            $parameterList->AddParameter($newparam);

            $retEvent = new ACF_Event('insertparameterEvent_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('eventname', $dataname));

            break;
        case 'view' :
            $viewDefManager = $controller->ViewDefinitionManager();
	    if($parentid)
		$viewDef = $viewDefManager->FindDefinition($layer,$parentid."/".$dataname);
	    else
		$viewDef = $viewDefManager->FindDefinition($layer,$dataname);
            $dataCollectors = $viewDef->DataCollectors();
            $dataCollector = $dataCollectors->FindInvocation($invocname);
            $parameterList = $dataCollector->ParameterList();
            $parameterList->AddParameter($newparam);

            $retEvent = new ACF_Event('insertparameterView_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('viewname', $dataname));

            break;
        default:
            throw new ACF_Exception("Unrecognized type for datatype ' in database: '$datatype'");
    }

    $controller->SaveFramework();

    $retEvent->SetParameterList($retParameterList);

    return $retEvent;
}

function AH_RemoveInvocation(ACF_ParameterList $inParameterList) {
    $dataname = $inParameterList->ParameterValue('dataname');
    $datatype = $inParameterList->ParameterValue('datatype');
    $invocname = $inParameterList->ParameterValue('invocname');
    if ($inParameterList->ParameterValue('realm')) {
        $realm = $inParameterList->ParameterValue('realm');
    }
    if ($inParameterList->ParameterValue('source')) {
        $source = $inParameterList->ParameterValue('source');
    }
    $parentid = $inParameterList->ParameterValue('parentid');

    $retParameterList = new ACF_ParameterList();
    $retParameterList->AddParameter(new ACF_DirectParameter('parentid', $parentid));

    $controller = GetCurrentController(true);
    $layer = $_SESSION['layer'];

    switch($datatype) {
        case 'event' :
            $processManager = $controller->ProcessManager();
            $process = $processManager->FindResponse($layer,$realm,$source,$dataname);
            $invocationsList = $process->InvocationList();
            $invocationsList->RemoveInvocation($invocname);
    
            $retEvent = new ACF_Event('removeinvocationEvent_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('eventname', $dataname));

            break;
        case 'view' :
            $viewDefManager = $controller->ViewDefinitionManager();
	    if($parentid)
		$viewDef = $viewDefManager->FindDefinition($layer,$parentid."/".$dataname);
	    else
		$viewDef = $viewDefManager->FindDefinition($layer,$dataname);
            $dataCollectors = $viewDef->DataCollectors();
            $dataCollectors->RemoveInvocation($invocname);

            $retEvent = new ACF_Event('removeinvocationView_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('viewname', $dataname));

            break;
        default:
            throw new ACF_Exception("Unrecognized type for datatype ' in database: '$datatype'");
    }

    $controller->SaveFramework();

    $retEvent->SetParameterList($retParameterList);

    return $retEvent;
}

function AH_RemoveParameter(ACF_ParameterList $inParameterList) {
    $datatype = $inParameterList->ParameterValue('datatype');
    $dataname = $inParameterList->ParameterValue('dataname');
    $paramname = $inParameterList->ParameterValue('paramname');
    $parentid = $inParameterList->ParameterValue('parentid');
    $invocname = $inParameterList->ParameterValue('invocname');
    if ($inParameterList->ParameterValue('realm')) {
        $realm = $inParameterList->ParameterValue('realm');
    }
    if ($inParameterList->ParameterValue('source')) {
        $source = $inParameterList->ParameterValue('source');
    }

    $controller = GetCurrentController(true);
    $layer = $_SESSION['layer'];
    $retParameterList = new ACF_ParameterList();

    switch($datatype) {
        case 'event' :
            $processManager = $controller->ProcessManager();
            $process = $processManager->FindResponse($layer,$realm,$source,$dataname);
            $invocationsList = $process->InvocationList();
            $invocation = $invocationsList->FindInvocation($invocname);
            $parameterList = $invocation->ParameterList();
            $parameterList->DeleteParameterName($paramname);

            $retEvent = new ACF_Event('removeparameterEvent_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('eventname', $dataname));

            break;
        case 'view' :
            $viewDefManager = $controller->ViewDefinitionManager();
	    if($parentid)
		$viewDef = $viewDefManager->FindDefinition($layer,$parentid."/".$dataname);
	    else
		$viewDef = $viewDefManager->FindDefinition($layer,$dataname);
            $dataCollectors = $viewDef->DataCollectors();
            $dataCollector = $dataCollectors->FindInvocation($invocname);
            $parameterList = $dataCollector->ParameterList();
            $parameterList->DeleteParameterName($paramname);

            $retEvent = new ACF_Event('removeparameterView_Success');
            $retParameterList->AddParameter(new ACF_DirectParameter('viewname', $dataname));

            break;
        default :
            throw new ACF_Exception("Unrecognized type for datatype ' in database: '$datatype'");
    }

    $controller->SaveFramework();

    $retEvent->SetParameterList($retParameterList);

    return $retEvent;
}

?>
