<?php

/*---------------------------------------------------------------------------------------------------------*/
/*---- This file has performs the logging in and logging out of the knowledge base pages               ----*/
/*---- These functions are invoked from the acf framework                                              ----*/
/*---------------------------------------------------------------------------------------------------------*/

require_once('../../libs/platypus/wow.inc.php');
require_once("../../libs/ccs/options.inc.php");

//---- load the CCS Options have to laod it here because this is the first manditory function call
if (!$_SESSION['ccs'])
    loadCCSOptions();

function AH_CheckEntry (ACF_ParameterList $inParameterList) {
    if ($inParameterList->ParameterValue('logintype') == "staff")
        return new ACF_Event("checkEntry_Staff");
    else
        return new ACF_Event("checkEntry_Default");
}

function AH_CheckLogin (ACF_ParameterList $inParameterList) {
    if (trim($inParameterList->ParameterValue('username'))) {
        $retEvent = new ACF_Event("DoLogin");
        $retEvent->SetParameterList($inParameterList);
    } else
        $retEvent = new ACF_Event("NoLogin");
    return $retEvent;
}

function AH_ProcessLogin (ACF_ParameterList $inParameterList) {
//---- grab the parameters that are passed in and assign them to a local variable for easy reference
    $username = trim($inParameterList->ParameterValue('username'));
    $password = trim($inParameterList->ParameterValue('password'));
    $status   = array();
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
    $_SESSION['knowledgebase']['username'] = '';
    $_SESSION['knowledgebase']['password'] = '';
    $_SESSION['knowledgebase']['status'] = array();
//---- assign values to the helpdesk object
    $helpdesk->setMethod('Login');
    $helpdesk->addAttribute('logintype', 'staff');
    $helpdesk->addParameter('cUsername', $username);
    $helpdesk->addParameter('cPassword', $password);
    if ($helpdesk->execute()) {
        $_SESSION['knowledgebase']['username'] = $username;
        $_SESSION['knowledgebase']['password'] = $password;
        $eventName = 'ProcessLogin_Success';
    } else {
        $status['code'] = 'LOGIN_ERROR';
        $status['text'] = $helpdesk->lastError;
        $eventName = 'ProcessLogin_Failure';
    }
    $parameterList = new ACF_ParameterList();
//---- override if in the client
    if ($inParameterList->ParameterValue('type') && $inParameterList->ParameterValue('id')) {
        if (strtolower($inParameterList->ParameterValue('type')) == 'article')
            $eventName = 'ProcessLogin_Article';
        else
            $eventName = 'ProcessLogin_Issue';
        $parameterList->AddParameter(new ACF_DirectParameter('id', $inParameterList->ParameterValue('id')));
    }
    $retEvent = new ACF_Event($eventName);
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}

function AH_ProcessLogout (ACF_ParameterList $inParameterList) {
    $status   = array();
//---- reset the session variables
    $_SESSION['knowledgebase']['username'] = '';
    $_SESSION['knowledgebase']['password'] = '';
    $eventName = 'ProcessLogout_Success';
    $retEvent = new ACF_Event($eventName);
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}

function AH_SetCookieData (ACF_ParameterList $inParameterList) {
    $type = $inParameterList->ParameterValue('type');
    $id   = $inParameterList->ParameterValue('id');
    if (!is_numeric($id)) {
        return new ACF_Event('SetCookie_Failure');
    }
    $newCookie = $type . "|" . $id;
    $cookie = array();
    if ($_COOKIE['kb_mru']) {
        $cookie = $_COOKIE['kb_mru'];
//---- cookies are not pulled in order
        ksort($cookie);
//---- if this item was clicked before remove it from the list so it can be added to the top
        foreach ($cookie as $index => $value)
            if ($value == $newCookie) {
                unset($cookie[$index]);
                break;
            }
    }
    array_unshift($cookie, $newCookie);
//---- write the cookies to expires in 120 days and only write the first 10
    foreach ($cookie as $index => $value) {
        if ($index < 10)
            setcookie("kb_mru[$index]", $value, time()+60*60*24*120);
    }
    return new ACF_Event('SetCookie_Success');
}

?>
