<?php

class Node{
    public $name     = null;
    public $value    = null;
    public $previous = null;
    public $next     = null;
    public $child    = null;

    function __construct($value = null, $name = ''){
        $this->name  = $name;
        $this->value = $value;
    }
}


class Tree{
    public $name        = '';
    private $firstNode  = null;
    private $lastNode   = null;
    public $currentNode = null;
    public $parent      = null;

    function __construct($name = 'root'){$this->name = $name;}

//---- node stuff
//---- functions to move around in the tree
    function moveFirst(){$this->currentNode = $this->firstNode;}

    function moveNext(){$this->currentNode = $this->currentNode->next;}

    function movePrevious(){$this->currentNode = $this->currentNode->previous;}

    function moveLast(){$this->currentNode = $this->lastNode;}

//---- functions to insert new nodes in the tree
    function insertAfter($value = null, $name = null){
//---- create a new node
        $node = new Node($value, $name);
        if (!$this->currentNode){
//---- no nodes exists so create the first one
            $this->firstNode = $node;
            $this->lastNode  = $this->firstNode;
        }else
            if ($this->currentNode->next){
//---- if the current node is not the last node insert the new node after the current
                $tail = $this->currentNode->next;
                $this->currentNode->next = $node;
                $node->next = $tail;
                $tail->previous = $node;
            }else{
//---- if the current node is the last node append the new node to the end of the link list
                $this->currentNode->next = $node;
                $this->lastNode = $node;
            }
//---- close the last link
        $node->previous = $this->currentNode;
//---- set the current node to the new node
        $this->currentNode = $node;
    }


    function insertBefore($value = null, $name = null){
//---- create a new node
        $node = new Node($value, $name);
        if (!$this->currentNode){
//---- no nodes exists so create the first one
            $this->firstNode = $node;
            $this->lastNode  = $this->firstNode;
        }else
            if ($this->currentNode->previous){
//---- if the current node is not the first node insert the new node infront of the current
                $head = $this->currentNode->previous;
                $this->currentNode->previous = $node;
                $node->previous = $head;
                $head->next = $node;
            }else{
//---- if the current node is the first node throw the new node in front
                $this->currentNode->previous = $node;
                $this->firstNode = $node;
            }
//---- close the last link
        $node->next = $this->currentNode;
//---- set the current node to the new node
        $this->currentNode = $node;
    }


    function insertHead($value = null, $name = null){
//---- create a new node
        $node = new Node($value, $name);
//---- if the last node does not exists use this one
        if (!$this->lastNode)
            $this->lastNode = $node;
//---- throw the new node at the beginning of the list
        $node->next                = $this->firstNode;
        $this->firstNode->previous = $node;
        $this->firstNode           = $node;
        $this->currentNode         = $node;
    }


    function insertTail($value = null, $name = null){
//---- create a new node
        $node = new Node($value, $name);
//---- if the first node does not exists use this one
        if (!$this->firstNode)
            $this->firstNode = $node;
//---- throw the new node at the end of the list
        $node->previous       = $this->lastNode;
        $this->lastNode->next = $node;
        $this->lastNode       = $node;
        $this->currentNode    = $node;
    }


//---- functions to remove nodes
    function removeNode(){
        $head = $this->currentNode->previous;
        $tail = $this->currentNode->next;
//---- relink the nodes that are before and after the current node
        if ($head)
            $head->next = $tail;
        if ($tail)
            $tail->previous = $head;
//---- move the current node to the next node if it exist otherwise set it to the previous
//---- if the current node has a child kill it
        if ($this->currentNode->child)
            $this->removeChild();
        $current = $this->currentNode;
        $this->currentNode = $tail ? $tail : $head;
//---- move the node pointers  to their new locations
        if (!$head)
            $this->firstNode = $tail;
        if (!$tail)
            $this->lastNode = $head;
//---- kill this node
        unset($current);
    }


//---- tree stuff
    function insertChild($value=null, $nodeName=null, $treeName=null){
//---- if the child already exist append the new node to the end
        if ($this->currentNode->child)
            $this->currentNode->child->insertTail($value, $nodeName);
        else{
//---- create a new tree
            $this->currentNode->child = new Tree($treeName);
            $this->currentNode->child->insertHead($value, $nodeName);
//---- the link back to the parent cuase an explosion!!!
//---- but without this link back the remove parent function will not work!!!
//            $this->currentNode->child->parent = &$this;
        }
    }


//---- functions to remove tree
    function removeTree(){
        if ($this->parent){
//---- grab a reference to the parent
            $parent = $this->parent;
            $this->parent = null;
            $parent->moveFirst();
//---- hunt down the child and sever it
            while ($parent->currentNode){
                if ($parent->currentNode->child === $this){
                    $parent->currentNode->child = null;
                    break;
                }
                $parent->moveNext();
            }
        }
        unset($this);
    }


    function removeChild(){
//---- grab a reference to the child
        $child = $this->currentNode->child;
//---- kill the reference to the child
        $this->currentNode->child = null;
//---- kill the child
        unset($child);
    }


    function removeParent(){
        if ($this->parent){
//---- get a reference to the parent
            $parent = $this->parent;
            $this->parent = null;
            $parent->moveFirst();
//---- hunt down the child and sever it
            while ($parent->currentNode){
                if ($parent->currentNode->child === $this){
                    $parent->currentNode->child = null;
                    break;
                }
                $parent->moveNext();
            }
//---- now go to the root
            while ($parent->parent){
                $parent = $parent->parent;
            }
//---- kill off the root and it's branches
            unset($parent);
        }
    }


//---- Misc functions
    function getParentPath(){
        $parent = $this;
        $path   = $this->currentNode->name;
        while ($parent){
            $path = $parent->name . "/" . $path;
            $parent = $parent->parent;
        }
        return $path;
    }


    function locateNode($path = ''){
//---- we do not want to modify the current tree so we dup it in memory
	$tree = $this;
	$index = explode("/", urldecode($path));
	$tree->moveFirst();
	for ($i = 1; $tree->currentNode; $i++){
	    if ($i == count($index) - 1){
//---- we found the right branch time to locate the node
		$tree->moveFirst();
		while($tree->currentNode){
		    if ($tree->currentNode->name == $index[$i]){
			break;
		    }
		    $tree->moveNext();
		}
		break;
	    }
	    $tree->moveFirst();
//---- search for the right branch
	    while($tree->currentNode){
		if ($tree->currentNode->child->name == $index[$i]){
		    $tree = $tree->currentNode->child;
		    $tree->moveFirst();
		    break;
		}
		$tree->moveNext();
	    }
	}
	$this->currentNode = $tree->currentNode;
    }
}
?>
