<?php
	function DC_GetTemplateStatus(ACF_ParameterList $inParameterList) {

		// This function is used by all modules to find the current status of each view
		$step = $inParameterList->ParameterValue('step');
		$source = $inParameterList->ParameterValue('source');

		$status = array();

		// Set Source (PIWI, CCS LANDING (default), etc..)
		if($source && (!$_SESSION['ccs']['source'] || $_SESSION['ccs']['source'] != 'piwi' )) // Passed in (can't override piwi)
			$_SESSION['ccs']['source'] = $source;

		$status['source'] = $_SESSION['ccs']['source'];

		// Set blog Caption
		$status['blogcaption'] = $_SESSION['ccs']['options']['driver-options']['service-modules']['blogware']['caption'];

		if(!$status['source'])
			$status['source'] = 'default';

		// Add Store ID
		if($status['source'] == 'piwi') {
			$status['serviceOverride'] = $_SESSION['piwi']['currentCRID'] ? true : false;
			$status['platypus'] = array('storeid' => $_SESSION['piwi']['storeid']);
		} else if($status['source'] == 'platsignup') {
		    $status['platypus'] = array('storeid' => $_SESSION['signup']['storeid']);
		    $_SESSION['modules']['billing']['storeid'] = $_SESSION['signup']['storeid'];
		} else if(isset($_SESSION['modules']['billing']['storeid']))
			$status['platypus'] = array('storeid' => $_SESSION['modules']['billing']['storeid']);
		else if(isset($_SESSION['modules']['billing']['passed']['storeid']))
			$status['platypus'] = array('storeid' => $_SESSION['modules']['billing']['passed']['storeid']);

		// Set Bundled Flag
		$status['isBundled'] = $_SESSION['ccs']['bundled'];

		// Set Bundle Info
		$status['bundle'] = $_SESSION['modules']['billing']['bundle'];

		// Set Service Type
		$status['servicetype'] = $_SESSION['modules']['requiredService'];
		$status['task'] = $_SESSION['modules'][$status['servicetype']]['task'];
		$status['alternate'] = $_SESSION['modules'][$status['servicetype']]['alternate'];

		// Get Blogware Options
		if($status['servicetype'] == 'blogware') {
			$status['blogname'] = $_SESSION['modules']['blogware']['blogname'];
			$status['blogid'] = $_SESSION['modules']['blogware']['currentBlog'];
			$status['hostname'] = $_SESSION['modules']['blogware']['hostname'];
			$status['user'] = $_SESSION['modules']['blogware']['user']['username'];
		    // Template Title
		    $status['title'] = $status['blogcaption'].': '.$status['hostname'];
		}

		$status['blogwareItem'] = $_SESSION['modules']['blogware']['chargeaction'];

		// Copy PIWI Header Info
		$status['custdata'] = $_SESSION['modules']['shared']['custdata'];

		// Copy PIWI Service List Info
		$status['serviceList'] = $_SESSION['modules']['shared']['serviceList'];
		$status['serviceTree'] = $_SESSION['modules']['shared']['serviceTree'];
		$status['serviceCommands'] = $_SESSION['modules']['shared']['serviceCommands'];

		// Get Skipped Steps
		if($_SESSION['ccs']['excludeSteps']) {
			$status['excludeSteps'] = array();
			foreach($_SESSION['ccs']['excludeSteps'] as $excludestep) {
				$status['excludeSteps'][$excludestep] = true;
			}
		}
		/********************************************
		Available Steps ----
		checkavail - Service Check Availability Step
		user - Select User Step (login/new screen)
		configure - Service Configuration Step
		contacts - Domain Contacts Configuration Step
		payment - Payment Step (Enter CC Info Screen)
		preview - Preview Step
		********************************************/

		// Get Service URL
		$status['serviceurl'] = $_SESSION['ccs']['options']['driver-options']['service-modules'][$status['servicetype']]['url'];
		$status['billingurl'] = $_SESSION['ccs']['options']['driver-options']['billing-module']['url'];
		$status['driverurl'] = $_SESSION['ccs']['options']['driver-options']['url'];

		// Get Domain Options
		if($status['servicetype'] == 'domain') {
			if ($_SESSION['modules']['shared']['user'])
			    $status['user'] = $_SESSION['modules']['shared']['user'];

			// Just get the first one
			if($_SESSION['modules']['domain']['register']['registerDomain']) {
				$basename = "";
				$domainlist = array();
				foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domainname=>$domain) {
					$basename = $domain['baseName'];
					array_push($domainlist,substr($domain['tld'],1));
				}
				$status['domainname'] = "www.".$basename.".".implode("; ",$domainlist);

				// Adjust caption length
				if(strlen($status['domainname']) > 70) {
					$status['domainname'] = substr($status['domainname'],0, 67)."...";
				}
			}

			if (!$status['domainname'] && is_array($_SESSION['modules']['domain']))
				if (array_key_exists('register-status',$_SESSION['modules']['domain']))
					$status['domainname'] = key($_SESSION['modules']['domain']['register-status']);

			if(!$status['domainname'])
				$status['domainname'] = $_SESSION['modules']['domain']['passed']['domain'];

			if(!$status['domainname'])
				$status['domainname'] = $_SESSION['modules']['shared']['domain'];

		    // Template Title
		    $status['title'] = 'Domain: '.$status['domainname'];
		}

		// Setting Step
		if(!$_SESSION['ccs']['knownSteps']) {
			$_SESSION['ccs']['knownSteps'] = array();
			$_SESSION['ccs']['maxStep'] = 0;
		}

		if($step && !in_array($step,$_SESSION['ccs']['knownSteps'])) { // New Step
			array_push($_SESSION['ccs']['knownSteps'],$step);
			$_SESSION['ccs']['maxStep']++;
		}

		$status['maxstep'] = $_SESSION['ccs']['maxStep'];
		if($_SESSION['GST'])
			$status['GST'] = $_SESSION['GST'];

		$paramList = new ACF_ParameterList();
		$paramList->AddParameter(new ACF_DirectParameter('templatestatus',$status));

		return $paramList;
	}

	function DC_GetServiceStatus(ACF_ParameterList $inParameterList) {
		$status = $_SESSION['modules'][$_SESSION['modules']['requiredService']]['status'];
		if($status)
			unset($_SESSION['modules'][$_SESSION['modules']['requiredService']]['status']);

		$paramList = new ACF_ParameterList();
		$paramList->AddParameter(new ACF_DirectParameter('status',$status));
		return $paramList;
	}

	function DC_GetSystemStatus(ACF_ParameterList $inParameterList) {
	    $status = $_SESSION['modules']['shared']['status'];
	    if($status)
		unset($_SESSION['modules']['shared']['status']);

	    $paramList = new ACF_ParameterList();
	    $paramList->AddParameter(new ACF_DirectParameter('status',$status));
	    return $paramList;
	}

function DC_Modules_GetCountriesList() {

    if (file_exists('../../etc/countrieslist.xml')) {
        $countryxml = simplexml_load_file('../../etc/countrieslist.xml');
        $countries = array();
        if($countryxml)
        foreach($countryxml as $country)
            $countries[(string) $country->code] = (string) $country->name;
    }

    if (!$countries) {
        $countries = array(
            "US" => "United States of America",
            "CA" => "Canada",
            "AF" => "Afghanistan",
            "AL" => "Albania",
            "DZ" => "Algeria",
            "AS" => "American Samoa",
            "AD" => "Andorra",
            "AI" => "Anguilla",
            "AQ" => "Antarctica",
            "AG" => "Antigua And Barbuda",
            "AR" => "Argentina",
            "AM" => "Armenia",
            "AW" => "Aruba",
            "AU" => "Australia",
            "AT" => "Austria",
            "AZ" => "Azerbaijan",
            "BS" => "Bahamas",
            "BH" => "Bahrain",
            "BD" => "Bangladesh",
            "BB" => "Barbados",
            "BY" => "Belarus",
            "BY" => "Belarus",
            "BE" => "Belgium",
            "BZ" => "Belize",
            "BJ" => "Benin",
            "BM" => "Bermuda",
            "BT" => "Bhutan",
            "BO" => "Bolivia",
            "BA" => "Bosnia And Herzegowina",
            "BW" => "Botswana",
            "BV" => "Bouvet Island",
            "BR" => "Brazil",
            "IO" => "British Indian Ocean Territory",
            "BN" => "Brunei Darussalam",
            "BG" => "Bulgaria",
            "BF" => "Burkina Faso",
            "BI" => "Burundi",
            "KH" => "Cambodia",
            "CM" => "Cameroon",
            "CV" => "Cape Verde",
            "KY" => "Cayman Islands",
            "CF" => "Central African Republic",
            "TD" => "Chad",
            "CL" => "Chile",
            "CN" => "China",
            "CX" => "Christmas Island",
            "CC" => "Cocos (Keeling) Islands",
            "CO" => "Colombia",
            "KM" => "Comoros",
            "CG" => "Congo",
            "CK" => "Cook Islands",
            "CR" => "Costa Rica",
            "CI" => "Cote D'ivoire",
            "HR" => "Croatia",
            "CY" => "Cyprus",
            "CZ" => "Czech Republic",
            "DK" => "Denmark",
            "DJ" => "Djibouti",
            "DM" => "Dominica",
            "DO" => "Dominican Republic",
            "TP" => "East Timor",
            "EC" => "Ecuador",
            "EG" => "Egypt",
            "SV" => "El Salvador",
            "GQ" => "Equatorial Guinea",
            "ER" => "Eritrea",
            "EE" => "Estonia",
            "ET" => "Ethiopia",
            "FK" => "Falkland Islands",
            "FO" => "Faroe Islands",
            "FJ" => "Fiji",
            "FI" => "Finland",
            "FR" => "France",
            "GF" => "French Guiana",
            "PF" => "French Polynesia",
            "TF" => "French Southern Territories",
            "GA" => "Gabon",
            "GM" => "Gambia",
            "GE" => "Georgia",
            "DE" => "Germany",
            "GH" => "Ghana",
            "GI" => "Gibraltar",
            "GR" => "Greece",
            "GL" => "Greenland",
            "GD" => "Grenada",
            "GP" => "Guadeloupe",
            "GU" => "Guam",
            "GT" => "Guatemala",
            "GN" => "Guinea",
            "GW" => "Guinea-Bissau",
            "GY" => "Guyana",
            "HT" => "Haiti",
            "HM" => "Heard And Mcdonald Islands",
            "HN" => "Honduras",
            "HK" => "Hong Kong",
            "HU" => "Hungary",
            "IS" => "Iceland",
            "IN" => "India",
            "ID" => "Indonesia",
            "IE" => "Ireland",
            "IL" => "Israel",
            "IT" => "Italy",
            "JM" => "Jamaica",
            "JP" => "Japan",
            "JO" => "Jordan",
            "KZ" => "Kazakstan",
            "KE" => "Kenya",
            "KI" => "Kiribati",
            "KR" => "Korea, Republic Of (South)",
            "KW" => "Kuwait",
            "KG" => "Kyrgyzstan",
            "LA" => "Lao People's Democratic Republic",
            "LV" => "Latvia",
            "LB" => "Lebanon",
            "LS" => "Lesotho",
            "LR" => "Liberia",
            "LI" => "Liechtenstein",
            "LT" => "Lithuania",
            "LU" => "Luxembourg",
            "MO" => "Macau",
            "MK" => "Macedonia",
            "MG" => "Madagascar",
            "MW" => "Malawi",
            "MY" => "Malaysia",
            "MV" => "Maldives",
            "ML" => "Mali",
            "MT" => "Malta",
            "MH" => "Marshall Islands",
            "MQ" => "Martinique",
            "MR" => "Mauritania",
            "MU" => "Mauritius",
            "YT" => "Mayotte",
            "MX" => "Mexico",
            "FM" => "Micronesia",
            "MD" => "Moldova, Republic Of",
            "MC" => "Monaco",
            "MN" => "Mongolia",
            "MS" => "Montserrat",
            "MA" => "Morocco",
            "MZ" => "Mozambique",
            "NA" => "Namibia",
            "NR" => "Nauru",
            "NP" => "Nepal",
            "NL" => "Netherlands",
            "AN" => "Netherlands Antilles",
            "NC" => "New Caledonia",
            "NZ" => "New Zealand",
            "NI" => "Nicaragua",
            "NE" => "Niger",
            "NG" => "Nigeria",
            "NU" => "Niue",
            "NF" => "Norfolk Island",
            "MP" => "Northern Mariana Islands",
            "NO" => "Norway",
            "OM" => "Oman",
            "PK" => "Pakistan",
            "PW" => "Palau",
            "PA" => "Panama",
            "PG" => "Papua New Guinea",
            "PY" => "Paraguay",
            "PE" => "Peru",
            "PH" => "Philippines",
            "PN" => "Pitcairn",
            "PL" => "Poland",
            "PT" => "Portugal",
            "PR" => "Puerto Rico",
            "QA" => "Qatar",
            "RE" => "Reunion",
            "RO" => "Romania",
            "RU" => "Russian Federation",
            "RW" => "Rwanda",
            "SH" => "Saint Helena",
            "KN" => "Saint Kitts And Nevis",
            "LC" => "Saint Lucia",
            "PM" => "Saint Pierre And Miquelon",
            "VC" => "Saint Vincent And The Grenadines",
            "WS" => "Samoa",
            "SM" => "San Marino",
            "ST" => "Sao Tome And Principe",
            "SA" => "Saudi Arabia",
            "SN" => "Senegal",
            "SC" => "Seychelles",
            "SL" => "Sierra Leone",
            "SG" => "Singapore",
            "SK" => "Slovakia",
            "SI" => "Slovenia",
            "SB" => "Solomon Islands",
            "SO" => "Somalia",
            "ZA" => "South Africa",
            "ES" => "Spain",
            "LK" => "Sri Lanka",
            "SR" => "Suriname",
            "SJ" => "Svalbard And Jan Mayen",
            "SZ" => "Swaziland",
            "SE" => "Sweden",
            "CH" => "Switzerland",
            "TW" => "Taiwan",
            "TJ" => "Tajikistan",
            "TZ" => "Tanzania",
            "TH" => "Thailand",
            "TG" => "Togo",
            "TK" => "Tokelau",
            "TO" => "Tonga",
            "TT" => "Trinidad And Tobago",
            "TN" => "Tunisia",
            "TR" => "Turkey",
            "TM" => "Turkmenistan",
            "TC" => "Turks And Caicos Islands",
            "TV" => "Tuvalu",
            "UG" => "Uganda",
            "UA" => "Ukraine",
            "AE" => "United Arab Emirates",
            "GB" => "United Kingdom",
            "UM" => "United States Minor Outlying Islands",
            "UY" => "Uruguay",
            "UZ" => "Uzbekistan",
            "VU" => "Vanuatu",
            "VA" => "Vatican City State (Holy See)",
            "VE" => "Venezuela",
            "VN" => "Viet Nam",
            "VG" => "Virgin Islands, (British)",
            "VI" => "Virgin Islands, (U.S.)",
            "WF" => "Wallis And Futuna",
            "EH" => "Western Sahara",
            "YE" => "Yemen",
            "YU" => "Yugoslavia",
            "ZM" => "Zambia",
            "ZW" => "Zimbabwe"
        );
    }

    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('countries',$countries));
    return $parameterList;
}
?>
