<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file performs updating of the domain manager data                                          ----*/
/*---- These functions are invoked from the acf framework                                              ----*/
/*---------------------------------------------------------------------------------------------------------*/

$_ENV['TCA_HOME']       = $_SESSION['ccs']['options']['tca-options']['tca-home'];
$_ENV['TCA_CARTRIDGES'] = $_SESSION['ccs']['options']['tca-options']['tca-cartridges'];
$_ENV['TCA_ETC']        = $_SESSION['ccs']['options']['tca-options']['tca-etc'];

set_include_path(get_include_path() . $_ENV['pathDelimiter'] . $_ENV['TCA_HOME']);

//---- create a global client object
require_once("client.inc.php");
require_once("./domain/domainFunctions.php");
global $client;
TCA_Client::initEnv();
$client = new TCA_Client(new TCA_XMLConfigReader("tucows-client.xml"));

if($_SESSION['ccs']['options']['driver-options']['capture-xml-communication-log'] === "yes")
    $client->setLogFile($_SESSION['ccs']['options']['driver-options']['xml-communication-log-file']);

require_once("../../libs/ccs/utilities.inc.php");

//---- Function to email the password to the owner/admin
function AH_SendPassword(ACF_ParameterList $inParameterList){
    global $client;
    $status = array();
    $sendTo = $inParameterList->ParameterValue('sendto') == 'owner' ? owner : 'admin';
    $domain = $inParameterList->ParameterValue('domain');
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.send_password_domain');
    $pSet = new TCA_ServiceRequestParameterSet();
    $pSet->addParameter(new TCA_ServiceRequestParameter('domain_name', $domain));
    $pSet->addParameter(new TCA_ServiceRequestParameter('send_to', $sendTo));
    $pSet->addParameter(new TCA_ServiceRequestParameter('sub_user', '0'));
    $request->addParameterSet('attributes',$pSet);
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $eventType       = 'SendPassword_Failure';
    }
    if ($result)
        if (!$result->isSuccess()){
            $status['code'] = $result->getResponseCode();
            $status['text'] = $result->getResponseText();
            $eventType      = 'SendPassword_Failure';
        }else
            $eventType      = 'SendPassword_Success';
//---- create an acf event object
    $retEvent = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to login
function AH_DoLogin(ACF_ParameterList $inParameterList){
//---- grab the global client
    $status = array();
    $domain = $inParameterList->ParameterValue('domain');
    if (!$domain){
        $status['code']  = 'Invalid Login Information';
        $status['text']  = ($status['text'] ? $status['text'] . "\n  " : '') . 'Please enter a valid domain';
    }
    if (!$inParameterList->ParameterValue('login')){
        $status['code']  = 'Invalid Login Information';
        $status['text']  = $status['text'] ? $status['text'] . ', username' : 'Please enter a valid username';
    }
    if (!$inParameterList->ParameterValue('password')){
        $status['code']  = 'Invalid Login Information';
        $status['text']  = $status['text'] ? $status['text'] . ', password' : 'Please enter a valid password';
    }

    if ($status){
//---- create an acf event object
        $retEvent        = new ACF_Event('dologin_failure');
//---- appended parameters
        $parameterList  = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
        if ($domain)
            $parameterList->AddParameter(new ACF_DirectParameter('domain', $domain));
        $retEvent->SetParameterList($parameterList);
//---- return the event
        return $retEvent;
    }
    $status = login($inParameterList->ParameterValue('login'), $inParameterList->ParameterValue('password'), $domain);
    if ($status){
        if ($status['code'] == 'Invalid input')
            $eventType = 'dologin';
        else
            $eventType = 'dologin_failure';
    }else
        $eventType = 'dologin_success';
    if ($eventType === 'dologin_success' && $_SESSION['modules']['domain']['passed']['target'])
        $eventType = 'TargetPage';
//---- create an acf event object
    $retEvent = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    if ($eventType === 'dologin_failure')
        $parameterList->AddParameter(new ACF_DirectParameter('domain', $domain));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to logout
function AH_Logout(ACF_ParameterList $inParameterList){
//---- grab the global client
    global $client;
    $status = array();
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.delete_cookie');
//---- create a paameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes', $pset);
//---- attempt to process the service request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
//---- an exception which should only happen if the cookie does not exist
        $status['code']  = 'Invalid input';
        $status['text']  = 'Please log in';
    }
//---- create an acf event object
    $parameterList  = new ACF_ParameterList();
//---- empty the session variables
    $_SESSION['modules']['shared']['username']            = false;
    $_SESSION['modules']['shared']['password']            = false;
    $_SESSION['modules']['shared']['domain']              = false;
    $_SESSION['modules']['shared']['domainManagerMode']   = false;
    $_SESSION['modules']['shared']['user']                = false;
    $_SESSION['modules']['domain']['manager']['ccsEvent'] = false;
    $_SESSION['ccs']['source'] = 'default';
    $retEvent = new ACF_Event('Logout_Success');
//---- appended parameters
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to change the contact info
function AH_UpdateContacts (ACF_ParameterList $inParameterList) {
//---- if this is a .ca domain run a different fucntion
    if ($_SESSION['modules']['domain']['manager']['domainSuffix'] == '.ca')
        return updateCAContacts($inParameterList);
//---- if this is a .eu domain run a different fucntion
    if ($_SESSION['modules']['domain']['manager']['domainSuffix'] == '.eu')
        return updateEUContacts($inParameterList);
    global $client;
    $status      = array();
    $contactInfo = stripslashes_deep($inParameterList->ParameterValue('contact'));
    $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
    $pSet = new TCA_ServiceRequestParameterSet();
    $pSet->addParameter(new TCA_ServiceRequestParameter('data', "contact_info"));
    $pSet->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
    $contactSet = new TCA_ServiceRequestParameterSet();
    foreach($contactInfo as $type=>$contactDetails){
        $contactSet->addParameter(new TCA_ServiceRequestParameter('also_apply_to', array($type)));
        $contact = new TCA_ServiceRequestParameterSet();
        $contactSet->addParameterSet($type, $contact);
//---- for all other domain suffixes
        $contact->addParameter(new TCA_ServiceRequestParameter('first_name', $contactDetails['firstName']));
        $contact->addParameter(new TCA_ServiceRequestParameter('last_name', $contactDetails['lastName']));
        $contact->addParameter(new TCA_ServiceRequestParameter('org_name', $contactDetails['organization']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address1', $contactDetails['address1']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address2', $contactDetails['address2']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address3', $contactDetails['address3']));
        $contact->addParameter(new TCA_ServiceRequestParameter('city', $contactDetails['city']));
        $contact->addParameter(new TCA_ServiceRequestParameter('state', $contactDetails['state']));
        $contact->addParameter(new TCA_ServiceRequestParameter('country', $contactDetails['country']));
        $contact->addParameter(new TCA_ServiceRequestParameter('postal_code', $contactDetails['postalCode']));
        $contact->addParameter(new TCA_ServiceRequestParameter('phone', $contactDetails['phone']));
        $contact->addParameter(new TCA_ServiceRequestParameter('fax', $contactDetails['fax']));
        $contact->addParameter(new TCA_ServiceRequestParameter('email', $contactDetails['email']));
    }
    $pSet->addParameterSet('contact_set', $contactSet);
    $request->addParameterSet('attributes',$pSet);
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $eventName       = 'UpdateContacts_Failure';
    }
    if ($result) {
        if (!$result->isSuccess()){
            $status['code']  = $result->getResponseCode();
            $status['text']  = $result->getResponseText();
            $eventName      = 'UpdateContacts_Failure';
        }else
            $eventName = 'UpdateContacts_Success';
    }
//---- create an acf event object
    $retEvent      = new ACF_Event($eventName);
//---- appended parameters
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to change the contact info
function updateCAContacts (ACF_ParameterList $inParameterList) {
    global $client;
    $status      = array();
    $contactInfo = stripslashes_deep($inParameterList->ParameterValue('contact'));
    $request     = $client->createServiceRequest('TUCOWS_domains.modify_domain');
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
    $pSet = new TCA_ServiceRequestParameterSet();
    $pSet->addParameter(new TCA_ServiceRequestParameter('data', "contact_info"));
    $pSet->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
    $contactSet = new TCA_ServiceRequestParameterSet();
    foreach($contactInfo as $type=>$contactDetails){
        $contactSet->addParameter(new TCA_ServiceRequestParameter('also_apply_to', array()));
        $contact = new TCA_ServiceRequestParameterSet();
        $contactSet->addParameterSet($type, $contact);
//---- if updating the admin or tech
        $contact->addParameter(new TCA_ServiceRequestParameter('lang', $contactDetails['language']));
        $contact->addParameter(new TCA_ServiceRequestParameter('org_name', $contactDetails['organization']));
        $contact->addParameter(new TCA_ServiceRequestParameter('first_name', $contactDetails['firstName']));
        $contact->addParameter(new TCA_ServiceRequestParameter('last_name', $contactDetails['lastName']));
        $contact->addParameter(new TCA_ServiceRequestParameter('org_name', $contactDetails['organization']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address1', $contactDetails['address1']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address2', $contactDetails['address2']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address3', $contactDetails['address3']));
        $contact->addParameter(new TCA_ServiceRequestParameter('city', $contactDetails['city']));
        $contact->addParameter(new TCA_ServiceRequestParameter('state', $contactDetails['state']));
        $contact->addParameter(new TCA_ServiceRequestParameter('country', $contactDetails['country']));
        $contact->addParameter(new TCA_ServiceRequestParameter('postal_code', $contactDetails['postalCode']));
        $contact->addParameter(new TCA_ServiceRequestParameter('phone', $contactDetails['phone']));
        $contact->addParameter(new TCA_ServiceRequestParameter('fax', $contactDetails['fax']));
        $contact->addParameter(new TCA_ServiceRequestParameter('email', $contactDetails['email']));
    }
    $pSet->addParameterSet('contact_set', $contactSet);
    $request->addParameterSet('attributes',$pSet);
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $eventName       = 'UpdateContacts_Failure';
    }
    if ($result) {
        if (!$result->isSuccess()){
            $status['code']  = $result->getResponseCode();
            $status['text']  = $result->getResponseText();
            $eventName      = 'UpdateContacts_Failure';
        }else
            $eventName = 'UpdateContacts_Success';
    }
//---- create an acf event object
    $retEvent      = new ACF_Event($eventName);
//---- appended parameters
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to change the contact info on a .eu domain
function updateEUContacts (ACF_ParameterList $inParameterList) {
    global $client;
    $status      = array();
    $contactInfo = stripslashes_deep($inParameterList->ParameterValue('contact'));
    $oriContact  = $_SESSION['modules']['domain']['manager']['oriContact'];
//---- variable used to force an error to display if a Trade happens, this reference should be removed once Trade is supported
    $forceFail   = false;

//---- see if we need to do a trade
    if (($contactInfo['owner']['organization'] != $oriContact['organization']) || 
	    (!$oriContact['organization'] && ($contactInfo['owner']['firstName'] != $oriContact['firstName'] ||
	    $contactInfo['owner']['lastName'] != $oriContact['lastName']))){
	$contactInfo['owner']['organization'] = $oriContact['organization'];
	$status['code'] = "Invalid Input";
	$status['text'] = "Please contact your Service Provider to make a change to the Owner's Company Name.";
	$forceFail = true;
/*---- once Trades are supported this section can be uncommentedA and the two lines above deleted
//---- create an acf event object
	$retEvent       = new ACF_Event('UpdateContacts_Trade');
//---- appended parameters
	$parameterList  = new ACF_ParameterList();
	$parameterList->AddParameter(new ACF_DirectParameter('contact', $contactInfo));
	$retEvent->SetParameterList($parameterList);
//---- return the event
	return $retEvent;
----*/
    }

    $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
    $pSet = new TCA_ServiceRequestParameterSet();
    $pSet->addParameter(new TCA_ServiceRequestParameter('data', "contact_info"));
    $pSet->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
    $contactSet = new TCA_ServiceRequestParameterSet();
    foreach($contactInfo as $type=>$contactDetails){
        $contactSet->addParameter(new TCA_ServiceRequestParameter('also_apply_to', array($type)));
        $contact = new TCA_ServiceRequestParameterSet();
        $contactSet->addParameterSet($type, $contact);
        $contact->addParameter(new TCA_ServiceRequestParameter('first_name', $contactDetails['firstName']));
        $contact->addParameter(new TCA_ServiceRequestParameter('last_name', $contactDetails['lastName']));
        $contact->addParameter(new TCA_ServiceRequestParameter('org_name', $contactDetails['organization']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address1', $contactDetails['address1']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address2', $contactDetails['address2']));
        $contact->addParameter(new TCA_ServiceRequestParameter('address3', $contactDetails['address3']));
        $contact->addParameter(new TCA_ServiceRequestParameter('city', $contactDetails['city']));
        $contact->addParameter(new TCA_ServiceRequestParameter('state', $contactDetails['state']));
        $contact->addParameter(new TCA_ServiceRequestParameter('country', $contactDetails['country']));
        $contact->addParameter(new TCA_ServiceRequestParameter('postal_code', $contactDetails['postalCode']));
        $contact->addParameter(new TCA_ServiceRequestParameter('phone', $contactDetails['phone']));
        $contact->addParameter(new TCA_ServiceRequestParameter('fax', $contactDetails['fax']));
        $contact->addParameter(new TCA_ServiceRequestParameter('email', $contactDetails['email']));
        $contact->addParameter(new TCA_ServiceRequestParameter('lang', $contactDetails['language']));
        $contact->addParameter(new TCA_ServiceRequestParameter('vat', $contactDetails['vat']));
    }
    $pSet->addParameterSet('contact_set', $contactSet);
    $request->addParameterSet('attributes',$pSet);
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
/*---- this will need to change once Trades are supported
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
----*/
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : "") . 'Invalid input';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : "") . $error->getMessage();
        $eventName       = 'UpdateContacts_Failure';
    }
    if ($result){
/*---- this will need to change once Trades are supported
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
----*/
	$responseCode = $result->getResponseCode();
	if ($responseCode != '200' && $responseCode != '210' && $status['code'])
	    $status['code'] .=  "\n" . $responseCode;
	elseif (!$status['code'])
	    $status['code']  =  $responseCode;
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : "") . $result->getResponseText();

        if (!$result->isSuccess() || $forceFail){
            $status['code'] = $result->getResponseCode();
            $status['text'] = $result->getResponseText();
        $eventName = 'UpdateContacts_Failure';
        }else
            $eventName      = 'UpdateContacts_Success';
    }
//---- create an acf event object
    $retEvent      = new ACF_Event($eventName);
//---- appended parameters
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to switch to a differnet domain
function AH_SwitchDomain (ACF_ParameterList $inParameterList) {
    global $client;
    $status = array();
    $newDomain =  $inParameterList->ParameterValue('domain') ? $inParameterList->ParameterValue('domain') : $_SESSION['modules']['domain']['manager']['domain'];
//---- valid tld check
    $tld = substr($newDomain, strrpos($newDomain, '.'));
    foreach ($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'] as $allowedServices){
        if (($allowedServices['tld'] === $tld) && (strtolower($allowedServices['active']) == "yes" || strtolower($allowedServices['active']) == "true" || $allowedServices['active'] == 1)){
            $validTld = true;
            break;
        }
    }
    if (!$validTld){
        $status['code']  = 'Invalid Login Information';
        $status['text']  = ($status['text'] ? $status['text'] . "\n  " : '') . 'Please enter a valid domain';
//---- create an acf event object
        $retEvent       = new ACF_Event('SwitchDomain_Failure');
//---- appended parameters
        $parameterList  = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
        $retEvent->SetParameterList($parameterList);
//---- return the event
        return $retEvent;
    }
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.set_cookie');
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.update_cookie');
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
    $pSet = new TCA_ServiceRequestParameterSet();
    $pSet->addParameter(new TCA_ServiceRequestParameter('domain', $_SESSION['modules']['domain']['manager']['domain']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('domain_new', $newDomain));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_username', 'x'));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', 'x'));
    $request->addParameterSet('attributes',$pSet);
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $retType         = 'SwitchDomain_Failure';
    }
    if ($result)
        if ($result->getResponseCode() != 200 && $result->getResponseCode() != 210){
            $status['code'] = $result->getResponseCode();
            $status['text'] = $result->getResponseText();
            $retType        = 'SwitchDomain_Failure';
        }else{
            $retType = 'SwitchDomain_Success';
            $_SESSION['modules']['domain']['manager']['domain']       = $newDomain;
            $_SESSION['modules']['domain']['manager']['domainSuffix'] = substr($newDomain, strrpos($newDomain, '.'));
            $_SESSION['modules']['shared']['domain']                  = $newDomain;
        }
//---- create an acf event object
    $retEvent       = new ACF_Event($retType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to direct a walk-in process
function AH_DirectTargetEntry(ACF_ParameterList $inParameterList){
//---- if they want the login page clear out the target
    if (strtolower($_SESSION['modules']['domain']['passed']['target']) == 'login')
        $_SESSION['modules']['domain']['passed']['target'] = false;
//---- if the loginInfo exist login
    if ($_SESSION['modules']['domain']['passed']['username'] && $_SESSION['modules']['domain']['passed']['password'] && $_SESSION['modules']['domain']['passed']['domain']){
        $status = login($_SESSION['modules']['domain']['passed']['username'], $_SESSION['modules']['domain']['passed']['password'], $_SESSION['modules']['domain']['passed']['domain']);
        $_SESSION['modules']['domain']['passed']['username'] = $_SESSION['modules']['domain']['passed']['password'] = '';
//---- if an error occured during login throw an error and send them to the login page
        if ($status){
            $retEvent = new ACF_Event('AutoLogin_Failed');
//---- appended parameters
            $parameterList = new ACF_ParameterList();
            $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['domain']['passed']['domain']));
            $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
            $retEvent->SetParameterList($parameterList);
            $_SESSION['modules']['domain']['passed']['domain']   = false;
            $_SESSION['modules']['domain']['passed']['username'] = false;
            $_SESSION['modules']['domain']['passed']['password'] = false;
//---- set the global error message
            $_SESSION['modules']['shared']['status'] = $status;
            $_SESSION['modules']['shared']['status']['isError'] = true;
            return $retEvent;
        }
    }else{
        if (($_SESSION['modules']['domain']['passed']['username'] || $_SESSION['modules']['domain']['passed']['password'] || $_SESSION['modules']['domain']['passed']['domain']) && !$_SESSION['modules']['domain']['manager']['cookie']){
//---- if the domain was passed in go to the alternate login pages
            if ($_SESSION['modules']['domain']['passed']['domain']){
                $retEvent = new ACF_Event('Target_ManagerLogin');
//---- appended parameters
                $parameterList = new ACF_ParameterList();
                $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['domain']['passed']['domain']));
                $retEvent->SetParameterList($parameterList);
                return $retEvent;
            }else{
                if (!$_SESSION['modules']['domain']['passed']['username'])
                    $errorText = 'The username is blank';
                if (!$_SESSION['modules']['domain']['passed']['password'])
                    $errorText = ($errorText ? $errorText . "\n" : '') . 'The password is blank';
                if (!$_SESSION['modules']['domain']['passed']['domain'])
                    $errorText = ($errorText ? $errorText . "\n" : '') . 'The domain is blank';
                if ($errorText){
                    $_SESSION['modules']['shared']['status']['code'] = "Auto Login Failed";
                    $_SESSION['modules']['shared']['status']['text'] = "Auto Login can not be completed because of the following errors:\n" . $errorText;
                    $_SESSION['modules']['shared']['status']['isError'] = true;
                }
            }
        }
    }
//---- if the cookie does not exist go to the login
    if (!$_SESSION['modules']['domain']['manager']['cookie']){
        $retEvent = new ACF_Event('Target_Login');
//---- appended parameters
        $parameterList = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['domain']['passed']['domain']));
        $retEvent->SetParameterList($parameterList);
        $_SESSION['modules']['domain']['passed']['domain']   = false;
        $_SESSION['modules']['domain']['passed']['username'] = false;
        $_SESSION['modules']['domain']['passed']['password'] = false;
    }else{
        switch (strtolower($_SESSION['modules']['domain']['passed']['target'])){
            case 'main':
                $retEvent = new ACF_Event('Target_Main');
                break;
            case 'contacts':
                $retEvent = new ACF_Event('Target_Contact');
                break;
            case 'renew':
                $retEvent = new ACF_Event('Target_Renew');
                break;
            case 'list':
                $retEvent = new ACF_Event('Target_List');
                break;
            case 'details':
                $retEvent = new ACF_Event('Target_Details');
                break;
            case 'extras':
                $retEvent = new ACF_Event('Target_Extras');
                break;
            case 'nameservers':
                $retEvent = new ACF_Event('Target_Nameservers');
                break;
            case 'localnameservers':
                $retEvent = new ACF_Event('Target_LocalNameservers');
                break;
            default:
                $retEvent = new ACF_Event('Target_Main');
                if ($_SESSION['modules']['domain']['passed']['target']){
                    $_SESSION['modules']['domain']['manager']['status']['code'] = "Invalid Target";
                    $_SESSION['modules']['domain']['manager']['status']['text'] = "The target that was requested does not exist: " . strtolower($_SESSION['modules']['domain']['passed']['target']);
                }
                break;
        }
        unset($_SESSION['modules']['domain']['passed']);
    }
//---- return the event
    return $retEvent;
}


//---- function to perform the login process
function login($username, $password, $domain){
    global $client;
    $status = array();
//---- valid tld check
    $tld = substr($domain, strrpos($domain, '.'));
    foreach ($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'] as $allowedServices){
        if (($allowedServices['tld'] === $tld) && (strtolower($allowedServices['active']) == "yes" || strtolower($allowedServices['active']) == "true" || $allowedServices['active'] == 1)){
            $validTld = true;
            break;
        }
    }
    if (!$validTld){
        $status['code']  = 'Invalid Login Information';
        $status['text']  = ($status['text'] ? $status['text'] . "\n  " : '') . 'Please enter a valid domain';
        $_SESSION['modules']['shared']['domain']   = false;
        $_SESSION['modules']['shared']['username'] = false;
        $_SESSION['modules']['shared']['password'] = false;
        return $status;
    }
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.set_cookie');
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_username', $username));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $password));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']                            = 'Invalid input';
        $status['text']                            = $error->getMessage();
        $_SESSION['modules']['shared']['domain']   = false;
        $_SESSION['modules']['shared']['username'] = false;
        $_SESSION['modules']['shared']['password'] = false;
    }
    if ($result){
        if (!$result->isSuccess()){
            $status['code']                                     = $result->getResponseCode();
            $status['text']                                     = $result->getResponseText();
            $_SESSION['modules']['shared']['domain']            = false;
            $_SESSION['modules']['shared']['username']          = false;
            $_SESSION['modules']['shared']['password']          = false;
            $_SESSION['modules']['domain']['manager']['cookie'] = false;
            $_SESSION['modules']['shared']['domainManagerMode'] = false;
            $_SESSION['modules']['shared']['user']              = false;
        }else{
//---- the login was successful so set 
            $_SESSION['modules']['domain']['manager']['cookie']       = $result->getResponseData()->getElement("attributes")->getElement('cookie')->getValue();
            $_SESSION['modules']['domain']['manager']['domain']       = $domain;
            $_SESSION['modules']['domain']['manager']['domainSuffix'] = $tld;
            $_SESSION['modules']['domain']['manager']['username']     = $username;
            $_SESSION['modules']['shared']['domain']                  = $domain;
            $_SESSION['modules']['shared']['username']                = $username;
            $_SESSION['modules']['shared']['password']                = $password;
            $_SESSION['modules']['shared']['domainManagerMode']       = true;
            if(!$_SESSION['ccs']['source'] || ($_SESSION['ccs']['source'] == 'default'))
                    $_SESSION['ccs']['source']                        = 'domain';
//---- remove the user step for registration
            if (is_array($_SESSION['ccs']['excludeSteps']))
                array_push($_SESSION['ccs']['excludeSteps'], 'user');
            else
                $_SESSION['ccs']['excludeSteps'] = array('user');
//---- grab the contact name
//---- create a service request object
            $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
            $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
            $pSet = new TCA_ServiceRequestParameterSet();
//---- attach the parameters to the parameter set
            $type = $_SESSION['modules']['domain']['manager']['domainSuffix'] == ".ca" ? 'admin' : 'owner';
            $pSet->addParameter(new TCA_ServiceRequestParameter('type', $type));
//---- attach the parameter set to the service request object
            $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
            try{
                $result = $client->processServiceRequest($request);
            }
            catch(Exception $error){}
            if ($result && $result->isSuccess()){
                $refPoint = $result->getResponseData()->getElement("attributes")->getElement('contact_set')->getElement($type);
                $_SESSION['modules']['shared']['user'] = $refPoint->getElement('first_name')->getValue('first_name') . " " . $refPoint->getElement('last_name')->getValue('last_name');
            }
        }
    }
    return $status;
}


//---- Function to renew a domain
function AH_RenewDomain (ACF_ParameterList $inParameterList) {
    global $client;
    $_SESSION['modules']['domain']['manager']['backupData'] = array();
    $autoRenewOption   = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-renew-change'];
    $changeRenewOption = $autoRenewOption == "yes" || $autoRenewOption == "1";
    $autoRenewOption   = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-renew-default'];
    $autoRenew         = ($autoRenewOption == "yes" || $autoRenewOption == "1") ? 1 : 0;
    if ($changeRenewOption && $inParameterList->ParameterValue('autoRenew'))
        $autoRenew = $inParameterList->ParameterValue('autoRenew');
    $status = array();
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.renew_domain');
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
    $pSet = new TCA_ServiceRequestParameterSet();
    $pSet->addParameter(new TCA_ServiceRequestParameter('domain', $_SESSION['modules']['domain']['manager']['domain']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('currentexpirationyear', $inParameterList->ParameterValue('expireyear')));
    $pSet->addParameter(new TCA_ServiceRequestParameter('period', (int) $inParameterList->ParameterValue('period')));
    $pSet->addParameter(new TCA_ServiceRequestParameter('auto_renew', $autoRenew));
    $pSet->addParameter(new TCA_ServiceRequestParameter('handle', $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['handle-action']));

    $request->addParameterSet('attributes',$pSet);
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $retType         = 'ChargeServiceItem_Failed';
    }
    if ($result){
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
        if ($status['code'] != 200 && $status['code'] != 210)
            $retType = 'ChargeServiceItem_Failed';
        else
            $retType = 'ChargeServiceItem_Commit';
    }
    $_SESSION['modules']['domain']['manager']['backupData']['status'] = $status;
    $_SESSION['modules']['domain']['manager']['ccsEvent']             = $retType;
//---- return the event
    return new ACF_Event('HopProject');
}


//---- function to trade a domain
function AH_TradeDomain(ACF_ParameterList $inParameterList){
    global $client;
    $_SESSION['modules']['domain']['manager']['backupData'] = array();
//---- grab the needed options
    $options           = &$_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options'];
    $contacts          = $inParameterList->ParameterValue('contact');
    $nameservers       = array();
    $contact_set       = array();
//---- convert the contacts array to a proper array for TCA
    foreach($contacts as $key => $contact){
	$details = array();
	$details['lang']        = $contact['language']     ? $contact['language']     : '';
	$details['first_name']  = $contact['firstName']    ? $contact['firstName']    : '';
	$details['last_name']   = $contact['lastName']     ? $contact['lastName']     : '';
	$details['org_name']    = $contact['organization'] ? $contact['organization'] : '';
	$details['email']       = $contact['email']        ? $contact['email']        : '';
	$details['phone']       = $contact['phone']        ? $contact['phone']        : '';
	$details['fax']         = $contact['fax']          ? $contact['fax']          : '';
	$details['address1']    = $contact['address1']     ? $contact['address1']     : '';
	$details['address2']    = $contact['address2']     ? $contact['address2']     : '';
	$details['address3']    = $contact['address3']     ? $contact['address3']     : '';
	$details['city']        = $contact['city']         ? $contact['city']         : '';
	$details['state']       = $contact['state']        ? $contact['state']        : '';
	$details['country']     = $contact['country']      ? $contact['country']      : '';
	$details['postal_code'] = $contact['postalCode']   ? $contact['postalCode']   : '';
	$details['vat']         = $contact['vat']          ? $contact['vat']          : '';
	$contact_set[$key] = $details;
    }
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('type', 'nameservers'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result){
//---- get data into an array
        if ($result->isSuccess()){
            $refPoint = $result->getResponseData()->getElement('attributes')->getElement('nameserver_list');
//---- loop through the data to build the array
            if ($refPoint)
                for ($index=0; $refPoint->getElement($index); $index++){
//---- flush the array
                    $details               = array();
                    $details['sortorder']  = $refPoint->getElement($index)->getElement('sortorder')->getValue('sortorder');
                    $details['name']       = $refPoint->getElement($index)->getElement('name')->getValue('name');
//---- push the array onto the parent
                    array_push($nameservers, $details);
                }
        }
    }
    $request = null;
    $result  = null;
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.sw_register_domain');
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('domain', $_SESSION['modules']['domain']['manager']['domain']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_type', 'transfer'));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_username', $_SESSION['modules']['shared']['username']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $_SESSION['modules']['shared']['password']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('change_contact', 1));
    $pSet->addParameter(new TCA_ServiceRequestParameter('handle', $options['handle-action']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('custom_tech_contact', 1));
    $pSet->addParameter(new TCA_ServiceRequestParameter('period', 1));
    $pSet->addParameter(new TCA_ServiceRequestParameter('custom_nameservers', 1));
    $pSet->addParameter(new TCA_ServiceRequestParameter('nameserver_list', $nameservers));
    $pSet->addParameter(new TCA_ServiceRequestParameter('contact_set', $contact_set));
    $pSet->addParameter(new TCA_ServiceRequestParameter('affiliate_id', 'platypus'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $retType         = 'ChargeServiceItem_Failed';
    }
    if ($result){
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
        if ($status['code'] != 200 && $status['code'] != 210)
            $retType = 'ChargeServiceItem_Failed';
        else
            $retType = 'ChargeServiceItem_Commit';
    }
    $_SESSION['modules']['domain']['manager']['backupData']['status'] = $status;
    $_SESSION['modules']['domain']['manager']['ccsEvent']             = $retType;
//---- return the event
    return new ACF_Event('HopProject');
}

function AH_ChargeItem(ACF_ParameterList $inParameterList){
    $options = array();
    switch ($inParameterList->ParameterValue('item')){
        case 'renewal':
            $options = array('autoRenew' => $inParameterList->ParameterValue('autoRenew'),
                             'quantity' => $inParameterList->ParameterValue('quantity'),
                             'expireyear' => $inParameterList->ParameterValue('expireyear'));
            break;
	case 'trade':
	    $options = array('quantity' => '1',
                             'contact' => $inParameterList->ParameterValue('contact'));
	    //$event   = 'ChargeItem_Auth';
	    $event   = 'chargeserviceitem_auth';
	    break;
    }
    setDomainChargeItem($inParameterList->ParameterValue('item'),$options);
//---- create an acf event object
    $retEvent = new ACF_Event('HopProject');
    if ($event){
	$parameterList = new ACF_ParameterList();
	$parameterList->AddParameter(new ACF_DirectParameter('event', $event));
	$retEvent->SetParameterList($parameterList);
    }
    return $retEvent;
}


function AH_ChargeItemComplete(ACF_ParameterList $inParameterList){
    $status = $_SESSION['modules']['domain']['manager']['backupData']['status'];
    $parameterList = new ACF_ParameterList();
    switch ($_SESSION['modules']['shared']['chargeItem']['description']){
        case 'Domain Renewal':
            $retType = 'RenewDomain_Success';
            $parameterList->AddParameter(new ACF_DirectParameter('period', $_SESSION['modules']['domain']['manager']['backupData']['period']));
            $parameterList->AddParameter(new ACF_DirectParameter('autoRenew', $_SESSION['modules']['domain']['manager']['backupData']['autoRenew']));
            $parameterList->AddParameter(new ACF_DirectParameter('expireYear', $_SESSION['modules']['domain']['manager']['backupData']['expireYear']));
            break;
	case 'Domain Trade':
            $retType = 'TradeDomain_Success';
            $parameterList->AddParameter(new ACF_DirectParameter('contact', $_SESSION['modules']['domain']['manager']['backupData']['contact']));
	    break;
    }
    $_SESSION['modules']['domain']['manager']['backupData'] = array();
//---- create an acf event object
    $retEvent = new ACF_Event($retType);
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}


function AH_CommitChargeComplete(ACF_ParameterList $inParameterList){
    $status = $_SESSION['modules']['domain']['manager']['backupData']['status'];
    $parameterList = new ACF_ParameterList();
    switch ($_SESSION['modules']['shared']['chargeItem']['description']){
        case 'Domain Renewal':
            $retType = 'DomainRenewal';
            break;
        case 'Domain Trade':
            $retType = 'DomainTrade';
            break;
    }
    $_SESSION['modules']['domain']['manager']['backupData'] = array();
//---- Finished with billing modules so reset charge Item / 12/12/05 JCW
    $_SESSION['modules']['shared']['chargeItem']            = array();
//---- create an acf event object
    $retEvent = new ACF_Event($retType);
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}


function AH_RollbackChargeComplete(ACF_ParameterList $inParameterList){
    $status = $_SESSION['modules']['domain']['manager']['backupData']['status'];
    $parameterList = new ACF_ParameterList();
    switch ($_SESSION['modules']['shared']['chargeItem']['description']){
        case 'Domain Renewal':
            $retType = 'DomainRenewal';
            break;
        case 'Domain Trade':
            $retType = 'DomainTrade';
            break;
    }
    $_SESSION['modules']['domain']['manager']['backupData'] = array();
//---- Finished with billing modules so reset charge Item / 12/12/05 JCW
    $_SESSION['modules']['shared']['chargeItem']            = array();
//---- create an acf event object
    $retEvent = new ACF_Event($retType);
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}


function AH_SetCCSEvent(ACF_ParameterList $inParameterList){
    $_SESSION['modules']['domain']['manager']['ccsEvent'] = $inParameterList->ParameterValue('event');
    $retEvent = new ACF_Event('SetCCSEvent_Success');
    return $retEvent;
}


function AH_DOM_MGR_HopProject(ACF_ParameterList $inParameterList){
//---- This activity handler redirects a user to the given url (for Project hopping)
        $event = $inParameterList->ParameterValue('event') ? $inParameterList->ParameterValue('event') : $_SESSION['modules']['domain']['manager']['ccsEvent'];
        if ($inParameterList->ParameterValue('status')){
            $_SESSION['modules']['shared']['status'] = $inParameterList->ParameterValue('status');
            $_SESSION['modules']['shared']['status']['isError'] = true;
        }
//---- Send HTML header
        header("location:".$_SESSION['ccs']['options']['driver-options']['url']."/".strtolower($event));
        $retEvent = new ACF_Event('HopProject_Success');
        return $retEvent;
}


function AH_AutoLogin(ACF_ParameterList $inParameterList){
    $domain   = $_SESSION['modules']['shared']['domain']   ? $_SESSION['modules']['shared']['domain']   : false;
    $login    = $_SESSION['modules']['shared']['username'] ? $_SESSION['modules']['shared']['username'] : false;
    $password = $_SESSION['modules']['shared']['password'] ? $_SESSION['modules']['shared']['password'] : false;

    if ($domain !== false && $login !== false && $password !== false){
        $retEvent      = new ACF_Event('AutoLogin_Success');
        $parameterList = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('domain', $domain));
        $parameterList->AddParameter(new ACF_DirectParameter('login', $login));
        $parameterList->AddParameter(new ACF_DirectParameter('password', $password));
        $retEvent->SetParameterList($parameterList);
    }else
        $retEvent = new ACF_Event('AutoLogin_Failed');
    return $retEvent;
}


//---- Function to lock a domain
function AH_LockDomain (ACF_ParameterList $inParameterList) {
    global $client;
    $status = array();
//---- create the service request object
    $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
    $pset = new TCA_ServiceRequestParameterSet();
    $pset->addParameter(new TCA_ServiceRequestParameter('data', 'status'));
    $pset->addParameter(new TCA_ServiceRequestParameter('lock_state', $inParameterList->ParameterValue('state')));
    $pset->addParameter(new TCA_ServiceRequestParameter('affect_domains', $inParameterList->ParameterValue('updateAllDomains') ? 1 : 0));
    $request->addParameterSet('attributes',$pset);
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
    }
    if ($result){
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
    }
//---- create an acf event object
    $retEvent       = new ACF_Event('Lockdomain_Success');
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to update the extra domain info
function AH_UpdateDomainExtras (ACF_ParameterList $inParameterList) {
    global $client;
    $status = array();
    $extras = $inParameterList->ParameterValue('extras');
//---- create the service request object
    $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
    $pset = new TCA_ServiceRequestParameterSet();
//---- get the approprate type
    switch ($_SESSION['modules']['domain']['manager']['domainSuffix']){
        case '.com':
        case '.net':
        case '.info':
        case '.biz':
        case '.org':
        case '.mobi':
            $type = 'domain_auth_info';
            $pset->addParameter(new TCA_ServiceRequestParameter($type, $extras['Domain Authorization Code']));
            $pset->addParameter(new TCA_ServiceRequestParameter('data', $type));
            $pset->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
            break;
        case '.ca':
//---- create the service request object
            $request = $client->createServiceRequest('TUCOWS_domains.modify_ca_owner');
            $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
            $pset = new TCA_ServiceRequestParameterSet();
            $pset->addParameter(new TCA_ServiceRequestParameter('data', 'contact_info'));
            $pset->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
            $contactSet = new TCA_ServiceRequestParameterSet();
            $contactSet->addParameter(new TCA_ServiceRequestParameter('also_apply_to', array()));
            $contact = new TCA_ServiceRequestParameterSet();
            $contactSet->addParameterSet('owner', $contact);
//---- update the owner
            $contact->addParameter(new TCA_ServiceRequestParameter('legal_type', $extras['Legal Type']));
            $pset->addParameterSet('contact_set', $contactSet);
            $request->addParameterSet('attributes',$pset);
            try{
                $result = $client->processServiceRequest($request);
            }
            catch(Exception $error){
                $status['code']  = 'Invalid input';
                $status['text']  = $error->getMessage();
            }
//---- if something went wrong leave
            if ($result && !$result->isSuccess()){
                $status['code']  = $result->getResponseCode();
                $status['text']  = $result->getResponseText();
//---- create an acf event object
                $retEvent       = new ACF_Event('UpdateDomainExtras_Failed');
//---- appended parameters
                $parameterList  = new ACF_ParameterList();
                $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
                $retEvent->SetParameterList($parameterList);
//---- return the event
                return $retEvent;
            }
            break;
        default:
//---- if it does not match the above then there is no reason to continue
//---- create an acf event object
            $retEvent       = new ACF_Event('UpdateDomainExtras_Success');
//---- appended parameters
            $parameterList  = new ACF_ParameterList();
            $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
            $retEvent->SetParameterList($parameterList);
//---- return the event
            return $retEvent;
    }
    $request->addParameterSet('attributes',$pset);
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $eventType       = 'UpdateDomainExtras_Failed';
    }
    if ($result){
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
        if ($result->isSuccess()){
            if ($status['text'])
                $_SESSION['modules']['domain']['manager']['status']['note'] = $status['text'];
            $eventType = 'UpdateDomainExtras_Success';
        }else
            $eventType = 'UpdateDomainExtras_Failed';
    }

//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to add a name server
function AH_AddLocalNameServer (ACF_ParameterList $inParameterList) {
//---- grab the global client
    global $client;
    $status = array();
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.create_nameserver');
//---- put the cookie onto the service request object
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a paameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('name', $inParameterList->ParameterValue('name').".".$_SESSION['modules']['domain']['manager']['domain']));
    $pset->addParameter(new TCA_ServiceRequestParameter('ipaddress',  $inParameterList->ParameterValue('ip')));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes', $pset);
//---- attempt to process the service request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
//---- an exception was thrown so grab the error
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $eventType       = 'AddLocalNameserver_Failure';
    }
    if ($result){
//---- get the results
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
//---- check code
        $eventType       = 'AddLocalNameserver_Success';
    }
//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to modify a name server
function AH_ModifyLocalNameServer (ACF_ParameterList $inParameterList) {
    global $client;
    $status = array();
//---- create a service object
    $request = $client->createServiceRequest('TUCOWS_domains.modify_nameserver');
//---- add the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add the parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('name', $inParameterList->ParameterValue('name')));
    if ($inParameterList->ParameterValue('name') != $inParameterList->ParameterValue('newName').".".$_SESSION['modules']['domain']['manager']['domain'])
        $pset->addParameter(new TCA_ServiceRequestParameter('new_name', $inParameterList->ParameterValue('newName').".".$_SESSION['modules']['domain']['manager']['domain']));
    $pset->addParameter(new TCA_ServiceRequestParameter('ipaddress',  $inParameterList->ParameterValue('ipAddress')));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pset);
//---- attempt to process the request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = 'Invalid input';
        $status['text']  = $error->getMessage();
        $eventType       = 'ModifyLocalNameserver_Failure';
    }
    if ($result){
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
        $eventType       = 'ModifyLocalNameserver_Success';
    }
//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to delete a name server
function AH_DeleteLocalNameServer (ACF_ParameterList $inParameterList) {
    global $client;
    $rmAll = $inParameterList->ParameterValue('all');
    $nameservers = $inParameterList->ParameterValue('nameservers');

    if($nameservers) {
        foreach($nameservers as $name => $nameserver) {
            if($nameserver['delete'] || $rmAll) {
                $status = array();
//---- create a service request object
                $request = $client->createServiceRequest('TUCOWS_domains.delete_nameserver');
//---- attach the cookie as a parameter
                $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
                $pset = new TCA_ServiceRequestParameterSet();
//---- add the parameters into the parameter set
                $pset->addParameter(new TCA_ServiceRequestParameter('name', $name));
                $pset->addParameter(new TCA_ServiceRequestParameter('ipaddress',  $nameserver['ipaddress']));
//---- attach the parameter set to the service request object
                $request->addParameterSet('attributes',$pset);
//---- attempt to process the request
                try {
                    $result = $client->processServiceRequest($request);
                }
                catch(Exception $error){
                    $status['code']  = 'Invalid input';
                    $status['text']  = $error->getMessage();
                    $eventType       = 'DeleteLocalNameserver_Failure';
                    break;
                }
            }
        }
    }
    if ($result){
        $status['code']  = $result->getResponseCode();
        $status['text']  = $result->getResponseText();
        $eventType       = 'DeleteLocalNameserver_Success';
    }

    if (!$eventType) { // Didn't select a domain
        $status['code']  = 'Invalid input';
        $status['text']  = 'Invalid Nameserver was selected.';
        $eventType       = 'DeleteLocalNameserver_Failure';
    }

//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to modify a remote name server
function AH_AddRemoteNameserver (ACF_ParameterList $inParameterList) {
    global $client;
    $nameservers = $inParameterList->ParameterValue('newnameservers');
    if($nameservers){
        $status = array();
//---- create the service request object
        $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
//---- attach the cookie as a parameter
        $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
        $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
        $pset->addParameter(new TCA_ServiceRequestParameter('data', 'nameserver_list'));
        $pset->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
        $nameserverList = array();
        foreach($nameservers as $details){
            $newNS = array();
            $newNS['name']   = $details;
            $newNS['action'] = 'add';
            array_push($nameserverList, $newNS);
        }
        $pset->addParameter(new TCA_ServiceRequestParameter('nameserver_list', $nameserverList));
//---- attach the parameter set to the service request object
        $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
        try {
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = 'Invalid input';
            $status['text']  = $error->getMessage();
            $eventType       = 'AddRemoteNameserver_Failure';
        }
        if ($result){
            $status['code']  = $result->getResponseCode();
            $status['text']  = $result->getResponseText();
            if ($status['code'] == 200){
                $status['note'] = 'Database changes will not be immediately visible.';
                $eventType      = 'AddRemoteNameserver_Success';
            }else
                $eventType      = 'AddRemoteNameserver_Failure';
        }
    }else
        return new ACF_Event('AddRemoteNameserver_Success');
//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to modify a remote name server
function AH_ModifyRemoteNameserver (ACF_ParameterList $inParameterList) {
    global $client;
    $status = array();
    $assign = $inParameterList->ParameterValue('nameservers');
//---- remove blank entries
    $empty = array();
    foreach($assign as $key=>$nameserver)
        if ($key > 1 && strlen(trim($nameserver)) == 0)
            array_push($empty, $key);
    if ($empty){
        $invalidTld = array_reverse($empty);
        foreach($empty as $rmKey)
            if (count($domainListArray) -1  == $rmKey)
                array_pop($assign);
            else
                array_splice($assign, $rmKey, $rmKey - count($assign) + 1);
    }
    if ($empty){
//---- create the service request object
        $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
        $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
        $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
        $pset->addParameter(new TCA_ServiceRequestParameter('data', 'nameserver_list'));
        $pset->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
        $nameserverList = array();
        foreach($empty as $key){
            $modNS = array();
            $modNS['sortorder'] = $key + 1;
            $modNS['action']    = 'remove';
            array_push($nameserverList, $modNS);
        }
        $pset->addParameter(new TCA_ServiceRequestParameter('nameserver_list', $nameserverList));
//---- attach the parameter set to the service request object
        $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
        try {
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = 'Invalid input';
            $status['text']  = $error->getMessage();
            $eventType       = 'ModifyRemoteNameserver_Failure';
        }
        if ($result){
            $status['code']  = $result->getResponseCode();
            $status['text']  = $result->getResponseText();
            if ($status['code'] == 200){
                $status['note'] = 'Database changes will not be immediately visible.';
                $eventType      = 'ModifyRemoteNameserver_Success';
            }else
                $eventType      = 'ModifyRemoteNameserver_Failure';
        }
    }
    if (!$status && $assign){
//---- create the service request object
        $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
        $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
        $assign = $inParameterList->ParameterValue('nameservers');
//---- create a parameter set
        $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
        $pset->addParameter(new TCA_ServiceRequestParameter('data', 'nameserver_list'));
        $pset->addParameter(new TCA_ServiceRequestParameter('affect_domains', 0));
        $nameserverList = array();
        $index = 1;
        foreach($assign as $details){
            $modNS = array();
            $modNS['sortorder'] = $index;
            $modNS['name']      = $details;
            $modNS['action']    = 'update';
            array_push($nameserverList, $modNS);
            $index++;
        }
        $pset->addParameter(new TCA_ServiceRequestParameter('nameserver_list', $nameserverList));
//---- attach the parameter set to the service request object
        $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
        try {
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = 'Invalid input';
            $status['text']  = $error->getMessage();
            $eventType       = 'ModifyRemoteNameserver_Failure';
        }
        if ($result){
            $status['code']  = $result->getResponseCode();
            $status['text']  = $result->getResponseText();
            if ($status['code'] == 200){
                $status['note'] = 'Database changes will not be immediately visible.';
                $eventType      = 'ModifyRemoteNameserver_Success';
            }else
                $eventType      = 'ModifyRemoteNameserver_Failure';
        }
    }
//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to change ownership
function AH_RequestOwnershipChange (ACF_ParameterList $inParameterList) {
    global $client;
    $status = array();
    if ($inParameterList->ParameterValue('password') !== $inParameterList->ParameterValue('confirmPassword')){
        $status['code']  = 'Invalid Password';
        $status['text']  = 'The passwords did not match';
    }else{
//---- create the service request object
        $request = $client->createServiceRequest('TUCOWS_domains.change_ownership');
        $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
        $pSet = new TCA_ServiceRequestParameterSet();
        $pSet->addParameter(new TCA_ServiceRequestParameter('username', $inParameterList->ParameterValue('username')));
        $pSet->addParameter(new TCA_ServiceRequestParameter('password', $inParameterList->ParameterValue('password')));
        if ($inParameterList->ParameterValue('existing')){
            if ($inParameterList->ParameterValue(regDomain))
                $pSet->addParameter(new TCA_ServiceRequestParameter('reg_domain', $inParameterList->ParameterValue('regDomain')));
            else{
                $status['code'] = 'Invalid input';
                $status['text'] = 'Please specify a domain profile to transfer to.';
//---- create an acf event object
                $retEvent       = new ACF_Event('RequestOwnershipChange_Failure');
//---- appended parameters
                $parameterList  = new ACF_ParameterList();
                $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
                $retEvent->SetParameterList($parameterList);
//---- return the event
                return $retEvent;
            }
        }else
           $pSet->addParameter(new TCA_ServiceRequestParameter('move_all', $inParameterList->ParameterValue('moveAll') ? 1:0));
        $request->addParameterSet('attributes',$pSet);
        try{
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = 'Invalid input';
            $status['text']  = $error->getMessage();
            $eventType       = 'RequestOwnershipChange_Failure';
        }
        if ($result){
            $status['code']  = $result->getResponseCode();
            $status['text']  = $result->getResponseText();
        }
    }
    if ($result && $result->isSuccess() && $status['code'] == 200){
        $request = $client->createServiceRequest('TUCOWS_domains.modify_domain');
        $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
        try{
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = ($status['code'] ? $status['code'] . "\n " : '') . 'Invalid input';
            $status['text']  = ($status['text'] ? $status['text'] . "\n " : '') . $error->getMessage();
        }
        $retEvent = new ACF_Event('RequestOwnershipChange_Success');
//---- appended parameters
        $parameterList  = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['domain']['manager']['domain']));
        $parameterList->AddParameter(new ACF_DirectParameter('login', $inParameterList->ParameterValue('username')));
        $parameterList->AddParameter(new ACF_DirectParameter('password', $inParameterList->ParameterValue('password')));
        $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
        $retEvent->SetParameterList($parameterList);
//---- return the event
        return $retEvent;
    }else{
//---- create an acf event object
        $retEvent       = new ACF_Event('RequestOwnershipChange_Failure');
//---- appended parameters
        $parameterList  = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
        $retEvent->SetParameterList($parameterList);
//---- return the event
        return $retEvent;
    }
}


//---- Function to change the password
function AH_UpdatePassword (ACF_ParameterList $inParameterList) {
    global $client;
    $status = array();
//---- compare the new passwords
    if ($inParameterList->ParameterValue('confirmpassword') != $inParameterList->ParameterValue('newpassword')){
        $status['code'] = 'Passwords do not match';
        $status['text'] = 'The password was not changed.';
        $eventType      = 'UpdatePassword_Failure';
    }else{
//---- create a service request object
        $request = $client->createServiceRequest('TUCOWS_domains.change_password');
        $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
        $pSet = new TCA_ServiceRequestParameterSet();
        $pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $inParameterList->ParameterValue('newpassword')));
        $request->addParameterSet('attributes',$pSet);
        try{
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = 'Invalid input';
            $status['text']  = $error->getMessage();
            $eventType       = 'UpdatePassword_Failure';
        }
        if ($result){
            $status['code']  = $result->getResponseCode();
            $status['text']  = $result->getResponseText();
            if ($status['code'] == "200")
                $eventType   = 'UpdatePassword_Success';
            else
                $eventType   = 'UpdatePassword_Failure';
        }
    }
//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}
?>
