<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file performs updating of the domain registration data                                     ----*/
/*---- These functions are invoked from the acf framework                                              ----*/
/*---------------------------------------------------------------------------------------------------------*/

$_ENV['TCA_HOME']       = $_SESSION['ccs']['options']['tca-options']['tca-home'];
$_ENV['TCA_CARTRIDGES'] = $_SESSION['ccs']['options']['tca-options']['tca-cartridges'];
$_ENV['TCA_ETC']        = $_SESSION['ccs']['options']['tca-options']['tca-etc'];

set_include_path(get_include_path() . $_ENV['pathDelimiter'] . $_ENV['TCA_HOME']);

require_once("client.inc.php");
require_once("./domain/domainFunctions.php");
global $client;
TCA_Client::initEnv();
$client = new TCA_Client(new TCA_XMLConfigReader("tucows-client.xml"));

if($_SESSION['ccs']['options']['driver-options']['capture-xml-communication-log'] === "yes")
    $client->setLogFile($_SESSION['ccs']['options']['driver-options']['xml-communication-log-file']);

require_once("../../libs/ccs/utilities.inc.php");


//---- function to set the environment
function AH_SetEnvironment(ACF_ParameterList $inParameterList){
    $config = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options'];
//---- this removes or addes the domain configuration step based on option settings
    if ($config['allow-domain-lock-change'] == "no" && ($config['allow-register-nameservers'] == "no" || $config['allow-custom-nameservers'] == "no"))
        if (is_array($_SESSION['ccs']['excludeSteps'])){
            array_push($_SESSION['ccs']['excludeSteps'], 'configure');
        }else
            $_SESSION['ccs']['excludeSteps'] = array('configure');
    elseif (isset($_SESSION['ccs']['excludeSteps']['configure']))
	unset($_SESSION['ccs']['excludeSteps']['configure']);
//---- this grabs the searchable tld groups, this is done so other applications can override what is searchable
    if (!isset($_SESSION['ccs']['tldGroups']))
	$_SESSION['ccs']['tldGroups'] = $config['related-tld-groups'];
    return new ACF_Event("SetEnvironment_Success");
}


//---- function to check to see if the domain being searched for is a .ca
function AH_CheckCA(ACF_ParameterList $inParameterList){
    $inDomain = $inParameterList->ParameterValue('domain');
    if (is_array($inDomain))
        $inDomain = implode("", $inDomain);
    $domain = explode(".", $inDomain);
//---- create an acf event object
    if (strtolower($domain[(count($domain) - 1)]) === "ca"){
        $retEvent = new ACF_Event("CheckCA_Success");
//---- appended parameters
        $retEvent->SetParameterList($inParameterList);
    }else
        $retEvent = new ACF_Event("CheckCA_Failure");
    return $retEvent;
}


//---- Function to check the requested domain
function AH_CheckDomain (ACF_ParameterList $inParameterList){
    global $client;
    $status         = array();
    $domain         = array();
    $domain['name'] = $inParameterList->ParameterValue('domain');
    $tldList        = &$_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'];
    $customTech     = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-custom-tech-info'];
    $customTech     = $customTech == "yes" || $customTech === true || $customTech == 1 ? true : false;
//---- strip the domain name apart
    if (strpos($domain['name'], '.') !== false){
        $domain['baseName'] = strtolower(substr($domain['name'], 0, strrpos($domain['name'], '.')));
        $domain['tld']      = strtolower(substr($domain['name'], strrpos($domain['name'], '.')));
    }else{
        $domain['baseName'] = strtolower($domain['name']);
        $domain['tld']      = '';
    }
//---- assign the domain parts to a session variable
    $_SESSION['modules']['domain']['register']['domain']['name']     = $domain['name'];
    $_SESSION['modules']['domain']['register']['domain']['baseName'] = $domain['baseName'];
    $_SESSION['modules']['domain']['register']['domain']['tld']      = $domain['tld'];
    $_SESSION['modules']['domain']['register']['domain_list'] = array();
    unset($domain);
    $selected = array();
    $period   = $inParameterList->ParameterValue('period');
//---- if domains are being transfered and are being passed in as a list make it into an array
    if ($inParameterList->ParameterValue('domainOwned')){
        $owned = $inParameterList->ParameterValue('domainOwned');
        if (!is_array($owned)){
            if (!$_SESSION['modules']['requiredService'])
                $_SESSION['modules']['requiredService'] = 'domain';
            $massTransfer = true;
            $domainList   = str_replace("\r\n", ",", $owned);
            $owned        = array('Bulk Transfer');
//---- if any are empty clear them out
            $domainList   = str_replace(" ", "", $domainList);
            while (strpos($domainList, ",,") !== false)
                $domainList = str_replace(",,", ",", $domainList);
            if (strpos($domainList, ",") === 0)
                $domainList = substr($domainList, 1);
            if (strrpos($domainList, ",") === (strlen($domainList) - 1))
                $domainList = substr($domainList, 0 , strlen($domainList) - 1);
            $domainListArray = explode(",", $domainList);
            $domainList = array();
//---- remove any domain that is not in the active tld list
            $invalidTld= array();
            $_SESSION['modules']['domain']['register']['invalidList'] = array();
//---- validate the domains
            $validationResult = AH_Validate(array("domain_list" => $domainListArray), 'CheckTransferDomain_Failed');
            if (is_object($validationResult)){                                           //---- if an object is returned the validation has failed return it
                $parameterList = $validationResult->ParameterList();
                $parameterList->AddParameter(new ACF_DirectParameter('domain_list', implode("\r\n", $domainListArray)));
                if (!$domainListArray && $invalidTld)
                    $parameterList->AddParameter(new ACF_DirectParameter('status', array()));
                $validationResult->SetParameterList($parameterList);
                return $validationResult;
            }
            foreach($domainListArray as $key=>$domainItem){
                $found = false;
                foreach($tldList as $tldDetails)
                    if ((strtolower($tldDetails['active']) == 'yes' || strtolower($tldDetails['active']) == 'true' || $tldDetails['active'] == '1') && $tldDetails['tld'] == strtolower(substr($domainItem, strrpos($domainItem, '.')))){
//---- if the tld is found check to make sure the domain is transferable
//---- create a service request object
                        $request = $client->createServiceRequest('TUCOWS_domains.check_transfer');
//---- create a parameter set
                        $pset = new TCA_ServiceRequestParameterSet();
//---- add the parameters to the parameter set
                        $pset->addParameter(new TCA_ServiceRequestParameter('domain', $domainItem));
                        $pset->addParameter(new TCA_ServiceRequestParameter('get_request_address', 1));
//---- add the parameter set to the service request object
                        $request->addParameterSet('attributes', $pset);
//---- attempt to process the service request
                        try{
                            $result = $client->processServiceRequest($request);
                        }
                        catch(Exception $error){
                            $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
                            $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
                        }
//---- check to see if the requested domain is available
                        if ($result){
//---- Get the data into an array
                            if ($result->getResponseCode() == 200){
                                $bulkTLD  = strtolower(substr($domainItem, strrpos($domainItem, '.')));
                                $refPoint = $result->getResponseData()->getElement('attributes');
                                if ($refPoint->getElement('transferrable')->getValue('transferrable'))
                                    $found = true;
                                else
                                    $_SESSION['modules']['domain']['register']['invalidList'][$domainItem] = "Domain not transferable: " . $refPoint->getElement('reason')->getValue('reason');
                            }
                        }
                        break;
                    }
                if (!$found)
                    array_push($invalidTld, $key);
            }
            if ($invalidTld){
                $invalidTld = array_reverse($invalidTld);
                foreach($invalidTld as $rmKey){
                    if (!$_SESSION['modules']['domain']['register']['invalidList'][$domainListArray[$rmKey]])
                        $_SESSION['modules']['domain']['register']['invalidList'][$domainListArray[$rmKey]] =  "Domain not transferable: Domain does not contain a valid tld.";
                    if (count($domainListArray) -1  == $rmKey)
                        array_pop($domainListArray);
                    else
                        array_splice($domainListArray, $rmKey, $rmKey - count($domainListArray) + 1);
                }
            }
//---- if there is only one domain being transfered switch the transfer type to no longer be bulk
            if (count($domainListArray) == 1){
                $owned = array(array_pop($domainListArray));
                unset($domainListArray);
            }else{
                if (!$domainListArray){
                    $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
                    $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . 'Please enter a valid domain';
                    $retEvent = new ACF_Event('CheckTransferDomain_Failed');
                    $parameterList = new ACF_ParameterList();
                    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
                    $retEvent->SetParameterList($parameterList);
                    return $retEvent;
                }
                foreach ($domainListArray as $domain)
                    $domainList[$domain] = $domain;
//----- set the domain list session variable
                $_SESSION['modules']['domain']['register']['domain_list'] = $domainList;
            }
        }
        $selected = $owned;
    }
//---- merge all of the domains together
    if ($inParameterList->ParameterValue('all'))
        $selected = array_merge($selected, $_SESSION['modules']['domain']['register']['searchResults']);
    elseif ($inParameterList->ParameterValue('domainSelect'))
        $selected = array_merge($selected, $inParameterList->ParameterValue('domainSelect'));
//---- if the array is empty leave
    if (!$selected){
        $status['code'] = 'No Domain Selected';
        $status['text'] = 'Please select a domain before continuing.';
//---- create an acf event object
        $retEvent = new ACF_Event('invalid_input');
//---- appended parameters
        $parameterList  = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
        $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['domain']['register']['domain']['name']));
        $retEvent->SetParameterList($parameterList);
    }else{
//---- Build the session variable
        $_SESSION['modules']['domain']['register']['registerDomain'] = array();
//---- make a short cut reference to save on typing space
        $sessionRef = &$_SESSION['modules']['domain']['register']['registerDomain'];
//---- if the user has already register a domain during this session clear it out
        if (isset($_SESSION['modules']['domain']['register']['loginInfo']['domain']))
            unset($_SESSION['modules']['domain']['register']['loginInfo']['domain']);
        $defaultLock = (strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'])
            == "yes" || strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'])
            === true || strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'])
            == "locked" || $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'] == 1)
            ? 1 : 0;
//---- grab the tlds out of the session variable and find out which ones allow locking
        $tldList = &$_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'];
        $tldLockable = array();
        foreach($tldList as $tldDetails)
            if ((strtolower($tldDetails['allow-locking']) == 'yes') || (strtolower($tldDetails['allow-locking']) === true) || (strtolower($tldDetails['allow-locking']) == 1))
                array_push($tldLockable, $tldDetails['tld']);
//---- loop through the selected domains and build the session variable
        foreach($selected as $domain){
            $sessionRef[$domain]                = array();
            $sessionRef[$domain]['baseName']    = strtolower(substr($domain, 0, strrpos($domain, '.')));
            if ($domain == 'Bulk Transfer')
                $sessionRef[$domain]['tld']     = $bulkTLD;
            else
                $sessionRef[$domain]['tld']     = strtolower(substr($domain, strrpos($domain, '.')));
            $sessionRef[$domain]['default']     = $_SESSION['modules']['domain']['register']['domain']['name'] == $domain ? 1 : 0;
            $sessionRef[$domain]['domain_lock'] = $defaultLock;
            switch ($sessionRef[$domain]['tld']){
                case '.ca':
                    $sessionRef[$domain]['contact_set'] = array("owner" => array(), "admin" => array());
                    if ($customTech)
                        $sessionRef[$domain]['contact_set']['tech'] = array();
                    break;
                case '.eu':
                    $sessionRef[$domain]['contact_set'] = array("owner" => array());
                    if ($customTech)
                        $sessionRef[$domain]['contact_set']['tech'] = array();
                    break;
                default:
                    $sessionRef[$domain]['contact_set'] = array("owner" => array(), "admin" => array(), "billing" => array());
                    if ($customTech)
                    $sessionRef[$domain]['contact_set']['tech'] = array();
            }
//---- check to see if the tld allows locking
            $sessionRef[$domain]['allowLocking'] = array_search($sessionRef[$domain]['tld'], $tldLockable) !== false ? 1 : 0;
//---- check to see if this is a domain transfer and set the period
            if (is_array($owned)){
                $sessionRef[$domain]['owned']  = array_search($domain, $owned) !== false ? 1 : 0;
                $sessionRef[$domain]['period'] = 1;                                          //---- hardcoding the period of a domain transfer for now
            }else
                $sessionRef[$domain]['period'] = $period[$domain];
//---- validate the data
            $validationResult = AH_Validate(array("period" => $sessionRef[$domain]['period']), 'CheckDomain_Failed');
            if (is_object($validationResult)){                                            //---- if an object is returned the validation has failed return it
                $parameterList = $validationResult->ParameterList();
                $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['domain']['register']['domain']['name']));
                $validationResult->SetParameterList($parameterList);
                $_SESSION['modules']['domain']['register']['domain']['name']     = '';
                $_SESSION['modules']['domain']['register']['domain']['baseName'] = '';
                $_SESSION['modules']['domain']['register']['domain']['tld']      = '';
                return $validationResult;
            }
        }
//---- set variables for bundle mode
        if ($_SESSION['ccs']['bundled'])
            $_SESSION['modules']['domain']['register-status'] = $_SESSION['modules']['domain']['register']['registerDomain'];

//---- setup chargeItem for billing Modules
        setDomainChargeItem('register');

//---- create an acf event object
        $parameterList = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('status', ''));
        $retEvent = new ACF_Event('valid_input');
        $retEvent->SetParameterList($parameterList);
    }
//---- if the domain is not being transfered skip some stuff this can only be hit with piwi
    if ($inParameterList->ParameterValue('transfer') == 'no'){
        $_SESSION['ccs']['excludeService'] = true;
        if (is_array($_SESSION['ccs']['excludeSteps'])){
            if (array_search('contacts', $_SESSION['ccs']['excludeSteps']) !== false)
		array_push($_SESSION['ccs']['excludeSteps'], 'contacts');
            if (array_search('configure', $_SESSION['ccs']['excludeSteps']) !== false)
		array_push($_SESSION['ccs']['excludeSteps'], 'configure');
        }else
            $_SESSION['ccs']['excludeSteps'] = array('contacts', 'configure');
        if ($selected)
            $retEvent = new ACF_Event('HopProject');
    }else{
//---- otherwise make sure nothing is skipped
        if (isset($_SESSION['ccs']['excludeService']))
            unset($_SESSION['ccs']['excludeService']);
        if (is_array($_SESSION['ccs']['excludeSteps'])){
            $rmKey = array_search('contacts', $_SESSION['ccs']['excludeSteps']);
            if ($rmKey !== false)
                unset($_SESSION['ccs']['excludeSteps'][$rmKey]);
            $rmKey = array_search('configure', $_SESSION['ccs']['excludeSteps']);
            if ($rmKey !== false)
                unset($_SESSION['ccs']['excludeSteps'][$rmKey]);
        }
//---- if this is a mass transfer remove the configure step
	$config = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options'];
        if (($_SESSION['modules']['domain']['register']['domain_list']) || ($config['allow-domain-lock-change'] == "no"
		&& ($config['allow-register-nameservers'] == "no" || $config['allow-custom-nameservers'] == "no"))){
            if (is_array($_SESSION['ccs']['excludeSteps']))
                array_push($_SESSION['ccs']['excludeSteps'], 'configure');
            else
                $_SESSION['ccs']['excludeSteps'] = array('configure');
        }
    }
//---- return the event
    return $retEvent;
}


//---- Function to check the requested domain
function AH_CheckUsername (ACF_ParameterList $inParameterList){
    $status = array();
    $_SESSION['modules']['domain']['register']['loginInfo'] = array();
    if (!$inParameterList->ParameterValue('login') || !$inParameterList->ParameterValue('password') || $inParameterList->ParameterValue('password') != $inParameterList->ParameterValue('Confirmpassword')){
        $status['code'] = 'Invalid Input';
        $status['text'] = 'Please enter a username and password before continuing.';
//---- create an acf event object
        $eventType = 'invalid_input';
    }else{
        $_SESSION['modules']['domain']['register']['loginInfo']['username'] = trim($inParameterList->ParameterValue('login'));
        $_SESSION['modules']['domain']['register']['loginInfo']['password'] = trim($inParameterList->ParameterValue('password'));
//---- validate the data
        $validationResult = AH_Validate(array("username" => $_SESSION['modules']['domain']['register']['loginInfo']['username']), 'CheckUsername_Failed');
        if (is_object($validationResult)){                                            //---- if an object is returned the validation has failed return it
/*---- this passes back the password to the signup page
            $parameterList = $validationResult->ParameterList();
            $parameterList->AddParameter(new ACF_DirectParameter('pass', $_SESSION['modules']['domain']['register']['loginInfo']['password']));
            $validationResult->SetParameterList($parameterList);
----*/
            return $validationResult;
        }
        $validationResult = AH_Validate(array("password" => $_SESSION['modules']['domain']['register']['loginInfo']['password']), 'CheckUsername_Failed');
        if (is_object($validationResult)){                                            //---- if an object is returned the validation has failed return it
            $parameterList = $validationResult->ParameterList();
            $parameterList->AddParameter(new ACF_DirectParameter('user', $_SESSION['modules']['domain']['register']['loginInfo']['username']));
            $validationResult->SetParameterList($parameterList);
            return $validationResult;
        }
        if (!($_SESSION['ccs']['bundled'] || $inParameterList->ParameterValue('terms'))){
            $status['code'] = 'Invalid Input';
            $status['text'] = 'You must agree with the Terms and Conditions before continuing.';
            $eventType = 'invalid_input';
        }else{
//---- set the shared username and password
            $_SESSION['modules']['shared']['username'] = $domainSuffix['username'];
            $_SESSION['modules']['shared']['password'] = $domainSuffix['password'];
//---- create an acf event object
            $eventType = 'valid_input';
            if ($_SESSION['modules']['shared']['contact_info']){
                foreach($_SESSION['modules']['domain']['register']['registerDomain'] as &$domainData){
                    foreach($domainData['contact_set'] as &$contact){
                        $contact['first_name']  = $_SESSION['modules']['shared']['contact_info']['first_name'];
                        $contact['last_name']   = $_SESSION['modules']['shared']['contact_info']['last_name'];
                        $contact['org_name']    = $_SESSION['modules']['shared']['contact_info']['company'];
                        $contact['address1']    = $_SESSION['modules']['shared']['contact_info']['addr1'];
                        $contact['address2']    = $_SESSION['modules']['shared']['contact_info']['addr2'];
                        $contact['address3']    = $_SESSION['modules']['shared']['contact_info']['addr3'];
                        $contact['city']        = $_SESSION['modules']['shared']['contact_info']['city'];
                        $contact['state']       = $_SESSION['modules']['shared']['contact_info']['state'];
                        $contact['country']     = $_SESSION['modules']['shared']['contact_info']['country'];
                        $contact['postal_code'] = $_SESSION['modules']['shared']['contact_info']['postal_code'];
                        $contact['phone']       = $_SESSION['modules']['shared']['contact_info']['phone'];
                        $contact['fax']         = $_SESSION['modules']['shared']['contact_info']['fax'];
                        $contact['email']       = $_SESSION['modules']['shared']['contact_info']['email'];
                    }
                }
            }
        }
    }
    $retEvent = new ACF_Event($eventType);
//---- appended parameters
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    if ($eventType == "invalid_input"){
        if ($inParameterList->ParameterValue('login'))
            $parameterList->AddParameter(new ACF_DirectParameter('user', $inParameterList->ParameterValue('login')));
/*---- this passes back the password to the signup page
        if ($inParameterList->ParameterValue('password') == $inParameterList->ParameterValue('Confirmpassword'))
            $parameterList->AddParameter(new ACF_DirectParameter('pass', $inParameterList->ParameterValue('password')));
----*/
    }
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to login
function AH_DoLogin (ACF_ParameterList $inParameterList){
//---- grab the global client
    global $client;
    $status        = array();
    $loginUsername = $inParameterList->ParameterValue('login');
    $loginPassword = $inParameterList->ParameterValue('password');
    $loginDomain   = $inParameterList->ParameterValue('domain');
    $customTech    = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-custom-tech-info'];
    $customTech    = $customTech == "yes" || $customTech === true || $customTech == 1 ? true : false;
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.set_cookie');
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('domain', $loginDomain));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_username', $loginUsername));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $loginPassword));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = "Login Failed";
        $status['text']  = $error->getMessage();
        $eventType = 'doLogin_Failed';
    }
    if ($result){
        $status['code'] = $result->getResponseCode();
        $status['text'] = $result->getResponseText();
        if (!$result->isSuccess())
            $eventType  = 'doLogin_Failed';
        else{
//---- the login was successful so grab the cookie
            $cookie             = $result->getResponseData()->getElement("attributes")->getElement('cookie')->getValue();
            $loginDomainSuffix  = substr($loginDomain, strrpos($loginDomain, '.'));
//---- get the contact information
//---- create a service request object
            $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
            $request->addParameter(new TCA_ServiceRequestParameter('cookie', $cookie));
//---- create a parameter set
            $pSet = new TCA_ServiceRequestParameterSet();
//---- attach the parameters to the parameter set
            $pSet->addParameter(new TCA_ServiceRequestParameter('type', 'all_info'));
//---- attach the parameter set to the service request object
            $request->addParameterSet('attributes', $pSet);
//---- attempt to process the service request
            try{
                $result = $client->processServiceRequest($request);
            }
            catch(Exception $error){
                $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . "Exception Thrown";
                $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
                $eventType       = 'doLogin_Failed';
            }
            if ($result)
                if (!$result->isSuccess()){
                    $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
                    $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
                    $eventType       = 'doLogin_Failed';
                }else{
                    $_SESSION['modules']['domain']['register']['loginInfo']['domain']   =  $loginDomain;
                    $_SESSION['modules']['domain']['register']['loginInfo']['username'] =  $loginUsername;
                    $_SESSION['modules']['domain']['register']['loginInfo']['password'] =  $loginPassword;
                    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain => &$domainDetails){
                        switch ($domainDetails['tld']){
                            case ".ca":
                                $type = array('owner', 'admin');
                                if ($customTech)
                                    array_push($type, 'tech');
                                break;
                            case ".eu":
                                $type = array('owner');
                                if ($customTech)
                                    array_push($type,'tech');
                                break;
                            default:
                                $type = array('owner', 'admin', 'billing');
                                if ($customTech)
                                    array_push($type, 'tech');
                        }

                        for ($index=0; $index<count($type); $index++){
                            $contact  = array();
                            $refPoint = $result->getResponseData()->getElement("attributes")->getElement('contact_set')->getElement($type[$index]);
//---- all other accounts non .ca owner
                            if ($domainDetails['tld'] == '.ca'){
                                $refOwner = $result->getResponseData()->getElement('attributes');
//---- .ca account -- owner
                                if ($type[$index] == 'owner'){
                                    $contact['legal_type'] = $refOwner->getElement('legal_type') ?
                                        $refOwner->getElement('legal_type')->getValue('legal_type') : '';
                                    $contact['org_name']    = $refPoint && $refPoint->getElement('org_name') ?
                                        $refPoint->getElement('org_name')->getValue('org_name') : '';
                                    $_SESSION['modules']['domain']['register']['registerDomain'][$domain]['legal_type'] = $contact['legal_type'];
                                }
//---- .ca account -- all other contacts
                                $contact['language'] = $refPoint && $refPoint->getElement('lang') ?
                                    $refPoint->getElement('lang')->getValue('lang') :
                                    ($refPoint && $refPoint->getElement('language') ?
                                        $refPoint->getElement('language')->getValue('language') : 'EN');
                            }
//---- .eu accounts
                            if ($domainDetails['tld'] == '.eu'){
                                $contact['language'] = $refPoint && $refPoint->getElement('lang') ?
                                    $refPoint->getElement('lang')->getValue('lang') :
                                    ($refPoint && $refPoint->getElement('language') ?
                                        $refPoint->getElement('language')->getValue('language') : 'EN');
                                $contact['vat']      = $refPoint && $refPoint->getElement('vat') ?
                                    $refPoint->getElement('vat')->getValue('vat') : '';
                            }
                            $contact['first_name']  = $refPoint && $refPoint->getElement('first_name') ?
                                $refPoint->getElement('first_name')->getValue('first_name') : '';
                            $contact['last_name']   = $refPoint && $refPoint->getElement('last_name') ?
                                $refPoint->getElement('last_name')->getValue('last_name') : '';
                            $contact['org_name']    = $refPoint && $refPoint->getElement('org_name') ?
                                $refPoint->getElement('org_name')->getValue('org_name') : '';
                            $contact['address1'] = $refPoint && $refPoint->getElement('address1') ?
                                $refPoint->getElement('address1')->getValue('address1') : '';
                            $contact['address2'] = $refPoint && $refPoint->getElement('address2') ?
                                $refPoint->getElement('address2')->getValue('address2') : '';
                            $contact['address3'] = $refPoint && $refPoint->getElement('address3') ?
                                $refPoint->getElement('address3')->getValue('address3') : '';
                            $contact['city']        = $refPoint && $refPoint->getElement('city') ?
                                $refPoint->getElement('city')->getValue('city') : '';
                            $contact['state']       = $refPoint && $refPoint->getElement('state') ?
                                $refPoint->getElement('state')->getValue('state') : '';
                            $contact['country']     = $refPoint && $refPoint->getElement('country') ?
                                $refPoint->getElement('country')->getValue('country') : '';
                            $contact['postal_code'] = $refPoint && $refPoint->getElement('postal_code') ?
                                $refPoint->getElement('postal_code')->getValue('postal_code') : '';
                            $contact['phone']       = $refPoint && $refPoint->getElement('phone') ?
                                $refPoint->getElement('phone')->getValue('phone') : '';
                            $contact['fax']         = $refPoint && $refPoint->getElement('fax') ?
                                $refPoint->getElement('fax')->getValue('fax') : '';
                            $contact['email']       = $refPoint && $refPoint->getElement('email') ?
                                $refPoint->getElement('email')->getValue('email') : '';
                            $domainDetails['contact_set'][$type[$index]] = array();
                            $domainDetails['contact_set'][$type[$index]] = $contact;
                        }
                    }
                    $eventType = 'doLogin_Success';
                    $_SESSION['modules']['shared']['username'] = $loginUsername;
                    $_SESSION['modules']['shared']['password'] = $loginPassword;
                    $_SESSION['modules']['shared']['domain']   = $loginDomain;
                }
        }
    }
//---- create an acf event object
    $retEvent       = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- Function to collect the contact info
function AH_AddContactData (ACF_ParameterList $inParameterList){
    $contactData   = stripslashes_deep($inParameterList->ParameterValue('contact'));
    $status        = array();
    foreach($contactData as $domain => $domainData){
        $domainParts = explode(".", $domain);
        foreach($domainData as $type=>$details){
//            if (!($domainParts[count($domainParts) - 1] == 'ca' && $type == 'owner')){
//---- for all .ca tlds
                if ($domainParts[count($domainParts) - 1] == 'ca'){
                    $contact['lang'] = $details['language'] ? $details['language'] : '';
//----   copy legal type from owner contact to main array
                    if ($type == 'owner'){
						$_SESSION['modules']['domain']['register']['registerDomain'][$domain]['legal_type'] = $details['legal_type'] ? $details['legal_type'] : '';
					}
                }
//---- for all .eu tlds
                if ($domainParts[count($domainParts) - 1] == 'eu'){
                    $contact['lang']      = $details['language']    ? $details['language']     : '';
                    $contact['vat']       = $details['vat']         ? $details['vat']          : '';
                }
//---- for all other domain tlds
                $contact['first_name']   = $details['firstName']    ? $details['firstName']    : '';
                $contact['last_name']    = $details['lastName']     ? $details['lastName']     : '';
                $contact['org_name']     = $details['organization'] ? $details['organization'] : '';
                $contact['address1']     = $details['address1']     ? $details['address1']     : '';
                $contact['address2']     = $details['address2']     ? $details['address2']     : '';
                $contact['address3']     = $details['address3']     ? $details['address3']     : '';
                $contact['city']         = $details['city']         ? $details['city']         : '';
                $contact['state']        = $details['state']        ? $details['state']        : '';
                $contact['country']      = $details['country']      ? $details['country']      : '';
                $contact['postal_code']  = $details['postalCode']   ? $details['postalCode']   : '';
                $contact['phone']        = $details['phone']        ? $details['phone']        : '';
                $contact['fax']          = $details['fax']          ? $details['fax']          : '';
                $contact['email']        = $details['email']        ? $details['email']        : '';
//---- update Session variable with the contact array
                if ($type == 'owner'){
                    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as &$domainType){
                        $domainType['contact_set']['owner'] = array();
                        $domainType['contact_set']['owner'] = $contact;
                    }
                }
                if ($type == 'admin'){
                    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as &$domainType){
                        $domainType['contact_set']['admin'] = array();
                        $domainType['contact_set']['admin'] = $contact;
                    }
                }
                if ($type == 'billing'){
                    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as &$domainType){
                        if ($domainParts[count($domainParts) - 1] != 'ca'){
                            $domainType['contact_set']['billing'] = array();
                            $domainType['contact_set']['billing'] = $contact;
                        }
                    }
                }
                if ($type == 'tech'){
                    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as &$domainType){
                        $domainType['contact_set']['tech'] = array();
                        $domainType['contact_set']['tech'] = $contact;
                    }
                }
//            }
        }
    }

//---- check for missing contacts
    $eventType  = 'addContact_Complete';
    $customTech = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-custom-tech-info'];
    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain => &$domainType){
        $currentDomain = $domain;
        if (!$domainType['contact_set']['owner']){
            $eventType      = 'addContact_Failure';
            $status['text'] = 'Please specify the owner contact information';
            break;
        }
        if ((!$domainType['contact_set']['admin']) && (!in_array($domainType['tld'], array('.eu', '.be')))){
            $eventType      = 'addContact_Failure';
            $status['text'] = 'Please specify the administrator contact information';
            break;
        }
        if (!$domainType['contact_set']['billing'] && !in_array($domainType['tld'], array('.ca', '.eu', '.be'))){
            $eventType      = 'addContact_Failure';
            $status['text'] = 'Please specify the billing contact information';
            break;
        }
        if (!$domainType['contact_set']['tech'] && (strtolower($customTech) == 'yes' || $customTech === true || $customTech == 1)){
            $eventType      = 'addContact_Failure';
            $status['text'] = 'Please specify the technical contact information';
            break;
        }
    }
//---- validate the data
    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain => &$domainData){
        $validateSuffix = substr($domainData['tld'], 1);
        if ($domainData['contact_set']['owner']){
            $validationResult = AH_Validate(array("owner" => $domainData['contact_set']['owner']), 'AddContact_Failure', $validateSuffix);
            if (is_object($validationResult))                                            //---- if an object is returned the validation has failed return it
                return $validationResult;
        }
        if ($domainData['contact_set']['admin']){
            $validationResult = AH_Validate(array("admin" => $domainData['contact_set']['admin']), 'AddContact_Failure', $validateSuffix);
            if (is_object($validationResult))                                            //---- if an object is returned the validation has failed return it
                return $validationResult;
        }
        if ($validateSuffix != "ca" && $domainData['contact_set']['billing']){
            $validationResult = AH_Validate(array("billing" => $domainData['contact_set']['billing']), 'AddContact_Failure', $validateSuffix);
            if (is_object($validationResult))                                            //---- if an object is returned the validation has failed return it
                return $validationResult;
        }
        if ($domainData['contact_set']['tech'] && (strtolower($customTech) == 'yes' || $customTech === true || $customTech == 1)){
            $validationResult = AH_Validate(array("tech" => $domainData['contact_set']['tech']), 'AddContact_Failure', $validateSuffix);
            if (is_object($validationResult))                                            //---- if an object is returned the validation has failed return it
                return $validationResult;
        }
    }
//---- create an acf event object
    $retEvent = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- function to check to see if any domains need to collect extra data i.e. .ca
function AH_CheckCIRAData(ACF_ParameterList $inParameterList){
    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain => $domainData){
        if ($domainData['tld'] == '.ca'){
            return new ACF_Event('CollectData');
            break;
        }
    }
    return new ACF_Event('CheckCIRAData_Success');
}


//---- function that gathers and test the ca registrant data
function AH_AddRegistrantData(ACF_ParameterList $inParameterList){
    $status  = array();
    $options = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options'];

    if ($inParameterList->ParameterValue('contact')){
        $contactData = stripslashes_deep($inParameterList->ParameterValue('contact'));
        $eventType = 'VerifyRegistrant';
        $sessionDomains = &$_SESSION['modules']['domain']['register']['registerDomain'];
        foreach($contactData as $domain => $details){
            $contact = array();
            $contact['legal_type']  = $details['legalType']    ? $details['legalType']                  : '';
            $contact['org_name']    = $details['organization'] ? $details['organization']               : '';
//---- set required parameters
//---- set domain specific variables
            if (!is_array($sessionDomains[$domain]['contact_set']))
                $sessionDomains[$domain]['contact_set']      = array();
            $sessionDomains[$domain]['legal_type']           = $contact['legal_type'];
        }
//---- validate the owner contact
        foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain => &$domainData){
            $validateSuffix = substr($domainData['tld'], 1);
            if ($domainData['contact_set']['owner']){
                $validationResult = AH_Validate(array("owner" => $domainData['contact_set']['owner']), 'AddRegistrant_Failure', $validateSuffix);
                if (is_object($validationResult))                       //---- if an object is returned the validation has failed return it
                    return $validationResult;
            }
        }
    }else{
        $registrantNo = $inParameterList->ParameterValue('registrant');
        global $client;
        $request = $client->createServiceRequest('TUCOWS_domains.query_registrant_ca');
//---- create a parameter set
        $pSet = new TCA_ServiceRequestParameterSet();
//---- attach the parameters to the parameter set
        $pSet->addParameter(new TCA_ServiceRequestParameter('rant_no', $registrantNo));
        $pSet->addParameter(new TCA_ServiceRequestParameter('domain', 'testing.ca'));
//---- attach the parameter set to the service request object
        $request->addParameterSet('attributes', $pSet);
//---- attempt to process the service request
        try{
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . "Exception Thrown";
            $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
            $eventType       = 'AddRegistrant_Failure';
        }
        if ($result)
            if (!$result->isSuccess()){
                $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
                $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
                $eventType       = 'AddRegistrant_Failure';
            }else{
                $regRef = $result->getResponseData()->getElement('attributes');
                foreach ($_SESSION['modules']['domain']['register']['registerDomain'] as $domainName => &$domainDetails){
                    if ($domainDetails['tld'] == ".ca"){
                        $domainDetails['contact_set'] = array();
                        $domainDetails['contact_set']['admin']['first_name']  = $regRef->getElement('admin_first_name')->getValue('admin_first_name');
                        $domainDetails['contact_set']['admin']['last_name']   = $regRef->getElement('admin_last_name')->getValue('admin_last_name');
                        $domainDetails['contact_set']['admin']['org_name']    = $regRef->getElement('admin_org_name')->getValue('admin_org_name')
                                                                                ? $regRef->getElement('admin_org_name')->getValue('admin_org_name') : 'temp';
                        $domainDetails['contact_set']['admin']['address1']    = $regRef->getElement('admin_address1')->getValue('admin_address1');
                        $domainDetails['contact_set']['admin']['address2']    = $regRef->getElement('admin_address2')->getValue('admin_address2');
                        $domainDetails['contact_set']['admin']['city']        = $regRef->getElement('admin_city')->getValue('admin_city');
                        $domainDetails['contact_set']['admin']['state']       = $regRef->getElement('admin_state')->getValue('admin_state');
                        $domainDetails['contact_set']['admin']['country']     = $regRef->getElement('admin_country')->getValue('admin_country');
                        $domainDetails['contact_set']['admin']['postal_code'] = $regRef->getElement('admin_postal_code')->getValue('admin_postal_code');
                        $domainDetails['contact_set']['admin']['phone']       = $regRef->getElement('admin_phone')->getValue('admin_phone');
                        $domainDetails['contact_set']['admin']['fax']         = $regRef->getElement('admin_fax')->getValue('admin_fax');
                        $domainDetails['contact_set']['admin']['email']       = $regRef->getElement('admin_email')->getValue('admin_email');
                        $domainDetails['contact_set']['admin']['language']    = $regRef->getElement('admin_lang_pref')->getValue('admin_lang_pref');

                        $domainDetails['contact_set']['owner']['legal_type']  = $regRef->getElement('legal_type')->getValue('legal_type');
                        $domainDetails['contact_set']['owner']['org_name']    = $regRef->getElement('owner_org_name')->getValue('owner_org_name');
                        $domainDetails['contact_set']['owner']['first_name']  = $regRef->getElement('owner_first_name')->getValue('owner_first_name');
                        $domainDetails['contact_set']['owner']['last_name']   = $regRef->getElement('owner_last_name')->getValue('owner_last_name');
                        $domainDetails['contact_set']['owner']['org_name']    = $regRef->getElement('owner_org_name')->getValue('owner_org_name')
                                                                                ? $regRef->getElement('owner_org_name')->getValue('owner_org_name') : 'temp';
                        $domainDetails['contact_set']['owner']['address1']    = $regRef->getElement('owner_address1')->getValue('owner_address1');
                        $domainDetails['contact_set']['owner']['address2']    = $regRef->getElement('owner_address2')->getValue('owner_address2');
                        $domainDetails['contact_set']['owner']['city']        = $regRef->getElement('owner_city')->getValue('owner_city');
                        $domainDetails['contact_set']['owner']['state']       = $regRef->getElement('owner_state')->getValue('owner_state');
                        $domainDetails['contact_set']['owner']['country']     = $regRef->getElement('owner_country')->getValue('owner_country');
                        $domainDetails['contact_set']['owner']['postal_code'] = $regRef->getElement('owner_postal_code')->getValue('owner_postal_code');
                        $domainDetails['contact_set']['owner']['phone']       = $regRef->getElement('owner_phone')->getValue('owner_phone');
                        $domainDetails['contact_set']['owner']['fax']         = $regRef->getElement('owner_fax')->getValue('owner_fax');
                        $domainDetails['contact_set']['owner']['email']       = $regRef->getElement('owner_email')->getValue('owner_email');
                        $domainDetails['contact_set']['owner']['language']    = $regRef->getElement('owner_lang_pref')->getValue('owner_lang_pref');

                        if ($options['allow-custom-tech-info'] == "yes" || $options['allow-custom-tech-info'] === true || $options['allow-custom-tech-info'] == 1){
                            $domainDetails['contact_set']['tech']['first_name']   = $regRef->getElement('tech_first_name')->getValue('tech_first_name');
                            $domainDetails['contact_set']['tech']['last_name']    = $regRef->getElement('tech_last_name')->getValue('tech_last_name');
                            $domainDetails['contact_set']['tech']['city']         = $regRef->getElement('tech_city')->getValue('tech_city');
                            $domainDetails['contact_set']['tech']['postal_code']  = $regRef->getElement('tech_postal_code')->getValue('tech_postal_code');
                            $domainDetails['contact_set']['tech']['org_name']     = $regRef->getElement('tech_org_name')->getValue('tech_org_name')
                                                                                    ? $regRef->getElement('tech_org_name')->getValue('tech_org_name') : 'temp';
                            $domainDetails['contact_set']['tech']['address1']     = $regRef->getElement('tech_address1')->getValue('tech_address1');
                            $domainDetails['contact_set']['tech']['address2']     = $regRef->getElement('tech_address2')->getValue('tech_address2');
                            $domainDetails['contact_set']['tech']['city']         = $regRef->getElement('tech_city')->getValue('tech_city');
                            $domainDetails['contact_set']['tech']['state']        = $regRef->getElement('tech_state')->getValue('tech_state');
                            $domainDetails['contact_set']['tech']['country']      = $regRef->getElement('tech_country')->getValue('tech_country');
                            $domainDetails['contact_set']['tech']['phone']        = $regRef->getElement('tech_phone')->getValue('tech_phone');
                            $domainDetails['contact_set']['tech']['fax']          = $regRef->getElement('tech_fax')->getValue('tech_fax');
                            $domainDetails['contact_set']['tech']['email']        = $regRef->getElement('tech_email')->getValue('tech_email');
                            $domainDetails['contact_set']['tech']['language']     = $regRef->getElement('tech_lang_pref')->getValue('tech_lang_pref');
                        }
//---- set required parameters
                        $domainDetails['legal_type']  = $domainDetails['contact_set']['owner']['legal_type'];
                    }
                }
            }
    }
    if ((!$status && $registrantNo) || !$eventType)
        $eventType = 'AddRegistrant_Success';
//---- create an acf event object
    $retEvent = new ACF_Event($eventType);
//---- appended parameters
    $parameterList  = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $retEvent->SetParameterList($parameterList);
//---- return the event
    return $retEvent;
}


//---- function to check the extra data and see if there should be more i.e. .ca accounts
function AH_SetDomainExtra(ACF_ParameterList $inParameterList){
    if ($inParameterList->ParameterValue('trademark'))
        $trademark = $inParameterList->ParameterValue('trademark');
    if ($inParameterList->ParameterValue('existing'))
        $existing = $inParameterList->ParameterValue('existing');
    if ($inParameterList->ParameterValue('member'))
        $member = $inParameterList->ParameterValue('member');
    if ($inParameterList->ParameterValue('authcode'))
        $authcode = $inParameterList->ParameterValue('authcode');
    if ($trademark){
        foreach($trademark as $key => $details){
            $_SESSION['modules']['domain']['register']['registerDomain'][$key]['trademark']     = $details;
            $_SESSION['modules']['domain']['register']['registerDomain'][$key]['cira_existing'] = $existing[$key];
            $_SESSION['modules']['domain']['register']['registerDomain'][$key]['cira_member']   = $member[$key];
            $eventName = 'GetCAExtra';
        }
    }else{
//---- not sure if this is needed
        $eventName = 'SetDomainExtra_Success';
    }
    return new ACF_Event($eventName);
}


//---- function to skip the configure page
function AH_CheckTransfer(ACF_ParameterList $inParameterList){
    $eventName = (is_array($_SESSION['ccs']['excludeSteps']) && array_search('configure', $_SESSION['ccs']['excludeSteps']) !== false) ? 'BulkTransfer' : 'Continue';
    return new ACF_Event($eventName);
}


//---- Function to check the requested domain
function AH_SetDomainAttrib (ACF_ParameterList $inParameterList){
    $autoRenew   = $inParameterList->ParameterValue('autoRenew') ? $inParameterList->ParameterValue('autoRenew') : 0;
    $lock        = $inParameterList->ParameterValue('lockDomain') ? $inParameterList->ParameterValue('lockDomain') : 0;
    $customRenew = (strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-auto-renew-change'])
                    == 'yes' || $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-auto-renew-change']
                    === true || $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-auto-renew-change']
                    == 1) ? true : false;
    $autoRenewDefault = (strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['auto-renew-default'])
                        == 'yes' || $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['auto-renew-default']
                        === true || $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['auto-renew-default']
                        == 1) ? 1 : 0 ;

    $offerings = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'];
    $defaultLock = (strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'])
            == "yes" || strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'])
            == "true" || strtolower($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'])
            == "locked" || $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['default-status-locked'] == 1)
            ? 1 : 0;

    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as &$domainDetails){
        $domainDetails['auto_renew'] = $customRenew ? $autoRenew : $autoRenewDefault;
        foreach($offerings as $serviceType)
            if ($serviceType['tld'] == $domainDetails['tld'] && (strtolower($serviceType['allow-locking']) == "no"
                    || strtolower($serviceType['allow-locking']) == "false" || $serviceType['allow-locking'] == "0")){
                $domainDetails['domain_lock'] = $defaultLock;
                break;
            }else
                $domainDetails['domain_lock'] = $lock ? 1 : 0;
    }
    if ($inParameterList->ParameterValue('nameserver')){
        $nameservers = $inParameterList->ParameterValue('nameserver');
        foreach($_SESSION['modules']['domain']['register']['registerDomain'] as &$domainDetails){
            $index = 0;
            $validateItems = array();
            $domainDetails['nameserver_list'] = array();
            for ($i=0; $i<count($nameservers); $i++){
                if (strlen($nameservers[$i]) > 0){
                    $details                 = array();
                    $details['sortorder']    = $index + 1;
                    $details['name']         = $nameservers[$i];
                    $validateItems[($index)] = $details['name'];
                    array_push($domainDetails['nameserver_list'], $details);
                    $index++;
                }
            }
//---- validate the data
            $validationResult = AH_Validate(array("dns" => $validateItems), 'SetDomainAttributes_Failed');
            if (is_object($validationResult))                                            //---- if an object is returned the validation has failed return it
                return $validationResult;
        }
        if ($inParameterList->ParameterValue('morenameservers'))
            return new ACF_Event('MoreNameservers');
    }
//---- return the event
    return new ACF_Event('DataStored');
}


//---- function to validate data
function AH_Validate($validateData, $onFailure, $currentSuffix = false){
    $invalidData   = array();
    $status        = array();

    foreach($validateData as $key => $value){
        $ruleLists    = array();
        switch ($key){
            case "domain":                                                                           //---- accepts array("domain" => "<domain>")
                $domainDisp = new TCA_DomainDispatcher;
                try{
                    $ruleRef = $domainDisp->getDomain($value)->getRuleList()->getElement('attributes');
                }
                catch(Exception $error){
                    $status['code']  = 'Invalid Input';
                    $status['text']  = $error->getMessage();
                    break;
                }
                $ruleLists = $ruleRef->getElement('domain');
                $ruleListKeys = $ruleLists->keys();
                for($index=0; $index<count($ruleListKeys); $index++){
                    $ruleSet = $ruleLists->getElement($ruleListKeys[$index]);
                    foreach($ruleSet as $rules){
                        $result = $rules->validate($value);
                        if (!$result->isValid()){
                            if (!$invalidData[$key])
                                $invalidData[$key] = array();
                            array_push($invalidData[$key], $result->__toString());
                        }
                    }
                }
                break;
            case "domain_list":                                                                      //---- accepts array("domain_list" =>
                $rule = TCA_TransferListValidationRule::createRule(1);                               //---- array(0 => "<domain>", ...)
                $result = $rule->validate($value);
                if (!$result->isValid()){
                    if (!$invalidData[$key])
                        $invalidData[$key] = array();
                    array_push($invalidData[$key], $result->__toString());
                }
                break;
            case "username":                                                                         //---- accepts array("username" => "<username>")
                $rule = TCA_NullValidationRule::createRule(1);
                $result = $rule->validate($value);
                if (!$result->isValid()){
                    if (!$invalidData[$key])
                        $invalidData[$key] = array();
                    array_push($invalidData[$key], $result->__toString());
                }
                break;
            case "password":                                                                         //---- accepts array("password" => "<password>")
                $rule = TCA_DomainPasswordValidationRule::createRule(1);
                $result = $rule->validate($value);
                if (!$result->isValid()){
                    if (!$invalidData[$key])
                        $invalidData[$key] = array();
                    array_push($invalidData[$key], $result->__toString());
                }
                break;
            case "owner":                                                                            //---- accepts array("owner" =>
            case "admin":                                                                            //---- array("first_name" => "<first_name>",
            case "tech":                                                                             //---- "last_name" => "<last_name>", ... etc))
            case "billing":
                $domainDisp = new TCA_DomainContactDispatcher();
                try{
                    $ruleRef = $domainDisp->getContact($currentSuffix, $key)->getRuleList()->getElement('attributes');
                }
                catch(Exception $error){
                    $status['code']  = 'Invalid Input';
                    $status['text']  = $error->getMessage();
                    break;
                }
                $ruleLists = $ruleRef->getElement('contact_set')->getElement($key);
                foreach ($value as $reference => $item){
                    try{
                        $ruleSet = $ruleLists->getElement($reference);
                    }
                    catch(Exception $error){
                        $status['code']  = 'Invalid Input';
                        $status['text']  = $error->getMessage();
                        break;
                    }
                    if ($ruleSet){
                        $ruleSetKeys = $ruleSet->keys();
                        for($index=0; $index<count($ruleSetKeys); $index++){
                            $ruleGroup = $ruleSet->getElement($ruleSetKeys[$index]);
                            foreach($ruleGroup as $rules){
                                $result = $rules->validate($item);
                                if (!$result->isValid()){
                                    if (!$invalidData[$reference])
                                        $invalidData[$reference] = array();
                                    array_push($invalidData[$reference], $result->__toString());
                                }
                            }
                        }
                    }
                }
                break;
            case "period":                                                                           //---- accepts array("period" => "<num>")
                $rule = TCA_EnumValidationRule::createRule(0, range(1,10));
                $result = $rule->validate($value);
                if (!$result->isValid()){
                    if (!$invalidData[$key])
                        $invalidData[$key] = array();
                    array_push($invalidData[$key], $result->__toString());
                }
                break;
            case "dns":                                                                              //---- accepts array("nameserverlist" => array(
                $rules  = TCA_MinArrayLengthValidationRule::createRule(1, 2);                        //---- 0 => "<nameserver>", 1 => "<nameserver>"))
                $result = $rules->validate($value);
                if (!$result->isValid()){
                    if (!$invalidData[$key])
                        $invalidData[$key] = array();
                    array_push($invalidData[$key], $result->__toString());
                }
                $index = 0;                                                                          //---- the first two are required so use a flag
                foreach($value as $index=>$dns){
                    $rules  = TCA_HostnameValidationRule::createRule($index < 2 ? 1 : 0);
                    $result = $rules->validate($dns);
                    if (!$result->isValid()){
                        if (!$invalidData[$dns])
                            $invalidData[$dns] = array();
                        array_push($invalidData[$dns], $result->__toString());
                    }else{
//---- manual check to see if there are duplicates
                        $dupCheck = array_keys($value, $dns);
//---- one show the message once per duplicate
                        if (count($dupCheck) > 1 && ((int) $dupCheck[0] >= $index)){
                            if (!$invalidData[$dns])
                                $invalidData[$dns] = array();
                            array_push($invalidData[$dns], "Duplicate nameservers detected.\n");
                        }
                    }
                    $index++;
                }
                break;
            default:
                $status['code']  = 'Invalid Input';
                $status['text']  = 'Invalid type';
        }
    }
    if ($invalidData || $status){
        if ($invalidData)
            foreach($invalidData as $key => $data){
                $status['code'] = 'Invalid Data';
                foreach($data as $dataDetail)
                    $status['text'] = ($status['text'] ? $status['text'] : '') . $key . ': ' . $dataDetail;
            }
//---- create an acf event object
        $retEvent       = new ACF_Event($onFailure);
//---- appended parameters
        $parameterList  = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
        $retEvent->SetParameterList($parameterList);
//---- return the event
        return $retEvent;
    }else
        return true;
}


//---- Function to process the  domain
function AH_ProcessDomain (ACF_ParameterList $inParameterList){
    $_SESSION['modules']['domain']['register']['isSuccess'] = true;
    $_SESSION['modules']['domain']['register']['status']    = array();
    $status = array();
    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain => &$details){
        try{
            $result = registerDomain($domain, $details);
        }
        catch(Exception $error){
            $status['code']  = 'Invalid Input';
            $status['text']  = $error->getMessage();
            $eventType       = 'processDomain_Failed';
            $_SESSION['modules']['domain']['register']['isSuccess'] = false;
            break;
        }
        if ($result){
            $status['code'] = $result->getResponseCode();
            $status['text'] = $result->getResponseText();
            if (!$result->isSuccess()){
                $eventType  = 'processDomain_Failed';
                $_SESSION['modules']['domain']['register']['isSuccess'] = false;
                break;
            }else{
                $refPoint = $result->getResponseData()->getElement("attributes");
                if ($domain === "Bulk Transfer")
                    $details['processId']   = $refPoint->getElement('bulk_transfer_id')->getValue('bulk_transfer_id');
                else{
                    $details['admin_email'] = $refPoint->getElement('admin_email')->getValue('admin_email');
                    $details['processId']   = $refPoint->getElement('id')->getValue('id');
                    $_SESSION['modules']['shared']['email'] = $details['admin_email'];
                }
                $eventType = 'processDomain_Success';
//---- set the domain variable if it is not set so all other domains will be appended to it
                if (!isset($_SESSION['modules']['domain']['register']['loginInfo']['domain']))
                    $_SESSION['modules']['domain']['register']['loginInfo']['domain'] = $domain;
                if (!$_SESSION['modules']['domain']['register']['master_order_id'])
                    $_SESSION['modules']['domain']['register']['master_order_id'] = $details['processId'];
            }
        }
    }
    if ($_SESSION['modules']['domain']['register']['isSuccess']){
        $_SESSION['modules']['shared']['domain']  = $_SESSION['modules']['domain']['register']['loginInfo']['domain'];
        $_SESSION['modules']['shared']['orderid'] = $_SESSION['modules']['domain']['register']['master_order_id'];
    }
    $_SESSION['modules']['domain']['status'] = $status;
//---- create an acf event object
    $retEvent = new ACF_Event($eventType);
//---- return the event
    return $retEvent;
}


function registerDomain ($domain, &$details){
//---- grab the global client
    global $client;
//---- grab the needed options
    $options           = &$_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options'];
    $contacts          = $details['contact_set'];
    $customTech        = ((strtolower($options['allow-custom-tech-info']) == 'yes' || $options['allow-custom-tech-info'] === true || $options['allow-custom-tech-info'] == 1) && count($contacts['tech'])) ? 1 : 0;
    $customNameServers = (strtolower($options['allow-custom-nameservers']) == 'yes' || $options['allow-custom-nameservers'] === true || $options['allow-custom-nameservers'] == 1) ? 1 : 0;
    $defaultLock       = $options['allow-domain-lock-change'] == "no" ? ($options['default-status-locked'] == 'locked' || $options['default-status-locked'] == 1 || $options['default-status-locked'] === true || $options['default-status-locked'] == 'yes' ? 1 : 0) : false;
//---- create a service request object
    if ($domain === "Bulk Transfer")
        $request = $client->createServiceRequest('TUCOWS_domains.bulk_transfer_domain');
    else
        $request = $client->createServiceRequest('TUCOWS_domains.sw_register_domain');
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    if ($domain !== "Bulk Transfer"){
        $pSet->addParameter(new TCA_ServiceRequestParameter('domain', $domain));
        $pSet->addParameter(new TCA_ServiceRequestParameter('reg_type', $details['owned'] ? 'transfer' : 'new'));
        $pSet->addParameter(new TCA_ServiceRequestParameter('auto_renew', $details['auto_renew']));
        $pSet->addParameter(new TCA_ServiceRequestParameter('f_lock_domain', $defaultLock !== false ? $defaultLock : $details['domain_lock']));
        $pSet->addParameter(new TCA_ServiceRequestParameter('custom_nameservers', $customNameServers));
        $pSet->addParameter(new TCA_ServiceRequestParameter('nameserver_list', $customNameServers ? $details['nameserver_list'] ? $details['nameserver_list']: array() : array()));
        $pSet->addParameter(new TCA_ServiceRequestParameter('link_domains', isset($_SESSION['modules']['domain']['register']['loginInfo']['domain']) ? 1:0));
        $pSet->addParameter(new TCA_ServiceRequestParameter('handle', $options['handle-action']));
        $pSet->addParameter(new TCA_ServiceRequestParameter('custom_tech_contact', $customTech));
        $pSet->addParameter(new TCA_ServiceRequestParameter('period', $details['period']));
    }else
        $pSet->addParameter(new TCA_ServiceRequestParameter('domain_list', explode(",", implode(",", $_SESSION['modules']['domain']['register']['domain_list']))));

    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_username', $_SESSION['modules']['domain']['register']['loginInfo']['username']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('reg_password', $_SESSION['modules']['domain']['register']['loginInfo']['password']));
    $pSet->addParameter(new TCA_ServiceRequestParameter('affiliate_id', 'platypus'));
    if (isset($_SESSION['modules']['domain']['register']['loginInfo']['domain'])){
        $pSet->addParameter(new TCA_ServiceRequestParameter('reg_domain', $_SESSION['modules']['domain']['register']['loginInfo']['domain']));
        if ($_SESSION['modules']['domain']['register']['master_order_id'])
            $pSet->addParameter(new TCA_ServiceRequestParameter('master_order_id', $_SESSION['modules']['domain']['register']['master_order_id']));
    }
    if ($details['tld'] == ".ca"){
        $pSet->addParameter(new TCA_ServiceRequestParameter('legal_type', $details['legal_type']));
    }

    if (!$customTech && isset($contacts['tech']))
        unset($contacts['tech']);
    foreach($contacts as $key => $contact)
	if (!count($contact))
	    unset($contacts[$key]);
    $pSet->addParameter(new TCA_ServiceRequestParameter('contact_set', $contacts));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
    return $client->processServiceRequest($request);
}


//---- this function directs the incoming check availablity call to the proper event
function AH_DirectCheckAvailability(ACF_ParameterList $inParameterList){
//---- if a domain is passed in search for it
    if ($_SESSION['modules']['domain']['passed']['domain']){
        $parameterList  = new ACF_ParameterList();
        $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['domain']['passed']['domain']));
        $retEvent = new ACF_Event('lookupDomain');
        $retEvent->SetParameterList($parameterList);
        $_SESSION['modules']['domain']['passed']['domain'] = false;
    }else
        $retEvent = new ACF_Event('default');
    return $retEvent;
}


//---- this function directs the incoming gather info call to the proper event
function AH_DirectGatherInfo(ACF_ParameterList $inParameterList){
    if ($_SESSION['ccs']['bundled'] || $_SESSION['modules']['shared']['domainManagerMode']){
//---- if a username exist in the session use a different event
        if ($_SESSION['modules']['shared']['username']){
            $parameterList  = new ACF_ParameterList();
            $parameterList->AddParameter(new ACF_DirectParameter('login', $_SESSION['modules']['shared']['username']));
            $parameterList->AddParameter(new ACF_DirectParameter('password', $_SESSION['modules']['shared']['password']));
//---- if a domain exist in the session log them in otherwise sign them up
            if ($_SESSION['modules']['shared']['domain']){
                $retEvent = new ACF_Event('login');
                $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['shared']['domain']));
            }else{
                $retEvent = new ACF_Event('signup');
                $parameterList->AddParameter(new ACF_DirectParameter('confirmpassword', $_SESSION['modules']['shared']['password']));
            }
            $retEvent->SetParameterList($parameterList);
        }else
            $retEvent = new ACF_Event('default');
    }else{
//---- always prompt for login
        $retEvent = new ACF_Event('default');
        if ($_SESSION['modules']['shared']['domain']){
            $parameterList  = new ACF_ParameterList();
            $parameterList->AddParameter(new ACF_DirectParameter('domain', $_SESSION['modules']['shared']['domain']));
            $retEvent->SetParameterList($parameterList);
        }
    }
    return $retEvent;
}


function AH_DOM_REG_HopProject(ACF_ParameterList $inParameterList){
//---- This activity handler redirects a user to the given url (for Project hopping)
    // Check for end of GatherInfo and Insert Data into Charge items
    if($inParameterList->ParameterValue('event') == "gatherinformation")
        setDomainChargeItemInfo();

    header("location:".$_SESSION['ccs']['options']['driver-options']['url']."/".$inParameterList->ParameterValue('event')); // Send HTML header
    $retEvent = new ACF_Event('HopProject_Success');
    return $retEvent;
}

?>
