<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file performs gathers the domain manager data                                              ----*/
/*---- These functions are invoked from the acf framework                                              ----*/
/*---------------------------------------------------------------------------------------------------------*/

$_ENV['TCA_HOME']       = $_SESSION['ccs']['options']['tca-options']['tca-home'];
$_ENV['TCA_CARTRIDGES'] = $_SESSION['ccs']['options']['tca-options']['tca-cartridges'];
$_ENV['TCA_ETC']        = $_SESSION['ccs']['options']['tca-options']['tca-etc'];

set_include_path(get_include_path() . $_ENV['pathDelimiter'] . $_ENV['TCA_HOME']);

require_once("./domain/domainFunctions.php");
//---- create a global client object
require_once("client.inc.php");
global $client;
TCA_Client::initEnv();
$client = new TCA_Client(new TCA_XMLConfigReader("tucows-client.xml"));

if($_SESSION['ccs']['options']['driver-options']['capture-xml-communication-log'] === "yes")
    $client->setLogFile($_SESSION['ccs']['options']['driver-options']['xml-communication-log-file']);

//---- this function gets the status for the login screen
function DC_GetLogin(ACF_ParameterList $inParameterList) {
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function returns back the options
function DC_GetOptions(ACF_ParameterList $inParameterList){
//---- assign a variable for quick reference
    $sessionOptions = &$_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options'];
    $properties     = array('allow_domain_lock_change'   => $sessionOptions['allow-domain-lock-change'],
                            'allow_custom_tech_info'     => $sessionOptions['allow-custom-tech-info'],
                            'allow_custom_nameservers'   => $sessionOptions['allow-custom-nameservers'],
                            'allow_manage_nameservers'   => $sessionOptions['allow-manage-nameservers'],
                            'allow_register_nameservers' => $sessionOptions['allow-register-nameservers'],
                            'allow_password_updates'     => $sessionOptions['allow-password-updates'],
                            'allow_change_ownership'     => $sessionOptions['allow-change-ownership'],
                            'isBundle'                   => $_SESSION['ccs']['bundled'] ? true : false);
//---- unify the options
    foreach($properties as &$value)
        $value =  (strtolower($value == 'yes') || $value === true || $value == 1) ? 1 : 0;
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the parameter to the object
    $parameterList->AddParameter(new ACF_DirectParameter('properties', $properties));
    $parameterList->AddParameter(new ACF_DirectParameter('domainmanager', $_SESSION['modules']['domain']['manager']));
//---- if any errors exist grab them
    if ($_SESSION['modules']['domain']['manager']['status']['text'] || $_SESSION['modules']['domain']['manager']['status']['note']){
        $status = array();
        $status = $_SESSION['modules']['domain']['manager']['status'];
        $_SESSION['modules']['domain']['manager']['status'] = array();
    }
//---- if any errors are being passed in append them to the existing errors
    if ($inParameterList->ParameterValue('status')){
        if (!$status)
            $status = array();
        $stat           = $inParameterList->ParameterValue('status');
        $status['text'] = $stat['text'] . "\n" . $status['text'];
        $status['code'] = $stat['code'] . "\n" . $status['code'];
        if ($stat['note'])
            if ($status['note'])
                $status['note'] = $stat['note'] . "\n" . $status['note'];
            else
                $status['note'] = $stat['note'];
    }
    if ($status)
        $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function gets an overview of the domain for the details page
function DC_GetDomainOverview(ACF_ParameterList $inParameterList){
    global $client;
    $contact     = array('tld' => $_SESSION['modules']['domain']['manager']['domainSuffix']);
    $nameservers = array('list' => array());
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- attach the parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('type', 'all_info'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
    try{
	$result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
	$status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
	$status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result){
	if (!$result->isSuccess()){
	    $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
	    $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
	}else{
//---- grab the contact info
	    switch ($contact['tld']){
		case ".ca":
		    $types = array('owner','admin','tech');
		    break;
		case ".eu":
		    $types = array('owner','tech');
		    break;
		default:
		    $types = array('owner','admin','billing','tech');
	    }
	    foreach($types as $type){
//---- put the data into an array to pass back through the acf
		$contact[$type] = array();
		$refPoint = $result->getResponseData()->getElement("attributes")->getElement('contact_set')->getElement($type);
		$contact[$type]['firstName']    = $refPoint->getElement('first_name')->getValue('first_name');
		$contact[$type]['lastName']     = $refPoint->getElement('last_name')->getValue('last_name');
		$contact[$type]['email']        = $refPoint->getElement('email')->getValue('email');
		$contact[$type]['phone']        = $refPoint->getElement('phone')->getValue('phone');
//---- get the approprate title
		switch ($type){
		    case 'admin':
			$contact[$type]['formalType'] = 'Administrative';
			break;
		    case 'billing':
			$contact[$type]['formalType'] = 'Billing';
			break;
		    case 'tech':
			$contact[$type]['formalType'] = 'Technical';
			break;
		    case 'owner':
			$contact[$type]['formalType'] = 'Owner';
			break;
		    default:
			$contact[$type]['formalType'] = "Unknown";
		}
	    }
//---- grab the nameservers
            $refPoint = $result->getResponseData()->getElement('attributes')->getElement('nameserver_list');
//---- loop through the data to build the array
            if ($refPoint)
                for ($index=0; $refPoint->getElement($index); $index++){
//---- flush the array
                    $details               = array();
                    $details['ipAddress']  = $refPoint->getElement($index)->getElement('ipaddress')->getValue('ipaddress');
                    $details['sortOrder']  = $refPoint->getElement($index)->getElement('sortorder')->getValue('sortorder');
                    $details['name']       = $refPoint->getElement($index)->getElement('name')->getValue('name');
//---- push the array onto the parent
                    array_push($nameservers['list'], $details);
                }
        }
    }



//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the parameter to the object
    $parameterList->AddParameter(new ACF_DirectParameter('contact', $contact));
    $parameterList->AddParameter(new ACF_DirectParameter('nameservers', $nameservers));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- countrylist is left for backwards compatablity
    $parameterList->AddParameter(new ACF_DirectParameter('countrylist', getCountry()));
//---- return the parameter list
    return $parameterList;


}


//---- this function gets the contact information based on the type
function DC_GetContact(ACF_ParameterList $inParameterList){
    global $client;
//---- this variable is used for .eu trades clear it out if it exists
    if (isset($_SESSION['modules']['domain']['manager']['oriContact']))
	unset($_SESSION['modules']['domain']['manager']['oriContact']);
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- get the needed options for the page
    $contact = array('tld' => $_SESSION['modules']['domain']['manager']['domainSuffix']);
//---- set the contact types
    switch ($contact['tld']){
	case ".ca":
	    $stat = getCAContactSet($contact);
	    if ($stat){
		$status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $stat['code'];
		$status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $stat['text'];
	    }
	    break;
	case ".eu":
	    $stat = getEUContactSet($contact);
	    if ($stat){
		$status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $stat['code'];
		$status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $stat['text'];
	    }
	    break;
	default:
	    $types = array('owner','admin','billing','tech');
//---- create a service request object
	    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
	    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
	    $pSet = new TCA_ServiceRequestParameterSet();
//---- attach the parameters to the parameter set
	    $pSet->addParameter(new TCA_ServiceRequestParameter('type', 'all_info'));
//---- attach the parameter set to the service request object
	    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
	    try{
		$result = $client->processServiceRequest($request);
	    }
	    catch(Exception $error){
		$status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
		$status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
	    }
	    if ($result){
		if (!$result->isSuccess()){
		    $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
		    $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
		}else{
		    foreach($types as $type){
//---- put the data into an array to pass back through the acf
			$contact[$type] = array();
			$refPoint = $result->getResponseData()->getElement("attributes")->getElement('contact_set')->getElement($type);
			$contact[$type]['firstName']    = $refPoint->getElement('first_name')->getValue('first_name');
			$contact[$type]['lastName']     = $refPoint->getElement('last_name')->getValue('last_name');
			$contact[$type]['organization'] = $refPoint->getElement('org_name')->getValue('org_name');
			$contact[$type]['email']        = $refPoint->getElement('email')->getValue('email');
			$contact[$type]['address1']     = $refPoint->getElement('address1')->getValue('address1');
			$contact[$type]['address2']     = $refPoint->getElement('address2')->getValue('address2');
			$contact[$type]['address3']     = $refPoint->getElement('address3')->getValue('address3');
			$contact[$type]['city']         = $refPoint->getElement('city')->getValue('city');
			$contact[$type]['state']        = $refPoint->getElement('state')->getValue('state');
			$contact[$type]['postalCode']   = $refPoint->getElement('postal_code')->getValue('postal_code');
			$contact[$type]['country']      = $refPoint->getElement('country')->getValue('country');
			$contact[$type]['phone']        = $refPoint->getElement('phone')->getValue('phone');
			$contact[$type]['fax']          = $refPoint->getElement('fax')->getValue('fax');
//---- get the approprate title
			switch ($type){
			    case 'admin':
				$contact[$type]['formalType'] = 'Administrative';
				break;
			    case 'billing':
				$contact[$type]['formalType'] = 'Billing';
				break;
			    case 'tech':
				$contact[$type]['formalType'] = 'Technical';
				break;
			    case 'owner':
				$contact[$type]['formalType'] = 'Owner';
				break;
			    default:
				$contact[$type]['formalType'] = "Unknown";
			}
                    }
            }
        }
    }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the parameter to the object
    $parameterList->AddParameter(new ACF_DirectParameter('contact', $contact));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- countrylist is left for backwards compatablity
    $parameterList->AddParameter(new ACF_DirectParameter('countrylist', getCountry()));
//---- return the parameter list
    return $parameterList;
}


//---- Function to pull the CA data
function getCAContactSet($contact){
    global $client;
    $status = array();
    $types = array('owner','admin','tech');
    foreach($types as $type){
        $details = array();
//---- create a service request object
        $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
        $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
        $pSet = new TCA_ServiceRequestParameterSet();
//---- attach the parameters to the parameter set
        $pSet->addParameter(new TCA_ServiceRequestParameter('type', $type));
//---- attach the parameter set to the service request object
        $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
        try{
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
            $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
        }
        if ($result){
            if (!$result->isSuccess()){
                $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
                $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
            }else{
//---- put the data into an array to pass back through the acf
                $contact[$type] = array();
                $refPoint = $result->getResponseData()->getElement("attributes")->getElement('contact_set')->getElement($type);

                if ($type == 'owner'){
                    $details['legalType']     = $refPoint->getElement('legal_type')->getValue('legal_type');
                }
                $details['language']           = $refPoint->getElement('lang')->getValue('lang');
                $details['firstName']          = $refPoint->getElement('first_name')->getValue('first_name');
                $details['lastName']           = $refPoint->getElement('last_name')->getValue('last_name');
                $details['organization']       = $refPoint->getElement('org_name')->getValue('org_name');
                $details['address1']           = $refPoint->getElement('address1')->getValue('address1');
                $details['address2']           = $refPoint->getElement('address2')->getValue('address2');
                $details['address3']           = $refPoint->getElement('address3')->getValue('address3');
                $details['city']               = $refPoint->getElement('city')->getValue('city');
                $details['state']              = $refPoint->getElement('state')->getValue('state');
                $details['country']            = $refPoint->getElement('country')->getValue('country');
                $details['postalCode']         = $refPoint->getElement('postal_code')->getValue('postal_code');
                $details['phone']              = $refPoint->getElement('phone')->getValue('phone');
                $details['fax']                = $refPoint->getElement('fax')->getValue('fax');
                $details['email']              = $refPoint->getElement('email')->getValue('email');

//---- get the approprate title
                switch ($type){
                    case 'admin':
                        $details['formalType'] = 'Administrative';
                        break;
                    case 'tech':
                        $details['formalType'] = 'Technical';
                        break;
                    case 'owner':
                        $details['formalType'] = 'Owner';
                        break;
                    default:
                        $details['formalType'] = "Unknown";
                }
            }
        }
        $contact[$type] = $details;
    }
    return $status;
}


//---- function to grab the eu contact set
function getEUContactSet($contact){
    global $client;
    $status = array();
    $types = array('owner','tech');
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- attach the parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('type', 'all_info'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
    try{
	$result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
	$status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
	$status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result){
	if (!$result->isSuccess()){
	    $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
	    $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
	}else{
	    foreach($types as $type){
//---- put the data into an array to pass back through the acf
		$contact[$type] = array();
		$refPoint = $result->getResponseData()->getElement("attributes")->getElement('contact_set')->getElement($type);
		$contact[$type]['language']     = $refPoint->getElement('lang')->getValue('lang');
		$contact[$type]['firstName']    = $refPoint->getElement('first_name')->getValue('first_name');
		$contact[$type]['lastName']     = $refPoint->getElement('last_name')->getValue('last_name');
		$contact[$type]['organization'] = $refPoint->getElement('org_name')->getValue('org_name');
		$contact[$type]['email']        = $refPoint->getElement('email')->getValue('email');
		$contact[$type]['address1']     = $refPoint->getElement('address1')->getValue('address1');
		$contact[$type]['address2']     = $refPoint->getElement('address2')->getValue('address2');
		$contact[$type]['address3']     = $refPoint->getElement('address3')->getValue('address3');
		$contact[$type]['city']         = $refPoint->getElement('city')->getValue('city');
		$contact[$type]['state']        = $refPoint->getElement('state')->getValue('state');
		$contact[$type]['postalCode']   = $refPoint->getElement('postal_code')->getValue('postal_code');
		$contact[$type]['country']      = $refPoint->getElement('country')->getValue('country');
		$contact[$type]['phone']        = $refPoint->getElement('phone')->getValue('phone');
		$contact[$type]['fax']          = $refPoint->getElement('fax')->getValue('fax');
		$contact[$type]['vat']          = $refPoint->getElement('vat')->getValue('vat');
//---- get the approprate title
		switch ($type){
		    case 'owner':
			$contact[$type]['formalType'] = 'Owner';
			$_SESSION['modules']['domain']['manager']['oriContact']['firstName']    = $contact['owner']['firstName'];
			$_SESSION['modules']['domain']['manager']['oriContact']['lastName']     = $contact['owner']['lastName'];
			$_SESSION['modules']['domain']['manager']['oriContact']['organization'] = $contact['owner']['organization'];
			break;
                    case 'tech':
                        $details['formalType'] = 'Technical';
			break;
		    default:
			$contact[$type]['formalType'] = "Unknown";
		}
	    }
	}
    }
    return $status;
}


//---- this function gets available domains
function DC_GetDomainList(ACF_ParameterList $inParameterList) {
    global $client;
//---- grab the current status
    $status         = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- get the needed options for the page
    $domain         = array('current' => $_SESSION['modules']['domain']['manager']['domain'], 'list' => array());
    $domain['page'] = $inParameterList->ParameterValue('page') ? $inParameterList->ParameterValue('page') : "0";
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('type', 'list'));
    $pSet->addParameter(new TCA_ServiceRequestParameter('page', $domain['page']));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result)
        if (!$result->isSuccess())
            $domain['error'] = "Information failed to load";
        else{
//---- put the data into an array to pass back through the acf
            $refPoint = $result->getResponseData()->getElement("attributes")->getElement("domain_list");
            $index = 0;
            while ($refPoint->getElement($index)){
                $data = array();
                $data['name'] = $refPoint->getElement($index)->getValue($index);
                $refPoint2 = $result->getResponseData()->getElement("attributes")->getElement("ext_results")->getElement($index)->getElement($data['name']);
                $data['let_expire']     = $refPoint2->getElement('let_expire')->getValue('let_expire');
                $data['sponsoring_rsp'] = $refPoint2->getElement('sponsoring_rsp')->getValue('sponsoring_rsp');
                $data['auto_renew']     = $refPoint2->getElement('auto_renew')->getValue('auto_renew');
                $data['lock_state']     = $refPoint2->getElement('lock_state')->getValue('lock_state');
                $data['expiredate']     = $refPoint2->getElement('expiredate')->getValue('expiredate');
//---- valid tld check
                $tld = substr($data['name'], strrpos($data['name'], '.'));
                $validTld = false;
                foreach ($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'] as $allowedServices){
                    if (($allowedServices['tld'] === $tld) && (strtolower($allowedServices['active']) == "yes" || strtolower($allowedServices['active']) == "true" || $allowedServices['active'] == 1)){
                        $validTld = true;
                        break;
                    }
                }
                $data['validTLD'] = $validTld ? 1 : 0;
                array_push($domain['list'], $data);
                $index++;
            }
            $domain['count']     = $result->getResponseData()->getElement("attributes")->getElement("count")->getValue("count");
            $domain['remainder'] = $result->getResponseData()->getElement("attributes")->getElement("remainder")->getValue("remainder");
        }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the parsmeters to the object
    $parameterList->AddParameter(new ACF_DirectParameter('domain', $domain));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function loads the Name Servers
function DC_GetNameServers(ACF_ParameterList $inParameterList) {
    global $client;
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- get the needed options for the page
    $nameservers = array('list' => array());
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('type', 'nameservers'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result){
//---- get data into an array
        if ($result->isSuccess()){
            $refPoint = $result->getResponseData()->getElement('attributes')->getElement('nameserver_list');
//---- loop through the data to build the array
            if ($refPoint)
                for ($index=0; $refPoint->getElement($index); $index++){
//---- flush the array
                    $details               = array();
                    $details['ipAddress']  = $refPoint->getElement($index)->getElement('ipaddress')->getValue('ipaddress');
                    $details['sortOrder']  = $refPoint->getElement($index)->getElement('sortorder')->getValue('sortorder');
                    $details['name']       = $refPoint->getElement($index)->getElement('name')->getValue('name');
//---- push the array onto the parent
                    array_push($nameservers['list'], $details);
                }
        }else{
            $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
            $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
        }
    }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the parameters to the object
    $parameterList->AddParameter(new ACF_DirectParameter('nameservers', $nameservers));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function gathers various domain details
function DC_GetDomainDetails(ACF_ParameterList $inParameterList) {
    global $client;
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- get the needed options for the page
    $domainDetails = array();
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('type', 'status'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result){
//---- get data into an array
        if ($result->isSuccess()){
            $domainDetails['can_modify']      = $result->getResponseData()->getElement('attributes')->getElement('can_modify')->getValue('can_modify');
            $domainDetails['domain_supports'] = $result->getResponseData()->getElement('attributes')->getElement('domain_supports')->getValue('domain_supports');
            $domainDetails['lock_state']      = $result->getResponseData()->getElement('attributes')->getElement('lock_state')->getValue('lock_state') ? 'Locked' : 'Unlocked';
	    $domainDetails['name']            = $_SESSION['modules']['domain']['manager']['domain'];
            $result = null;
//---- create a service request object
            $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
            $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
            $pset = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
            $pset->addParameter(new TCA_ServiceRequestParameter('type', 'auto_renew_flag'));
//---- attach the parameter set to the service request object
            $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
            try {
                $result = $client->processServiceRequest($request);
            }
            catch(Exception $error){
                $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
                $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
            }
            if ($result){
//---- get data into an array
                if ($result->isSuccess()){
                    $domainDetails['expiredate'] = $result->getResponseData()->getElement('attributes')->getElement('expiredate')->getValue('expiredate');
                }else{
                    $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
                    $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
                }
            }
        }else{
            $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
            $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
        }
    }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the parameters to the object
    $parameterList->AddParameter(new ACF_DirectParameter('domaindetails', $domainDetails));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function gets the extra data for the domain
function DC_GetDomainExtras(ACF_ParameterList $inParameterList) {
    global $client;
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- get the needed options for the page
    $extras  = array('details' => array());
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- get the approprate type
    switch ($_SESSION['modules']['domain']['manager']['domainSuffix']){
        case '.com':
        case '.net':
        case '.info':
        case '.biz':
        case '.org':
        case '.mobi':
            $type = 'domain_auth_info';
            break;
        case '.ca':
            $type = 'all_info';
            break;
        default:
//---- create a ACF parameter object to attach the data to
            $parameterList = new ACF_ParameterList();
            $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
            return $parameterList;
    }
//---- attach the parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('type', $type));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result)
        if (!$result->isSuccess()){
            $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . $result->getResponseCode();
            $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $result->getResponseText();
        }else{
            $refPoint = $result->getResponseData()->getElement("attributes");
//---- get the correct data
            switch ($_SESSION['modules']['domain']['manager']['domainSuffix']){
                case '.com':
                case '.net':
                case '.info':
                case '.biz' :
                case '.org' :
                case '.mobi':
                    $detail          = array();
                    $detail['name']  = 'Domain Authorization Code';                                                 //---- this is the title for this item
                    $detail['value'] = $refPoint->getElement('domain_auth_info')->getValue('domain_auth_info');     //---- this is the current value
                    array_push($extras['details'], $detail);
                    break;
                case '.ca':
                    $detail            = array();
                    $detail['name']    = 'Legal Type';
                    $detail['value' ]  = $refPoint->getElement('legal_type')->getValue('legal_type');
                    array_push($extras['details'], $detail);
                    break;
            }
        }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $parameterList->AddParameter(new ACF_DirectParameter('extras', $extras));
//---- return the parameter list
    return $parameterList;
}


//---- this function loads the local Name Servers
function DC_GetLocalNameServers(ACF_ParameterList $inParameterList) {
    global $client;
    $_SESSION['modules']['domain']['manager']['nameserverList'] = array();
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- put the data into an array to pass it back through the acf
    $localNameServers            = array('list' => array(), 'domain' => $_SESSION['modules']['domain']['manager']['domain']);
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_nameserver');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add the parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('name', 'all'));
//---- add the parameter set to the service request object
    $request->addParameterSet('attributes',$pset);
//---- attempt to process the service request
    try {
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result){
//---- Get the data into an array
        if ($result->getResponseData()->getElement('attributes')){
            $refPoint = $result->getResponseData()->getElement('attributes')->getElement('nameserver_list');
//---- loop through the data to build the array
            for ($index=0; $refPoint->getElement($index); $index++){
//---- flush the array
                $details               = array();
                $details['ipAddress']  = $refPoint->getElement($index)->getElement('ipaddress')->getValue('ipaddress');
                $details['canDelete']  = $refPoint->getElement($index)->getElement('can_delete')->getValue('can_delete');
                $details['name']       = $refPoint->getElement($index)->getElement('name')->getValue('name');
                if ($details['canDelete'])
                    array_push($_SESSION['modules']['domain']['manager']['nameserverList'], $details['name']);
//---- strip the domain off of the name server
                $details['name']       = substr($details['name'], 0, strrpos($details['name'], $_SESSION['modules']['domain']['manager']['domain']) - 1);
//---- push the array onto the parent
                array_push($localNameServers['list'], $details);
            }
        }
    }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the data to the parameter list
    $parameterList->AddParameter(new ACF_DirectParameter('localnameservers', $localNameServers));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function gets the domain renewal information
function DC_GetDomainRenewal(ACF_ParameterList $inParameterList) {
    global $client;
//---- grab the current status
    $status  = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
    $domain  = array();
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('type', 'all_info'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result)
        if (!$result->isSuccess())
            $domain['error'] = "Information failed to load";
        else{
//---- put the data into an array to pass back through the acf
            $refPoint                  = $result->getResponseData()->getElement("attributes");
            $domain['currentExpiry']   = $refPoint->getElement('expiredate')->getValue('expiredate');
            $domain['expireYear']      = date(Y, strtotime($domain['currentExpiry']));
            $domain['autoRenew']       = $refPoint->getElement('auto_renew')->getValue('auto_renew');
        }

//---- search the config options to set the renewal years by
    if ($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering']){
		foreach($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'] as $tld){
	   		if ($tld['tld'] == $_SESSION['modules']['domain']['manager']['domainSuffix']){
//---- if the domain is a .eu or .mobi only allow 1 year renewals
				if ($tld['tld'] == ".eu" || $tld['tld'] == ".mobi"){
		    		$years = array('1'  => '1 year');
		    		break;
				}

				if (isset($tld['price']['period-pricing-renew']) && is_array($tld['price']['period-pricing-renew'])){
		    		$years = array();
		    		foreach ($tld['price']['period-pricing-renew'] as $year => $price)
						if($price !== false)
			    			$years[$year + 1] = ($year + 1) . ($year == 0 ? " year" : " years");
				}

				break;
	    	}
		}
    }

    if (!$years)
		$years = array('0' => '0 years');
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
//---- attach the parsmeters to the object
    $parameterList->AddParameter(new ACF_DirectParameter('domain', $domain));
    $parameterList->AddParameter(new ACF_DirectParameter('renewyears', $years));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function gets the list of countries
function DC_GetCountryList(ACF_ParameterList $inParameterList) {
//---- get the needed options for the page
    $country = getCountry();
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('country', $country));
//---- return the parameter list
    return $parameterList;
}


//---- this function gets the reseller information
function DC_GetResellerContact(ACF_ParameterList $inParameterList) {
    global $client;
//---- grab the current status
    $status    = $inParameterList->ParameterValue('status') ? $inParameterList->ParameterValue('status') : array();
//---- get the needed options for the page
    $contact   = array('type' => $inParameterList->ParameterValue('type'), 'details' => array(), error => '');
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.get_domain');
//---- attach the cookie as a parameter
    $request->addParameter(new TCA_ServiceRequestParameter('cookie', $_SESSION['modules']['domain']['manager']['cookie']));
//---- create a parameter set
    $pSet = new TCA_ServiceRequestParameterSet();
//---- add parameters to the parameter set
    $pSet->addParameter(new TCA_ServiceRequestParameter('type', 'rsp_whois_info'));
//---- attach the parameter set to the service request object
    $request->addParameterSet('attributes',$pSet);
//---- attempt to process the service request object
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code']  = ($status['code'] ? $status['code'] . "\n" : '') . 'Exception Thrown';
        $status['text']  = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
    if ($result)
        if (!$result->isSuccess())
            $contact['error'] = "Information failed to load";
        else{
//---- put the data into an array to pass back through the acf
            $refPoint                  = $result->getResponseData()->getElement("attributes");
            $contact['url']            = $refPoint->getElement('url')->getValue('url');
            $contact['domainEnabled']  = $refPoint->getElement('domain_enabled')->getValue('domain_enabled');
            $contact['rspEnabled']     = $refPoint->getElement('rsp_enabled')->getValue('rsp_enabled');
            $contact['fax']            = $refPoint->getElement('fax')->getValue('fax');
            $contact['business']       = $refPoint->getElement('business')->getValue('business');
            $contact['optInfo']        = $refPoint->getElement('opt_info')->getValue('opt_info');
            $contact['email']          = $refPoint->getElement('email')->getValue('email');
            $contact['phone']          = $refPoint->getElement('phone')->getValue('phone');
            $contact['formalType']     = 'Reseller';
        }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('contact', $contact));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function is used to build the contact page
function DC_GetContactQuerySet(ACF_ParameterList $inParameterList){
//---- $query['attrib'] = string exists on each query, the template will add these directly into the html tag
    $select   = array();
    $querySet = array();
    switch ($_SESSION['modules']['domain']['manager']['domainSuffix']){
        case '.ca':
            $types = array('admin','tech');
            foreach($types as $type){
//---- put the data into an array
                $querySet[$type] = array('queryList' => array());
//---- Language
                $query = GenerateTemplateField('language', 'select', 'Preferred Language:', true, '', '', 'languageList');
                array_push($querySet[$type]['queryList'], $query);
//---- add to the select array
                $select['languageList'] = array('EN' => 'English', 'FR' => 'French');
//---- First Name
                $query = GenerateTemplateField('firstName', 'text', 'First Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Last Name
                $query = GenerateTemplateField('lastName', 'text', 'Last Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Organization Name
                $query = GenerateTemplateField('organization', 'text', 'Company Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Email
                $query = GenerateTemplateField('email', 'text', 'Email:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Phone Number
                $query = GenerateTemplateField('phone', 'text', 'Phone Number:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Fax Number
                $query = GenerateTemplateField('fax', 'text', 'Fax Number:');
                array_push($querySet[$type]['queryList'], $query);
//---- Street Address
                $query = GenerateTemplateField('address1', 'text', 'Street Address:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Suite Number
                $query = GenerateTemplateField('address2', 'text', 'Suite #:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Address 2
                $query = GenerateTemplateField('address3', 'text', 'Address 2:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- City
                $query = GenerateTemplateField('city', 'text', 'City:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- State
                $query = GenerateTemplateField('state', 'text', 'State/Province:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Country
                $query = GenerateTemplateField('country', 'select', 'Country:', true, '', '', 'countryList');
                array_push($querySet[$type]['queryList'], $query);
//---- add to the select array
                $select['countryList'] = getCountry();
//---- Postal Code
                $query = GenerateTemplateField('postalCode', 'text', 'Zip/Postal Code:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- get the approprate title
                switch ($type){
                    case 'admin':
                        $querySet[$type]['name'] = 'Admin';
                        break;
                    case 'tech':
                        $querySet[$type]['name'] = 'Technical';
                        break;
                    default:
                        $querySet[$type]['name'] = "Unknown";
                }
            }
            break;
        case '.eu':
            $types = array('owner', 'tech');
            foreach($types as $type){
//---- put the data into an array
                $querySet[$type] = array('queryList' => array());
//---- Language
                $query = GenerateTemplateField('language', 'select', 'Preferred Language:', true, '', '', 'languageList');
                array_push($querySet[$type]['queryList'], $query);
//---- add to the select array
                $select['languageList'] = array("cs" => "Czech",
                                                "da" => "Danish",
                                                "de" => "German",
                                                "et" => "Estonian",
                                                "el" => "Greek",
                                                "en" => "English",
                                                "es" => "Spanish",
                                                "fi" => "Finnish",
                                                "fr" => "French",
                                                "it" => "Italian",
                                                "lv" => "Latvian",
                                                "lt" => "Lithuanian",
                                                "hu" => "Hungarian",
                                                "mt" => "Maltese",
                                                "nl" => "Dutch",
                                                "pl" => "Polish",
                                                "pt" => "Portuguese",
                                                "sk" => "Slovak",
                                                "sl" => "Slovenian",
                                                "sv" => "Swedish");
//---- First Name
                $query = GenerateTemplateField('firstName', 'text', 'First Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Last Name
                $query = GenerateTemplateField('lastName', 'text', 'Last Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Organization Name
//---- When trades get implemented the if condition needs to be removed
		if ($type == 'owner')
		    $query = GenerateTemplateField('organization', 'text', 'Company Name:', false, '', ' READONLY');
		else
		    $query = GenerateTemplateField('organization', 'text', 'Company Name:');
                array_push($querySet[$type]['queryList'], $query);
//---- Email
                $query = GenerateTemplateField('email', 'text', 'Email:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Phone Number
                $query = GenerateTemplateField('phone', 'text', 'Phone Number:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Fax Number
                $query = GenerateTemplateField('fax', 'text', 'Fax Number:');
                array_push($querySet[$type]['queryList'], $query);
//---- Street Address
                $query = GenerateTemplateField('address1', 'text', 'Street Address:');
                array_push($querySet[$type]['queryList'], $query);
//---- Suite Number
                $query = GenerateTemplateField('address2', 'text', 'Suite #:');
                array_push($querySet[$type]['queryList'], $query);
//---- Address 2
                $query = GenerateTemplateField('address3', 'text', 'Address 2:');
                array_push($querySet[$type]['queryList'], $query);
//---- City
                $query = GenerateTemplateField('city', 'text', 'City:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- State
                $query = GenerateTemplateField('state', 'text', 'State/Province:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Country
                $query = GenerateTemplateField('country', 'select', 'Country:', true, '', '', 'countryList');
                array_push($querySet[$type]['queryList'], $query);
//---- add to the select array
                $select['countryList'] = getCountry('.eu');
//---- Postal Code
                $query = GenerateTemplateField('postalCode', 'text', 'ZIP/Postal Code:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- VAT Registration Number
                $query = GenerateTemplateField('vat', 'text', 'VAT Registration Number:', false, '', 'maxlength="20"');
                array_push($querySet[$type]['queryList'], $query);
//---- get the approprate title
                switch ($type){
                    case 'owner':
                        $querySet[$type]['name'] = 'Owner';
                        break;
                    case 'tech':
                        $querySet[$type]['name'] = 'Technical';
                        break;
                    default:
                        $querySet[$type]['name'] = "Unknown";
                }
            }
            break;
        default:
            $types = array('owner','admin','billing','tech');
            foreach($types as $type){
//---- put the data into an array
                $querySet[$type] = array('queryList' => array());
//---- First Name
                $query = GenerateTemplateField('firstName', 'text', 'First Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Last Name
                $query = GenerateTemplateField('lastName', 'text', 'Last Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Organization Name
                $query = GenerateTemplateField('organization', 'text', 'Company Name:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Email
                $query = GenerateTemplateField('email', 'text', 'Email:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Phone Number
                $query = GenerateTemplateField('phone', 'text', 'Phone Number:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Fax Number
                $query = GenerateTemplateField('fax', 'text', 'Fax Number:');
                array_push($querySet[$type]['queryList'], $query);
//---- Street Address
                $query = GenerateTemplateField('address1', 'text', 'Street Address:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Suite Number
                $query = GenerateTemplateField('address2', 'text', 'Suite #:');
                array_push($querySet[$type]['queryList'], $query);
//---- Address 2
                $query = GenerateTemplateField('address3', 'text', 'Address 2:');
                array_push($querySet[$type]['queryList'], $query);
//---- City
                $query = GenerateTemplateField('city', 'text', 'City:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- State
                $query = GenerateTemplateField('state', 'text', 'State/Province:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- Country
                $query = GenerateTemplateField('country', 'select', 'Country:', true, '', '', 'countryList');
                array_push($querySet[$type]['queryList'], $query);
//---- add to the select array
                $select['countryList'] = getCountry();
//---- Postal Code
                $query = GenerateTemplateField('postalCode', 'text', 'ZIP/Postal Code:', true);
                array_push($querySet[$type]['queryList'], $query);
//---- get the approprate title
                switch ($type){
                    case 'admin':
                        $querySet[$type]['name'] = 'Admin';
                        break;
                    case 'billing':
                        $querySet[$type]['name'] = 'Billing';
                        break;
                    case 'tech':
                        $querySet[$type]['name'] = 'Technical';
                        break;
                    case 'owner':
                        $querySet[$type]['name'] = 'Owner';
                        break;
                    default:
                        $querySet[$type]['name'] = "Unknown";
                }
            }
    }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('queryset', $querySet));
    if ($select)
        $parameterList->AddParameter(new ACF_DirectParameter('select', $select));
//---- return the parameter list
    return $parameterList;
}
?>
