<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file performs gathers the domain registration data                                         ----*/
/*---- These functions are invoked from the acf framework                                              ----*/
/*---------------------------------------------------------------------------------------------------------*/

$_ENV['TCA_HOME']       = $_SESSION['ccs']['options']['tca-options']['tca-home'];
$_ENV['TCA_CARTRIDGES'] = $_SESSION['ccs']['options']['tca-options']['tca-cartridges'];
$_ENV['TCA_ETC']        = $_SESSION['ccs']['options']['tca-options']['tca-etc'];

set_include_path(get_include_path() . $_ENV['pathDelimiter'] . $_ENV['TCA_HOME']);

require_once('whois.inc.php');
require_once("./domain/domainFunctions.php");
//---- create a global client object
require_once("client.inc.php");

require_once('../../libs/ccs/utilities.inc.php');

global $client;
TCA_Client::initEnv();
$client = new TCA_Client(new TCA_XMLConfigReader("tucows-client.xml"));

if($_SESSION['ccs']['options']['driver-options']['capture-xml-communication-log'] === "yes")
    $client->setLogFile($_SESSION['ccs']['options']['driver-options']['xml-communication-log-file']);

//---- this function pulls the default data that is need on almost every page
function DC_GetDefaultData(ACF_ParameterList $inParameterList){
    $status = array();
//---- assign a variable for quick reference
    $sessionOptions = &$_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options'];
    $properties     = array('allow_domain_lock_change'   => $sessionOptions['allow-domain-lock-change'],
                            'default_status_locked'      => $sessionOptions['default-status-locked'],
                            'allow_password_recovery'    => $sessionOptions['allow-password-recovery'],
                            'allow_custom_tech_info'     => $sessionOptions['allow-custom-tech-info'],
                            'allow_custom_nameservers'   => $sessionOptions['allow-custom-nameservers'],
                            'allow_manage_nameservers'   => $sessionOptions['allow-manage-nameservers'],
                            'allow_register_nameservers' => $sessionOptions['allow-register-nameservers'],
                            'allow-auto-renew-change'    => $sessionOptions['allow-auto-renew-change'],
                            'auto-renew-default'         => $sessionOptions['auto-renew-default'],
                            'isBundle'                   => $_SESSION['ccs']['bundled'] ? true : false);
//---- unify the options
    foreach($properties as &$value)
        $value =  (strtolower($value == 'yes') || $value === true || $value == 1) ? 1 : 0;
//---- create a ACF parameter object to attach the data
    $parameterList = new ACF_ParameterList();
//---- if any errors exist grab them
    if ($_SESSION['modules']['domain']['register']['status']['text']){
        $status = $_SESSION['modules']['domain']['register']['status'];
        $_SESSION['modules']['domain']['register']['status'] = array();
    }
//---- if any errors are being passed in append them to the existing errors
    if ($inParameterList->ParameterValue('status')){
        $stat           = $inParameterList->ParameterValue('status');
        $status['text'] = $stat['text'] . "\n" . $status['text'];
        $status['code'] = $stat['code'] . "\n" . $status['code'];
        if ($stat['note'])
            if ($status['note'])
                $status['note'] = $stat['note'] . "\n" . $status['note'];
            else
                $status['note'] = $stat['note'];
    }
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    $parameterList->AddParameter(new ACF_DirectParameter('registerDomain', $_SESSION['modules']['domain']['register']['registerDomain']));
    if ($_SESSION['modules']['domain']['register']['invalidList'])
        $parameterList->AddParameter(new ACF_DirectParameter('invalid_list', $_SESSION['modules']['domain']['register']['invalidList']));
    if ($_SESSION['modules']['domain']['register']['domain_list'])
        $parameterList->AddParameter(new ACF_DirectParameter('domain_list', $_SESSION['modules']['domain']['register']['domain_list']));
    if ($_SESSION['modules']['domain']['register']['domain']['details']['parts']['municipal'])
        $parameterList->AddParameter(new ACF_DirectParameter('municipal', true));
    $parameterList->AddParameter(new ACF_DirectParameter('properties', $properties));
//---- return the parameter list
    return $parameterList;
}


//---- this function checks to see if a domain is being transfered or not
function DC_CheckTransfer(ACF_ParameterList $inParameterList){
    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain){
        if ($domain['owned']){
            $transfer = true;
            break;
        }
    }
    $parameterList = new ACF_ParameterList();
    if ($_SESSION['modules']['domain']['register']['domain_list'] || $transfer)
        $parameterList->AddParameter(new ACF_DirectParameter('transfer', true));
//---- return the parameter list
    return $parameterList;
}


//---- this function pulls the available tlds and their prices
function DC_GetAvailableTLDs(ACF_ParameterList $inParameterList){
//---- get the domain services that are offered
    $serviceOfferings = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'];
    $tldList = array();
//---- Loop through the services and grab the tlds and their prices
    foreach($serviceOfferings as $service)
        if (strtolower($service['active']) == "yes" || strtolower($service['active']) == "true" || $service['active'] == 1){
            $details          = array();
            $details['tld']   = $service['tld'];
            $details['price'] = $service['price']['new'];
            array_push($tldList, $details);
        }
//---- create a ACF parameter object to attach the data
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('tldlist', $tldList));
    return $parameterList;
}


//---- this function searches for the requested domain
function DC_LookupDomain(ACF_ParameterList $inParameterList){
    global $client;
    $status = array();
    $_SESSION['modules']['domain']['register']['searchResults'] = array();
//---- create the session array
    if (!isset($_SESSION['modules']['domain']['register']['registerDomain']))
        $_SESSION['modules']['domain']['register']['registerDomain'] = array();
//---- strip the domain name apart
    $inDomain = $inParameterList->ParameterValue('domain');
    if (is_array($inDomain))
        $inDomain = implode("", $inDomain);
    $inDomain = strtolower(trim($inDomain));
    $domain = array('details'=>array(), 'name'=>$inDomain);
    if (strrpos($inDomain, '.') !== false){
        $domain['baseName'] = substr($inDomain, 0, strrpos($inDomain, '.'));
        $domain['tld']   = substr($inDomain, strrpos($inDomain, '.'));
    }else{
        $domain['baseName'] = $inDomain;
        $domain['tld']   = '';
    }
//---- find out what group of tlds to look for
    $defaultGroup = array();
    $tldGroup     = array();
    $tldGroups    = $_SESSION['ccs']['tldGroups'];

    foreach($tldGroups['group'] as $group){
        if (is_array($group['tld']) && in_array($domain['tld'], $group['tld']))
            $tldGroup = $group['tld'];
        elseif ($domain['tld'] ===  $group['tld'])
            array_push($tldGroup, $group['tld']);
//---- grab the default group while we are looping through
        if (is_array($group['tld']) && in_array('default', $group['tld']))
            $defaultGroup = $group['tld'];
        elseif ('default' ===  $group['tld'])
            array_push($defaultGroup, $group['tld']);
    }
//---- if no matches where found use the default group
    if (!$tldGroup)
        $tldGroup = $defaultGroup;

//---- get the domain services that are offered
    $serviceOfferings = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'];
    $tld   = array();
//---- Loop through the services and grab the tlds
    foreach($serviceOfferings as $service)
        if (in_array($service['tld'], $tldGroup) && ((($domain['tld'] == '.ca' && $service['tld'] == '.ca') || ($domain['tld'] != '.ca' && $service['tld'] != '.ca')) && (strtolower($service['active']) == "yes" || strtolower($service['active']) == "true" || $service['active'] == 1)))
            array_push($tld, $service['tld']);
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.lookup_domain');
//---- loop through the available domain tlds
    for($index=0; $index<count($tld) && !$exceptionError; $index++){
        $result = null;
//---- create a parameter set
        $pset = new TCA_ServiceRequestParameterSet();
//---- add the parameters to the parameter set
        $pset->addParameter(new TCA_ServiceRequestParameter('domain', $domain['baseName'].$tld[$index]));
//---- add the parameter set to the service request object
        $request->addParameterSet('attributes', $pset);
//---- attempt to process the service request
        try{
            $result = $client->processServiceRequest($request);
        }
        catch(Exception $error){
            $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
            $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
            $exceptionError = true;
        }
//---- check to see if the requested domain is available
        if ($result){
//---- Get the data into an array
            $details = array('text' => $result->getResponseText(), 'code' => $result->getResponseCode());
//---- if there was an error grab it
            if (!$result->isSuccess() && $domain['tld'] === $tld[$index]){
                $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . $details['code'];
                $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $tld[$index] . " Search : " .$details['text'];
            }
//---- if this is a .ca account get extra data
            if ($tld[$index] == ".ca"){
                $details['domain'] = $domain['baseName'].$tld[$index];
                if ($result->isSuccess()){
                    $details['status'] = $result->getResponseData()->getElement('attributes')->getElement('status')->getValue('status');
                    getCAData($details);
                    $domain['baseName'] = $details['parts']['base'];
                    if ($details['code'] == 500){
                        $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . $details['code'];
                        $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $tld[$index] . " Search : " .$details['text'];
                    }
                }
                $_SESSION['modules']['domain']['register']['domain']['details'] = $details;
                $domain['details'] =  $details;
                array_push($_SESSION['modules']['domain']['register']['searchResults'], $details['domain']);
            }else{
                if (($details['code'] == '210') || ($details['code'] == '211' && $domain['tld'] === $tld[$index])){
                    if ($result->getResponseData()->getElement('attributes')){
                        $details['domain'] = $domain['baseName'].$tld[$index];
                        $details['tld']    = $tld[$index];
                        $details['status'] = $result->getResponseData()->getElement('attributes')->getElement('status')->getValue('status');
                        $_SESSION['modules']['domain']['register']['domain']['details'] = $details;
                        array_push($domain['details'], $details);
                        array_push($_SESSION['modules']['domain']['register']['searchResults'], $details['domain']);
                    }
                }
            }
        }
    }
    if ($inParameterList->ParameterValue('status')){
        $stat           = $inParameterList->ParameterValue('status');
        $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . $stat['code'];
        $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $stat['text'];
    }
//---- check to see if the tld that was searched for exists in the system
    if ($domain['tld'] && !in_array($domain['tld'], $tld)){
        $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . "Invalid TLD";
        $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . "Extension of domain name that you have entered is not recognized: syntax of
this domain name is either invalid, or this is currently unsupported extension";
    }

//---- create a ACF parameter object to attach the data
    $parameterList = new ACF_ParameterList();
//---- attach the data to the parameter list
    $parameterList->AddParameter(new ACF_DirectParameter('domain', $domain));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
    if ($_SESSION['modules']['domain']['passed']['register_period']) {
	$parameterList->AddParameter(new ACF_DirectParameter('period', $_SESSION['modules']['domain']['passed']['register_period']));
    }
//---- return the parameter list
    return $parameterList;
}


//---- this function is attempts to get the admin email addr
function DC_GetWhois(ACF_ParameterList $inParameterList){
//---- if they did not request the data leave
    if (!$inParameterList->ParameterValue('whois'))
        return new ACF_ParameterList();
    global $client;
    $status    = array();
    $whoisData = array('domain' => $inParameterList->ParameterValue('whois'));
//---- create a service request object
    $request = $client->createServiceRequest('TUCOWS_domains.check_transfer');
//---- create a parameter set
    $pset = new TCA_ServiceRequestParameterSet();
//---- add the parameters to the parameter set
    $pset->addParameter(new TCA_ServiceRequestParameter('domain', $whoisData['domain']));
    $pset->addParameter(new TCA_ServiceRequestParameter('get_request_address', 1));
//---- add the parameter set to the service request object
    $request->addParameterSet('attributes', $pset);
//---- attempt to process the service request
    try{
        $result = $client->processServiceRequest($request);
    }
    catch(Exception $error){
        $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
        $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
    }
//---- check to see if the requested domain is available
    if ($result){
//---- Get the data into an array
        if ($result->getResponseCode() != 200){
            $status['text'] = $result->getResponseText();
            $status['code'] = $result->getResponseCode();
        }else{
            $refPoint = $result->getResponseData()->getElement('attributes');
            if ($refPoint->getElement('transferrable')->getValue('transferrable')){
                $whoisData['type'] = $refPoint->getElement('type')->getValue('type');
                if ($refPoint->getElement('request_address'))
                    $whoisData['request_address'] = $refPoint->getElement('request_address')->getValue('request_address');
            }else
                $whoisData['reason'] = $refPoint->getElement('reason')->getValue('reason');
        }
    }
//---- create a ACF parameter object to attach the data
    $parameterList = new ACF_ParameterList();
//---- attach the data to the parameter list
    $parameterList->AddParameter(new ACF_DirectParameter('whoisdata', $whoisData));
    $parameterList->AddParameter(new ACF_DirectParameter('status', $status));
//---- return the parameter list
    return $parameterList;
}


//---- this function gets a detailed whois query
function DC_GetDetailedWhois(ACF_ParameterList $inParameterList){
    $status = array();
    $wr = new whois_request($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['whois-server']);
    $wr->query($inParameterList->ParameterValue('domain'));
    $response = $wr->getResponse();
//---- create a ACF parameter object to attach the data
    $parameterList = new ACF_ParameterList();
//---- attach the data to the parameter list
    $parameterList->AddParameter(new ACF_DirectParameter('whoisdata', $response));
//---- return the parameter list
    return $parameterList;
}


//---- this function returns a country list array
function DC_GetCountryList(ACF_ParameterList $inParameterList){
//---- create a ACF parameter object to attach the data
    $parameterList = new ACF_ParameterList();
//---- attach the data to the parameter list
    $parameterList->AddParameter(new ACF_DirectParameter('country', getCountry()));
//---- return the parameter list
    return $parameterList;
}


//---- function to do .ca specific lookups
function getCAData(&$details){
    global $client;
    $caData = array();
    $details['available'] = array();
//---- pull the domain apart
    $domainParts = explode('.', $details['domain']);
    $details['parts'] = array();
    $details['parts']['municipal'] = (count($domainParts) == 4) ? $domainParts[0] . "." : '';
    $details['parts']['base']      = (count($domainParts) >= 3) ? $domainParts[(count($domainParts) - 3)] : $domainParts[(count($domainParts) - 2)];
    $details['parts']['province']  = (count($domainParts) >= 3) ? "." . $domainParts[(count($domainParts) - 2)] : '';
    $details['parts']['tld']       = '.ca';
    switch ($details['code']){
        case '210':
//---- all is good leave
            break;
        case '211':
//---- the domain is taken get other possiblities
            $provinces = array('.ca', '.ab.ca', '.bc.ca', '.mb.ca', '.nb.ca', '.nl.ca', '.ns.ca',
                                '.nt.ca',  '.nu.ca', '.on.ca', '.pe.ca', '.qc.ca', '.sk.ca', '.yk.ca');
//---- create a service request object
            $request = $client->createServiceRequest('TUCOWS_domains.lookup_domain');
            foreach($provinces as $province){
                if ($province !== $details['parts']['province'] . $details['parts']['tld']){
                    $domain = $details['parts']['municipal'] . $details['parts']['base'] . $province;
//---- create a parameter set
                    $pset = new TCA_ServiceRequestParameterSet();
//---- add the parameters to the parameter set
                    $pset->addParameter(new TCA_ServiceRequestParameter('domain', $domain));
//---- add the parameter set to the service request object
                    $request->addParameterSet('attributes', $pset);
//---- attempt to process the service request
                    try{
                        $result = $client->processServiceRequest($request);
                    }
                    catch(Exception $error){
                        $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
                        $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
                    }
//---- check to see if the requested domain is available
                    if ($result && ($result->getResponseCode() == 246 || $result->getResponseCode() == 210 || $result->getResponseCode() == 200))
                        array_push($details['available'], $domain);
                }
            }
            break;
        case '246':
        case '247':
        case '500':
//---- get the list of blocking domains
            $request = $client->createServiceRequest('TUCOWS_domains.ca_blocker_list_domain');
            $pset = new TCA_ServiceRequestParameterSet();
            $pset->addParameter(new TCA_ServiceRequestParameter('domain', $details['domain']));
            $request->addParameterSet('attributes', $pset);
//---- attempt to process the service request
            try{
                $result = $client->processServiceRequest($request);
            }
            catch(Exception $error){
                $status['code'] = ($status['code'] ? $status['code'] . "\n" : '') . 'Invalid input';
                $status['text'] = ($status['text'] ? $status['text'] . "\n" : '') . $error->getMessage();
            }
            if ($result){
                if ($result->getResponseCode() == 200){
                    if ($result->getResponseData()->getElement('attributes')->getElement(blocker_list)){
                        $details['blocker_list'] = array();
                        $blockerRef  = $result->getResponseData()->getElement('attributes')->getElement(blocker_list);
                        $blockerKeys = $blockerRef->keys();
                        foreach ($blockerKeys as $domainName){
                            $blocker = array('domain' => $domainName);
                            $blocker['registrar']      = $blockerRef->getElement($domainName)->getElement('registrar')->getValue('registrar');
                            $blocker['owner_org_name'] = $blockerRef->getElement($domainName)->getElement('owner_org_name')->getValue('owner_org_name');
                            $blocker['tech_email']     = $blockerRef->getElement($domainName)->getElement('tech_email')->getValue('tech_email');
                            $blocker['admin_email']    = $blockerRef->getElement($domainName)->getElement('admin_email')->getValue('admin_email');
                            array_push($details['blocker_list'], $blocker);
                        }
                    }
                }
            }
            break;
    }
}


//---- this function returns registration pricing
function DC_GetNewPricing(ACF_ParameterList $inParameterList){
//---- create a ACF parameter object to attach the data
    $parameterList = new ACF_ParameterList();

    $periodPricing = array();
    foreach($_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['service-offering-list']['service-offering'] as $offering){
        $periodPricing[$offering['tld']]['register'] = array();
        $periodPricing[$offering['tld']]['transfer'] = array();
        if($offering['price']['period-pricing-new']){
            foreach($offering['price']['period-pricing-new'] as $key=>$price){
                if($price !== false){
                    array_push($periodPricing[$offering['tld']]['register'],array('period'=>$key+1,'price'=>trim(fixMoney($price))));
//---- if the domain is a .eu only allow one year
                    if ($offering['tld'] == '.eu')
                        break;
                }
            }
        }

        if($offering['price']['period-pricing-transfer']){
            foreach($offering['price']['period-pricing-transfer'] as $key=>$price){
                if($price !== false){
                    array_push($periodPricing[$offering['tld']]['transfer'],array('period'=>$key+1,'price'=>trim(fixMoney($price))));
//---- if the domain is a .eu only allow one year
                    if ($offering['tld'] == '.eu')
                        break;
                }
            }
        }
    }
//---- attach the data to the parameter list
    $parameterList->AddParameter(new ACF_DirectParameter('pricing', $periodPricing));
//---- return the parameter list
    return $parameterList;
}


//---- this function is used to build the contact page
function DC_GetContactQuerySet(ACF_ParameterList $inParameterList){
    $select     = array();
    $querySet   = array();
    $customTech = $_SESSION['ccs']['options']['driver-options']['service-modules']['domain']['service-options']['allow-custom-tech-info'];
    $customTech = $customTech == "yes" || $customTech === true || $customTech == 1 ? true : false;
    foreach($_SESSION['modules']['domain']['register']['registerDomain'] as $domain){
        $querySet[$domain['tld']] = array();
        switch ($domain['tld']){
            case '.ca':
                if (is_array($caSet)){
                    $querySet[$domain['tld']] = $caSet;
                    break;
                }
                $types = array('owner', 'admin');
                if ($customTech)
                    array_push($types, 'tech');
                foreach($types as $type){
//---- put the data into an array
                    $querySet[$domain['tld']][$type] = array('queryList' => array());
//---- Legal Type (added by Justin 9/22/2011)
					if ($type == 'owner') {
	                    $query = GenerateTemplateField('legal_type', 'select', 'Legal Type:', true, '', '', 'legalType');
	                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);

						$select['legalType'] = array('CCO' => 'Corporation (Canada or Canadian province or territory)',
													 'CCT' => 'Canadian citizen',
													 'RES' => 'Permanent Resident of Canada',
													 'GOV' => 'Government or government entity in Canada',
													 'EDU' => 'Canadian Educational Institution',
													 'ASS' => 'Canadian Unincorporated Association',
													 'HOP' => 'Canadian Hospital',
													 'PRT' => 'Partnership Registered in Canada',
													 'TDM' => 'Trade-mark registered in Canada (by a non-Canadian owner)',
													 'TRD' => 'Canadian Trade Union',
													 'PLT' => 'Canadian Political Party',
													 'LAM' => 'Canadian Library, Archive or Museum',
													 'TRS' => 'Trust established in Canada',
													 'ABO' => 'Aboriginal Peoples (individuals or groups) indigenous to Canada',
													 'INB' => 'Indian Band recognized by the Indian Act of Canada',
													 'LGR' => 'Legal Representative of a Canadian Citizen or Permanent Resident',
													 'OMK' => 'Official mark registered in Canada',
													 'MAJ' => 'Her Majesty the Queen');
					}
//---- Language
                    $query = GenerateTemplateField('language', 'select', 'Preferred Language:', true, '', '', 'languageList');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- add to the select array
                    $select['languageList'] = array('EN' => 'English', 'FR' => 'French');
//---- First Name
                    $query = GenerateTemplateField('firstName', 'text', 'First Name:', true, 'first_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Last Name
                    $query = GenerateTemplateField('lastName', 'text', 'Last Name:', true, 'last_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Organization Name
                    $query = GenerateTemplateField('organization', 'text', 'Company Name:', true, 'org_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Email
                    $query = GenerateTemplateField('email', 'text', 'Email:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Phone Number
                    $query = GenerateTemplateField('phone', 'text', 'Phone Number:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Fax Number
                    $query = GenerateTemplateField('fax', 'text', 'Fax Number:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Street Address
                    $query = GenerateTemplateField('address1', 'text', 'Street Address:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Suite Number
                    $query = GenerateTemplateField('address2', 'text', 'Suite #:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Address 2
                    $query = GenerateTemplateField('address3', 'text', 'Address 2:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- City
                    $query = GenerateTemplateField('city', 'text', 'City:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- State
                    $query = GenerateTemplateField('state', 'text', 'State/Province:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Country
                    $query = GenerateTemplateField('country', 'select', 'Country:', true, '', '', 'countryList');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- add to the select array
                    $select['countryList'] = getCountry();
//---- Postal Code
                    $query = GenerateTemplateField('postalCode', 'text', 'ZIP/Postal Code:', true, 'postal_code');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- get the approprate title
                    switch ($type){
                        case 'owner':
                            $querySet[$domain['tld']][$type]['name'] = 'Owner';
                            break;
                        case 'admin':
                            $querySet[$domain['tld']][$type]['name'] = 'Admin';
                            break;
                        case 'tech':
                            $querySet[$domain['tld']][$type]['name'] = 'Technical';
                            break;
                        default:
                            $querySet[$domain['tld']][$type]['name'] = "Unknown";
                    }
                }
                $caSet = $querySet[$domain['tld']];
                break;
            case '.eu':
                if (is_array($euSet)){
                    $querySet[$domain['tld']] = $euSet;
                    break;
                }
                $types = array('owner');
                if ($customTech)
                    array_push($types, 'tech');
                foreach($types as $type){
//---- put the data into an array
                    $querySet[$domain['tld']][$type] = array('queryList' => array());
//---- Language
                    $query = GenerateTemplateField('language', 'select', 'Preferred Language:', true, '', '', 'languageList');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- add to the select array
                    $select['languageList'] = array("cs" => "Czech",
                                                    "da" => "Danish",
                                                    "de" => "German",
                                                    "et" => "Estonian",
                                                    "el" => "Greek",
                                                    "en" => "English",
                                                    "es" => "Spanish",
                                                    "fi" => "Finnish",
                                                    "fr" => "French",
                                                    "it" => "Italian",
                                                    "lv" => "Latvian",
                                                    "lt" => "Lithuanian",
                                                    "hu" => "Hungarian",
                                                    "mt" => "Maltese",
                                                    "nl" => "Dutch",
                                                    "pl" => "Polish",
                                                    "pt" => "Portuguese",
                                                    "sk" => "Slovak",
                                                    "sl" => "Slovenian",
                                                    "sv" => "Swedish");
//---- First Name
                    $query = GenerateTemplateField('firstName', 'text', 'First Name:', true, 'first_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Last Name
                    $query = GenerateTemplateField('lastName', 'text', 'Last Name:', true, 'last_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Organization Name
                    $query = GenerateTemplateField('organization', 'text', 'Company Name:', true, 'org_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Email
                    $query = GenerateTemplateField('email', 'text', 'Email:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Phone Number
                    $query = GenerateTemplateField('phone', 'text', 'Phone Number:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Fax Number
                    $query = GenerateTemplateField('fax', 'text', 'Fax Number:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Street Address
                    $query = GenerateTemplateField('address1', 'text', 'Street Address:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Suite Number
                    $query = GenerateTemplateField('address2', 'text', 'Suite #:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Address 2
                    $query = GenerateTemplateField('address3', 'text', 'Address 2:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- City
                    $query = GenerateTemplateField('city', 'text', 'City:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- State
                    $query = GenerateTemplateField('state', 'text', 'State/Province:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Country
                    $query = GenerateTemplateField('country', 'select', 'Country:', true, '', '', 'countryList');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- add to the select array
                    $select['countryList'] = getCountry('.eu');
//---- Postal Code
                    $query = GenerateTemplateField('postalCode', 'text', 'ZIP/Postal Code:', true, 'postal_code');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- VAT Registration Number
                    $query = GenerateTemplateField('vat', 'text', 'VAT Registration Number:', false, '', 'maxlength="20"');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- get the approprate title
                    switch ($type){
                        case 'owner':
                            $querySet[$domain['tld']][$type]['name'] = 'Owner';
                            break;
                        case 'tech':
                            $querySet[$domain['tld']][$type]['name'] = 'Technical';
                            break;
                        default:
                            $querySet[$domain['tld']][$type]['name'] = "Unknown";
                    }
                }
                $euSet = $querySet[$domain['tld']];
                break;
            default:
                if (is_array($defaultSet)){
                    $querySet[$domain['tld']] = $defaultSet;
                    break;
                }
                $types = array('owner','admin','billing');
                if ($customTech)
                    array_push($types, 'tech');
                foreach($types as $type){
//---- put the data into an array
                    $querySet[$domain['tld']][$type] = array('queryList' => array());
//---- First Name
                    $query = GenerateTemplateField('firstName', 'text', 'First Name:', true, 'first_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Last Name
                    $query = GenerateTemplateField('lastName', 'text', 'Last Name:', true, 'last_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Organization Name
                    $query = GenerateTemplateField('organization', 'text', 'Company Name:', true, 'org_name');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Email
                    $query = GenerateTemplateField('email', 'text', 'Email:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Phone Number
                    $query = GenerateTemplateField('phone', 'text', 'Phone Number:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Fax Number
                    $query = GenerateTemplateField('fax', 'text', 'Fax Number:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Street Address
                    $query = GenerateTemplateField('address1', 'text', 'Street Address:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Suite Number
                    $query = GenerateTemplateField('address2', 'text', 'Suite #:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Address 2
                    $query = GenerateTemplateField('address3', 'text', 'Address 2:');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- City
                    $query = GenerateTemplateField('city', 'text', 'City:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- State
                    $query = GenerateTemplateField('state', 'text', 'State/Province:', true);
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- Country
                    $query = GenerateTemplateField('country', 'select', 'Country:', true, '', '', 'countryList');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- add to the select array
                    $select['countryList'] = getCountry();
//---- Postal Code
                    $query = GenerateTemplateField('postalCode', 'text', 'ZIP/Postal Code:', true, 'postal_code');
                    array_push($querySet[$domain['tld']][$type]['queryList'], $query);
//---- get the approprate title
                    switch ($type){
                        case 'admin':
                            $querySet[$domain['tld']][$type]['name'] = 'Admin';
                            break;
                        case 'billing':
                            $querySet[$domain['tld']][$type]['name'] = 'Billing';
                            break;
                        case 'tech':
                            $querySet[$domain['tld']][$type]['name'] = 'Technical';
                            break;
                        case 'owner':
                            $querySet[$domain['tld']][$type]['name'] = 'Owner';
                            break;
                        default:
                            $querySet[$domain['tld']][$type]['name'] = "Unknown";
                    }
                }
                $defaultSet = $querySet[$domain['tld']];
        }
    }
//---- create a ACF parameter object to attach the data to
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('queryset', $querySet));
    if ($select)
        $parameterList->AddParameter(new ACF_DirectParameter('select', $select));
//---- return the parameter list
    return $parameterList;
}

function DC_GetInfoText(ACF_ParameterList $inParameterList) {
	$type = array();
	$infotext = array();
	$type = explode(",", $inParameterList->ParameterValue('type'));

	if(isset($type) && is_array($type)) {
		foreach($type as $filename) {
			if(file_exists("../../etc/infotext/".$filename.".info"))
				$infotext[$filename] = file_get_contents("../../etc/infotext/".$filename.".info");
		}
	}

	if(!$infotext)
		$infotext = "Information Not Found";

	$paramList = new ACF_ParameterList();
	$paramList->AddParameter(new ACF_DirectParameter('infotext',$infotext));
	return $paramList;
}

?>
