<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file performs the ticket interactions for the helpdesk pages                               ----*/
/*---- These functions are invoked from the acf framework                                              ----*/
/*---------------------------------------------------------------------------------------------------------*/

require_once('../../libs/platypus/wow.inc.php');
require_once("../../libs/ccs/options.inc.php");
//---- load the CCS Options have to laod it here because this is the first manditory function call
if (!$_SESSION['ccs'])
    loadCCSOptions();


function AH_ReassignTicket(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('ticket_Reassign');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $inParameterList->ParameterValue('ticketid'), 'int');
    $helpdesk->addParameter('nStaffID', $inParameterList->ParameterValue('staff'), 'int');
//---- execute it
    if (!$helpdesk->execute())
        $error = $helpdesk->lastError;
    if ($inParameterList->ParameterValue('returnpage'))
	$eventName = 'reassignticket_ticket';
    else
	$eventName = "reassignTicket_Success";

//---- return the proper event
    $retEvent = new ACF_Event($eventName);
    $parameterList = new ACF_ParameterList();
    if ($inParameterList->ParameterValue('xml')){
//---- convert the results to xml
	$oDomDoc = new domDocument();
	if ($error){
	    $root = $oDomDoc->createElement('error');
	    $oDomDoc->appendChild($root);
	    $root->appendChild($oDomDoc->createTextNode(trim($error)));
	}else{
	    $root = $oDomDoc->createElement('ticketid');
	    $oDomDoc->appendChild($root);
	    $root->appendChild($oDomDoc->createTextNode($inParameterList->ParameterValue('ticketid')));
	}
	$parameterList->AddParameter(new ACF_DirectParameter('xml', $oDomDoc->savexml()));
    }else{
	if ($helpdesk->lastError)
	    $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
	$parameterList->AddParameter(new ACF_DirectParameter('ticketid', $inParameterList->ParameterValue('ticketid')));
    }
    $retEvent->SetParameterList($parameterList);
    return $retEvent;
}


function AH_StealTicket(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('ticket_Steal');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $_SESSION['helpdesk']['ticketid'], 'int');
//---- execute it
    $helpdesk->execute();

    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- return the proper event
    $retEvent = new ACF_Event('stealticket_success');
    return $retEvent;
}


function AH_UpdateTicket(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
    $ticket = $inParameterList->ParameterValue('ticket');
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('ticket_Save');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $_SESSION['helpdesk']['ticketid'], 'int');
    if (isset($ticket['category']))
        $helpdesk->addParameter('nCategory', $ticket['category'], 'int');
    if ($inParameterList->ParameterValue('contact'))
        $helpdesk->addParameter('cContact', $inParameterList->ParameterValue('contact'));
    if ($inParameterList->ParameterValue('custid'))
        $helpdesk->addParameter('nCustomer_id', $inParameterList->ParameterValue('custid'), 'int');
    if (isset($ticket['email']))
        $helpdesk->addParameter('cEmail', trim($ticket['email']));
    if (isset($ticket['hidden']))
        $helpdesk->addParameter('cHidden', $ticket['hidden']);
    if (isset($ticket['knowledgebase_id']))
        $helpdesk->addParameter('nKnowledgebase_id', $ticket['knowledgebase_id'], 'int');
    if (isset($ticket['knowledgebase_type']))
        $helpdesk->addParameter('cKnowledgebase_type', $ticket['knowledgebase_type']);
    if (isset($ticket['priority']))
        $helpdesk->addParameter('nPriority', $ticket['priority'], 'int');
    if (isset($ticket['problem']))
        $helpdesk->addParameter('cProblem', trim($ticket['problem']));
    if (isset($ticket['project_id']))
        $helpdesk->addParameter('nProject_id', $ticket['project_id'], 'int');
    if (isset($ticket['solution']))
        $helpdesk->addParameter('cSolution', trim($ticket['solution']));
    if (isset($ticket['staff_assigned']))
        $helpdesk->addParameter('nStaff_assigned', $ticket['staff_assigned'], 'int');
    if (isset($ticket['status']))
        $helpdesk->addParameter('nStatus', $ticket['status'], 'int');
    if (isset($ticket['status_progression']))
        $helpdesk->addParameter('nStatus_progression', $ticket['status_progression'], 'int');
    if (isset($ticket['type']))
        $helpdesk->addParameter('nType', $ticket['type'], 'int');
    if ($inParameterList->ParameterValue('svc_id'))
        $helpdesk->addParameter('nService', $inParameterList->ParameterValue('svc_id'), 'int');
    else
        $helpdesk->addParameter('nService', 0, 'int');
    if ($inParameterList->ParameterValue('svc_instance_id'))
        $helpdesk->addParameter('nInstance', $inParameterList->ParameterValue('svc_instance_id'), 'int');
    else
        $helpdesk->addParameter('nInstance', 0, 'int');
//---- execute it
    if ($helpdesk->execute()){
	if ($inParameterList->ParameterValue('loadCust') == "Y"){
	    $retEvent = new ACF_Event('LoadCustomer');
	    $parameterList = new ACF_ParameterList();
	    if (strtoupper($inParameterList->ParameterValue('contact')) == "Y")
		$parameterList->AddParameter(new ACF_DirectParameter('custid', 0));
	    else
		$parameterList->AddParameter(new ACF_DirectParameter('custid', $inParameterList->ParameterValue('custid')));
	    $retEvent->SetParameterList($parameterList);
	}else
	    $retEvent = new ACF_Event('updateticket_ticket');
    }else
	$retEvent = new ACF_Event('updateticket_ticket');
    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- return the proper event
    return $retEvent;
}

function AH_CancelNewMessage(ACF_ParameterList $inParameterList) {
	$helpdesk = new WOW_Request('','','wombat');

	$helpdesk->setMethod('email_Cancel');

	$helpdesk->setusername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');

	$helpdesk->addParameter('cAction','new','string');
	$helpdesk->addParameter('nEmail',$inParameterList->ParameterValue('messageid'),'int');
	$helpdesk->addParameter('cType','ticket','string');
	$helpdesk->addParameter('nTypeID',$_SESSION['helpdesk']['ticketid'],'int');

	$helpdesk->execute();

	return new ACF_Event('cancelnewmessage_Done');
}

function AH_DeleteEmailAttachment(ACF_ParameterList $inParameterList) {
	$helpdesk = new WOW_Request('','','wombat');

	$helpdesk->setMethod('attachment_Delete');

	$helpdesk->setusername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');

	$helpdesk->addParameter('cID',$inParameterList->ParameterValue('attachid'),'string');
	$helpdesk->addParameter('cType','ticket','string');
	$helpdesk->addParameter('nTypeId',$_SESSION['helpdesk']['ticketid'],'int');
	$helpdesk->addParameter('nEmail',$inParameterList->ParameterValue('emailid'),'int');

	$helpdesk->execute();

	return new ACF_Event('deleteemailattachment_done');
}

function AH_UpdateMessage(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
//---- create the helpdesk object
    $helpdesk  = new WOW_Request('', '', 'wombat');
    $message   = $inParameterList->ParameterValue('message');
    $messageId = $inParameterList->ParameterValue('messageid') ? $inParameterList->ParameterValue('messageid') : 0;
//---- assign values to the wow
    switch (strtolower($inParameterList->ParameterValue('action'))){
        case 'save':
            $helpdesk->setMethod('email_Save');
            break;
        case 'send':
            $helpdesk->setMethod('email_Send');
            break;
        default:
            $helpdesk->setMethod('email_Cancel');
    }

    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');

    $helpdesk->addParameter('cAction', $inParameterList->ParameterValue('type'));
    $helpdesk->addParameter('nEmail', $messageId, 'int');
    $helpdesk->addParameter('cFrom', $message['from']);
    $helpdesk->addParameter('cTo', $message['to']);
    $helpdesk->addParameter('cCc', $message['cc']);
    $helpdesk->addParameter('cBcc', $message['bcc']);
    $helpdesk->addParameter('cSubject', $message['subject']);
    $helpdesk->addParameter('cBody', $message['body']);
    $helpdesk->addParameter('nAttachments', $message['attachmentCount'], 'int');
    $helpdesk->addParameter('cType', 'ticket');
    $helpdesk->addParameter('nTypeId', $_SESSION['helpdesk']['ticketid'], 'int');
//---- execute it
    if ($helpdesk->execute())
	$retEvent = new ACF_Event('UpdateTicketMessage_Success');
    else{
	$retEvent = new ACF_Event('UpdateTicketMessage_Failure');
        $parameterList = new ACF_ParameterList();
	if (strtolower($inParameterList->ParameterValue('action')) == "send"){
//---- grab the error message
	    if ($helpdesk->lastError)
		$_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- the send failed so attempt to save the message so no data is lost

//---- create the helpdesk object
	    $helpdesk = new WOW_Request('', '', 'wombat');
            $helpdesk->setMethod('email_Save');

	    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
	    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
	    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');

	    $helpdesk->addParameter('cAction', $inParameterList->ParameterValue('type'));
	    $helpdesk->addParameter('nEmail', $messageId, 'int');
	    $helpdesk->addParameter('cFrom', $message['from']);
            $helpdesk->addParameter('cTo', $message['to']);
            $helpdesk->addParameter('cCc', $message['cc']);
            $helpdesk->addParameter('cBcc', $message['bcc']);
            $helpdesk->addParameter('cSubject', $message['subject']);
            $helpdesk->addParameter('cBody', $message['body']);
	    $helpdesk->addParameter('nAttachments', $message['attachmentCount'], 'int');
	    $helpdesk->addParameter('cType', 'ticket');
	    $helpdesk->addParameter('nTypeId', $_SESSION['helpdesk']['ticketid'], 'int');
//---- execute it
	    if (!$helpdesk->execute())
		$saveFailed == true;
	}

	$parameterList->AddParameter(new ACF_DirectParameter('messageid', $messageId));
        $parameterList->AddParameter(new ACF_DirectParameter('action', 'continue'));
        $retEvent->SetParameterList($parameterList);
    }

    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
    if ($saveFailed)
	$_SESSION['helpdesk']['error'] .= "\nAttempt to save the message to the database has failed.";
//---- return the proper event
    return $retEvent;
}


function AH_SaveCall(ACF_ParameterList $inParameterList){
//---- grab the parameters that are passed in and assign them to a local variable for easy reference
    $call = $inParameterList->ParameterValue('call');
    $_SESSION['helpdesk']['error'] = '';
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('call_Save');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('cType', 'ticket');
    $helpdesk->addParameter('nTypeId', $_SESSION['helpdesk']['ticketid'], 'int');
    $helpdesk->addParameter('cBody', trim($call['body']));
    $helpdesk->addParameter('nTime', time() - $_SESSION['helpdesk']['callStartTime'], 'int');
    $helpdesk->addParameter('cCallType', $call['type']);
    $helpdesk->addParameter('cContactNumber', $call['number']);
    $helpdesk->addParameter('cContactName', $call['contact']);
    $helpdesk->addParameter('bReopen', true, 'boolean');
//---- execute it
    if ($helpdesk->execute())
        $eventName = 'SaveTicketCall_Success';
    else
        $eventName = 'SaveTicketCall_Failure';
    if ($helpdesk->lastError)
	$_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- return the proper event
    $retEvent = new ACF_Event($eventName);
    return $retEvent;
}


function AH_AddTicketNote(ACF_ParameterList $inParameterList){
//---- grab the parameters that are passed in and assign them to a local variable for easy reference
    $note    = $inParameterList->ParameterValue('note');
    $body    = trim($note['body']);
    $visible = $note['visible'] ? 'Y' : 'N';
    $_SESSION['helpdesk']['error'] = '';
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('note_Save');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('cBody', $body);
    $helpdesk->addParameter('cVisible', $visible);
    $helpdesk->addParameter('cType', 'ticket');
    $helpdesk->addParameter('nTypeId', $_SESSION['helpdesk']['ticketid'], 'int');
    $helpdesk->addParameter('bReopen', true, 'boolean');
//---- execute it
    if ($helpdesk->execute())
        $eventName = 'AddTicketNote_Success';
    else
        $eventName = 'AddTicketNote_Failure';
    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- return the proper event
    $retEvent = new ACF_Event($eventName);
    return $retEvent;
}


function AH_UploadFile(ACF_ParameterList $inParameterList){
	$descriptionData = $inParameterList->ParameterValue('description');
	$_SESSION['helpdesk']['error'] = '';
	$eventName = strtolower($inParameterList->ParameterValue('type')) == 'email' ? 'UploadMessageFile_Failure' : 'UploadTicketFile_Failure';
	if(isset($_FILES['file']) && is_array($_FILES['file'])) {
		foreach($_FILES['file']['tmp_name'] as $thisFileIndex=>$this_tmp_name) {
			if(is_uploaded_file($this_tmp_name)) {
//---- create the helpdesk object
				$helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
				$helpdesk->setMethod('attachment_Save');
				$helpdesk->setUsername($_SESSION['helpdesk']['username']);
				$helpdesk->setPassword($_SESSION['helpdesk']['password']);
				$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
//---- make a pointer to the file
				$filePointer = fopen($this_tmp_name, 'r');
//---- encode the file and attach it as a parameter
				$helpdesk->addParameter('cFile', $_FILES['file']['name'][$thisFileIndex]);
				$helpdesk->addParameter('cFileData', base64_encode(fread($filePointer, $_FILES['file']['size'][$thisFileIndex])));
				fclose($filePointer);
				if(strtolower($inParameterList->ParameterValue('type')) == 'email') {
					$helpdesk->addParameter('nTypeId', $inParameterList->ParameterValue('messageid'), 'int');
				} else {
					$helpdesk->addParameter('nTypeId', $_SESSION['helpdesk']['ticketid'], 'int');
					$helpdesk->addParameter('cDescription', $descriptionData[$thisFileIndex]);
					$helpdesk->addParameter('bReopen', true, 'boolean');
				}

				$helpdesk->addParameter('cType', $inParameterList->ParameterValue('type'));
//---- execute it
				if($helpdesk->execute()) {
					$eventName = strtolower($inParameterList->ParameterValue('type')) == 'email' ? 'UploadMessageFile_Success' : 'UploadTicketFile_Success';
				} else {
					$eventName = strtolower($inParameterList->ParameterValue('type')) == 'email' ? 'UploadMessageFile_Failure' : 'UploadTicketFile_Failure';
				}

				if($helpdesk->lastError) {
					$_SESSION['helpdesk']['error'] = $helpdesk->lastError;
				} else {
					if(isset($_FILES['file']['error'][$thisFileIndex])) {
						switch($_FILES['file']['error'][$thisFileIndex]){
							case 0:
								break;
							case 1:
							case 2:
								$_SESSION['helpdesk']['error'] = "The file you are trying to upload is too big.";
								break;
							case 3:
								$_SESSION['helpdesk']['error'] = "The file you are trying upload was only partially uploaded.";
								break;
							case 4:
								$_SESSION['helpdesk']['error'] = "You must select a file to upload.";
								break;
							default:
								$_SESSION['helpdesk']['error'] = "There was a problem with your upload.";
						}
					}
				}
			} else {
				$_SESSION['helpdesk']['error'] = 'Please select a file to upload.';
			}
		}
	}
//---- return the proper event

	$retEvent = new ACF_Event($eventName);
	return $retEvent;
}


function AH_ProgressStatus(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('ticket_Change_Status');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $_SESSION['helpdesk']['ticketid'], 'int');
    $helpdesk->addParameter('nProgression', $inParameterList->ParameterValue('status'), 'int');
//---- execute it
    $helpdesk->execute();

    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- return the proper event
    $retEvent = new ACF_Event('ProgressTicketStatus_Success');
    return $retEvent;
}


function AH_RemoveItem(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
    $type = trim($inParameterList->ParameterValue('type'));

    if (in_array($type, array('call', 'note', 'attachment', 'email')))
	$method = $type . "_Delete";
    else{
	$_SESSION['helpdesk']['error'] = 'Invalid Type';
	return new ACF_Event('RemoveTicketItem_Failure');
    }
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod($method);
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('cID', $inParameterList->ParameterValue('id'));
    $helpdesk->addParameter('cType', 'ticket');
    $helpdesk->addParameter('nTypeId', $_SESSION['helpdesk']['ticketid'], 'int');
//---- execute it
    if ($helpdesk->execute())
        $eventName = 'RemoveTicketItem_Success';
    else
        $eventName = 'RemoveTicketItem_Failure';
    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- return the proper event
    $retEvent = new ACF_Event($eventName);
    return $retEvent;
}


function AH_DeleteTicket(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('ticket_Delete');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $inParameterList->ParameterValue('ticketid'), 'int');
    $helpdesk->addParameter('bDeleteEmails', $inParameterList->ParameterValue('deleteemails'), 'boolean');
//---- execute it
    if ($helpdesk->execute())
        $eventName = 'DeleteTicket_Success';
    else
        $eventName = 'DeleteTicket_Failure';
    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
//---- return the proper event
    $retEvent = new ACF_Event($eventName);
    return $retEvent;
}


function AH_CheckTicket(ACF_ParameterList $inParameterList){
    if ($inParameterList->ParameterValue('ticketid')){
	    $_SESSION['helpdesk']['ticketid'] = $inParameterList->ParameterValue('ticketid');
	    $_SESSION['helpdesk']['ticketLoaded'] = false;
//---- get the customer id for the ticket
	$helpdesk = new WOW_Request('', '', 'wombat');
	$helpdesk->setMethod('ticket_Get_Customer_Id');
	$helpdesk->setUsername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
	$helpdesk->addParameter('nTicket', $_SESSION['helpdesk']['ticketid'], 'int');
//---- this is the initial load of the ticket
	if($helpdesk->execute()){
	    $custid = $helpdesk->getAttribute(0);
	    if ($custid['ticket_Get_Customer_Id'] > -1){
		$_SESSION['helpdesk']['ticketid'] = $_SESSION['helpdesk']['ticketid'];
		if (!$_SESSION['helpdesk']['useCustWebTicket']){
		    $retEvent = new ACF_Event('LoadCustomer');
		    $parameterList = new ACF_ParameterList();
		    $parameterList->AddParameter(new ACF_DirectParameter('custid', $custid['ticket_Get_Customer_Id']));
		    $retEvent->SetParameterList($parameterList);
		}else{
		    if ($custid['ticket_Get_Customer_Id'] == $_SESSION['helpdesk']['viewCustomerTickets'])
			$retEvent = new ACF_Event('CheckTicket_Success');
		    else{
			$_SESSION['helpdesk']['error'] = "Ticket #" . $inParameterList->ParameterValue('ticketid') . " does not belong to customer #" . $_SESSION['helpdesk']['viewCustomerTickets'];
			$retEvent = new ACF_Event('CheckTicket_Failure');
		    }
		}
	    }else{
		$_SESSION['helpdesk']['error'] = "Ticket #" . $inParameterList->ParameterValue('ticketid') . " not found.";
		$retEvent = new ACF_Event('CheckTicket_Failure');
	    }
	}else{
	    $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
	    $retEvent = new ACF_Event('CheckTicket_Failure');
	}
    }else
	$retEvent = new ACF_Event('CheckTicket_Success');
    return $retEvent;
}

function AH_SetCallStartTime () {
	$_SESSION['helpdesk']['callStartTime'] = time();
}

function AH_CreateNewMessage(ACF_ParameterList $inParameterList) {
	$helpdesk = new WOW_Request('','','wombat');

	$helpdesk->setMethod('email_New');

	$helpdesk->setusername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');

	$helpdesk->addParameter('cAction','NEW','string');
	$helpdesk->addParameter('nEmail',0,'int');
	$helpdesk->addParameter('cType','ticket','string');
	$helpdesk->addParameter('nTypeID',$_SESSION['helpdesk']['ticketid'],'int');

	if($helpdesk->execute()) {
		$newmessageData = $helpdesk->getAttribute(0);

		$domdoc = new domDocument();
		$root = $domdoc->createElement('newmessage');
		$root = $domdoc->appendChild($root);
		$id = $domdoc->createElement('id');
		$id = $root->appendChild($id);
		$idData = $domdoc->createTextNode($newmessageData['em_id']);
		$idData = $id->appendChild($idData);
		$to = $domdoc->createElement('to');
		$to = $root->appendChild($to);
		$toData = $domdoc->createTextNode($newmessageData['em_to']);
		$toData = $to->appendChild($toData);
		$from = $domdoc->createElement('from');
		$from = $root->appendChild($from);
		$fromData = $domdoc->createTextNode($newmessageData['em_from']);
		$fromData = $from->appendChild($fromData);
		$cc = $domdoc->createElement('cc');
		$cc = $root->appendChild($cc);
		$ccData = $domdoc->createTextNode($newmessageData['em_cc']);
		$ccData = $cc->appendChild($ccData);
		$bcc = $domdoc->createElement('bcc');
		$bcc = $root->appendChild($bcc);
		$bccData = $domdoc->createTextNode($newmessageData['em_bcc']);
		$bccData = $bcc->appendChild($bccData);
		$subject = $domdoc->createElement('subject');
		$subject = $root->appendChild($subject);
		$subjectData = $domdoc->createTextNode($newmessageData['em_subject']);
		$subjectData = $subject->appendChild($subjectData);
		$body = $domdoc->createElement('body');
		$body = $root->appendChild($body);
		$bodyData = $domdoc->createTextNode($newmessageData['em_body']);
		$bodyData = $body->appendChild($bodyData);

		$xml = $domdoc->saveXML();
	} else {
		$domdoc = new domDocument();
		$root = $domdoc->createElement('error');
		$root = $domdoc->appendChild($root);
		$error = $domdoc->createTextNode($helpdesk->lastError);
		$error = $root->appendChild($error);

		$xml = $domdoc->saveXML();
	}

	$paramList = new ACF_ParameterList();
	$paramList->AddParameter(new ACF_DirectParameter('xml',$xml));
	$retEvent = new ACF_Event('createnewmessage_done');
	$retEvent->SetParameterList($paramList);
	return $retEvent;
}

?>
