<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file performs the tickettree interactions for the helpdesk pages                           ----*/
/*---- These functions are invoked from the acf framework                                              ----*/
/*---------------------------------------------------------------------------------------------------------*/

require_once('../../libs/platypus/wow.inc.php');
require_once("../../libs/ccs/options.inc.php");
//---- load the CCS Options have to laod it here because this is the first manditory function call
if (!$_SESSION['ccs'])
    loadCCSOptions();


function AH_SetNewCall(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['newCall']       = array('active' => 1);
    $_SESSION['helpdesk']['callStartTime'] = time();
//---- return the proper event
    $retEvent = new ACF_Event('SetNewTicketCall_Success');
    return $retEvent;
}


function AH_UpdateCall(ACF_ParameterList $inParameterList){
    if ($inParameterList->ParameterValue('action') == 'update'){
        $_SESSION['helpdesk']['newCall']           = $inParameterList->ParameterValue('call');
        $_SESSION['helpdesk']['newCall']['active'] = 1;
        $eventName = $inParameterList->ParameterValue('view') == 'tree' ? "ToTickettree" : "ToSearch";
    }else{
        $_SESSION['helpdesk']['newCall'] = array();
        $eventName = "ToTickettree";
    }
//---- return the proper event
    $retEvent = new ACF_Event($eventName);
    return $retEvent;
}


function AH_SaveTicket(ACF_ParameterList $inParameterList){
    $_SESSION['helpdesk']['error'] = '';
    $ticket = $inParameterList->ParameterValue('ticket');
    if ($inParameterList->ParameterValue('call'))
        $call = $inParameterList->ParameterValue('call');
//---- create the helpdesk object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- assign values to the wow
    $helpdesk->setMethod('ticket_Save');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('bForceSave', 'true', 'boolean');
    $helpdesk->addParameter('nTicket', 0, 'int');
    if ($ticket['category'])
        $helpdesk->addParameter('nCategory', $ticket['category'], 'int');
    if ($inParameterList->ParameterValue('custid'))
        $helpdesk->addParameter('nCustomer_id', $inParameterList->ParameterValue('custid'), 'int');
    else
        $helpdesk->addParameter('nCustomer_id', 0, 'int');
    if ($inParameterList->ParameterValue('contact'))
        $helpdesk->addParameter('cContact', strtoupper($inParameterList->ParameterValue('contact')));
    else
        //bug fix - if the customer is logged in, this value will not be set but the value is required
        $helpdesk->addParameter('cContact', 'N');
    if ($ticket['email'])
        $helpdesk->addParameter('cEmail', $ticket['email']);
    if ($ticket['hidden'])
        $helpdesk->addParameter('cHidden', strtoupper($ticket['hidden']));
    if ($ticket['knowledgebase_id'])
        $helpdesk->addParameter('nKnowledgebase_id', $ticket['knowledgebase_id']);
    if ($ticket['knowledgebase_type'])
        $helpdesk->addParameter('cKnowledgebase_type', $ticket['knowledgebase_type']);
    if ($ticket['priority'])
        $helpdesk->addParameter('nPriority', $ticket['priority'], 'int');
    if ($ticket['problem'])
        $helpdesk->addParameter('cProblem', trim($ticket['problem']));
    if ($ticket['project_id'])
        $helpdesk->addParameter('nProject_id', $ticket['project_id'], 'int');
    if ($ticket['solution'])
        $helpdesk->addParameter('cSolution', trim($ticket['solution']));
    if ($ticket['staff_assigned'])
        $helpdesk->addParameter('nStaff_assigned', $ticket['staff_assigned'], 'int');
    if ($ticket['status'])
        $helpdesk->addParameter('nStatus', $ticket['status'], 'int');
    if ($ticket['status_progression'])
        $helpdesk->addParameter('nStatus_progression', $ticket['status_progression'], 'int');
    if ($ticket['type'])
        $helpdesk->addParameter('nType', $ticket['type'], 'int');
    if ($inParameterList->ParameterValue('svc_id'))
        $helpdesk->addParameter('nService', $inParameterList->ParameterValue('svc_id'), 'int');
    else
        $helpdesk->addParameter('nService', 0, 'int');
    if ($inParameterList->ParameterValue('svc_instance_id'))
        $helpdesk->addParameter('nInstance', $inParameterList->ParameterValue('svc_instance_id'), 'int');
    else
        $helpdesk->addParameter('nInstance', 0, 'int');
//---- if a call exists return the object so we can grab the ticket id
//    if ($call)
        $helpdesk->addParameter('bReturnObject', true, 'boolean');
//---- execute it
    if ($helpdesk->execute()){
        $eventName = "SaveNewTicket_Success";
            $results  = $helpdesk->getAttribute(0);
            $ticketId = $results['tkt_id'];
//---- added the call to the ticket
        if ($call){
            $helpdesk->setMethod('call_Save');
            $helpdesk->setUsername($_SESSION['helpdesk']['username']);
            $helpdesk->setPassword($_SESSION['helpdesk']['password']);
	    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
            $helpdesk->addParameter('cType', 'ticket');
            $helpdesk->addParameter('nTypeId', $ticketId, 'int');
            $helpdesk->addParameter('cBody', trim($call['body']));
            $helpdesk->addParameter('nTime', time() - $_SESSION['helpdesk']['callStartTime'], 'int');
            $helpdesk->addParameter('cCallType', $call['type']);
            $helpdesk->addParameter('cContactNumber', $call['number']);
            $helpdesk->addParameter('cContactName', $call['contact']);
//---- execute it
            if ($helpdesk->execute())
                $_SESSION['helpdesk']['newCall'] = array();
            else
                $_SESSION['helpdesk']['error'] = $helpdesk->lastError;
        }
    }else
        $eventName = "SaveNewTicket_Failure";
    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "  \r\n" : "") . $helpdesk->lastError;
//---- return the proper event
    $retEvent = new ACF_Event($eventName);
	if ($eventName == "SaveNewTicket_Success") {
		$paramList = new ACF_ParameterList();
		$paramList->AddParameter(new ACF_DirectParameter('ticketid',$ticketId));
		$retEvent->SetParameterList($paramList);
	}

    return $retEvent;
}


function AH_SetView(ACF_ParameterList $inParameterList){
//---- if a different view type is passed in set the session variable
    if ($inParameterList->ParameterValue('view')){
        $_SESSION['helpdesk']['view'] = $inParameterList->ParameterValue('view');
//---- if the select view was passed in process it
        $selectView = $inParameterList->ParameterValue('selectview');
        foreach($_SESSION['helpdesk']['tickettree']['viewable'] as $key => &$treeViews)
            foreach($treeViews as $viewKey => &$viewDetails)
                $viewDetails['shown'] = $selectView[$key][$viewKey] ? "Y" : "N";
    }
//---- grab/update the category filter
    if ($_SESSION['helpdesk']['tickettree']['categoryFilters']){
        if ($inParameterList->ParameterValue('catFilter'))
            $newCatFilter = $inParameterList->ParameterValue('catFilter');
        $catFilter = "0";
        foreach($_SESSION['helpdesk']['tickettree']['categoryFilters'] as $key => &$filter){
            if ($newCatFilter)
                if ($key == $newCatFilter)
                    $filter['current'] = true;
                else
                    $filter['current'] = false;
        }
    }
//---- if the staff id is passed in change the session variable of who we are looking at
    if ($inParameterList->ParameterValue('staffid') >= -1){
        if ($inParameterList->ParameterValue('staffid') > -1)
	    $_SESSION['helpdesk']['viewStaffTickets']  = $inParameterList->ParameterValue('staffid');
        $_SESSION['helpdesk']['currentTickettreeView'] = 'staff';
    }
    if ($inParameterList->ParameterValue('custid') > -1){
        $_SESSION['helpdesk']['viewCustomerTickets']   = $inParameterList->ParameterValue('custid');
        $_SESSION['helpdesk']['currentTickettreeView'] = 'customer';
        $_SESSION['helpdesk']['customerContact']       = $inParameterList->ParameterValue('contact') ? $inParameterList->ParameterValue('contact') : 'N';
    }
    return new ACF_Event('SetTickettreeView_Success');
}


function AH_LoadBranch(ACF_ParameterList $inParameterList){
    $catFilter  = 0;
    $branchData = array();
    $branches   = explode(",", $inParameterList->ParameterValue('sections'));
//---- grab the category filter
    if (is_array($_SESSION['helpdesk']['tickettree']['categoryFilters'])) {
		foreach($_SESSION['helpdesk']['tickettree']['categoryFilters'] as $key => &$filter) {
			if ($filter['current']) {
			$catFilter = $filter['id'];
			break;
			}
		}
    }
//---- build the branch array
    foreach ($branches as $branch) {
		if ($branch == "root") {
			$details = array();
			$details['path']   = "root";
			$details['viewId'] = 0;
			$details['count']  = 0;
			$details['month']  = 0;
			$details['year']   = 0;
			array_push($branchData, $details);
		} else {
			$parentId = 0;
			$refPoint = $_SESSION['helpdesk']['tickettree']['tree'];
			$path     = $branch;
			while($path && isset($refPoint)) {
				$pos = strpos($path, "/");
				if ($pos && isset($refPoint[substr($path, 0, $pos)])) {
					$refPoint = $refPoint[substr($path, 0, $pos)];
					$path = substr($path, $pos+1);
					if (isset($refPoint['id']))
						$parentId = $refPoint['id'];
				} else if ($pos === false && isset($refPoint[$path])) {
					$refPoint = $refPoint[$path];
					$path = '';
				} else
					unset($refPoint);
			}
			if(!is_array($refPoint))
				continue;
			$details = array();
			$details['path']   = $branch;
			$details['viewId'] = isset($refPoint['id']) ? $refPoint['id'] : $parentId;
			$details['count']  = isset($refPoint['tkt_total_count']) ? $refPoint['tkt_total_count'] : 0;
			$details['month']  = isset($refPoint['tkt_date_month'])  ? $refPoint['tkt_date_month']  : 0;
			$details['year']   = isset($refPoint['tkt_date_year'])   ? $refPoint['tkt_date_year']   : 0;
			array_push($branchData, $details);
		}
    }
//---- if a ticket was passed in this array will be empty so leave
    if (!$branchData)
		return new ACF_Event('LoadBranch_Failure');
//---- get the branches
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('ticketTree_Get_Branch');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('cViewType', $_SESSION['helpdesk']['view']);
    $helpdesk->addParameter('aBranch', $branchData, 'array');
//---- add the viewable data
    foreach($_SESSION['helpdesk']['tickettree']['viewable'] as $key => $viewableData) {
		$viewableParam = "";
		foreach($viewableData as $id=>$viewableDetails) {
			if ($viewableDetails['shown'] == "Y")
			$viewableParam .= $id. ",";
		}
		if ($viewableParam)
			$viewableParam = substr($viewableParam, 0, strlen($viewableParam) - 1);
		$helpdesk->addParameter("c" . ucfirst($key) . "Viewable", $viewableParam);
    }
    $helpdesk->addParameter('nCatFilter', $catFilter, 'int');
    if ($_SESSION['helpdesk']['currentTickettreeView'] == 'staff')
		$helpdesk->addParameter('nID', $_SESSION['helpdesk']['viewStaffTickets'], 'int');
    else {
		$helpdesk->addParameter('nID', $_SESSION['helpdesk']['viewCustomerTickets'], 'int');
		$helpdesk->addParameter('cContact', $_SESSION['helpdesk']['customerContact']);
		$helpdesk->addParameter('bCustomer', true, 'boolean');
    }
    if($helpdesk->execute()){
		$results = $helpdesk->getAttributesArray();
//---- if the array came back empty clear it out and leave
		if (!is_array($results[0]['root']) || $results[0]['root'] == "false") {
			$_SESSION['helpdesk']['tickettree']['tree'] = array();
			return new ACF_Event('LoadBranch_Failure');
		}
		$prefix = substr($_SESSION['helpdesk']['view'], 0, 1);
//---- if the root node was passed back reset the array to lose vacant data
		if (isset($results[0]['root']['items'])) {
//---- this is the root node clean out the array
			$_SESSION['helpdesk']['tickettree']['tree']['root'] = array();
			$resultData = $results[0]['root']['items'];
//---- if there are no views they will fall into here
			if (!is_array($resultData['items_item']))
				return new ACF_Event('LoadBranch_Failure');
//---- fix the array for easier coding
			if (isset($resultData['items_item']['tkt_total_count']))
				$resultData['items_item'] = array($resultData['items_item']);
//---- loop through the views
			foreach($resultData['items_item'] as $view) {
				$title = $view[$prefix . 't_name'];
//---- remove odd characters
				$name = preg_replace('( )', '_', strtolower($title));
				if (!isset($_SESSION['helpdesk']['tickettree']['tree']['root'][$name]))
					$_SESSION['helpdesk']['tickettree']['tree']['root'][$name] = array();
				$_SESSION['helpdesk']['tickettree']['tree']['root'][$name]['id']              = $view[$prefix . 't_id'];
				$_SESSION['helpdesk']['tickettree']['tree']['root'][$name]['title']           = $title;
				$_SESSION['helpdesk']['tickettree']['tree']['root'][$name]['tkt_bold_count']  = $view['tkt_bold_count'];
				$_SESSION['helpdesk']['tickettree']['tree']['root'][$name]['tkt_total_count'] = $view['tkt_total_count'];
			}
		}
//---- loop through the results to build the rest of the tree
		foreach($results[0]['root'] as $key => $resultData) {
			if ($key != "items") {
//---- this is a view
//---- views should already exist
				if (!isset($_SESSION['helpdesk']['tickettree']['tree']['root'][$key]))
					continue;
				if (isset($resultData['items'])) {
//---- this view has either tickets or groups
//---- fix the array for easier coding
					if (isset($resultData['items']['items_item']['tkt_id']) || isset($resultData['items']['items_item']['tkt_date_year']))
						$resultData['items']['items_item'] = array($resultData['items']['items_item']);
					if (isset($resultData['items']['items_item'][0]['tkt_id'])){
						$_SESSION['helpdesk']['tickettree']['tree']['root'][$key]['tickets'] = array();
//---- copy the tickets on to the tree
					foreach($resultData['items']['items_item'] as $ticket)
						array_push($_SESSION['helpdesk']['tickettree']['tree']['root'][$key]['tickets'], $ticket);
					} else {
//---- make the groups
						$_SESSION['helpdesk']['tickettree']['tree']['root'][$key]['groups'] = array();
						foreach($resultData['items']['items_item'] as $group)
							$_SESSION['helpdesk']['tickettree']['tree']['root'][$key]['groups']['group_' . $group['tkt_date_month'] . '_' . $group['tkt_date_year']] = $group;
					}
				}
				if (isset($resultData['groups'])) {
//---- this is a group
					foreach($resultData['groups'] as $groupName => $groupData) {
						$_SESSION['helpdesk']['tickettree']['tree']['root'][$key]['groups'][$groupName]['tickets'] = array();
//---- fix the array for easier coding
						if (isset($groupData['items']['items_item']['tkt_id']))
							$groupData['items']['items_item'] = array($groupData['items']['items_item']);
//---- copy the tickets on to the tree
						foreach($groupData['items']['items_item'] as $ticket)
							array_push($_SESSION['helpdesk']['tickettree']['tree']['root'][$key]['groups'][$groupName]['tickets'], $ticket);
					}
				}
			}
		}
    } else
		if ($helpdesk->lastError !== "ADODB.RecordSet is empty.")
			$error = $helpdesk->lastError;
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']) {
        $error = ($error ? $error . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }
    if ($helpdesk->lastError)
        $_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "  \r\n" : "") . $helpdesk->lastError;
//---- return the proper event
    $eventName = "loadBranch_Success";
    $retEvent = new ACF_Event($eventName);
    return $retEvent;
}
?>
