<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file generates data for the header template                                                ----*/
/*---- All functions pull the information from the object by invoking the helpdesk object function.    ----*/
/*---- Then this data is put into a collection of arrays (this makes the ACF less cumbersome)          ----*/
/*---- This data is then passed to the Smarty object to be used in conjunction with the template file  ----*/
/*---------------------------------------------------------------------------------------------------------*/

require_once('../../libs/platypus/wow.inc.php');
require_once("../../libs/ccs/options.inc.php");
//---- load the CCS Options have to laod it here because this is the first manditory function call
if (!$_SESSION['ccs'])
    loadCCSOptions();


function DC_GetStaffList(ACF_ParameterList $inParameterList){
    if ($inParameterList->ParameterValue('staffid') > -1 )
        $_SESSION['helpdesk']['viewStaffTickets'] = $inParameterList->ParameterValue('staffid');
    $staff  = array('list' => array(), 'current' => $_SESSION['helpdesk']['viewStaffTickets']);

    if (!$_SESSION['helpdesk']['useCustWebTicket']){
//---- get a list of staff members
	$helpdesk = new WOW_Request('', '', 'wombat');
	$helpdesk->setMethod('get_Staff_List');
	$helpdesk->setUsername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', 'staff');
	if ($inParameterList->ParameterValue('type'))
	    $helpdesk->addParameter('cType', $inParameterList->ParameterValue('type'));
	if ($inParameterList->ParameterValue('ticketid')){
	    $helpdesk->addParameter('cIDType', 'ticket');
	    $helpdesk->addParameter('nId', $inParameterList->ParameterValue('ticketid'), 'int');
	}
	if($helpdesk->execute())
	    $staff['list'] = $helpdesk->getAttributesArray();
	else
	    $error = $helpdesk->lastError;
    }
    $parameterList = new ACF_ParameterList();                              //---- create a parameter object
    if ($inParameterList->ParameterValue('xml')){
//---- convert the results to xml
	$oDomDoc = new domDocument();
	if ($error){
	    $root = $oDomDoc->createElement('error');
	    $oDomDoc->appendChild($root);
	    $root->appendChild($oDomDoc->createTextNode(trim($error)));
	}
	$root = $oDomDoc->createElement('stafflist');
	$oDomDoc->appendChild($root);
	foreach($staff['list'] as $item){
	    $keyDom = $root->appendChild($oDomDoc->createElement('staff'));
	    $name   = $keyDom->appendChild($oDomDoc->createAttribute('name'));
	    $name->appendChild($oDomDoc->createTextNode($item['name']));
	    $id     = $keyDom->appendChild($oDomDoc->createAttribute('id'));
	    $id->appendChild($oDomDoc->createTextNode($item['num']));
	    if ($item['num'] == $staff['current']){
		$current = $keyDom->appendChild($oDomDoc->createAttribute('current'));
		$current->appendChild($oDomDoc->createTextNode('true'));
	    }
	}
	$parameterList->AddParameter(new ACF_DirectParameter('xml', $oDomDoc->savexml()));
    }else{
//---- add values to the parameter object
	$parameterList->AddParameter(new ACF_DirectParameter('staff', $staff));
	if ($error)
	    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    }
    return $parameterList;
}


function DC_GetStaffTools(ACF_ParameterList $inParameterList){
    $parameterList = new ACF_ParameterList();
    if (!$_SESSION['helpdesk']['useCustWebTicket'])
	$parameterList->AddParameter(new ACF_DirectParameter('stafftools', true));
    return $parameterList;
}

function DC_GetError(ACF_ParameterList $inParameterList){
    $parameterList = new ACF_ParameterList();
    if ($_SESSION['helpdesk']['error'])
	$parameterList->AddParameter(new ACF_DirectParameter('error', $_SESSION['helpdesk']['error']));
    $_SESSION['helpdesk']['error'] = '';
    return $parameterList;
}

function DC_GetAccessControl(ACF_ParameterList $inParameterList){
    $access    = array();
    $ticket_id = $inParameterList->ParameterValue('ticketid') ? $inParameterList->ParameterValue('ticketid') : $_SESSION['helpdesk']['ticketid'];
//---- get the ticket security
    if ($ticket_id > 0) {
//---- create the wow object
        $helpdesk = new WOW_Request('', '', 'wombat');
        $helpdesk->setMethod('get_Access_Settings');
        $helpdesk->setUsername($_SESSION['helpdesk']['username']);
        $helpdesk->setPassword($_SESSION['helpdesk']['password']);
        $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
        $helpdesk->addParameter('nTicket', $ticket_id, 'int');
        if($helpdesk->execute()){
            $results = $helpdesk->getAttributesArray();
//---- this array can override the security array
            foreach ($results as $result)
                $access[$result['tag']] = $result['value'];
        } else {
            $error = $helpdesk->lastError;
        }
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
    if ($inParameterList->ParameterValue('xml')){
//---- convert the results to xml
        $oDomDoc = new domDocument();
        if ($error){
            $root = $oDomDoc->createElement('error');
            $oDomDoc->appendChild($root);
            $root->appendChild($oDomDoc->createTextNode(trim($error)));
        }else{
            $root = $oDomDoc->createElement('customerdata');
            $oDomDoc->appendChild($root);
            foreach($_SESSION['helpdesk']['tickettree']['security'] as $key=>$item){
                $keyDom = $root->appendChild($oDomDoc->createElement('tag'));
                $name   = $keyDom->appendChild($oDomDoc->createAttribute('name'));
                $name->appendChild($oDomDoc->createTextNode($key));
                $value  = $keyDom->appendChild($oDomDoc->createAttribute('value'));
//---- if the acees array overwrites the main security grab it instead
                $value->appendChild($oDomDoc->createTextNode(isset($access[$key]) ? $access[$key] : $item));
            }
        }
        $parameterList->AddParameter(new ACF_DirectParameter('xml', $oDomDoc->savexml()));
    }else{
        foreach($_SESSION['helpdesk']['tickettree']['security'] as $key=>$item){
            if (!isset($access[$key]))
                $access[$key] = $item;
        }
        $parameterList->AddParameter(new ACF_DirectParameter('access', $access));
    }
    return $parameterList;
}


function DC_GetCustomerTickets(ACF_ParameterList $inParameterList){
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('get_Customer_Ticket_List');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nCustid', $inParameterList->ParameterValue('custid'), 'int');
    if ($inParameterList->ParameterValue('contact'))
	$helpdesk->addParameter('cContact', $inParameterList->ParameterValue('contact'));
    if($helpdesk->execute())
	$results = $helpdesk->getAttributesArray();
    else
	$error = $helpdesk->lastError;
//---- convert the results to xml
    $oDomDoc = new domDocument();
    if ($error){
	$root = $oDomDoc->createElement('error');
	$oDomDoc->appendChild($root);
	$root->appendChild($oDomDoc->createTextNode(trim($error)));
    }else{
	$root = $oDomDoc->createElement('ticketlist');
	$oDomDoc->appendChild($root);
	foreach($results as $item){
	    $ticket = $root->appendChild($oDomDoc->createElement('ticket'));
	    $id = $ticket->appendChild($oDomDoc->createAttribute('id'));
	    $id->appendChild($oDomDoc->createTextNode(trim($item['tkt_id'])));
	    $date = $ticket->appendChild($oDomDoc->createAttribute('date'));
	    $date->appendChild($oDomDoc->createTextNode(trim($item['tkt_date_entered'])));
	    $status = $ticket->appendChild($oDomDoc->createAttribute('status'));
	    $status->appendChild($oDomDoc->createTextNode(trim($item['st_name'])));
	    $problem = $ticket->appendChild($oDomDoc->createAttribute('problem'));
	    $problem->appendChild($oDomDoc->createTextNode(trim($item['tkt_problem'])));
	}
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('xml', $oDomDoc->savexml()));
    return $parameterList;
}
?>
