<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file generates data for the ticket templates                                               ----*/
/*---- All functions pull the information from the object by invoking the helpdesk object function.    ----*/
/*---- Then this data is put into a collection of arrays (this makes the ACF less cumbersome)          ----*/
/*---- This data is then passed to the Smarty object to be used in conjunction with the template file  ----*/
/*---------------------------------------------------------------------------------------------------------*/

require_once('../../libs/platypus/wow.inc.php');
require_once("../../libs/ccs/options.inc.php");
require_once("zipfunctions.php");

//---- load the CCS Options have to laod it here because this is the first manditory function call
if (!$_SESSION['ccs'])
    loadCCSOptions();


function DC_GetTicketInfo(ACF_ParameterList $inParameterList){
    $ticketInfo = array();
    $staff      = array();
    $ticketId   = $inParameterList->ParameterValue('ticketid');
//---- get the ticket info if requested
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('get_Ticket_Info');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $ticketId, 'int');
    $helpdesk->addParameter('nType', 2, 'int');
    if($helpdesk->execute()){
        $results = $helpdesk->getAttribute(0);
	$ticketInfo = is_array($results['display_fields']['display_fields_item']) ? $results['display_fields']['display_fields_item'] : array();
    }else
        $error = $helpdesk->lastError;
//---- put data into xml format
    $oDomDoc = new domDocument();
    if ($error){
	$root = $oDomDoc->createElement('error');
	$oDomDoc->appendChild($root);
	$root->appendChild($oDomDoc->createTextNode(trim($error)));
    }else{
	$root = $oDomDoc->createElement('ticketdata');
	$oDomDoc->appendChild($root);
	if(isset($ticketInfo) && is_array($ticketInfo)) {
	foreach($ticketInfo as $section){
	    $data = $root->appendChild($oDomDoc->createElement('data'));
	    $data->appendChild($oDomDoc->createTextNode(trim($section['displayvalue'])));
	    $name = $data->appendChild($oDomDoc->createAttribute('name'));
	    $name->appendChild($oDomDoc->createTextNode(trim($section['displayname'])));
	}
	}
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('xml', $oDomDoc->savexml()));
    return $parameterList;
}


function DC_GetStaffReassignList(ACF_ParameterList $inParameterList){
    $staff = array();
//---- get the staff list if logged in as staff
    if(!$_SESSION['helpdesk']['useCustWebTicket']) {
        $helpdesk = new WOW_Request('', '', 'wombat');
        $helpdesk->setMethod('get_Staff_List');
        $helpdesk->setUsername($_SESSION['helpdesk']['username']);
        $helpdesk->setPassword($_SESSION['helpdesk']['password']);
        $helpdesk->addAttribute('logintype', 'staff');
        $helpdesk->addParameter('cType', 'reassign');
        $helpdesk->addParameter('cIdType', 'ticket');
        $helpdesk->addParameter('nId', $_SESSION['helpdesk']['ticketid'], 'int');
        if($helpdesk->execute())
            $staff = $helpdesk->getAttributesArray();
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('staff', $staff));
    return $parameterList;
}


function DC_GetGeneralInfo(ACF_ParameterList $inParameterList){
    $ticket    = array('details' => array(), 'customer' =>array(), 'currentPage' => 'general');
    $ticket_id = $_SESSION['helpdesk']['ticketid'];
    if (!$_SESSION['helpdesk']['useCustWebTicket']){
//---- get the ticket's history
	$helpdesk = new WOW_Request('', '', 'wombat');
	$helpdesk->setMethod('ticket_Get_History');
	$helpdesk->setUsername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
	$helpdesk->addParameter('nTicket', $ticket_id, 'int');
	if($helpdesk->execute())
	    $ticket['history'] = $helpdesk->getAttributesArray();
	else
	    $error = $helpdesk->lastError;
    }

//---- get the ticket information
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('get_Ticket_Info');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $ticket_id, 'int');
    $helpdesk->addParameter('nType', 1, 'int');
    if($helpdesk->execute()){
	$results = $helpdesk->getAttribute(0);
	if($results['display_fields']['display_fields_item'][0]) {
		$ticket['details'] = $results['display_fields']['display_fields_item'];
	} else {
		$ticket['details'][0] = $results['display_fields']['display_fields_item'];
	}
//---- get the customer information
	$helpdesk = new WOW_Request('', '', 'wombat');
	$helpdesk->setMethod('ticket_Get_Customer_Info');
	$helpdesk->setUsername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
	$helpdesk->addParameter('nTicket', $ticket_id, 'int');
	$helpdesk->addProperty('tkt_id', $ticket_id, 'int');
	if($helpdesk->execute()){
		$results = $helpdesk->getAttribute(0);
		if($results['display_fields']['display_fields_item'][0]) {
			$ticket['customer'] = $results['display_fields']['display_fields_item'];
		} else {
			$ticket['customer'][0] = $results['display_fields']['display_fields_item'];
		}
        }else
	    if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
		$error = $helpdesk->lastError;
    }else
        $error = $helpdesk->lastError;
    if (!$error){
	if ($_SESSION['helpdesk']['ticketid'] != $_SESSION['helpdesk']['oldticketid'])
	    $_SESSION['helpdesk']['oldticketid'] = $_SESSION['helpdesk']['ticketid'];
	if ($_SESSION['helpdesk']['oldticketid'] == '0')
	    $_SESSION['helpdesk']['oldticketid'] = $ticket_id;
    }
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
            $error = ($error ? $error . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('ticket', $ticket));
    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    return $parameterList;
}


function DC_GetHistoryInfo(ACF_ParameterList $inParameterList){
    $ticket    = array('history' => array(), 'currentPage' => 'history');
    $ticket_id = $_SESSION['helpdesk']['ticketid'];
//---- get the ticket's history
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('ticket_Get_History');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $ticket_id, 'int');
    if($helpdesk->execute())
	$ticket['history'] = $helpdesk->getAttributesArray();
    else
	if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
	    $error = $helpdesk->lastError;
    if (!$error){
	if ($_SESSION['helpdesk']['ticketid'] != $_SESSION['helpdesk']['oldticketid'])
	    $_SESSION['helpdesk']['oldticketid'] = $_SESSION['helpdesk']['ticketid'];
	if ($_SESSION['helpdesk']['oldticketid'] == '0')
	    $_SESSION['helpdesk']['oldticketid'] = $ticket_id;
    }
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
            $error = ($error ? $error . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('ticket', $ticket));
    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    return $parameterList;
}


function DC_GetProblemInfo(ACF_ParameterList $inParameterList){
    $ticket    = array('attachments' => array(), 'currentPage' => 'problem');
    $ticket_id = $_SESSION['helpdesk']['ticketid'];
//---- get the ticket data
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('ticket_Load');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $ticket_id, 'int');
    $helpdesk->addParameter('bLoadExtra', true, 'boolean');
    if($helpdesk->execute()){
        $ticket['details'] = $helpdesk->getAttribute(0);
//---- get the ticket's attachments
        $helpdesk->setMethod('ticket_Get_Attachments');
        $helpdesk->setUsername($_SESSION['helpdesk']['username']);
        $helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
        $helpdesk->addParameter('nTicket', $ticket_id, 'int');
	$helpdesk->addParameter('bIncludeEmail',false,'boolean');
        if($helpdesk->execute()){
	    $ticket['attachments'] = $helpdesk->getAttributesArray();
        }else
            if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
                $error = $helpdesk->lastError;
    }else
        $error = $helpdesk->lastError;
    if (!$error){
	if ($_SESSION['helpdesk']['ticketid'] != $_SESSION['helpdesk']['oldticketid'])
	    $_SESSION['helpdesk']['oldticketid'] = $_SESSION['helpdesk']['ticketid'];
	if ($_SESSION['helpdesk']['oldticketid'] == '0')
	    $_SESSION['helpdesk']['oldticketid'] = $ticket_id;
    }
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
        $error = ($error ? $error . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }
// moved the following line out of the IF ticket_Get_Attachments.execute() section above
// path needs to be set even if no attachments are returned, because the new email attachment toggle could return attachments
    $ticket['path'] = dirname($_SERVER["SCRIPT_NAME"]);
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('ticket', $ticket));
    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    return $parameterList;
}


function DC_GetMessagesInfo(ACF_ParameterList $inParameterList){
    $ticket    = array('messages' => array(), 'currentPage' => 'messages');
    $ticket_id = $_SESSION['helpdesk']['ticketid'];
//---- get the message data
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('ticket_Get_Emails');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $ticket_id, 'int');
    if($helpdesk->execute()){
	$ticket['messages'] = $helpdesk->getAttributesArray();
        if ($inParameterList->ParameterValue('messageid')){
//---- if a message id was passed in load the message
            $action = $inParameterList->ParameterValue('action') ? $inParameterList->ParameterValue('action') : 'view';
            if ($inParameterList->ParameterValue('fileattached') || in_array($action, array('continue', 'edit', 'view')))
		$method = 'email_Load';
	    else
		$method = 'email_New';
	    $helpdesk->setMethod($method);
            $helpdesk->setUsername($_SESSION['helpdesk']['username']);
            $helpdesk->setPassword($_SESSION['helpdesk']['password']);
	    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
            $helpdesk->addParameter('cType', 'ticket');
            $helpdesk->addParameter('nTypeId', $ticket_id, 'int');
//---- if an attachment was added (or attempted to be added) load the message differently
            if ($inParameterList->ParameterValue('fileattached'))
                switch(strtolower($action)){
                    case 'new':
                    case 'reply':
                    case 'forward':
                    case 'continue':
                        $helpdesk->addParameter('cAction', 'continue');
                        break;
                    case 'edit':
                        $helpdesk->addParameter('cAction', 'edit');
                        break;
                    default:
                        $helpdesk->addParameter('cAction', 'view');
                }
            else
                $helpdesk->addParameter('cAction', $action);
            $helpdesk->addParameter('nEmail', $inParameterList->ParameterValue('messageid'), 'int');
            if($helpdesk->execute()){
                $message = array();
                $message = $helpdesk->getAttribute(0);
                $message['action'] = $action;
//---- get the attachments if they exist
                $helpdesk->setMethod('email_Get_Attachments');
                $helpdesk->setUsername($_SESSION['helpdesk']['username']);
                $helpdesk->setPassword($_SESSION['helpdesk']['password']);
		$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
                $helpdesk->addParameter('cType', 'ticket');
                $helpdesk->addParameter('nTypeId', $ticket_id, 'int');
                $helpdesk->addParameter('cAction', 'view');
                $helpdesk->addParameter('nEmail', $inParameterList->ParameterValue('messageid'), 'int');
                if($helpdesk->execute()){
                    $message['attachments'] = array();
		    $message['attachments'] = $helpdesk->getAttributesArray();
                    $message['path']        = dirname($_SERVER["SCRIPT_NAME"]);
                }else
                    if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
                        $error = $helpdesk->lastError;
            }else
		if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
		    $error = $helpdesk->lastError;
        }
    }else
	if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
	    $error = $helpdesk->lastError;
//---- creating a new email load the default data
    if (!$error && $inParameterList->ParameterValue('action') == 'new' && $inParameterList->ParameterValue('messageid') == 0){
        $helpdesk->setMethod('email_New');
        $helpdesk->setUsername($_SESSION['helpdesk']['username']);
        $helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
        $helpdesk->addParameter('cType', 'ticket');
        $helpdesk->addParameter('nTypeId', $ticket_id, 'int');
        $helpdesk->addParameter('cAction', 'new');
        $helpdesk->addParameter('nEmail', 0, 'int');
        if($helpdesk->execute()){
            $message = array();
            $message = $helpdesk->getAttribute(0);
            $message['action'] = 'new';
        }else
	    $error = $helpdesk->lastError;
    }
    if (!$error){
	if ($_SESSION['helpdesk']['ticketid'] != $_SESSION['helpdesk']['oldticketid'])
	    $_SESSION['helpdesk']['oldticketid'] = $_SESSION['helpdesk']['ticketid'];
	if ($_SESSION['helpdesk']['oldticketid'] == '0')
	    $_SESSION['helpdesk']['oldticketid'] = $ticket_id;
    }
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
        $error = ($error ? $error . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }

    $ticket['path'] = dirname($_SERVER["SCRIPT_NAME"]);
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('ticket', $ticket));
    if (isset($message))
        $parameterList->AddParameter(new ACF_DirectParameter('message', $message));
    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    return $parameterList;
}


function DC_GetCallsInfo(ACF_ParameterList $inParameterList){
    $ticket    = array('calls' => array(), 'currentPage' => 'calls');
    $newCall   = array();
    $ticket_id = $_SESSION['helpdesk']['ticketid'];
//---- get the call data
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('ticket_Get_Calls');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $ticket_id, 'int');
    if($helpdesk->execute())
		$ticket['calls'] = $helpdesk->getAttributesArray();
    else
		if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
			$error = $helpdesk->lastError;
//---- if adding a new call or if adding a call failed get the customer data
    if ($inParameterList->ParameterValue('customerdata')){
        $_SESSION['helpdesk']['callStartTime'] = time();
        $helpdesk->setMethod('call_New');
        $helpdesk->setUsername($_SESSION['helpdesk']['username']);
        $helpdesk->setPassword($_SESSION['helpdesk']['password']);
		$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
        $helpdesk->addParameter('cType', 'ticket');
        $helpdesk->addParameter('nTypeId', $ticket_id, 'int');
        if($helpdesk->execute()){
            $newCall['default'] = $helpdesk->getAttribute(0);
        }
		else
            if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
                $error = $helpdesk->lastError;
            else
                $newCall['default'] = array();
    }
//---- if adding the call failed preserve the data or if the call is sitting in the session
    if (($inParameterList->ParameterValue('call') && $_SESSION['helpdesk']['error']) || (isset($_SESSION['helpdesk']['newCall']['active']))){
//---- if the session call variable is set assign it to the local variable and clear out the session variable
        if (isset($_SESSION['helpdesk']['newCall']['active'])){
            $newCall['data'] = $_SESSION['helpdesk']['newCall'];
            $_SESSION['helpdesk']['newCall'] = array();
        }
		else
            $newCall['data'] = $inParameterList->ParameterValue('call');
		$newCall['default']['number']  = $newCall['data']['number'];
		$newCall['default']['contact'] = $newCall['data']['contact'];
    }
    if (!$error){
		if ($_SESSION['helpdesk']['ticketid'] != $_SESSION['helpdesk']['oldticketid'])
			$_SESSION['helpdesk']['oldticketid'] = $_SESSION['helpdesk']['ticketid'];
		if ($_SESSION['helpdesk']['oldticketid'] == '0')
			$_SESSION['helpdesk']['oldticketid'] = $ticket_id;
    }
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
        $error = ($error ? $error  . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['status'] = '';
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    if ($newCall)
        $parameterList->AddParameter(new ACF_DirectParameter('newcall', $newCall));
    $parameterList->AddParameter(new ACF_DirectParameter('ticket', $ticket));
    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    return $parameterList;
}


function DC_GetNotesInfo(ACF_ParameterList $inParameterList){
    $ticket    = array('notes' => array(), 'currentPage' => 'notes');
    $newNote   = array();
    $ticket_id = $_SESSION['helpdesk']['ticketid'];
    if ($inParameterList->ParameterValue('startnote'))
		$newNote['active'] = true;
//---- get the note data
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('ticket_Get_Notes');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $ticket_id, 'int');
    if($helpdesk->execute()){
        $results = $helpdesk->getAttribute(0);
        $ticket['notecount'] = $results['itemcount'];
//---- to keep the smarty code simple the array is forced to be indexed if there is only one value
		switch($ticket['notecount']) {
			case 0:
				//$ticket['notes'] = false;
				break;
			case 1:
				array_push($ticket['notes'], $results['items']['items_item']);
				break;
			default:
				$ticket['notes'] = $results['items']['items_item'];
		}

//---- remove non-visible notes if logged in as customer
		if($_SESSION['helpdesk']['useCustWebTicket'] && isset($ticket['notes']) && is_array($ticket['notes'])) {
			foreach($ticket['notes'] as $thisNoteIndex=>$thisNoteData) {
				if(strtoupper($thisNoteData['note_visible']) == 'N') {
					unset($ticket['notes'][$thisNoteIndex]);
					$ticket['notecount']--;
				}
			}
		}

    }
	else
		if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
			$error = $helpdesk->lastError;

    if (!$error){
		if ($_SESSION['helpdesk']['ticketid'] != $_SESSION['helpdesk']['oldticketid'])
			$_SESSION['helpdesk']['oldticketid'] = $_SESSION['helpdesk']['ticketid'];
		if ($_SESSION['helpdesk']['oldticketid'] == '0')
			$_SESSION['helpdesk']['oldticketid'] = $ticket_id;
    }
//---- if adding the note failed preserve the data
    if ($inParameterList->ParameterValue('note') && $_SESSION['helpdesk']['error'])
		$newNote['data'] = $inParameterList->ParameterValue('note');
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
        $error = ($error ? $error . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    if ($newNote)
        $parameterList->AddParameter(new ACF_DirectParameter('newnote', $newNote));
    $parameterList->AddParameter(new ACF_DirectParameter('ticket', $ticket));
    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    return $parameterList;
}


function DC_GetStatusButtons(ACF_ParameterList $inParameterList){
    $buttons = array('data' => array(), 'status' => array());
//---- get the message data
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod('ticket_Get_Status_Progression_List');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $_SESSION['helpdesk']['ticketid'], 'int');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $buttons['data'] = $results['display_fields']['display_fields_item'];
	else
	    array_push($buttons['data'], $results['display_fields']['display_fields_item']);
    }else
        if ($helpdesk->lastError != "ADODB.RecordSet is empty.")
            $buttons['status']['text'] = $helpdesk->lastError;
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('buttons', $buttons));
    return $parameterList;
}


function DC_GetHeaderData(ACF_ParameterList $inParameterList){
    $headerData = array('ticket' => array(), 'status' => array(), 'views' => array('status' => array(), 'category' => array(), 'priority' => array(), 'type' => array()));
//---- create the wow object
    $helpdesk = new WOW_Request('', '', 'wombat');
//---- get the ticket data
    $helpdesk->setMethod('ticket_Load');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('nTicket', $_SESSION['helpdesk']['ticketid'], 'int');
    $helpdesk->addParameter('bLoadExtra', !$_SESSION['helpdesk']['ticketLoaded'], 'boolean'); // if ticketLoaded, don't load extra
    $_SESSION['helpdesk']['ticketLoaded'] = true; // set flag to true to indicate loaded ticket
    if($helpdesk->execute())
        $headerData['ticket'] = $helpdesk->getAttribute(0);
    else
        $headerData['status']['text'] = (isset($headerData['status']['text']) ? $headerData['status']['text'] . "\n" : "") . $helpdesk->lastError;
//---- grab the category
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Category_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $headerData['views']['category'] = $list;
//---- grab the priority
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Priority_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $headerData['views']['priority'] = $list;
//---- grab the status
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Status_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $headerData['views']['status']   = $list;
//---- grab the type
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Type_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $headerData['views']['type']     = $list;
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
        $headerData['status']['text'] = (isset($headerData['status']['text']) ? $headerData['status']['text'] . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }

//---- convert $headerdata.tkt_total_time to HH:MM:SS
// mod total time by 60 to get remainder of seconds from minutes
    $timeSeconds = $headerData['ticket']['tkt_total_time'] % 60;
// subtract timeSeconds from totaltime and div by 60 to get total mintues;
// mod result by 60 to get remainder of minutes from hours
    $timeMinutes = (($headerData['ticket']['tkt_total_time'] - $timeSeconds) / 60) % 60;
// subtract timeSeconds and timeMinutes*60 from totaltime and div by 3600 to get total hours
    $timeHours = ($headerData['ticket']['tkt_total_time'] - $timeSeconds - ($timeMinutes * 60)) / 3600;
// for minutes and seconds, display leading 0 if value is less than 9 (ie, so it will show 1:06:08 instead of 1:6:8)
// absolute value is taken for minutes and seconds so that negative times (which normally don't occur) won't show up all weird...
    $headerData['ticket']['tkt_total_time'] = $timeHours . ":" . (abs($timeMinutes) > 9 ? abs($timeMinutes) : "0" . abs($timeMinutes)) . ":" . (abs($timeSeconds) > 9 ? abs($timeSeconds) : "0" . abs($timeSeconds));

//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    if (!$_SESSION['helpdesk']['useCustWebTicket'])
	$parameterList->AddParameter(new ACF_DirectParameter('stafftools', true));
    $parameterList->AddParameter(new ACF_DirectParameter('ticketheaderdata', $headerData));
    $parameterList->AddParameter(new ACF_DirectParameter('ticketid', $_SESSION['helpdesk']['ticketid']));
    return $parameterList;
}


function DC_GetFile(ACF_ParameterList $inParameterList){
    $filename = '';
    $filedata = '';
//---- get the message data
    $helpdesk = new WOW_Request('', '', 'wombat');
    if ($inParameterList->ParameterValue('emailid')){
	$id   = $inParameterList->ParameterValue('emailid');
	$type = 'email';
    }else{
	$id   = $_SESSION['helpdesk']['ticketid'];
	$type = 'ticket';
    }
    $helpdesk->setMethod('attachment_Load');
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('cId', $inParameterList->ParameterValue('id'));
    $helpdesk->addParameter('cType', $type);
    $helpdesk->addParameter('nTypeId', $id, 'int');
    if($helpdesk->execute()){
        $results    = $helpdesk->getAttribute(0);
        $filename   = $results['at_filename'];
        $filedata   = $results['at_file'];

//---- base 64 decode the file
        $filedata = base64_decode(substr($filedata, strpos($filedata, "\n\n") + 1, strlen($filedata) - strpos($filedata, "\n\n") + 1));

//---- if the file is compressed strip off the header and decompress it
        if (strtoupper($results['at_compressed']) != "N") {
	        $filedata = decompress_first_file_from_zip($filedata);
	    }
    }else{
        $filename = 'error.txt';
        $filedata = $helpdesk->lastError;
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('filename', $filename));
    $parameterList->AddParameter(new ACF_DirectParameter('filedata', $filedata));
    return $parameterList;
}

function DC_LoadMessage(ACF_ParameterList $inParameterList) {
	$action = $inParameterList->ParameterValue('action');
	$method = ($action == 'continue') ? 'email_Load' : 'email_New';

	$helpdesk = new WOW_Request('','','wombat');

	$helpdesk->setMethod($method);

	$helpdesk->setusername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');

	$helpdesk->addParameter('cAction',strtoupper($action),'string');
	$helpdesk->addParameter('nEmail',$inParameterList->ParameterValue('messageid'),'int');
	$helpdesk->addParameter('cType','ticket','string');
	$helpdesk->addParameter('nTypeID',$_SESSION['helpdesk']['ticketid'],'int');

	if($helpdesk->execute()) {
		$newmessageData = $helpdesk->getAttribute(0);
		if($newmessageData['em_attachments'] > 0) { // attachments are there, load them... somehow...
			$helpdesk->setMethod('email_Get_Attachments');

			$helpdesk->setusername($_SESSION['helpdesk']['username']);
			$helpdesk->setPassword($_SESSION['helpdesk']['password']);
			$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');

			$helpdesk->addParameter('cAction',strtoupper($action),'string');
			$helpdesk->addParameter('nEmail',$newmessageData['em_id'],'int');
			$helpdesk->addParameter('cType','ticket','string');
			$helpdesk->addParameter('nTypeId',$_SESSION['helpdesk']['ticketid'],'int');

			if($helpdesk->execute()) {
				$newmessageData['attachmentData'] = $helpdesk->getAttributesArray();
			}
		}

		$domdoc = new domDocument();
		$root = $domdoc->createElement('newmessage');
		$root = $domdoc->appendChild($root);
		$id = $domdoc->createElement('id');
		$id = $root->appendChild($id);
		$idData = $domdoc->createTextNode($newmessageData['em_id']);
		$idData = $id->appendChild($idData);
		$to = $domdoc->createElement('to');
		$to = $root->appendChild($to);
		$toData = $domdoc->createTextNode($newmessageData['em_to']);
		$toData = $to->appendChild($toData);
		$from = $domdoc->createElement('from');
		$from = $root->appendChild($from);
		$fromData = $domdoc->createTextNode($newmessageData['em_from']);
		$fromData = $from->appendChild($fromData);
		$cc = $domdoc->createElement('cc');
		$cc = $root->appendChild($cc);
		$ccData = $domdoc->createTextNode($newmessageData['em_cc']);
		$ccData = $cc->appendChild($ccData);
		$bcc = $domdoc->createElement('bcc');
		$bcc = $root->appendChild($bcc);
		$bccData = $domdoc->createTextNode($newmessageData['em_bcc']);
		$bccData = $bcc->appendChild($bccData);
		$subject = $domdoc->createElement('subject');
		$subject = $root->appendChild($subject);
		$subjectData = $domdoc->createTextNode($newmessageData['em_subject']);
		$subjectData = $subject->appendChild($subjectData);
		$body = $domdoc->createElement('body');
		$body = $root->appendChild($body);
		$bodyData = $domdoc->createTextNode($newmessageData['em_body']);
		$bodyData = $body->appendChild($bodyData);

		if(isset($newmessageData['attachmentData']) && is_array($newmessageData['attachmentData'])) {
			$attachments = $domdoc->createElement('attachments');
			$attachments = $root->appendChild($attachments);
			foreach($newmessageData['attachmentData'] as $thisAttachment) {
				$attachment = $domdoc->createElement('attachment');
				$attachment = $attachments->appendChild($attachment);
				$attachid = $domdoc->createElement('attachment_id');
				$attachid = $attachment->appendChild($attachid);
				$attachidData = $domdoc->createTextNode($thisAttachment['at_id']);
				$attachidData = $attachid->appendChild($attachidData);
				$attachname = $domdoc->createElement('attachment_filename');
				$attachname = $attachment->appendChild($attachname);
				$attachnameData = $domdoc->createTextNode($thisAttachment['exp']);
				$attachnameData = $attachname->appendChild($attachnameData);
				$attachsize = $domdoc->createElement('attachment_size');
				$attachsize = $attachment->appendChild($attachsize);
				$attachsizeData = $domdoc->createTextNode($thisAttachment['at_size']);
				$attachsizeData = $attachsize->appendChild($attachsizeData);
			}
		}

		$xml = $domdoc->saveXML();

		$parameterList = new ACF_ParameterList();
		$parameterList->AddParameter(new ACF_DirectParameter('xml',$xml));
		return $parameterList;
	}
}

function DC_PopulateWombatControl(ACF_ParameterList $inParameterList) {
	$type	= $inParameterList->ParameterValue('wombattype');
	$typeid	= $inParameterList->ParameterValue('typeid');
	$ticketid = $inParameterList->ParameterValue('ticketid');

	$helpdesk = new WOW_Request('', '', 'wombat');

	switch($type) {
		case "message":
			$helpdesk->setMethod('email_Load');
			$helpdesk->setUsername($_SESSION['helpdesk']['username']);
			$helpdesk->setPassword($_SESSION['helpdesk']['password']);
			$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
			$helpdesk->addParameter('cType', 'ticket');
			$helpdesk->addParameter('nTypeId', $ticketid, 'int');
//---- if an attachment was added (or attempted to be added) load the message differently
			$helpdesk->addParameter('cAction', 'view');
			$helpdesk->addParameter('nEmail', $typeid, 'int');

			if($helpdesk->execute()){
				$message = array();
				$message = $helpdesk->getAttribute(0);
//---- get the attachments if they exist
				$helpdesk->setMethod('email_Get_Attachments');
				$helpdesk->setUsername($_SESSION['helpdesk']['username']);
				$helpdesk->setPassword($_SESSION['helpdesk']['password']);
				$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
				$helpdesk->addParameter('cType', 'ticket');
				$helpdesk->addParameter('nTypeId', $ticketid, 'int');
				$helpdesk->addParameter('cAction', 'view');
				$helpdesk->addParameter('nEmail', $typeid, 'int');

				if($helpdesk->execute()) {
					$message['attachments'] = array();
					$message['attachments'] = $helpdesk->getAttributesArray();
					$message['path']        = dirname($_SERVER["SCRIPT_NAME"]);
				} else {
					if($helpdesk->lastError != "ADODB.RecordSet is empty.") {
						$error = $helpdesk->lastError;
					}
				}
			} else {
				if($helpdesk->lastError != "ADODB.RecordSet is empty.") {
					$error = $helpdesk->lastError;
				}
			}
			break;
		case "note":
			$helpdesk->setMethod('ticket_Get_Notes');
			$helpdesk->setUsername($_SESSION['helpdesk']['username']);
			$helpdesk->setPassword($_SESSION['helpdesk']['password']);
			$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
			$helpdesk->addParameter('nTicket', $ticketid, 'int');
			if($helpdesk->execute()){
				$results = $helpdesk->getAttribute(0);
				if($results['itemcount'] > 1 && isset($results['items']['items_item']) && is_array($results['items']['items_item'])) {
					foreach($results['items']['items_item'] as $thisItem) {
						if($thisItem['note_id'] == $typeid) { // the selected note
							$note = $thisItem;
						}
					}
				} else {
					if($results['items']['items_item']['note_id'] == $typeid) { // the selected note
						$note = $results['items']['items_item'];
					}
				}
			} else {
				if ($helpdesk->lastError != "ADODB.RecordSet is empty.") {
					$error = $helpdesk->lastError;
				}
			}

			$displayedText = $note['note_body'];
			$noteVisible = $note['note_visible'];
			break;
		case "call":
			$helpdesk->setMethod('ticket_Get_Calls');
			$helpdesk->setUsername($_SESSION['helpdesk']['username']);
			$helpdesk->setPassword($_SESSION['helpdesk']['password']);
			$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
			$helpdesk->addParameter('nTicket', $ticketid, 'int');
			if($helpdesk->execute()) {
				$results = $helpdesk->getAttributesArray();
				if(isset($results) && is_array($results)) {
					foreach($results as $thisCall) {
						if($thisCall['call_id'] == $typeid) { // the selected call
							$call = $thisCall;
						}
					}
				}
			} else {
				if ($helpdesk->lastError != "ADODB.RecordSet is empty.") {
					$error = $helpdesk->lastError;
				}
			}

			$displayedText = $call['call_note'];
			break;
		case "attachment":
			if($inParameterList->ParameterValue('emailattachments')) {
				$showemailattachments = $inParameterList->ParameterValue('emailattachments');
			} else {
				$showemailattachments = false;
			}

			$helpdesk->setMethod('ticket_Get_Attachments');
			$helpdesk->setUsername($_SESSION['helpdesk']['username']);
			$helpdesk->setPassword($_SESSION['helpdesk']['password']);
			$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
			$helpdesk->addParameter('nTicket', $ticketid, 'int');
			$helpdesk->addParameter('bIncludeEmail',$showemailattachments,'boolean');
			if($helpdesk->execute()) {
				$attachments = $helpdesk->getAttributesArray();
			} else {
				if ($helpdesk->lastError != "ADODB.RecordSet is empty.") {
					$error = $helpdesk->lastError;
				}
			}
			break;
	}

	$domdoc = new domDocument();
	$root = $domdoc->createElement($type.'data');
	$root = $domdoc->appendChild($root);

	switch($type) {
		case "message":
			$messageroot = $domdoc->createElement('message');
			$messageroot = $root->appendChild($messageroot);
			$messagetext = $domdoc->createTextNode($message['em_body']);
			$messagetext = $messageroot->appendChild($messagetext);

			$messagedirection = $domdoc->createElement('direction');
			$messagedirection = $root->appendChild($messagedirection);
			$messagedirectionValue = $domdoc->createTextNode($message['em_type']);
			$messagedirectionValue = $messagedirection->appendChild($messagedirectionValue);

			$messagesent = $domdoc->createElement('sent');
			$messagesent = $root->appendChild($messagesent);
			$messagesentValue = $domdoc->createTextNode($message['em_sent']);
			$messagesentValue = $messagesent->appendChild($messagesentValue);

			if(isset($message['attachments']) && is_array($message['attachments'])) {
				$attachmentdata = $domdoc->createElement('attachments');
				$attachmentdata = $root->appendChild($attachmentdata);
				foreach($message['attachments'] as $thisAttachment) {
					$attachment = $domdoc->createElement('attachment');
					$attachment = $attachmentdata->appendChild($attachment);
					$fileid = $domdoc->createElement('fileid');
					$fileid = $attachment->appendChild($fileid);
					$fileidtext = $domdoc->createTextNode($thisAttachment['at_id']);
					$fileidtext = $fileid->appendChild($fileidtext);
					$filename = $domdoc->createElement('filename');
					$filename = $attachment->appendChild($filename);
					$filenametext = $domdoc->createTextNode($thisAttachment['exp']);
					$filenametext = $filename->appendChild($filenametext);
					$filesize = $domdoc->createElement('filesize');
					$filesize = $attachment->appendChild($filesize);
					$filesizetext = $domdoc->createTextNode($thisAttachment['at_size']);
					$filesizetext = $filesize->appendChild($filesizetext);
					$filedesc = $domdoc->createElement('filedesc');
					$filedesc = $attachment->appendChild($filedesc);
					$filedesctext = $domdoc->createTextNode('Email Attachment');
					$filedesctext = $filedesc->appendChild($filedesctext);
				}
			}
			break;
		case "attachment":
			if(isset($attachments) && is_array($attachments)) {
				foreach($attachments as $thisAttachment) {
					$attachment = $domdoc->createElement('attachment');
					$attachment = $root->appendChild($attachment);
					$fileid = $domdoc->createElement('fileid');
					$fileid = $attachment->appendChild($fileid);
					$fileidtext = $domdoc->createTextNode($thisAttachment['at_id']);
					$fileidtext = $fileid->appendChild($fileidtext);
					$filename = $domdoc->createElement('filename');
					$filename = $attachment->appendChild($filename);
					$filenametext = $domdoc->createTextNode($thisAttachment['at_filename']);
					$filenametext = $filename->appendChild($filenametext);
					$filesize = $domdoc->createElement('filesize');
					$filesize = $attachment->appendChild($filesize);
					$filesizetext = $domdoc->createTextNode($thisAttachment['at_size']);
					$filesizetext = $filesize->appendChild($filesizetext);
					$filedesc = $domdoc->createElement('filedesc');
					$filedesc = $attachment->appendChild($filedesc);
					$filedesctext = $domdoc->createTextNode($thisAttachment['at_description']);
					$filedesctext = $filedesc->appendChild($filedesctext);
					$filetype = $domdoc->createElement('filetype');
					$filetype = $attachment->appendChild($filetype);
					$filetypetext = $domdoc->createTextNode(strtolower($thisAttachment['at_idtype']));
					$filetypetext = $filetype->appendChild($filetypetext);
					$filetypeid = $domdoc->createElement('filetypeid');
					$filetypeid = $attachment->appendChild($filetypeid);
					$filetypeidtext = $domdoc->createTextNode($thisAttachment['at_idnum']);
					$filetypeidtext = $filetypeid->appendChild($filetypeidtext);
				}
			}
			break;
		case "note":
			$data = $domdoc->createElement($type);
			$data = $root->appendChild($data);
			$datatext = $domdoc->createTextNode($displayedText);
			$datatext = $data->appendChild($datatext);
			$external = $domdoc->createElement('external');
			$external = $root->appendChild($external);
			$externalValue = $domdoc->createTextNode($noteVisible);
			$externalValue = $external->appendChild($externalValue);
			break;
		default:
			$data = $domdoc->createElement($type);
			$data = $root->appendChild($data);
			$datatext = $domdoc->createTextNode($displayedText);
			$datatext = $data->appendChild($datatext);
	}

	$xml = $domdoc->saveXML();

	$parameterList = new ACF_ParameterList();
	$parameterList->AddParameter(new ACF_DirectParameter('xml',$xml));
	return $parameterList;
}

?>
