<?php
/*---------------------------------------------------------------------------------------------------------*/
/*---- This file generates data for the tickettree templates                                           ----*/
/*---- All functions pull the information from the object by invoking the helpdesk object function.    ----*/
/*---- Then this data is put into a collection of arrays (this makes the ACF less cumbersome)          ----*/
/*---- This data is then passed to the Smarty object to be used in conjunction with the template file  ----*/
/*---------------------------------------------------------------------------------------------------------*/

require_once('../../libs/platypus/wow.inc.php');
require_once("../../libs/ccs/options.inc.php");
//---- load the CCS Options have to laod it here because this is the first manditory function call
if (!$_SESSION['ccs'])
    loadCCSOptions();


function DC_GetViewList(ACF_ParameterList $inParameterList){
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('viewable', $_SESSION['helpdesk']['tickettree']['viewable']));
    $parameterList->AddParameter(new ACF_DirectParameter('view', array('default' => $_SESSION['helpdesk']['view'])));
    return $parameterList;
}


function DC_GetFilterList(ACF_ParameterList $inParameterList){
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('catFilters', $_SESSION['helpdesk']['tickettree']['categoryFilters']));
    return $parameterList;
}

function DC_PerformSearch(ACF_ParameterList $inParameterList){
//---- get the return set from the query
    if ($inParameterList->ParameterValue('query') !== '')
        $_SESSION['helpdesk']['query'] = $inParameterList->ParameterValue('query');
    $searchResults = array();
    if ($_SESSION['helpdesk']['query']){
	$helpdesk = new WOW_Request('', '', 'wombat');
	$helpdesk->setMethod('customer_Search');
	$helpdesk->setUsername($_SESSION['helpdesk']['username']);
	$helpdesk->setPassword($_SESSION['helpdesk']['password']);
	$helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
	$helpdesk->addParameter('cSearchString', $_SESSION['helpdesk']['query']);
	if($helpdesk->execute())
	    $searchResults = $helpdesk->getAttributesArray();
	else
	    $error = $helpdesk->lastError;
    }
//---- convert the results to xml
    $oDomDoc = new domDocument();
    if ($error){
	$root = $oDomDoc->createElement('error');
	$oDomDoc->appendChild($root);
	$root->appendChild($oDomDoc->createTextNode(trim($error)));
    }else{
	$root = $oDomDoc->createElement('customerdata');
	$oDomDoc->appendChild($root);
	foreach($searchResults as $item){
	    $customer = $root->appendChild($oDomDoc->createElement('customer'));
	    $id = $customer->appendChild($oDomDoc->createAttribute('id'));
	    $id->appendChild($oDomDoc->createTextNode(trim($item['id'])));
	    $name = $customer->appendChild($oDomDoc->createAttribute('name'));
	    $name->appendChild($oDomDoc->createTextNode(trim($item['name'])));
	    $username= $customer->appendChild($oDomDoc->createAttribute('username'));
	    $username->appendChild($oDomDoc->createTextNode(trim($item['username'])));
	    $contact= $customer->appendChild($oDomDoc->createAttribute('contact'));
	    $contact->appendChild($oDomDoc->createTextNode(trim($item['contact'])));
	    $attention= $customer->appendChild($oDomDoc->createAttribute('attention'));
	    $attention->appendChild($oDomDoc->createTextNode(trim($item['attn'])));
	    $phone= $customer->appendChild($oDomDoc->createAttribute('phone'));
	    $phone->appendChild($oDomDoc->createTextNode(trim($item['phone'])));
	    $type= $customer->appendChild($oDomDoc->createAttribute('type'));
	    $type->appendChild($oDomDoc->createTextNode(trim($item['type'])));
	    $data_id= $customer->appendChild($oDomDoc->createAttribute('data_id'));
	    $data_id->appendChild($oDomDoc->createTextNode(trim($item['data_id'])));
	    $active= $customer->appendChild($oDomDoc->createAttribute('active'));
	    $active->appendChild($oDomDoc->createTextNode(trim($item['active'])));
	}
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('xml', $oDomDoc->savexml()));
    return $parameterList;
}


function DC_GetNewTicketInfo(ACF_ParameterList $inParameterList){
    $ticket = array('staff' => array(), 'views' => array('category' => array(), 'priority' => array(), 'status' => array(), 'type' => array()));
    if ($inParameterList->ParameterValue('ticket'))
        $ticket = $inParameterList->ParameterValue('ticket');

    if(!$_SESSION['helpdesk']['useCustWebTicket']) {
        $helpdesk = new WOW_Request('', '', 'wombat');
//---- grab the staff list if logged in as staff
        $helpdesk->setMethod('get_Staff_List');
        $helpdesk->setUsername($_SESSION['helpdesk']['username']);
        $helpdesk->setPassword($_SESSION['helpdesk']['password']);
        $helpdesk->addAttribute('logintype', 'staff');
//---- calls use a different security tag
        if ($inParameterList->ParameterValue('call'))
            $helpdesk->addParameter('cType', 'newcallstafflist');
        else
            $helpdesk->addParameter('cType', 'newticketstafflist');

	if($helpdesk->execute())
            $ticket['staff'] = $helpdesk->getAttributesArray();
        else
            $error = $helpdesk->lastError;
    }

//---- grab the category
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Category_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('bCreateableOnly', true, 'boolean');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $ticket['views']['category'] = $list;
//---- grab the priority
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Priority_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('bCreateableOnly', true, 'boolean');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $ticket['views']['priority'] = $list;
//---- grab the status
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Status_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('bCreateableOnly', true, 'boolean');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $ticket['views']['status']   = $list;
//---- grab the type
    $list = array();
    $helpdesk = new WOW_Request('', '', 'wombat');
    $helpdesk->setMethod("ticket_Get_Type_List");
    $helpdesk->setUsername($_SESSION['helpdesk']['username']);
    $helpdesk->setPassword($_SESSION['helpdesk']['password']);
    $helpdesk->addAttribute('logintype', $_SESSION['helpdesk']['useCustWebTicket'] ? 'customer' : 'staff');
    $helpdesk->addParameter('bCreateableOnly', true, 'boolean');
//---- only grab the customer visible ones if a customer is logged in
    if ($_SESSION['helpdesk']['useCustWebTicket'])
	$helpdesk->addParameter('bCustomerVisible', true, 'boolean');
    if($helpdesk->execute()){
        $results  = $helpdesk->getAttribute(0);
	if ($results['display_fields']['display_fields_item'][0])
	    $list = $results['display_fields']['display_fields_item'];
	else
	    array_push($list, $results['display_fields']['display_fields_item']);
    }else
	$_SESSION['helpdesk']['error'] = ($_SESSION['helpdesk']['error'] ? $_SESSION['helpdesk']['error'] . "\n" : "") . $helpdesk->lastError;
    $ticket['views']['type']     = $list;
//---- if an error was set preserve it
    if ($_SESSION['helpdesk']['error']){
        $error = ($error ? $error . "\n" : "") . $_SESSION['helpdesk']['error'];
        $_SESSION['helpdesk']['error'] = '';
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('ticket', $ticket));
//---- add values to the parameter object
    $parameterList->AddParameter(new ACF_DirectParameter('customer', array('id'  => $inParameterList->ParameterValue('custid'), 'contact' => $inParameterList->ParameterValue('contact'))));
    $parameterList->AddParameter(new ACF_DirectParameter('error', $error));
    return $parameterList;
}


function DC_GetBranch(ACF_ParameterList $inParameterList){
    if ($inParameterList->ParameterValue('sections'))
	$sections = explode(",", $inParameterList->ParameterValue('sections'));
    else
	$sections = array('root');
    $tree = $_SESSION['helpdesk']['tickettree']['tree'];
    $oDomDoc = new domDocument();
    $treeDom = $oDomDoc->createElement('tree');
    $oDomDoc->appendChild($treeDom);
    if (isset($tree['root'])){
	foreach($sections as $section){
	    $tickets = false;
	    $groups  = false;
//---- grab headers
	    if ($section == 'root'){
		$root = $treeDom->appendChild($oDomDoc->createElement('root'));
		foreach ($tree['root'] as $key => $view){
		    $viewType = substr($_SESSION['helpdesk']['view'], 0, 1) . "t_name";
		    $viewDom  = $root->appendChild($oDomDoc->createElement('view'));
		    $name = $viewDom->appendChild($oDomDoc->createAttribute('name'));
		    $name->appendChild($oDomDoc->createTextNode($key));
		    $type = $viewDom->appendChild($oDomDoc->createAttribute('id'));
		    $type->appendChild($oDomDoc->createTextNode($view[id]));
		    $type = $viewDom->appendChild($oDomDoc->createAttribute('type'));
		    $type->appendChild($oDomDoc->createTextNode('folder'));
		    $title = $viewDom->appendChild($oDomDoc->createAttribute('title'));
		    $title->appendChild($oDomDoc->createTextNode($view['title'] . ' (' . $view['tkt_bold_count'] . ':' . $view['tkt_total_count'] . ')'));
		    $bold = $viewDom->appendChild($oDomDoc->createAttribute('bold'));
		    $bold->appendChild($oDomDoc->createTextNode($view['tkt_bold_count'] > 0 ? 'true' : 'false'));
		}
	    }else{
//---- strip off the xPath
		$sect = substr($section, strrpos($section, "/") + 1);
		if (strpos($sect, ":") !== false)
		    $sect = substr($sect, 0, strpos($sect, ":"));
		if (strpos($sect, "ticket") === 0)
		    $section = $treeDom->appendChild($oDomDoc->createElement($sect));
		else{
		    $refPoint = $tree;
		    $path = $section;
		    while($path && isset($refPoint)){
			$pos = strpos($path, "/");
			if ($pos && isset($refPoint[substr($path, 0, $pos)])){
			    $refPoint = $refPoint[substr($path, 0, $pos)];
			    $path = substr($path, $pos+1);
			}elseif ($pos === false && isset($refPoint[$path])){
				$refPoint = $refPoint[$path];
				break;
			    }else
				unset($refPoint);
		    }
		    if (!is_array($refPoint))
			continue;
		    else{
			if (isset($refPoint['tickets'])){
			    $refPoint = $refPoint['tickets'];
			    $tickets  = true;
			}elseif (isset($refPoint['groups'])){
				$viewId   = $refPoint['id'];
				$refPoint = $refPoint['groups'];
				$groups   = true;
			    }else{
				unset($refPoint);
				continue;
			    }
		    }
		    $viewDom = $treeDom->appendChild($oDomDoc->createElement('view'));
		    $name = $viewDom->appendChild($oDomDoc->createAttribute('path'));
		    $name->appendChild($oDomDoc->createTextNode($section . ":folder"));
		    if ($tickets){
//---- grab tickets
			foreach ($refPoint as $ticketData){
			    $titleText = trim($ticketData['tkt_customer_name']) . ' - ' . str_replace('T', ' ', $ticketData['tkt_date_entered']) . ($ticketData['tkt_staff_assigned'] < 1 ? ' - (Unassigned)' : '') . ' - ' . trim($ticketData['tkt_problem']);
			    $ticketDom = $viewDom->appendChild($oDomDoc->createElement('ticket'));
			    $ticketDom->appendChild($oDomDoc->createTextNode(trim($ticketData['tkt_id'])));
			    $id = $ticketDom->appendChild($oDomDoc->createAttribute('id'));
			    $id->appendChild($oDomDoc->createTextNode(trim($ticketData['tkt_id'])));
			    $type = $ticketDom->appendChild($oDomDoc->createAttribute('type'));
			    $type->appendChild($oDomDoc->createTextNode('leaf'));
			    $title = $ticketDom->appendChild($oDomDoc->createAttribute('title'));
			    $title->appendChild($oDomDoc->createTextNode($titleText));
			    $bold = $ticketDom->appendChild($oDomDoc->createAttribute('bold'));
			    $bold->appendChild($oDomDoc->createTextNode($ticketData['tkt_modified'] == 'Y' ? 'true' : 'false'));
			}
		    }elseif ($groups){
//---- grab groups
			foreach ($refPoint as $groupKey => $groupData){
			    $month = '';
			    switch($groupData['tkt_date_month']){
				case "1"  :
				case "01" :
				    $month = 'January';
				    break;
				case "2"  :
				case "02" :
				    $month = 'February';
				    break;
				case "3"  :
				case "03" :
				    $month = 'March';
				    break;
				case "4"  :
				case "04" :
				    $month = 'April';
				    break;
				case "5"  :
				case "05" :
				    $month = 'May';
				    break;
				case "6"  :
				case "06" :
				    $month = 'June';
				    break;
				case "7"  :
				case "07" :
				    $month = 'July';
				    break;
				case "8"  :
				case "08" :
				    $month = 'August';
				    break;
				case "9"  :
				case "09" :
				    $month = 'September';
				    break;
				case "10" :
				    $month = 'October';
				    break;
				case "11" :
				    $month = 'November';
				    break;
				case "12" :
				    $month = 'December';
				    break;
			    }
			    $titleText = 'Group for ' . $month . ' ' .  $groupData['tkt_date_year'] . ' (' . $groupData['tkt_bold_count'] .'/' . $groupData['tkt_total_count'] . ')';
			    $groupDom = $viewDom->appendChild($oDomDoc->createElement('view'));
			    $groupDom->appendChild($oDomDoc->createTextNode($groupKey));
			    $title = $groupDom->appendChild($oDomDoc->createAttribute('title'));
			    $title->appendChild($oDomDoc->createTextNode($titleText));
			    $month = $groupDom->appendChild($oDomDoc->createAttribute('month'));
			    $month->appendChild($oDomDoc->createTextNode(trim($groupData['tkt_date_month'])));
			    $year = $groupDom->appendChild($oDomDoc->createAttribute('year'));
			    $year->appendChild($oDomDoc->createTextNode(trim($groupData['tkt_date_year'])));
			    $name = $groupDom->appendChild($oDomDoc->createAttribute('name'));
			    $name->appendChild($oDomDoc->createTextNode("groups/" . $groupKey));
			    $type = $groupDom->appendChild($oDomDoc->createAttribute('id'));
			    $type->appendChild($oDomDoc->createTextNode($viewId));
			    $type = $groupDom->appendChild($oDomDoc->createAttribute('type'));
			    $type->appendChild($oDomDoc->createTextNode('folder'));
			    $bold = $groupDom->appendChild($oDomDoc->createAttribute('bold'));
			    $bold->appendChild($oDomDoc->createTextNode($groupData['tkt_bold_count'] > 0 ? 'true' : 'false'));
			}
		    }
		}
	    }
	}
    }
//---- create a parameter object
    $parameterList = new ACF_ParameterList();
    $parameterList->AddParameter(new ACF_DirectParameter('xml', $oDomDoc->savexml()));
    return $parameterList;
}
?>
